#usr/bin/perl -w
#
#This program aims to run PennCNV, GNOSIS and QuantiSNPv1.1 in two clicks. These prediction algorithms need to already be present on the machine prior to using this program.
#To run a specific section of this program use one of the options below eg --cleanup.

#The location of each program should be written below:
$QTloc = 'F:/QuantiSNPv1.1/';
$PNloc = 'F:/penncnv/';
#$GNloc = 'F:/GNOSIS0.999/';
#$MUloc = 'F:/Murim/v0.76_012309/';

#This is the location of the files for annotation
$ANloc = 'C:/Annotation/';
$QT2loc = 'F:/QuantiSNPv2.3/';
#Many of the options look for FinalReport in file names, to change this alter the option below (eg $namematch = 'Hapmap_';)
$namematch = 'FinalReport';
#This is the name of the program so that it knows how to refer to itself in batch files
$progname = "Combined_CNVv1.73.pl";
#Autoflush - allows communication with Java for questions
$|=1;

use Getopt::Long;
use threads;
use threads::shared;
use Cwd;
use POSIX qw(strftime);

#Identifies current directory
$cdir = getcwd;
print "Current: $cdir\n";
$curr_dir = $cdir . "/";
$res_dir = "$curr_dir" . "01_Raw_Results/";
$pro_dir = "$curr_dir" . "02_Processed/";
$merge_dir = "$curr_dir" . "03_Merged/";
$fin_dir = "$curr_dir" . "04_Final_Results/";
$plink_dir = "$curr_dir" . "07_Plink/";

#Checks the time
$timeanddate = &showtime;
#This prevents Excel interpreting the date:
#$timeanddate = '\''.$timeanddate.'\'';
#print "Time: $timeanddate\n";


GetOptions(  'start' => \$startall,
             'start2' => \$startall2,
             'setup' => \$setmeup,
             'sexsift' => \$sexsift,
             'sexsiftbatch' => \$sexsift_batch,
             'input' => \$inputer,
             'input2' => \$inputer2,
             'input3' => \$inputer3,
             'convert' => \$convert,
             'convertbatch' => \$convert_batch,
             'qtbatch' => \$qt_batch,
             'qt2batch' => \$qt2_batch,
             'pnbatch' => \$pn_batch,
             'cnv' => \$cnvfind,
             'cnv2' => \$cnvfind2,
             'cnvbatch' => \$cnv_batch,
             'cnvbatch2' => \$cnv_batch2,
             'moveGNres' => \$moveGNres,
             'gnformat' => \$GNformat,
             'pnformat' => \$PNformat,
             'qtformat' => \$QTformat,
             'qt2format' => \$QT2format,
             'hdformat' => \$HDformat,
             'merge' => \$merge_prog,
             'rare' => \$rarealg,
             'denovo' => \$denovoalg,
             'inquire' => \$inquire,
             'annotate' => \$annotatefxn,
             'cleanup' => \$clean,
             'moveMU' => \$moveMU,
             'muformat' => \$MUformat,
             'homodel' => \$homodelfind,
             'homodelbatch' => \$homodelfind_batch,
             'plot' => \$plot,
             'denovo2' => \$denovo2,
             'parent' => \$parent_origin,
             'zoomout' => \$zoomout, #used with plot
             'zoomin' => \$zoomin, #used with plot
             'realdata' => \$giverealdata, #used with denovo2
             'loci' => \$locifind,
             'locianno' => \$loci_annotate,
             'filelist' => \$filelistprog,
             'movefiles' => \$movestuff,
             'copyfiles' => \$copystuff,
             'help' => \$help,
             'bigcnv' => \$bigcnv,
             'homozyg' => \$homozyg,
             'badchip' => \$nastychip,
             'pedmod' => \$pedmodify,
             'vlookup' => \$vlookup,
             'split' => \$spliter,
             'combine' => \$combo,
             'freq' => \$freqfind,
             'relfreq' => \$relfreq,
             'thresh' => \$thres, #used with freq or relfreq
             'freqdgv' => \$freq_dgv,
             'snpcount' => \$snpcounter,
             'dbsnpcount' => \$dbsnpcounter,
             'chiperror' => \$chiperror,
             'chiperrorbatch' => \$chiperror_batch,
             'fr_name' => \$fr_name,
             'snp_position' => \$snp_pos,
             'gene_position' => \$gene_pos,
             'gene_position2' => \$gene_pos2,
             'sorter' => \$sorter,
             'fix_gn' => \$gn_chip_input, #used to make GN input files, no longer necessary
             'fr_consensus' => \$fr_consensus,
             'fr_prune' => \$fr_prune,
             'plink_maker' => \$plinkmaker,
             'designqpcrprimers' => \$design_qpcr_primers, 
             'designpcrprimers' => \$design_pcr_primers,
             'qpcr' => \$qpcr_settings, #used as an option in design_primers
             'pcr' => \$pcr_settings, #used as an option in design_primers
             'getdna' => \$get_dna, #Given a list of regions returns the sequence in repeat masked fasta format
             'getdna2' => \$get_dna2, #Given a list of regions returns the sequence in repeat and SNP masked fasta format    
             'findgc' => \$find_gc, #Uses getdna to work out the GC content of each region
             'testfile' => \$testfile_maker, 
             'counter' => \$counter_command, #An idea for batch process, now defunct
             'smallinq' => \$smallinquire,
             'prop' => \$proportion,
             'make_template' => \$make_temp, #A suboption of Prop and Annotate
             'batch' => \$batch, #A suboption of Prop
             'propbatch' => \$propbatch, #An alternative to Prop
             'meanlogr' => \$mean_logr, #Calculates mean, median and stdev for autosomes per samples
             'chrlogr' => \$logr_per_chr,
             'xylogr' => \$xy_logr, #Looks at the LogR for ChrX and ChrY to find Turner's/XXY
             'blanks' => \$blank_remove, #Removes blankspaces in FinalReport names and folders
             'wiggle' => \$wiggle,
             'bedgraph' => \$bedGraph,
             'bedfile' => \$bedfile,
             'fr_gb' => \$fr_gb,
             'linkage' => \$linkage,
             'compare' => \$compare_alg, #Compares the results of two chips to give an idea of coverage
             'delete' => \$deleter, #Deletes file named as ARGV[0], used for batch processing
             'workdir' => \$work_dir, #Sets the Working Directory for the Java Shell
             'ancestry' => \$ancestry_finder,
             'family' => \$family_check,
             'fr_order' => \$fr_order_check,
             'plinkcheck' => \$plink_check,
             'etaqman' => \$e_taqman,
             'overlap' => \$overlap_check,
             'peddetail' => \$ped_detail,
             'marksamp' => \$mark_samples,
             'remhead' => \$rem_head,
             'remdup' => \$rem_dup,
             'reorder' => \$re_order,
             'difference' => \$file_difference,
             'gen_fr' => \$out_FinalReports, #This option generates FinalReport files
);

if ($propbatch == 1){
	$proportion = 1;
	$batch = 1;
}

if( $sexsift == 0
and $sexsift_batch == 0
and $startall2 == 0
and $inputer == 0
and $inputer2 == 0
and $inputer3 == 0
and $convert == 0
and $convert_batch == 0
and $moveGNres == 0
and $clean == 0
and $PNformat == 0
and $GNformat == 0
and $QTformat == 0
and $QT2format == 0
and $HDformat == 0
and $MUformat == 0
and $qt_batch == 0
and $qt2_batch == 0
and $pn_batch == 0
and $merge_prog == 0
and $rarealg == 0
and $denovoalg == 0
and $inquire == 0
and $moveMU == 0
and $MUformat == 0
and $homodelfind == 0
and $homodelfind_batch == 0
and $freqfind == 0
and $locifind == 0
and $loci_annotate == 0
and $filelistprog == 0
and $movestuff == 0
and $copystuff == 0
and $help == 0
and $bigcnv == 0
and $homozyg == 0
and $nastychip == 0
and $pedmodify == 0
and $vlookup == 0
and $relfreq == 0
and $spliter == 0
and $combo == 0
and $snpcounter == 0
and $dbsnpcounter == 0
and $chiperror == 0
and $chiperror_batch == 0
and $fr_name == 0
and $snp_pos == 0
and $gene_pos == 0
and $gene_pos2 == 0
and $sorter == 0
and $gn_chip_input == 0
and $fr_consensus == 0
and $fr_prune == 0
and $plot == 0
and $denovo2 == 0
and $parent_origin == 0
and $annotatefxn == 0
and $plinkmaker == 0
and $design_primers == 0
and $get_dna == 0
and $find_gc == 0
and $testfile_maker == 0
and $cnvfind == 0
and $cnv_batch == 0
and $cnvfind2 == 0
and $cnv_batch2 == 0
and $counter_command == 0
and $setmeup == 0
and $smallinquire == 0
and $proportion == 0
and $make_temp == 0
and $batch == 0
and $mean_logr == 0
and $logr_per_chr == 0
and $xy_logr == 0
and $blank_remove == 0
and $propbatch == 0
and $wiggle == 0
and $bedGraph == 0
and $bedfile == 0
and $fr_gb == 0
and $linkage == 0
and $compare_alg == 0
and $deleter == 0
and $freq_dgv == 0
and $work_dir == 0
and $ancestry_finder == 0
and $family_check == 0
and $fr_order_check == 0
and $plink_check == 0
and $e_taqman == 0
and $overlap_check == 0
and $ped_detail == 0
and $mark_samples == 0
and $rem_head == 0
and $rem_dup == 0
and $re_order == 0
and $file_difference == 0
and $out_FinalReports == 0){
	$startall = 1;
}

$javamethod = 0;
unless ($setmeup == 1 or $help == 1){
	#The location of the Java directory is:
	$JVloc = $curr_dir;
	$visor = $JVloc."CNVision.jar";
	$jvlocfile = $curr_dir."JavaLocator.txt";
	if (-e $visor){
		$javamethod = 1;
		print "Current working directory is:\n";
		$path = <STDIN>; chomp $path;
		$path =~ s/\\/\//gi;
		#print "Path: $path\n";
		$cdir = $path;
		$curr_dir = $cdir . "/";
		chdir $curr_dir; 
		$res_dir = "$curr_dir" . "01_Raw_Results/";
		$pro_dir = "$curr_dir" . "02_Processed/";
		$merge_dir = "$curr_dir" . "03_Merged/";
		$fin_dir = "$curr_dir" . "04_Final_Results/";
		$plink_dir = "$curr_dir" . "07_Plink/";
		$jvlocfile = $curr_dir."JavaLocator.txt";
		print "I am trying to open $jvlocfile\nCurrent directory is $curr_dir\nModified from $cdir\n";
		open (OUT, ">$jvlocfile") || die "Could not open file $jvlocfile; may already be open or may not have write permissions to that folder\n";
		print OUT $JVloc; close OUT;
	}
	elsif (-e $jvlocfile){
		$javamethod = 1;
		open (IN, "<$jvlocfile") || die "Could not open file $jvlocfile; check it has been created\n";
		@JVloc = <IN>; $JVloc = $JVloc[0];
		close IN;
	}
 
	#print "Dir: $curr_dir\n";
	#$cdir = getcwd;
	#print "Dir: $cdir\n";
	
	#Check to see if the PennCNV and QunatiSNP folders are set up correctly
	unless (-e $PNloc . "PN_setup.txt"){
		print "ERROR: PennCNV folder has not been installed correctly\nEnter this command: perl $progname --setup\n";
	}
	unless (-e $QTloc . "QT_setup.txt"){
		print "ERROR: QuantiSNP folder has not been installed correctly\nEnter this command: perl $progname --setup\n";
	}
	#print $ANloc . "AN_setup.txt\n";
	unless (-e $ANloc . "AN_setup.txt"){
		print "ERROR: Annotation folder has not been installed correctly\nEnter this command: perl $progname --setup\n";
	}
	$listout = "Latest_Output.txt";
	open (LISTOUT, ">$listout") || die "Could not open the list of output files $listout; may already be open\n";
}

print "=============================================================================\n";
print "   CNV Pipeline by Stephan Sanders        email: stephan.sanders\@yale.edu\n";
print "   and Christopher Mason                  \n";
print "=============================================================================\n";

#If a logfile is present this is opened to retrieve the Tag then opened in an appending state as LOG1
@files =<*>;
#print "Files: @files\n";
foreach $file (@files){
	if ($file =~ /Logfile/){
		#print "$file\n";
		open (TAGIN, "<$file") || die "Could not open the logfile $file; may already be open\n";
		@logs = <TAGIN>; close TAGIN;
		@tags = split ("\t", $logs[0]);
		$tag = $tags[3];
		chomp $tag;
		@processors = split ("\t", $logs[1]);
		$processors = $processors[3];
		chomp $processors;
		@chips = split ("\t", $logs[3]);
		$chip = $chips[3];
		chomp $chip;
		open (LOG1, ">>$file") || die "Could not open the logfile $file; may already be open\n";
	}
}

if($tag eq ""){
	$method = 1;
}
else{
	$method = 0;
}

#This is the default option, it will start if no specific option has been used or can be invoked with --start; it creates batch files to run the pipeline
if ($startall == 1 or $startall2 == 1){
	
	#print "=============================================================================\n";
	#print "   CNV Pipeline by Stephan Sanders        email: stephan.sanders\@yale.edu\n";
	#print "   and Christopher Mason                  email: christopher.mason\@yale.edu\n";
	#print "=============================================================================\n";
	#print "If this is not the option you want try writing \'$progname --help\'\n";
	
	if ($curr_dir =~ / /){
		@folders = split (/\//,$curr_dir);
		foreach (@folders){
			if ($_ =~ / /){
				die "\nERROR: There are spaces in name of the folder \'$_\'\n$progname will not work correctly unless these spaces are removed\n";
			}
		}
	}
	if ($tag ne ""){
		die "WARNING: There is already a logfile in this folder; move or delete this before proceeding\n";
	}
	$progloc = '"'.$JVloc.$progname.'"';
	mkdir ("01_Raw_Results", 0777);
	mkdir ("03_Merged", 0777);
	mkdir ("02_Processed", 0777);
	mkdir ("04_Final_Results", 0777);
	#Give the project a tag
	print "\n**********************\nTag Name:\n**********************\nPlease give a unique tag for this group of samples without spaces, slashes or\ncolons (eg SSC).\nThen press Enter\n";
	$tag = <STDIN>; chomp $tag;
	print "\n**********************\nNumber of Processors:\n**********************\nHow many processors do you want to use (eg 1, 3, 6).\nThen press Enter\n";
	$processors = <STDIN>; chomp $processors;
	
	#Opens the logfile and Samplelist file
	$out = $tag . "_Logfile.txt";
	open (LOG1, ">$out") || die "Could not open the logfile $out, may already be open\n";
	print LOG1 "$timeanddate\tTag\tINFO\t$tag\n";
	print LOG1 "$timeanddate\tProcessors\tINFO\t$processors\n";
	print LOG1 "$timeanddate\tProg\tINFO\t$progname\n";
	$samplist = $tag. "_Sample_list.txt";
	open (SAMP, ">$samplist") || die "Could not open $samplist, may already be open\n";
	
	
	print "Looking for FinalReport files\n";
	@files = <*>;
	$counthere = 0;
	@frpresent = &fr_name;
	
	if ($processors > 1){
		foreach (1..$processors){
			$prohere = "OUT" . $_;
			$outhere = $tag . "_Processor_list_" . $_ . ".txt";
			open ($prohere, ">$outhere") || die "Could not open Processor file $outhere; may already be open\n";
			$lastout
		}
	}
	print SAMP "SampleID\tFinalReport\tSNPs\tChip\tChip bpm\tProcessor\n";
	
	foreach (@frpresent){
		$filecount2++;
		@tab = split (/\t/,$_);
		chomp @tab;
		
		if ($processor_count >= $processors){
			$processor_count = 1;
		}
		else {
			$processor_count++;
		}
		if ($processors > 1){
			$prohere = "OUT" . $processor_count;
			print $prohere "$tab[0]\t$tab[1]\n";
			$lastout = "$processor_count";
		}
		else {$lastout = 1;}
		
		print SAMP "$tab[1]\t$tab[0]\t$tab[2]\t$tab[3]\t$tab[4]\t$lastout\n";
		if ($tab[3] eq "Unknown"){
			$newchip = $tab[2]."_probe_chip";
			$chipspresent{$newchip} = 1
		}
		else {$chipspresent{$tab[3]} = 1}
	}
	
	@chipkeys = keys %chipspresent;
	$chipnum = @chipkeys;
	
	if ($chipnum == 1){
		print LOG1 "$timeanddate\tChip\tINFO\t@chipkeys\n";
		print "\nChip type is @chipkeys and there are $filecount2 $namematch files\n";sleep(2);
		print LOG1 "$timeanddate\tStart\tINFO\tChip type is @chipkeys and there are $filecount2 $namematch files\n";
	}
	elsif ($chipnum == 0){
		print LOG1 "$timeanddate\tStart\tERROR\tThere are no $namematch files present\n"; sleep(2);
		print LOG1 "$timeanddate\tStart\tERROR\t$progname will now end\n";
		die "There are no $namematch files present\n";
	}
	else {
		print LOG1 "$timeanddate\tStart\tERROR\tThere are $chipnum chip types present in the $filecount2 $namematch files\nThe chip types are:\t";
		print "There are $chipnum chip types present in the $filecount2 $namematch files\nThe chip types are:";
		foreach (@chipkeys){
			print LOG1 "$_\t";
			print "$_\n";
		}
		print LOG1 "$timeanddate\tStart\tERROR\tThis is a serious issue, $progname will not run correctly without running the -fr_consensus and --fr_prune options\n";sleep(2);
		print LOG1 "$timeanddate\tStart\tERROR\t$progname will now end\n";
		die "This is a serious issue, $progname will not run correctly without running the -fr_consensus and --fr_prune options\n";
	}
	
	
	#chdir ("$MUloc");
	#@files = <*>;
	#foreach $file (@files){
	#	if ($file =~ /.tag/){
	#	unlink $MUloc . $file;
	#	}
	#}
	#chdir ("$curr_dir");
	
	#$mutag = $MUloc . "MU_$tag.tag";
	#open (MUTAG, ">$mutag") || "Couldn't open the results file $mutag, may already be open.\n";
	#close MUTAG;
	

	#$murim1 = "perl p00HetCountSort_Final.pl\n";
	#$murim2 = "perl p01DELcnv\n";
	#$murim3 = "perl p01DUPcnv\n";
	
	if ($startall2 == 0){
		open (BATCH, ">Click_me.bat") || die "couldn't open the batch file Click_me.bat, may be open\n";
		open (BATCH2, ">Batch2.bat") || die "couldn't open the batch file Batch2.bat, may be open\n";
	}
	push (@batch1, "perl $progloc --sexsiftbatch\n");
	push (@batch1, "perl $progloc --chiperrorbatch\n");
	push (@batch1, "perl $progloc --convertbatch\n");
	push (@batch1, "perl $progloc --input\n");
	
	#push (@batch1, "cd $MUloc\n");
	#push (@batch1, $murim1);
	#push (@batch1, $murim2);
	#push (@batch1, $murim3);
	#push (@batch1, "cd $curr_dir\n");
	#push (@batch1, "perl $progloc --moveMU\n");
	#push (@batch1, "perl $progloc --moveGN\n");
	push (@batch1, "perl $progloc --cnvbatch\n");
	push (@batch1, "perl $progloc --gnformat\n");
	push (@batch1, "perl $progloc --pnbatch\n");
	push (@batch1, "perl $progloc --pnformat\n");
	push (@batch1, "perl $progloc --qtbatch\n");
	
#	else{
#		$drive = substr $PNloc, 0, 2;
#		push (@batch1, "cd $PNloc\n");
#		push (@batch1, "$drive\n");
#		push (@batch1, $penncnv);
#		push (@batch1, $penncnvx);
#		$drive = substr $curr_dir, 0, 2;
#		push (@batch1, "cd $curr_dir\n");
#		push (@batch1, "$drive\n");
#		push (@batch1, "perl $progloc --pnformat\n");
#		$drive = substr $QTloc, 0, 2;
#		push (@batch1, "cd $QTloc\n");
#		push (@batch1, "$drive\n");
#		push (@batch1, $quantisnp);
#		$drive = substr $curr_dir, 0, 2;
#		push (@batch1, "cd $curr_dir\n");
#		push (@batch1, "$drive\n");
#	}
	push (@batch1, "Batch2.bat\n");
	
	push (@batch2, "perl $progloc --qtformat ".$res_dir.$tag."_QT_results_output.txt\n");
	#push (@batch2, "perl $progloc --muformat\n");
	push (@batch2, "perl $progloc --badchip\n");
	push (@batch2, "perl $progloc --merge " . $pro_dir . "QT_" . $tag . "_Merge.txt " . $pro_dir . "GN_" . $tag . "_Merge.txt " . $pro_dir . "PN_" . $tag . "_Merge.txt\n");
	push (@batch2, "perl $progloc --rare " . $merge_dir . $tag . "_Merged.txt " . $ANloc . "hg18_1_Loci_all_DGV.txt\n");
	push (@batch2, "perl $progloc --pedmod\n");
	push (@batch2, "perl $progloc --denovo " . $merge_dir . "Rare_" . $tag . "_Merged.txt\n");
	push (@batch2, "perl $progloc --inquire " . $merge_dir . "DN_Rare_" . $tag . "_Merged.txt\n");
	push (@batch2, "perl $progloc --annotate " . $merge_dir . $tag . "_CNV_Results.txt\n");
	push (@batch2, "perl $progloc --homodelbatch\n");
	push (@batch2, "perl $progloc --hdformat\n");
	push (@batch2, "perl $progloc --peddetail " . $pro_dir. "HD_" . $tag . "_Merge.txt\n");
	push (@batch2, "perl $progloc --marksamp " . $merge_dir. "Ped_HD_" . $tag . "_Merge.txt " . $merge_dir . $tag . "_Badchips.txt\n");
	push (@batch2, "perl $progloc --annotate " . $merge_dir . $tag . "_HD_Results.txt\n");
	push (@batch2, "perl $progloc --cleanup\n");
	
	foreach (@batch1){
		$_ =~ s/\//\\/gi;
		print BATCH $_;
	}
	
	foreach (@batch2){
		$_ =~ s/\//\\/gi;
		print BATCH2 $_;
	}
	
	#print LOG1 "$thetime\tStart\tMurim\t$murim1";
	#print LOG1 "$thetime\tStart\tMurim\t$murim2";
	#print LOG1 "$thetime\tStart\tMurim\t$murim3";
	
	$command1 = "Click_me.bat\n";
	#To enable true double click functionality activate the line below by removing the '#' at the start
	if ($startall2 == 0){
		system($command1);
	}
}

#Determines the working directory in this has not been given 
if ($work_dir == 1){
	$output = $JVloc."Working_directory.txt";
	open (OUT, ">$output")||die "Could not open $output; may already be open\n";
	
	print "What is the working directory of the files you want to analyse?\n";
	$user_workdir = <STDIN>; chomp $user_workdir;
	
	print OUT "$user_workdir\n";
	close OUT;
}

#This is the annotation function custom annotation file maker
if ($annotatefxn == 1 and $make_temp == 1){
	$outfile = $curr_dir."Custom_annotation.txt";
	open (OUT, ">$outfile") || die "Could not open file $outfile; may already be open\n";
	print OUT "Filename\tColumn 1 Name? (Y or N)\tColumn 1 percentage coverage? (Y or N)\tColumn 1 5' and 3' features? (Y or N)\tInclude column 5? (Name, Num, Total, Max or Min)\tInclude column 6? (Name, Num, Total, Max or Min)\tInclude column 7? (Name, Num, Total, Max or Min)\tInclude column 8? (Name, Num, Total, Max or Min)\tInclude column 9? (Name, Num, Total, Max or Min)\n";
	print LISTOUT $outfile."\n";
	chdir $ANloc;
	@files = <*>;
	@exclude = ( #"hg18_1%_Loci_all_DGV.txt",
	            "AN_setup.txt",
	            "AN_custom_annotation.txt",
	            "hg18_cnvsnps.txt",
	            "Annotation.zip",
	            "hg18_AIMs.txt",
	            "tempanno",
	            "Custom_annotation.txt",
	            "plink.exe",
	            "Hapmap_LD.prune.in",
	            "dbSNP_build130.txt",
	            "dbSNP_build130.zip",
	);
	
	#Genelist
	#$genesfile1 = "hg18_RefSeq_genes_merged_no_duplicate.txt";
	$genesfile1 = "hg18_UCSC_genes_merged_no_duplicate.txt";
	#$genesfile1 = "hg18_ENSEMBL_v50_exons.txt";
	#$genesfile1 = "hg18_Aceview_genes_merged_no_duplicate.txt";
	print OUT "$genesfile1\tGene_list\tN\tN\n";
	push (@exclude, $genesfile1);
	#DGV variation
	$dgvfile = "variation.hg18.v9.mar.2010.txt";
	print OUT "$dgvfile\tDGV_file\tN\tN\n";
	push (@exclude, $dgvfile);
	
	#Hash is made of files in the exclude array
	foreach $exclude (@exclude){
		$exclude{$exclude} = 1;
	}
	
	foreach $file (@files){
		unless ($exclude{$file} == 1){
			#print OUT "$file\t1\t1\n";
			open (IN, "<$file")|| die "Could not open file $file\n";
			while (<IN>){
				@line = split (/\t/,$_); chomp @line;
				print OUT "$file\tY\tN\tN\t$line[4]\t$line[5]\t$line[6]\t$line[7]\t$line[8]\n";
				close IN;
			}
		}
	}
	print "Template created\n";
	print "Output: $outfile\n";
}

#This is the annotation programe, the first bit lists the files that will be used for annotation using a custom annotation file if present
if ($annotatefxn == 1 and $make_temp != 1){
	
	#Looks to see if a custom file is present in the working directory (created by --annotate --make_temp) then loads this into memory
	$custfile = $curr_dir."Custom_annotation.txt";
	$custfile2 = $ANloc."Custom_annotation.txt";
	if (-e $custfile or -e $custfile2){
		print "Loading custom annotation file\n";
		if (-e $custfile){open (CUSTOM, "<$custfile")||die "Could not open custom annotation file $custfile\n";}
		elsif (-e $custfile2){open (CUSTOM, "<$custfile2")||die "Could not open custom annotation file $custfile2\n";}
		{ local $/=undef;  $input1=<CUSTOM>; }
		close CUSTOM;
		($\) = (/(\r\n|\r|\n)/);
		@custom=split (/\r\n|\r|\n/, $input1); chomp(@custom);
		$inputcounter = 1; shift @custom;
		foreach $line (@custom){
			@tab = split (/\t/, $line);
			if ($tab[1] =~ /Gene_list/){$genesfile1 = $tab[0]; $inputfiles[0]=$genesfile1;$firstcolhash{$genesfile1}="Y";$perhash{$genesfile1}="$tab[2]"; $disthash{$genesfile1} = "$tab[3]";}
			elsif ($tab[1] =~ /DGV_file/){$tdgvfile = $tab[0]; $inputfiles[1]=$tdgvfile;$firstcolhash{$tdgvfile}="Y";$perhash{$tdgvfile}="$tab[2]"; $disthash{$tdgvfile} = "$tab[3]";}
			else{
				#$disthash{$tab[0]} = "Y";
				$inputcounter++;
				$tabcount = 0; $firstcolhashinput = "N"; $perhashinput = "N"; $disthashinput = "N";
				$keepcount = 0;
				foreach $tab (@tab){
					$tabcount++;
					if (uc $tab ne "N" or $tab ne "") {$keepcount++;}
					if (uc $tab eq "Y" and $tabcount == 2){$firstcolhashinput = "Y";}
					elsif (uc $tab eq "Y" and $tabcount == 3){$perhashinput = "Y";}
					elsif (uc $tab eq "Y" and $tabcount == 4){$disthashinput = "Y";}
					elsif ($tabcount >= 5){
						#The column type is put into a hash called 'the filename' . 'the column'
						# 'N' or '' means do not include
						# 'Y' or 'Name' means return unique names
						# 'Num' means return number of unique names
						# 'Total' means return total of all calls (not reliable if there are duplicates)
						# 'Max' means return highest number
						# 'Min' means return lowest number
						chomp $tab;
						$hashname = $tab[0];
						$$hashname{$tabcount} = $tab;
						#print "Custom file: Hash: $hashname, Tab: $tabcount, Number: $tab\n";
					}
				}
				#This is where the files are put into the array to be loaded
				unless ($keepcount == 0){$inputfiles[$inputcounter]=$tab[0];}
				$disthash{$tab[0]}=$disthashinput;
				$perhash{$tab[0]}=$perhashinput;
				$firstcolhash{$tab[0]}=$firstcolhashinput;
			}
		}
	}
	else {
		#$genesfile1 = "hg18_RefSeq_genes_merged_no_duplicate.txt";
		$genesfile1 = "hg18_UCSC_genes_merged_no_duplicate.txt";
		#$genesfile1 = "hg18_ENSEMBL_v50_exons.txt";
		#$genesfile1 = "hg18_Aceview_genes_merged_no_duplicate.txt";
		push (@inputfiles, $genesfile1);
	}
	print "Genes: $genesfile1\n";
	print "DGV: $tdgvfile\n";
	
	chdir $ANloc;
	
	@tempfiles=<*>;
	
	@exclude = ( #"hg18_1%_Loci_all_DGV.txt",
	            "AN_setup.txt",
	            "AN_custom_annotation.txt",
	            "hg18_cnvsnps.txt",
	            "Annotation.zip",
	            "hg18_AIMs.txt",
	            "tempanno",
	            "Custom_annotation.txt",
	            "plink.exe",
	            "Hapmap_LD.prune.in",
	            "dbSNP_build130.txt",
	);
	$donotdisturb = join (",", @exclude);
	
	#If there is no custom annotation file in the working directory then all the files in the annotation folder are loaded
	unless (-e $custfile or -e $custfile2){
		#Checks file is not in exclude list
		foreach $foundfile (@tempfiles){
			if (-f $foundfile){
				if ($foundfile ne "AN_setup.txt" && $foundfile ne $genesfile1 && $donotdisturb !~ /$foundfile/){
					push (@inputfiles, $foundfile);
					if ($foundfile =~ /variation.hg18/ && $foundfile !~ /McC/){
						$tdgvfile = $foundfile;
					}
				}
				#print "$foundfile\n";
				#If in this hash with N do not give percent
				$perhash{$foundfile}="N";
				$firstcolhash{$foundfile}="Y";
			}
		}
		
		#This is the list of additional info from files; key is the file name
		open (CUSTOMFILES, "<Custom_annotation.txt") || die "Could not open Custom_annotation.txt in $ANloc.\nPlease check for the presence of this file\n";
		{ local $/=undef;  $input1=<CUSTOMFILES>; }
		close CUSTOMFILES;
		($\) = (/(\r\n|\r|\n)/);
		@customfiles=split (/\r\n|\r|\n/, $input1); chomp(@customfiles);
		foreach $section (@customfiles){
			($filec, $colc, $percc) = split (/\t/, $section); chomp($perc); chomp($colc);
			@columnshere = split (/,/, $colc);
			foreach (@columnshere){
				$$filec{$_} = "Y";
			}
			#$colhash{$filec}=$colc;
			$perhash{$filec}=$percc;
		}
	}

	chdir $curr_dir;
	
	$toprint = "cd $cdir\n";
	#print "we got @inputfiles\n";
	
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
		@folders = split (/\\/,$ARGV[0]);
		@output2 = split (/\//,$folders[-1]);
		#$output =~ s/$curr_dir//gi;
		$newfile1 = "Annot_" . $output2[-1];
		print LISTOUT $curr_dir.$newfile1."\n";
	}
	else{
		$method = 0;
		mkdir ("04_Final_Results", 0777);
		@folders = split (/\\/,$ARGV[0]);
		@output2 = split (/\//,$folders[-1]);
		$newfile1 = $fin_dir . "Annot_" . $output2[-1];
		print LISTOUT $newfile1."\n";
	}
	open(OUT, ">$newfile1")  || die "Could not open the results file $newfile1; may already be open\n";
	print "Loading input file for $ARGV[0]\n";
	if (-e $ARGV[0]){
		open (FINALDATA, "<$ARGV[0]") || die "Could not open the results file $ARGV[0]; check it is in the right directory\n";
	}
	else {
		chdir $fin_dir;
		if (-e $ARGV[0]){
			open (FINALDATA, "<$ARGV[0]") || die "Could not open the results file $ARGV[0]; check it is in the right directory\n";
		}
		else {
			chdir $merge_dir;
			if (-e $ARGV[0]){
				open (FINALDATA, "<$ARGV[0]") || die "Could not open the results file $ARGV[0]; check it is in the right directory\n";
			}
			else {
				chdir $pro_dir;
				if (-e $ARGV[0]){
					open (FINALDATA, "<$ARGV[0]") || die "Could not open the results file $ARGV[0]; check it is in the right directory\n";
				}
				else {
					chdir $res_dir;
					open (FINALDATA, "<$ARGV[0]") || die "Could not open the results file $ARGV[0]; check it is in the right directory\n";
				}
			}
		}
		chdir $curr_dir;
	}
	{ local $/=undef;  $input1=<FINALDATA>; }
	close FINALDATA;
	($\) = (/(\r\n|\r|\n)/);
	@finaldata=split (/\r\n|\r|\n/, $input1); 
	
	@firstone = split (/\t/, $finaldata[0]);
	
	if ($method == 0){
		$hereonly = $tag . "_HD";
		if ($ARGV[0] =~ /$hereonly/){
			$chromocol = 5;
			$upchromodist = 6;
			$downchromodist = 7;
			$headme = "y";
		}
		else {
			$chromocol = 7;
			$upchromodist = 8;
			$downchromodist = 9;
			$headme = "y";
		}
	}
	else {
		print "\n\nHere is the first line of your file:\n";
		for ($pt=0; $pt<=$#firstone; $pt++){
			$outme = ($pt +1);
			print "$outme) $firstone[$pt]\n";
		}
		print "Which column is the chromosome in (1, 2, 3 etc)?\n";
		$user_chromo = <STDIN>; chomp $user_chromo; $chromocol = $user_chromo - 1;
		print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
		$user_start = <STDIN>; chomp $user_start; $upchromodist = $user_start - 1;
		print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
		$user_stop = <STDIN>; chomp $user_stop; $downchromodist = $user_stop - 1;
		print "Is there a header ('y' or 'n')?\n";
		$headme = <STDIN>; chomp $headme;
	}

	&newannotate($ARGV[0], $chromocol, $upchromodist, $downchromodist, $headme);
}

#Overlap check - looks for regions that overlap in the input file, uses subroutine of annotator input files
if ($overlap_check == 1){
	$inputfile = $ARGV[0];
	$outputfile = "Nodup_".$inputfile;
	open (OUT, ">$outputfile")||die "Could not open file $outputfile; may already be open\n";
	open (IN, "<$inputfile")||die "Could not open file $inputfile; check it is in the right directory\n";
	@input1 = <IN>; close (IN); chomp @input1;
	@lineshow = split ("\t", $input1[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	print "Which column is the chromosome in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	print "Does any column have useful data (1, 2, 3, etc or blank if none)?\n";
	$user_column1 = <STDIN>; chomp $user_column1; $column1 = $user_column1 - 1; $colcount = 0;
	unless ($user_column1 eq ''){
		print "Does any other column have useful data (1, 2, 3, etc or blank if none)?\n";
		$user_column2 = <STDIN>; chomp $user_column2; $column2 = $user_column2 - 1; $colcount = 1;
	}
	unless ($user_column2 eq ''){
		print "Does any other column have useful data (1, 2, 3, etc or blank if none)?\n";
		$user_column3 = <STDIN>; chomp $user_column3; $column3 = $user_column3 - 1; $colcount = 2;
	}
	unless ($user_column3 eq ''){
		print "Does any other column have useful data (1, 2, 3, etc or blank if none)?\n";
		$user_column4 = <STDIN>; chomp $user_column4; $column4 = $user_column4 - 1; $colcount = 3;
	}
	unless ($user_column4 eq ''){
		print "Does any other column have useful data (1, 2, 3, etc or blank if none)?\n";
		$user_column5 = <STDIN>; chomp $user_column5; $column5 = $user_column5 - 1; $colcount = 4;
	}
	unless ($user_column5 eq ''){
		$colcount = 5;
	}
	print "Is there a header ('y' or 'n')?\n";
	$head = <STDIN>; chomp $head;
	if ($head =~ /y/){
		$thisthehead = $input1[0];
		shift @input1;
		@tab = split (/\t/, $thisthehead);
		$colin = '';
		chomp $colin;
		foreach $col (1..$colcount){
			$colname = "column".$col;
			$colin = $colin.$tab[$$colname]."\t";
		}
		chomp $colin;
		print OUT "Chr\tStart\tStop\t$colin"."Matches\n";
	}
	
	$uniquenum = 0;
	@input = ();
	print "Adjusting the column order\n";
	#print "Count1: $colcount\n";
	foreach $line (@input1){
		$uniquenum++;
		$uniquename = "Line".$uniquenum;
		unless ($javamethod == 1){
			print "\rDoing line $uniquenum";
		}
		@tab = split (/\t/, $line);
		$colin = '';
		chomp $colin;
		foreach $col (1..$colcount){
			$colname = "column".$col;
			$colin = $colin.$tab[$$colname]."\t";
		}
		#print "$colin\n";
		$inputformat = "$uniquename\t$tab[$chromo]\t$tab[$start]\t$tab[$stop]\t$colin";
		unless ($tab[$chromo] eq ''){
			push (@input, $inputformat);
		}
	}
	
	$flag=0;
	print "\nSorting input file by chromosome and start position\n";
	@input = sort { (split '\t', $a, 4)[2] <=> (split '\t', $b, 4)[2] } @input;
	@input = sort { (split '\t', $a, 4)[1] cmp (split '\t', $b, 4)[1] } @input;
	
	print "Now looking for overlap\n";
	#This looks for overlap in the reference file; if found flag = 1
	$pchromo=''; $pstop=0; $flag=0;
	foreach $piece (@input){
		@pnow = split (/\t/, $piece);
		if ($pnow[1] eq $pchromo && $pnow[2] <= $pstop){
			print "$pnow[1] eq $pchromo && $pnow[2] <= $pstop\n";
			$flag=1;
			last;
		}
		$pchromo=$pnow[1]; $pstop=$pnow[3];
	}
	
	#print "Count: $colcount\n";
	if ($flag==1){
		print "\nOverlapping regions detected; merging regions to correct\n";
		@input = &MergeAnnotation;
		#open (OUT, ">$outputfile")||die "Could not open file $outputfile; may already be open\n";
		print LISTOUT $curr_dir.$outputfile;
		foreach $line (@input){
			#print "Line: $line\n"; sleep (1);
			@tab = split (/\t/, $line);
			$colin = '';
			chomp $colin;
			foreach $col (1..$colcount){
				$colnum = $col + 3;
				$colin = $colin.$tab[$colnum]."\t";
			}
			chomp $colin;
			#print "$colin\n";
			$outputformat = "$tab[1]\t$tab[2]\t$tab[3]\t$colin"."$tab[-1]";
			print OUT "$outputformat\n";
		}
		print "\nOutput file $outputfile created; overlapping terms are separated by commas\n";
	}
	
	else {
		print "There were no overlapping regions in this file\n";
	}
	print "\nAll done\n";
}

#Checks that all FinalReports are in the same order
if ($fr_order_check == 1){
	&fr_columns;
	@files = <*>;
	$output = "Order_check.txt";
	open (OUT, ">$output") || die "Could not open $output; may already be open\n";
	print OUT "File\tSNPs in the wrong order\n";
	print LISTOUT $curr_dir.$output."\n";
	foreach $file (@files){
		if ($file =~ /$namematch/){
			$filecounter++;
			if ($filecounter == 1){
				print "Using file $file as the template\n";
			}
			print "$filecounter) Doing file $file\n";
			$linecount = 0;
			$error = 0;
			open (IN, "<$file") || die "Could not open $file\n";
			while (<IN>){
				$linecount++;
				if ($filecounter == 1){
					if ($linecount > 11){
						@tab = split (/\t/, $_);
						$order{$tab[$snpcol]} = $linecount;
					}
				}
				else {
					if ($linecount > 11){
						@tab = split (/\t/, $_);
						unless ($order{$tab[$snpcol]} == $linecount){
							$error++;
						}
					}
				}
			}
			if ($error > 0){
				print "ERROR: FinalReport $file has $error SNPs in a different order\n";
			}
			print OUT "$file\t$error\n";
		}
	}
	print "All done\n";
}

#Makes a plink file then rtuns check sex, mendel and ibd. Summaries results.
if ($family_check == 1){
	mkdir ("07_Plink", 0777);
	$pedinput1 = $ARGV[0];
	$pedinput = $pedinput1;
	$pedinput =~ s/.txt//gi;
	
	open (PED, "<$pedinput1")||die "Could not open pedfile $pedinput1; check it is present in this directory\n";
	@data = <PED>; chomp @data; close PED;
	$output = "Family_Check_Results.txt";
	open (OUT2, ">$output") || die "Could not open file $output; may already be open\n";
	#print LISTOUT $curr_dir.$output;
	
	#Gives list of all FinalReports
	#FinalReport name = 0, Sample = 1, SNPs = 2, Chip = 3, Bigchip = 4
	@frfiles = &fr_name;
	foreach $line (@frfiles){
		@tab = split (/\t/, $line);
		$frname{$tab[1]} = $tab[0];
		$frsnp{$tab[1]} = $tab[2];
	}
	
	#Copies plink into the current folder
	print "Plink: $plink_dir\n";
	$plink = "plink.exe.";
	chdir $ANloc;
	unless (-e $plink){die "Plink is not present in the Annotation folder ($ANloc)\nPlease download plink from http://pngu.mgh.harvard.edu/~purcell/plink/\n";}
	$newfile = $plink_dir . "/" . $plink;
	copy ($plink, $newfile);
	chdir $curr_dir;
	
	#Makes a batch file and runs the genome command using a pre-made list of SNPs in the Annotation folder (Hapmap indep-pairwise 50 5 0.2
	$ldprune1 = $ANloc."Hapmap_LD.prune.in";
	$output2 = "Plinkbatch.bat";
	open(OUT, ">$output2") || die "Could not open $output2; may already be open\n";
	print OUT "perl \"".$JVloc.$progname."\" --plink_maker ".$pedinput1."\n";
	print OUT "cd $plink_dir\n";
	print OUT "plink --file ".$pedinput." --make-bed --out Binary_".$pedinput."\n";
	print OUT "plink --bfile Binary_".$pedinput." --check-sex\n";
	print OUT "plink --bfile Binary_".$pedinput." --mendel\n";
	#Mind is missingness per individual, geno is missingness per genotype, maf is minor allele frequency, hwe-all means hardy weinberg is ignored
	print OUT "plink --bfile Binary_".$pedinput." --extract $ldprune1 --mind 0.05 --geno 0.1 --maf 0.01 --hwe-all --make-bed --out Binary_".$pedinput."_indep\n";
	print OUT "plink --bfile Binary_".$pedinput."_indep --genome --min 0.05 --out ".$pedinput."\n";
	close(OUT);
	#system("start Plinkbatch.bat");
	system("Plinkbatch.bat");
	print LISTOUT $curr_dir.$output;
	chdir $plink_dir;
	
	#This loads the pedfile into a series of hashes
	&propedfile(@data);
	
#	foreach $line (@data){
#		@tab = split (/\t/, $line);
#		$family = $tab[0];
#		$sample = $tab[1];
#		$father = $tab[2];
#		$mother = $tab[3];
#		$gender = $tab[4];
#		$phenotype = $tab[5];
#		
#		$pedfile{$sample} = $line;
#		$family_member{$family} = $family_member{$family}.$sample."\t";
#		$which_family{$sample} = $family;
#		$which_mother{$sample} = $mother;
#		$which_father{$sample} = $father;
#		unless ($father == 0){$which_child{$father} = $which_child{$father}.$sample."\t";}
#		unless ($mother == 0){$which_child{$mother} = $which_child{$mother}.$sample."\t";}
#		$gender{$sample} = $gender;
#		$phenotype{$sample} = $phenotype;
#	}
#	
#	#This works out who the siblings are (samples from the same family who have the same parents)
#	@samplelist = keys %gender;
#	foreach $sample (@samplelist){
#		$family = $which_family{$sample};
#		$members = $family_member{$family};
#		@members = split (/\t/,$members);
#		foreach $member (@members){
#			if ($member eq $sample){}
#			elsif ($member eq $which_mother{$sample}){}
#			elsif ($member eq $which_father{$sample}){}
#			else {
#				if ($which_mother{$sample} eq $which_mother{$member}
#				and $which_father{$sample} eq $which_father{$member}
#				and $which_mother{$sample} ne 0
#				and $which_father{$sample} ne 0){
#					$which_sibling{$sample} = $which_sibling{$sample}.$member.",";
#				}
#			}
#		}
#	}
	
	#Opens the sexcheck results file and puts this into a hash
	print "Loading results from Sexcheck file\n";
	$sexinput = "plink.sexcheck";
	open (SEX, "<$sexinput")||die "Could not open $sexinput; check check_sex has been run in plink\n";
	@sexcheck = <SEX>; chomp @sexcheck; close SEX;
	shift (@sexcheck);
	#Going through plink sexcheck file. Columns are labelled as follows
	#FID = 0, IID = 1, PEDSEX = 2, SNPSEX = 3, STATUS = 4, F = 5
	foreach $line (@sexcheck){
		#Removes blank space at the start for trailing blank space use s/\s+$//;
		$line =~ s/^\s+//;
		#Replaces all blank spaces with a tab
		$line =~ s/\s+/\t/g;
		#print "Line: $line,\n";
		@tab = split (/\t/,$line);
		$sexcheck{$tab[1]} = $line;
	}
	
	#Opens the mendel results file and puts this into a hash
	print "Loading results from Mendel file\n";
	$meninput = "plink.imendel";
	open (MEN, "<$meninput")||die "Could not open $meninput; check mendel has been run in plink\n";
	@mencheck = <MEN>; chomp @mencheck; close MEN;
	shift (@mencheck);
	#Going through plink mendel file. Columns are labelled as follows
	#FID = 0, IID = 1, N = 2
	foreach $line (@mencheck){
		#Removes blank space at the start for trailing blank space use s/\s+$//;
		$line =~ s/^\s+//;
		#Replaces all blank spaces with a tab
		$line =~ s/\s+/\t/g;
		#print "Line: $line,\n";
		@tab = split (/\t/,$line);
		$mencheck{$tab[1]} = $tab[2];
	}
	
	#Opens the genome results file and puts this into a hash
	print "Loading results from Genome file\n";
	$geninput = "$pedinput.genome";
	open (GEN, "<$geninput")||die "Could not open $geninput; check ibd has been run in plink\n";
	@gencheck = <GEN>; chomp @gencheck; close GEN;
	shift (@gencheck);
	#Going through plink genome file. Columns are labelled as follows:
	#FID1 = 0, IID1 = 1, FID2 = 2, IID2 = 3, RT = 4, EZ = 5, Z0 = 6, Z1 = 7, Z2 = 8, PI_HAT = 9, PHE = 10, DST = 11, PPC = 12, Ratio = 13
	$iid1 = 1;
	$iid2 = 3;
	$rt = 4;
	$z1 = 7;
	$pihat = 9;
	foreach $line (@gencheck){
		#Removes blank space at the start for trailing blank space use s/\s+$//;
		$line =~ s/^\s+//;
		#Replaces all blank spaces with a tab
		$line =~ s/\s+/\t/g;
		#print "Line: $line,\n";
		@tab = split (/\t/,$line);
		if ($tab[$pihat] >= 0.95) {
			$gidentical{$tab[$iid1]} = $gidentical{$tab[$iid1]}.$tab[$iid2].",";
			$gidentical{$tab[$iid2]} = $gidentical{$tab[$iid2]}.$tab[$iid1].",";
		}
		elsif ($tab[$pihat] >= 0.35) {
			if ($tab[$rt] eq "PO") {
				$gparentchild{$tab[$iid1]} = $gparentchild{$tab[$iid1]}.$tab[$iid2].",";
				$gparentchild{$tab[$iid2]} = $gparentchild{$tab[$iid2]}.$tab[$iid1].",";
			}
			elsif ($tab[$rt] eq "FS") {
				$gsibling{$tab[$iid1]} = $gsibling{$tab[$iid1]}.$tab[$iid2].",";
				$gsibling{$tab[$iid2]} = $gsibling{$tab[$iid2]}.$tab[$iid1].",";
			}
			else {
				$ghigh{$tab[$iid1]} = $ghigh{$tab[$iid1]}.$tab[$iid2].",";
				$ghigh{$tab[$iid2]} = $ghigh{$tab[$iid2]}.$tab[$iid1].",";
			}
		}
		elsif ($tab[$pihat] >= 0.1) {
			$glow{$tab[$iid1]} = $glow{$tab[$iid1]}.$tab[$iid2].",";
			$glow{$tab[$iid2]} = $glow{$tab[$iid2]}.$tab[$iid1].",";
		}
	}
	
	#Prints out the results of the sexcheck in the results file
	print "Preparing results file\n";
	print OUT2 "FinalReport\tOverall Status\tFamily\tSample\tFather\tMother\tGender\tPhenotype\tGender Status\tSNP Gender\t%Homozygosity\tMendel Status\tMendelian Errors\tSNPs\tPrecent Mendel Errors\tFather\tMother\tSiblings\tChildren\tIdentical\tParent-Child\tSibling\tHigh\tLow\n";
	@samplelist = sort { $which_gender{$a} cmp $which_gender{$b} } @samplelist;
	@samplelist = sort { $which_mother{$a} cmp $which_mother{$b} } @samplelist;
	@samplelist = sort { $which_family{$a} cmp $which_family{$b} } @samplelist;
	$statusprob = 0;
	foreach $sample (@samplelist){
		unless ($sample eq ''){
			$mendelout = "OK";
			$sexcheck = $sexcheck{$sample};
			$mencheck = $mencheck{$sample};
			$finalreport = $frname{$sample};
			
			#Mendel output
			$snpcount = $frsnp{$sample};
			if ($snpcount > 0){
				$percentmendel = ($mencheck / $snpcount * 100);
			}
			if ($percentmendel > 1){
				$mendelstatus = "PROBLEM";
			}
			else {
				$mendelstatus = "OK";
			}
			
			$mendelout = $mendelstatus."\t".$mencheck."\t".$snpcount."\t".$percentmendel."%";
			
			#Sex check output
			@tab = split (/\t/,$sexcheck);
			$percenthomo = ($tab[5] * 100);
			if ($percenthomo < 40){
				$correctgender = 2;
				if ($correctgender != $tab[2]){
					$genstatus = "PROBLEM";
				}
				else {
					$genstatus = "OK";
				}
			}
			elsif ($percenthomo > 90){
				$correctgender = 1;
				if ($correctgender != $tab[2]){
					$genstatus = "PROBLEM";
				}
				else {
					$genstatus = "OK";
				}
			}
			else {
				$correctgender = 0;
				$genstatus = "PROBLEM";
			}
			$sexout = $genstatus."\t".$correctgender."\t".$percenthomo."%";
			
			#Genome output
			@children = split (/\t/,$which_child{$sample});
			$children = @children;
			$child = "OK";
			if ($children == 0){$child = 0;}
			foreach $childhere (@children){
				if ($gparentchild{$sample} !~ /$childhere/){
					$child = "PROBLEM";
				}
			}
			
			if ($which_mother{$sample} eq 0){$mother = 0;}
			elsif ($gparentchild{$sample} =~ /$which_mother{$sample}/){$mother = "OK";}
			else {$mother = "PROBLEM";}
			
			if ($which_father{$sample} eq 0){$father = 0;}
			elsif ($gparentchild{$sample} =~ /$which_father{$sample}/){$father = "OK";}
			else {$father = "PROBLEM";}
			
			@siblings = split (/,/,$which_sibling{$sample});
			$sibling = 0;
			foreach $sib (@siblings){
				if ($gsibling{$sample} =~ /$sib/i and $sib ne ''){$sibling = "OK";}
				else {$sibling = "PROBLEM";}
			}
			$genomeout = $father."\t".$mother."\t".$sibling."\t".$child."\t".$gidentical{$sample}."\t".$gparentchild{$sample}."\t".$gsibling{$sample}."\t".$ghigh{$sample}."\t".$glow{$sample};
			
			if ($father ne "PROBLEM" and $mother ne "PROBLEM" and $sibling ne "PROBLEM" and $child ne "PROBLEM" and $genstatus ne "PROBLEM" and $mendelstatus ne "PROBLEM"){
				$overallstatus = "OK";
			}
			else {
				$overallstatus = "PROBLEM";
				$statusprob++;
			}
			
			print OUT2 $finalreport."\t".$overallstatus."\t".$pedfile{$sample}."\t".$sexout."\t".$mendelout."\t".$genomeout."\n";
		}
	}
	print "Overall there are $statusprob samples with problems in their pedfile\n";
}

#Determines the Ethnicity of each FinalReport using AIMs
if ($ancestry_finder == 1){
	&fr_columns;
	$ancest{CEU} = "European"; $ancest{YRI} = "African"; $ancest{JPT} = "Japanese"; $ancest{CHB} = "Chinese";
	$outfile = "Ancestry_data.txt";
	if (-e $outfile){
		print "This process has already been run; are you sure you want to rerun it? (\'y\' or \'n\')\n";
		$continue = <STDIN>; if ($continue =~ /n/i){die "User terminated process as it was already run\n";}
	}
	open(OUT, ">$outfile") || die "Could not open file $outfile; may already be open\n";
	$outfile2 = "Concise_Ancestry_data.txt";
	open(OUT2, ">$outfile2") || die "Could not open file $outfile2; may already be open\n";
	print OUT2 "FinalReport\tSample\tHapmap Ancestry\tContinent\tProbability\tNumber of AIMs\n";
	print LISTOUT $curr_dir.$outfile."\n".$curr_dir.$outfile2."\n";
	@files = <*>;
	%hash = buildHash();
	foreach $sample (@files){
		if ($sample =~ /$namematch/){
			$YRI = 1;
			$CEU = 1;
			$JPT = 1;
			$CHB = 1;
			$counter = 0;
			open(SAM,"<$sample") || die "Could not open file $sample\n";
			print "Opening file $sample\n";
			#print %hash;
			while(<SAM>){
				@tab = split("\t", $_);
				chomp @tab;
				#This looks for SNPs in the FinalReport that are AIMs and retuns the AIMs line for that SNP
				@tempVals = parseAIMS($tab[$snpcol]);
				chomp @tempVals;
				#Tests to make sure there is something in the returned AIM array (ie did parseAIMS find anything)
				if($tempVals[5] eq '+' || $tempVals[5] eq '-'){
					#The major and minor allele are put into a hash position 0 and 1
					$samplename = $tab[0];
					@tempAlleles = split("/", $tempVals[4]);
					#If the sample is homozygous for the major allele the probability of each ancestry is modified
					if(($tab[$gtcol1].$tab[$gtcol2]) eq ($tempAlleles[0].$tempAlleles[0])){
						#print "Informative SNP found\n";
						$YRI = $YRI * $tempVals[7];
						$CEU = $CEU * $tempVals[6];
						$JPT = $JPT * $tempVals[9];
						$CHB = $CHB * $tempVals[8];
						#print "Probabilities modified:\nYRI:" . $YRI . "\nCEU:" . $CEU . "\nCHB:" . $CHB . "\nJPT:" . $JPT . "\n";
						$counter++;
					}
				}
			}
			#Works out the highest probability
			$greatest = "YRI";
			if($CEU > $$greatest){
				$greatest = "CEU";
			}
			if($JPT > $$greatest){
				$greatest = "JPT";
			}
			if($CHB > $$greatest){
				$greatest = "CHB";
			}
			
			unless ($counter < 10){
				print OUT $sample . " matches " . $greatest . " origin most closely from $counter AIMs with a probability of " . $$greatest. "\n";
				print OUT "YRI:\t" . $YRI . "\nCEU:\t" . $CEU. "\nCHB:\t" . $CHB. "\nJPT:\t" . $JPT. "\n";
				print OUT2 $sample . "\t" . $samplename . "\t" . $greatest . "\t" . $ancest{$greatest} . "\t" . $$greatest . "\t" . $counter . "\n";
				#print $sample . "\t" . $greatest . "\t" . $ancest{$greatest} . "\t" . $$greatest . "\t" . $counter . "\n";
				print $sample . ": " . $greatest . "; $counter AIMs; Probability of " . $$greatest. "\n";
				print "YRI:\t" . $YRI . "\nCEU:\t" . $CEU. "\nCHB:\t" . $CHB. "\nJPT:\t" . $JPT. "\n";
				
			}
			else {
				print OUT $sample . " has only $counter AIMs; less than 10 makes ancestry information a poor indicator\n";
				print OUT2 $sample . "\t" . $samplename . "\tUnknown\tUnknown\t0\t" . $counter . "\n";
				print $sample . ": Insufficient AIMS: $counter\n";
			}
		}
	}
	close OUT; close OUT2;
}

#Deletes the file ARGV[0], used in the Pipeline
if ($deleter == 1){
	$file_to_delete = $ARGV[0];
	unlink $file_to_delete;
	print "File $file_to_delete has been deleted\n";
}

#Works out the mean, stdev and median logr per sample
if ($mean_logr == 1){
	$output = "LogR_Results.txt";
	open (OUT, ">$output")||die "Could not open $output; may already be open\n";
	print OUT "Sample\tMean LogR\tStDev LogR\tMedian LogR\tProbes\n";
	print LISTOUT $curr_dir.$output."\n";
	&fr_columns;
	@files = <*>;
	while ($file=<@files>){
		if ($file =~ /$namematch/){
			$samplecount++;
			close IN;
			open (IN, "<$file")||die "Could not open file $file\n";
			print "Examining $namematch $samplecount... ";
			$linecount = 0; $meant = 0; $snpcount = 0; @logr = (); $sdinput = 0;
			while ($line=<IN>){
				chomp $line;
				$linecount++;
				@tab = split(/\t/,$line);
				if ($linecount > 11 and $tab[$chromocol] !~ /X|Y/){
					$snpcount++;
					$meant += $tab[$log2col];
					push (@logr, $tab[$log2col]);
					$samp = $tab[$patname];
				}
			}
			$meanlogr = eval {$meant / $snpcount};
			@logr = sort {$a <=> $b} @logr;
			foreach $loger (@logr){
				$logdiff = $loger - $thismeanlogr;
				$logdiffsqd = $logdiff**2;
				$sdinput += $logdiffsqd; 
			}
			$thissdinput = eval {$sdinput / $snpcount };
			$sdout = sqrt($thissdinput);
			$half = $snpcount/2;
			if ($half == int($half)){$medianlogr = $logr[$half];}
			else {$medianlogr = ($logr[$half - 0.5] + $logr[$half + 0.5])/2;}
			print $meanlogr."\n";
			print OUT "$samp\t$meanlogr\t$sdout\t$medianlogr\t$snpcount\n";
		}
	}
	print "All done!\n";
}

#Works out the mean LogR and BAF for each chromosome in a FinalReport - useful for normalizing prior to mosaic detection
if ($logr_per_chr == 1){
	open (OUT, ">Chr_LogR_Results.txt")||die "Could not open Chr_LogR_Results.txt; may already be open\n";
	print LISTOUT $curr_dir."Chr_LogR_Results.txt\n";
	&fr_columns;
	@files = <*>;
	while (<@files>){
		if ($_ =~ /$namematch/){
			$samplecount++;
			open (IN, "<$_")||die "Could not open file $_\n";
			print "Examining $namematch $samplecount... ";
			$linecount = 0;
			$meant=0; $meanct=0; %mean=(); %meanc=(); %homo=(); $homo=0; %het=(); $het=0; %nc=(); $nc=0; %total=(); @logr=();
			while (<IN>){
				$linecount++;
				if ($linecount > 11){
					@tab = split(/\t/,$_);
					$samp = $tab[$patname];
					$input = $tab[$chromocol];
					chomp $input;
					if ($samplecount == 1){
						$chromos{$input} = 0;
					}
					#Sample total
					$meant += $tab[$log2col];
					push (@logr, $tab[$log2col]);
					$meanct += 1;
					
					#Total per chromosome
					$mean{$input} = $mean{$input} + $tab[$log2col];
					push (@$input, $tab[$log2col]);
					$meanc{$input} = $meanc{$input} + 1;
					
					#homozygous SNPs
					if (($tab[$ballele] > 0.9 or $tab[$ballele] < 0.1) and $tab[$gtcol1] !~ /-/){
						$homo{$input} = $homo{$input} + 1;
						$homo++;
					}
					
					#Heterozygous SNPs
					if ($tab[$ballele] >= 0.42 and $tab[$ballele] <= 0.58 and $tab[$gtcol1] !~ /-/){
						$het{$input} = $het{$input} + 1;
						$het++;
					}
					
					#No call SNPs
					if ($tab[$gtcol1] =~ /-/){
						$nc{$input} = $nc{$input} + 1;
						$nc++;
					}
					
					$total{$input} = $total{$input} + 1;
				}
			}
			if ($samplecount == 1){
				@chromos = keys %chromos;
				@chromos = sort { $a <=> $b } @chromos;
				print OUT "SampleID\tMeasure\tSample wide\t";
				foreach $chro (@chromos){
					print OUT "chr$chro\t";
				}
				print OUT "\n";
			}
			#Calculate sample LogR, homo and het
			if ($meanct > 0){
				$thismeanlogr = $meant/$meanct;
				$thishomo = $homo/$meanct;
				$thishet = $het/$meanct;
				$thisnc = $nc/$meanct;
			}
			else {$thismeanlogr = "Error"; $thishomo = "Error"; $thishet = "Error"; $thisnc = "Error";}
			
			$sdinput = 0;
			#Calculate sample SD
			foreach $loger (@logr){
				$logdiff = $loger - $thismeanlogr;
				$logdiffsqd = $logdiff**2;
				$sdinput += $logdiffsqd; 
			}
			if ($meanct > 0){
				$thissdinput = $sdinput / $meanct;
				$sdout = sqrt($thissdinput);
			}
			else {$sdout = "Error";}
			
			print "Mean LogR is $thismeanlogr\n";
			
			$mean1 = $thismeanlogr;
			$sd1 = $sdout;
			$diff2 = 0;
			$perhet3 = ($thishet*100)."%";
			$perhomo4 = ($thishomo*100)."%";
			$pernc5 = ($thisnc*100)."%";
			
			foreach $chro (@chromos){
				#Per chromosome mean and difference LogR
				if ($meanc{$chro} > 0){
					$meanlogr = $mean{$chro}/$meanc{$chro};
					$difflogr = $meanlogr - $thismeanlogr;
				}
				else {$meanlogr = "Error"; $difflogr = "Error";}
				
				$sdinputc = 0;
				#SD per chromosome
				foreach $loger (@$chro){
					$logdiff = $loger - $meanlogr;
					$logdiffsqd = $logdiff**2;
					$sdinputc += $logdiffsqd; 
					#print "L:$loger,M:$meanlogr,D:$logdiff,S:$logdiffsqd,T:$sdinputc\n";sleep(5);
				}
				@$chro = ();
				
				if ($meanct > 0){
					$thissdinput = $sdinputc / $meanct;
					$sdoutc = sqrt($thissdinput);
				}
				else {$sdoutc = "Error";}
				
				#Per chromosome percent heterozygous
				if ($meanc{$chro} > 0){
					$perhet = ($het{$chro} / $meanc{$chro} * 100)."%";
				}
				else {$perhet = "Error";}
				
				#Per chromosome percent homozygous
				if ($meanc{$chro} > 0){
					$perhomo = ($homo{$chro} / $meanc{$chro} * 100)."%";
				}
				else {$perhomo = "Error";}
				
				#Per chromosome percent no calls
				if ($meanc{$chro} > 0){
					$pernc = ($nc{$chro} / $meanc{$chro} * 100)."%";
				}
				else {$pernc = "Error";}
				
				#Make the output
				$mean1 = $mean1."\t".$meanlogr;
				$sd1 = $sd1."\t".$sdoutc;
				$diff2 = $diff2."\t".$difflogr;
				$perhet3 = $perhet3."\t".$perhet;
				$perhomo4 = $perhomo4."\t".$perhomo;
				$pernc5 = $pernc5."\t".$pernc;
			}
			print OUT "$samp\tMean LogR\t$mean1\t$meanct\n";
			print OUT "$samp\tStandard Deviation of LogR\t$sd1\n";
			print OUT "$samp\tLogR difference from sample mean\t$diff2\n";
			print OUT "$samp\tPercent heterozygote genotypes\t$perhet3\n";
			print OUT "$samp\tPercent homozygote genotypes\t$perhomo4\n";
			print OUT "$samp\tPercent no call genotypes\t$pernc5\n\n";
		}
	}
	print "\nAll done for $samplecount samples! See LogR_Results.txt for the results\n";
}

#Determines the mean LogR and homozygosity of the X and Y chromosomes to assess gender in complicated cases
if ($xy_logr == 1){
	open (OUT, ">LogR_XY.txt")||die "Could not open LogR_XY.txt; may already be open\n";
	print OUT "Sample\tLogR_ChrX\tHomo_ChrX\tHet_ChrX\tLogR_ChrY\tHomo_ChrY\tHet_ChrY\tLogR_ChrXY\tHomo_ChrXY\tHet_ChrXY\tGender\n";
	print LISTOUT $curr_dir."LogR_XY.txt\n";
	&fr_columns;
	@files = <*>;
	while (<@files>){
		if ($_ =~ /$namematch/){
			$samplecount++;
			open (IN, "<$_")||die "Could not open file $_\n";
			print "Examining $namematch $samplecount... \n";
			$linecount = 0;
			$meant=0; $meanct=0; %mean=(); %meanc=(); %homo=(); $homo=0; %het=(); $het=0; %nc=(); $nc=0; %total=(); @logr=();
			while ($line = <IN>){
				$linecount++;
				if ($linecount > 11 and ($line =~ /X/ or $line =~ /Y/)){
					@tab = split(/\t/,$line);
					$samp = $tab[$patname];
					$input = $tab[$chromocol];
					chomp $input;
					
					#Total per chromosome
					$mean{$input} += $tab[$log2col];
					$meanc{$input} += 1;
					
					#homozygous SNPs
					if (($tab[$ballele] > 0.9 or $tab[$ballele] < 0.1) and $tab[$gtcol1] !~ /-/){
						$homo{$input} += 1;
					}
					
					#Heterozygous SNPs
					if ($tab[$ballele] >= 0.42 and $tab[$ballele] <= 0.58 and $tab[$gtcol1] !~ /-/){
						$het{$input} += 1;
					}
					
					#No call SNPs
					if ($tab[$gtcol1] =~ /-/){
						$nc{$input} += 1;
					}
				}
			}
			close IN;
			$meanlogrX = sprintf("%.2f", eval {$mean{X} / $meanc{X}});
			$meanlogrY = sprintf("%.2f", eval {$mean{Y} / $meanc{Y}});
			$meanlogrXY = sprintf("%.2f", eval {$mean{XY} / $meanc{XY}});
			$totalprobes = ($meanc{X} + $meanc{Y} + $meanc{XY});
			#$meanlogr_allXY = eval {($mean{X} + $mean{Y} + $mean{XY}) / $totalprobes};
			
			$homoX = sprintf("%.2f", eval {$homo{X} / ($meanc{X} - $nc{X}) * 100});
			$homoY = sprintf("%.2f", eval {$homo{Y} / ($meanc{Y} - $nc{Y}) * 100});
			$homoXY = sprintf("%.2f", eval {$homo{XY} / ($meanc{XY} - $nc{XY}) * 100});
			$totalcallprobes = $totalprobes - $nc{X} - $nc{Y} - $nc{XY};
			#$homo_allXY = eval {($homo{X} + $homo{Y} + $homo{XY}) / $totalcallprobes * 100};
			
			$hetX = sprintf("%.2f", eval {$het{X} / ($meanc{X} - $nc{X}) * 100});
			$hetY = sprintf("%.2f", eval {$het{Y} / ($meanc{Y} - $nc{Y}) * 100});
			$hetXY = sprintf("%.2f", eval {$het{XY} / ($meanc{XY} - $nc{XY}) * 100});
			#$het_allXY = eval {($het{X} + $het{Y} + $het{XY}) / $totalcallprobes * 100};
			
			if ($meanlogrY < -1 and $meanlogrX > -0.25 and $meanlogrX < 0.2){
				$logrgender = "Female";
			}
			elsif ($meanlogrX < -0.25 and $meanlogrX > -1 and $meanlogrY > -0.25 and $meanlogrY < 0.2){
				$logrgender = "Male";
			}
			else {
				$logrgender = "Unknown";
			}
			if ($hetX > 10){
				$homogender = "Female";
			}
			else {
				$homogender = "Male";
			}
			if ($logrgender eq $homogender){
				$thegender = $homogender;
			}
			else {
				$thegender = "Error: LogR gender is $logrgender, BAF gender is $homogender";
			}
			
			print OUT $samp."\t".$meanlogrX."\t".$homoX."%\t".$hetX."%\t".$meanlogrY."\t".$homoY."%\t".$hetY."%\t".$meanlogrXY."\t".$homoXY."%\t".$hetXY."%\t".$thegender."\n";
		}
	}
}

#Assesses the burden of CNVs meeting specific criteria
if ($proportion == 1){
	$analysiscount = 0;
	$input = $ARGV[0];
	$input2 = "Genes_".$ARGV[0];
	$outputfile = "Prop_" . $input;
	$outputfile2 = "SimpProp_" . $input;
	$outputfile3 = "ttestcnv_Prop_" . $input;
	$outputfile4 = "ttestgene_Prop_" . $input;
	$outputfile5 = "ttestgenicCNV_Prop_" . $input;
	
	open (PROIN, "<$input") || die "Could not open the file $input; check it is spelled correctly\n";
	if ($batch == 1 and $make_temp == 0){
		open (TOUT2, ">$outputfile") || die "Could not open the file $outputfile; may already be open\n";
		open (TOUT3, ">$outputfile2") || die "Could not open the file $outputfile2; may already be open\n";
		open (TOUT4, ">$outputfile3") || die "Could not open the file $outputfile3; may already be open\n";
		open (TOUT5, ">$outputfile4") || die "Could not open the file $outputfile4; may already be open\n";
		open (TOUT6, ">$outputfile5") || die "Could not open the file $outputfile5; may already be open\n";
		open (GENES, ">$input2")||die "Could not open the file $input2; may already be open\n";
		#print LISTOUT $curr_dir."$input2\n";
	}
	elsif ($make_temp == 0) {
		open (TOUT2, ">>$outputfile") || die "Could not open the file $outputfile; may already be open\n";
		open (TOUT3, ">>$outputfile2") || die "Could not open the file $outputfile2; may already be open\n";
		#open (TOUT4, ">>$outputfile3") || die "Could not open the file $outputfile3; may already be open\n";
		#open (TOUT5, ">>$outputfile4") || die "Could not open the file $outputfile4; may already be open\n";
		if ((-s $outputfile) == 0){
			print TOUT2 "CNVs\tGenes\tIndividuals with at least one CNV\tIndividuals with no CNVs\tTotal Individuals\tProportion with at least one CNV\n";
			print TOUT3 "CNVs\tGenes\tIndividuals with at least one CNV\tIndividuals with no CNVs\tTotal Individuals\tProportion with at least one CNV\n";
		}
	}
	{ local $/=undef;  $input1=<PROIN>; }
	close PROIN;
	($\) = (/(\r\n|\r|\n)/);
	@propinput=split (/\r\n|\r|\n/, $input1); chomp @propinput;
		
	@head = split (/\t/,$propinput[0]);
	$thefileheader = $propinput[0];
	if ($make_temp == 1){
		$tempfile = "Prop_template_".$input;
		print LISTOUT $curr_dir."$tempfile\n";
		open (TOUT, ">$tempfile") || die "Could not open the file $tempfile; may already be open\n";
		print TOUT "Column\t\t";
		foreach (@head){
			$headcount++;
			print TOUT "$headcount\t";
		}
		print TOUT "\nHeader\tDescription\t$propinput[0]\n";
	}
	if ($batch == 1){
		$tempfile = "Prop_template_".$input;
		open (TIN, "<$tempfile") || die "Could not open the file $tempfile\nYou may need to run the --propbatch --make_template command\n";
		{ local $/=undef;  $input1=<TIN>; }
		close TIN;
		($\) = (/(\r\n|\r|\n)/);
		@batch=split (/\r\n|\r|\n/, $input1); chomp @batch;
	}
	print LISTOUT $curr_dir."$outputfile2\n";
	
	@head = split (/\t/,$propinput[0]);
	foreach (@head){
		chomp;
		$headcount++;
		$sampfind ='';
		if ($make_temp == 1){
			if ($_ =~ /SampleID/i or $_ eq "IID"){$sampfind = "SampleID";}
			if ($_ =~ /Gene/){$sampfind = "Genes";}
			push (@tout, "$sampfind\t");
		}
		elsif ($batch == 1){}
		else{
			print "$headcount) $_\n";
		}
	}
	shift @propinput;
	
	if ($make_temp == 1){
		print TOUT "Function\tWrite analysis description here\t@tout\nTerm\t\t@tout\nFunction\tWrite analysis description here\t@tout\nTerm\t\t@tout\n";
		print "Template created: Prop_template_".$input."\n";
	}
	
	#Here the template file is processed to make Analyses and questions
	if ($batch == 1 and $make_temp == 0){
		print TOUT2 "$batch[1]\tCNVs\tGenes\tIndividuals with at least one CNV\tIndividuals with no CNVs\tTotal Individuals\tProportion with at least one CNV\n";
		print TOUT3 "Description\tCNVs\tGenes\tIndividuals with at least one CNV\tIndividuals with no CNVs\tTotal Individuals\tProportion with at least one CNV\n";
		#The header and column numbers are removed
		shift @batch; shift @batch;
		foreach (@batch){
			#This prevents Term or Function being counted as a column
			$colcount = 0;
			$_ =~ s/\"//gi;
			@tab = split (/\t/,$_);
			$typeline = $tab[0];
			if ($typeline eq "Function"){$ananame{$analysiscount+1} = $tab[1];}
			shift @tab; shift @tab; 
			#If the line starts with Function a new analyses is started
			chomp (@tab);
			if ($typeline eq "Function"){
				$analysiscount++;
				$name1 = "func".$analysiscount;
				$name2 = "term".$analysiscount;
				%$name1 = (); %$name2 = ();
				#Columns with input are entered into a hash with column number as the key
				foreach (@tab){
					#print "This: $_, Count: $colcount\n";
					$_ =~ s/^\s+|\s+$//g;
					if ($_ eq "SampleID") {$samplecol = $colcount; }
					elsif ($_ eq "Genes") {$genecol = $colcount;}
					elsif ($_ ne "" and $_ ne "Function"){
						#print "Analysis: $analysiscount, Function: $_\n";
						$name = "func".$analysiscount;
						$$name{$colcount} = $_;
					}
					$colcount++;
				}
			}
			#If the line starts with Term the same analysis is used
			elsif ($typeline eq "Term"){
				#Columns with input are entered into a hash with column number as the key
				foreach (@tab){
					$_ =~ s/^\s+|\s+$//g;
					if ($_ eq "SampleID") {$samplecol = $colcount;}
					elsif ($_ eq "Genes") {$genecol = $colcount;}
					elsif ($_ ne "" and $_ ne "Term"){
						#print "Term: There is a term, it is:$_"."End\n";
						$name = "term".$analysiscount;
						$$name{$colcount} = $_;
					}
					$colcount++;
				}
			}
		}
		#The individual analyses are then sent to a subroutine for processing
		@analyses = (1 .. $analysiscount);
		foreach $ana (@analyses){
			$thiscount++;
			print "Analysis $thiscount of $analysiscount: $ananame{$ana}\n";
			$genehead = $genehead.$ananame{$ana}."\t";
			if ($giverealdata == 1){
				close CNVOUT;
				$cnvoutfile = $ananame{$ana} . ".txt";
				open (CNVOUT, ">$cnvoutfile") or die "Could not open $cnvoutfile; may already be open\n";
				print CNVOUT $thefileheader."\n";
			}
			@questions = ();
			push (@questions, "$samplecol\tSampleID");
			$name = "func".$ana;
			@columns = keys %$name;
			foreach $col (@columns){
				$name1 = "func".$ana;
				$name2 = "term".$ana;
				$quest = "$col\t".$$name1{$col}."\t".$$name2{$col};
				push (@questions, $quest);
			}
			&questprocess(@questions);
			$ttestcnvout = join ("\t",@results2);
			$ttestgeneout = join ("\t",@results3);
			$ttestgeniccnvout = join ("\t",@results4);
			if ($thiscount == 1){
				print TOUT4 "Samples\t$thesamplelist\n";
				print TOUT5 "Samples\t$thesamplelist\n";
				print TOUT6 "Samples\t$thesamplelist\n";
			}
			print TOUT4 "$ananame{$ana}\t$ttestcnvout\n";
			print TOUT5 "$ananame{$ana}\t$ttestgeneout\n";
			print TOUT6 "$ananame{$ana}\t$ttestgeniccnvout\n";
		}
		$termcount = 0;
		foreach (@batch){
			if (substr($_,0,4) eq "Func"){
				@tab = split ("\t",$_);
				print TOUT2 "$_\t$results[$termcount]\n";
				print TOUT3 "$tab[1]\t$results[$termcount]\n";
			}
			if (substr($_,0,4) eq "Term"){
				print TOUT2 "$_\n";
				$termcount++;
			}
		}
	}
	elsif ($make_temp == 0) {
		print "Which column is the SampleID info in (eg 1, 2, 3)?\n";
		$user_sample = <STDIN>; chomp $user_sample; $samplecol = ($user_sample - 1);
		$arrayin = "$samplecol\tSampleID";
		push (@questions, $arrayin);
		until ($nomore == 1){
			&askinput;
		}
		&questprocess(@questions);
		print TOUT2 "$results[0]\n";
		print TOUT3 "$results[0]\n";
	}
	#Print out the list of genes
	print GENES "$genehead\n";
	@numberofanalyses = keys %geneperanalysis;
	@numberofanalyses = sort {$a <=> $b} @numberofanalyses;
	@maxgenes = (0..$maxgenecount);
	foreach $num (@numberofanalyses){
		@geneshere = split (", ", $geneperanalysis{$num});
		foreach $num2 (@maxgenes){
			#print "num1 $num, num2 $num2, gene, $geneshere[$num2], list $geneperanalysis{$num}\n";
			$generesult[$num2] .= "$geneshere[$num2]\t";
		}
	}
	foreach $line (@generesult){
		chomp $line;
		print GENES $line."\n";
	}
}

#Removes balnk spaces from file names
if ($blank_remove == 1){
	print "Removing blank spaces from all files/folders with $namematch in their name\n";
	@files = <*>;
	foreach (@files){
		if ($_ =~ /$namematch/){
			$counter++;
			if ($javamethod == 0){print "\rDoing file $counter";}
			$old = $_;
			$newname = $old;
			$newname =~ s/ //g;
			$newname =~ s/^\s+|\s+$//g;
			rename ($old, $newname);
		}
	}
	print "\n\nAll done!\n";
}

#Modifies this script to have the correct folders for PennCNV, QuantiSNP and Annotation
if ($setmeup == 1){
	#This program modifies Combined_CNV by changing the file locations at the top 
	print "=============================================================================\n                        Welcome to $progname\n=============================================================================\n";
	print "For $progname to function it needs to know where PennCNV, QuantiSNP\nand the annotation files are kept:\n";
	open (IN, "<$progname") || die "Could not open $progname\n";
	@thescript = <IN>; close IN;
	$thePNloc = $thescript[7];
	unless ($thePNloc =~ /PNloc/){die "The location files are no on the expected lines\n";}
	print "\nPennCNV location is currently: $PNloc\nDo you want to change this? (\'y\' or \'n\')\n";
	$user_PNq = <STDIN>; chomp $user_PNq;
	if ($user_PNq =~ /y/i){
		print "Where is PennCNV located?\n";
		$user_PN = <STDIN>; chomp $user_PN;
		$user_PN =~ s/\\/\//gi;
		@PNchar = split (//,$user_PN);
		unless ($PNchar[-1] eq '/'){
			$user_PN = $user_PN . '/';
		}
		#print "$user_PN\n";
	}
	else {$user_PN = $PNloc};
	@tab = split(/\'/,$thePNloc);
	$output = $tab[0] . "\'" . $user_PN . "\';\n";
	#print "$output\n";
	$thescript[7] = $output;
	
	$theQTloc = $thescript[6];
	unless ($theQTloc =~ /QTloc/){die "The location files are no on the expected lines\n";}
	print "\nQuantiSNP location is currently: $QTloc\nDo you want to change this? (\'y\' or \'n\')\n";
	$user_QTq = <STDIN>; chomp $user_QTq;
	if ($user_QTq =~ /y/i){
		print "Where is QuantiSNP located?\n";
		$user_QT = <STDIN>; chomp $user_QT;
		$user_QT =~ s/\\/\//gi;
		@QTchar = split (//,$user_QT);
		unless ($QTchar[-1] eq '/'){
			$user_QT = $user_QT . '/';
		}
		#print "$user_QT\n";
	}
	else {$user_QT = $QTloc};
	@tab = split(/\'/,$theQTloc);
	$output = $tab[0] . "\'" . $user_QT . "\';\n";
	#print "$output\n";
	$thescript[6] = $output;
	
	$theANloc = $thescript[12];
	unless ($theANloc =~ /ANloc/){die "The location files are not on the expected lines\n";}
	print "\nAnnotation files location is currently: $ANloc\nDo you want to change this? (\'y\' or \'n\')\n";
	$user_ANq = <STDIN>; chomp $user_ANq;
	if ($user_ANq =~ /y/i){
		print "Where are the Annotation files located?\n";
		$user_AN = <STDIN>; chomp $user_AN;
		$user_AN =~ s/\\/\//gi;
		@ANchar = split (//,$user_AN);
		unless ($ANchar[-1] eq '/'){
			$user_AN = $user_AN . '/';
		}
		#print "$user_AN\n";
	}
	else {$user_AN = $ANloc};
	@tab = split(/\'/,$theANloc);
	$output = $tab[0] . "\'" . $user_AN . "\';\n";
	#print "$output\n";
	$thescript[12] = $output;
	
	print "Modifying script, do not stop the program... ";
	#To enable this function the line below must be deactivated
	#$progname = $progname . "2";
	open (OUT, ">$progname") || die "Could not open $progname\n";
	print OUT @thescript;
	
	print "Done.\n\nThe locations have been modified to:\n";
	print "PennCNV:    $user_PN\n";
	print "QuantiSNP:  $user_QT\n";
	print "Annotation: $user_AN\n\n";
	
	#Writes a file to each folder to check that the folder exists
	$PNsetupfile = $user_PN . "PN_setup.txt";
	open (PN, ">$PNsetupfile");
	$QTsetupfile = $user_QT . "QT_setup.txt";
	open (QT, ">$QTsetupfile");
	$ANsetupfile = $user_AN . "AN_setup.txt";
	open (AN, ">$ANsetupfile");
	
	#Checks to see if each file is in the folder
	if (-e $PNsetupfile){
		#print "I can write to PennCNV\n";
	}
	else {
		#print "I can't write to PennCNV\n";
		$error{PennCNV} = 1;
	}
	if (-e $QTsetupfile){
		#print "I can write to QuantiSNP\n";
	}
	else {
		#print "I can't write to QuantiSNP\n";
		$error{QuantiSNP} = 1;
	}
	if (-e $ANsetupfile){
		#print "I can write to Annotation\n";
	}
	else {
		#print "I can't write to Annotation\n";
		$error{Annotation} = 1;
	}
	
	#Works out if any of the folders failed the check above
	@thekeys = keys %error;
	foreach (@thekeys){
		if ($error{$_} == 1){
			print "ERROR: The folder for $_ could not be found, please repeat setup\n";
			$issues++;
		}
		else {
			print "Found $_ folder\n";
		}
	}
	if ($issues == 0){
		print "\nCongratulations! Setup is complete.\n";
	}
	else {
		print "\nERROR at least one of the folders could not be found, please repeat setup\n";
	}
	
}

#Runs the GNOSIS CNV algorithm using multiple processors
if ($cnv_batch == 1 or $cnv_batch2 == 1){
	&findthesex;
	mkdir ("01_Raw_Results", 0777);
	print LOG1 "$timeanddate\tGNOSIS\tINFO\tStarting GNOSIS CNV Prediction Algorithm\n";
	$progloc = '"'.$JVloc.$progname.'"';
	if ($processors <= 1){
		$command1 = "perl " . $progloc . " --cnv";
		system ($command1);
		print LOG1 "$timeanddate\tGNOSIS\tGNOSIS\t$command1\n";
	}
	else {
		foreach $pro (1..$processors){
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			push (@batchlist, $batchname);
			open (OUTHERE, ">$batchname") || die "Could not open $batchname; may already be open\n";
			$countfile = $tag . "_Count_gn_".$pro.".txt";
			open (OUT, ">$countfile")|| die "Could not open counter file $countfile; may already be open\n";
			close OUT;
			print "Opening GNOSIS window $pro at $timeanddate\n";
			if ($cnv_batch2 == 1){
				$command1 = "perl " . $progloc . " --cnv2 " . $tag . "_Processor_list_" . $pro . ".txt";
			}
			else {
				$command1 = "perl " . $progloc . " --cnv " . $tag . "_Processor_list_" . $pro . ".txt";
			}
			if ($pro == 1){print LOG1 "$timeanddate\tGNOSIS\tGNOSIS\t$command1\n";}
			$command2 = "perl " . $progloc . " --delete ".$countfile;
			print OUTHERE "$command1\n$command2\nexit\n";
			system ("start $batchname");
		}
		
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0;
			foreach $pro (1..$processors){
				$countfile = $tag . "_Count_gn_".$pro.".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (5);
		} until ($marker == 1);
		$counter = 0;
	
		foreach $pro (1..$processors){
			#print "I'm on $pro\n";
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			unlink $batchname;
			$cnvfile = $res_dir.$tag . "_GNOSIS_CNVs_".$pro.".txt";
			$badcnvfile = $res_dir.$tag . "_GNOSIS_badchips_".$pro.".txt";
			$thresout = $res_dir.$tag."_GNOSIS_thresholds_".$pro.".txt";
			$finalcnvfile = $res_dir.$tag . "_GNOSIS_CNVs_all.txt";
			$finalbadcnv = $res_dir.$tag . "_GNOSIS_badchips_all.txt";
			$finalthresout = $res_dir.$tag."_GNOSIS_thresholds_all.txt";
			open (INCNV, "<$cnvfile") || die "Could not open file $cnvfile; may not have been created\n";
			open (INBAD, "<$badcnvfile") || die "Could not open file $badcnvfile; may not have been created\n";
			open (INTHRES, "<$thresout") || die "Could not open file $thresout; may not have been created\n";
			if ($pro == 1){
				open (OUTCNV, ">$finalcnvfile") || die "Could not open results file $finalcnvfile; may already be open\n";
				open (OUTBAD, ">$finalbadcnv") || die "Could not open results file $finalbadcnv; may already be open\n";
				open (OUTTHRES, ">$finalthresout") || die "Could not open results file $finalthresout; may already be open\n";
				while (<INCNV>){print OUTCNV $_;} close INCNV;
				while (<INBAD>){print OUTBAD $_;} close INBAD;
				while (<INTHRES>){print OUTTHRES $_;} close INTHRES;
			}
			else {
				open (OUTCNV, ">>$finalcnvfile") || die "Could not open results file $finalcnvfile; may already be open\n";
				open (OUTBAD, ">>$finalbadcnv") || die "Could not open results file $finalbadcnv; may already be open\n";
				open (OUTTHRES, ">>$finalthresout") || die "Could not open results file $finalthresout; may already be open\n";
				@incnv = <INCNV>; shift @incnv; foreach (@incnv) {print OUTCNV $_;} close INCNV;
				while (<INBAD>){print OUTBAD $_;} close INBAD;
				@inthres = <INTHRES>; shift @inthres; foreach (@inthres) {print OUTTHRES $_;} close INTHRES;
			}
			unlink $cnvfile;
			unlink $badcnvfile;
			unlink $thresout;
		}
		unlink $countfile;
	}
	$timeanddate = &showtime;
	print LOG1 "$timeanddate\tGNOSIS\tINFO\tGNOSIS CNV Prediction Algorithm complete\n";
	print "\nGNOSIS complete at $timeanddate\n";
}

#Runs the GNOSIS CNV algorithm
if ($cnvfind == 1 or $cnvfind2 == 1){
	print "=============================================================================\n                          Loading FinalReport Genders\n=============================================================================\n";
	mkdir ("01_Raw_Results", 0777);
	&findthesex;
	&fr_columns;
	$inputcount = @ARGV;
	#print "$inputcount\n";
	
	if ($inputcount == 0){
		@files = <*>;
		$process_num = "all";
	}
	else {
		@files = ();
		$filename = $ARGV[0];
		$filename =~ s/.txt//gi;
		@filenamelist = split (/_/,$filename);
		$process_num = $filenamelist[-1];
		open (PROIN, "<$ARGV[0]")|| die "Cannot open file $ARGV[0], check it is in the right directory\n";
		while (<PROIN>) {
			@tab = split (/\t/,$_);
			chomp @tab;
			push (@files, $tab[0]);
		}
	}
	$filecount = 0;
	$cnvfile = $res_dir.$tag . "_GNOSIS_CNVs_".$process_num.".txt";
	$badcnvfile = $res_dir.$tag . "_GNOSIS_badchips_".$process_num.".txt";
	$newfile = $cnvfile;

	open (RESULTS,">$cnvfile") || die "Could not open the results file $cnvfile; may already be open \n"; 
	print RESULTS "SampleID\tChr\tStart\tStop\tStarting probe\tStopping probe\tNumber of probes\tMean LogR\tBAF Deviation\t% Heterozygosity\t% No Call\tCNV Type\tProbe type (0 Only SNP probes, 1 CNV probes present, 2 All CNV probes\tG Score\n";
	open (BADCHIP,">$badcnvfile") || die "Could not open the results file $badcnvfile; may already be open \n"; 

	$thresout = $res_dir.$tag."_GNOSIS_thresholds_".$process_num.".txt";
	open (THRESH, ">$thresout") || die "Cannot open the file for thresholds, $thresout; may already be open";
	print THRESH "Patient\torig_file\tpos_homo_thresh\tneg_homo_thresh\tpos_het_thresh\tneg_het_thresh\tpos_win_thresh\tneg_win_thresh\tarray logR avg\tcall rate\tchip status\tADS\tmolecular sex\tChrX Correction\n";
	
	#This opens the CNV snps file and makes a hash by SNP name with the value of 2 for a CNV probe, 1 for an intensity only rs probe and 0 for everything else
	$goodsnpsfile = $ANloc . "hg18_cnvsnps.txt";
	open (BESTSNPS, "<$goodsnpsfile") || die "Cannot open the file $goodsnpsfile!";
	while (<BESTSNPS>){
		@temp = split ("\t", $_);
		chomp ($temp[1]);
		$cnvsnps{$temp[0]} = $temp[1];
	}
	close (BESTSNPS); #print "done\n";
	
	foreach $file (@files){
		if ($file =~ /$namematch/gi){
			$filecount++;
			#print "Look I've found one! I've found one! I'm going to be rich, richer than... oh it's a FinalReport.\n";
			open (NEWFILE, "<$file" || die "Could not open FinalReport file $file\n");
			@data = '';
			print "Loading file $filecount...";
			@data = <NEWFILE>; chomp @data; close (NEWFILE);
			@tab = split (/\t/,$data[20]);
			$samplename = $tab[$patname];
			$sexnow = $sexysex{$samplename};
			print " sample $samplename is $sexnow\n";
			if ($filecount == 1){
				print "=============================================================================\n                         Building FinalReport Template\n=============================================================================\n";
				print "Sorting first file; this will serve as a template for all who follow...\n";
				@data = sort { (split '\t', $a, ($chromodist+2))[$chromodist] <=> (split '\t', $b, ($chromodist+2))[$chromodist]  } @data;
				@data = sort { (split '\t', $a, ($chromocol+2))[$chromocol] cmp (split '\t', $b, ($chromocol+2))[$chromocol] } @data;
				#@data = sort specificSort @data;
				print "Loading SNP positions from the first file...\n";
				foreach (@data){
					#This prevents the header section or headings from being included
					if ($_ =~ /$samplename/){
						@temp = split("\t", $_); chomp(@temp);
						$finalsnps{$temp[$snpcol]} = $counter;
						$difference = $temp[$chromodist] - $lastpos;
						#print "Chromo:$temp[$chromocol], SNP:$temp[$snpcol], Array:$finalsnps{$temp[$snpcol]}, Position:$temp[$chromodist], Difference:$difference\n"; sleep (1);
						$counter++;
						$lastpos = $temp[$chromodist];
					}
				}
				@thekeys = keys %finalsnps; $thekeys = @thekeys;
				@thevalues = values %finalsnps; $thevalues = @thevalues;
				print "Loaded $thevalues positions for $thekeys SNPs, now lets find some CNVs\n";
				print "=============================================================================\n                                 CNV Analysis\n=============================================================================\n";
				close (NEWFILE);
			}
			chomp($samplename);
			#print "we are getting sexy with $samplename\n";
			
			########################################################
			# Run CNV algorithm
			########################################################
			#[$row][$column]
			
			
			#Thresholds for different types of CNV
			$pvalue = 0.001;      #default is 0.001
			$otherp = (50*$pvalue);
			$windowp = (750*$pvalue);
			$min_homo_cnv_snps = (2); #default is 2
			$min_x_cnv_snps  = (3);  #default is 5
			$min_het_cnv_snps  = (5);  #default is 5
			$min_het_dup_snps  = (6);  #default is 5
			$chip_min_quality  = (10);	#default is 5
			$min_call_rate		 = (0.95); #default is 0.97
			
			$min_balleledev =(0.1);
			$max_nc_in_hetdel = (0.5);
			$max_het_in_hetdel = (0.33);
			$min_nc_in_homodel = (0.4);
			$max_het_in_homodel= (0.2);
			
			@canons=(0, 0.5, 1);
			$homolow=( -( 3.5));
			$homolowx=( -( 2.0));
			$hetperc=(0.2);
			
			#Nested window sizes (default 15, 30, 50)
			$zero=0;
			$view=15;
			$window=($view+$view);
			$miniview=($view+1);
			$newview=25; $newwindow=($newview+$newview);
			
			#Files needed for analysis
			$snpsfile= $array . ".txt";
			$starting_row=1;			#this means that the data starts on row 2 (perl loads at with row#1 as row#0)
			
			$genome = "hg18";
			#These are the genomic locations of XY homology on chrX in hg18
			$par_reg_1_start = 0;
			$par_reg_1_stop = 2692881;
			$xtr_reg_start = 88343459;
			$xtr_reg_stop = 92429752;
			$par_reg_2_start = 154494747;
			$par_reg_2_stop = 154824264;
			###################################################################################################################################################################
			&fr_columns;
			
			$success=0; $cnvsuccess=0;
			$total_ratio=0; $count=0; $divisor=0; $hets=0; $ncs=0; $count2=0; $total_diff=0; $totalsnps=0;
			
			$currenttime = &showtime;
			#print "GNOSIS Started: $currenttime\n";
			#print "Stuctural variation analysis running at alpha $pvalue and $otherp\n";
			
			
			$subname=$newfile;
			@subdata=''; @distribution2='';
			$hetcount=0; $allcount=0;
			$xchromo=0; $ychromo=0; $totalx=0; $xcount=0; $total2x=0; $x2count=0;
			
			#Process the array to normalize the X and Y chromosome logR values
			foreach (@data){
				chomp; 
				@temp = split("\t", $_);
				$spot = $finalsnps{$temp[$snpcol]};
				#Makes the array @subdata which is ordered by SNP position and contains the full FR data on each line
				$subdata[$spot] = $_;

				#If the probe is a proper rs probe (or is not in the cnv probes file...) the logR goes into the distribution array
				if ($cnvsnps{$temp[$snpcol]} == 0){
					push (@distribution2, $temp[$log2col]);
					if ($gtcol1 eq "-"){$totalncs++;}
					$totalsnps++;
				}
				
				if ($cnvfind2 == 1){
					if ($temp[$chromocol] eq "X" 
					and $temp[$chromodist] > $par_reg_1_stop
					and ($temp[$chromodist] < $xtr_reg_start or $temp[$chromodist] > $xtr_reg_stop)
					and $temp[$chromodist] < $par_reg_2_start){
						$total2x += $temp[$log2col];
						$x2count++;
					}
				}
				#In men on the X chromosome the total logR is calculated and the number of probes recorded if it is not in a region of XY homology
				#if ($sexnow eq "Male" and $temp[$chromocol] eq "X"){$totalx+=$temp[$log2col]; $xcount++;}
				elsif ($sexnow eq "Male" 
				and $temp[$chromocol] eq "X" 
				and $temp[$chromodist] > $par_reg_1_stop
				and ($temp[$chromodist] < $xtr_reg_start or $temp[$chromodist] > $xtr_reg_stop)
				and $temp[$chromodist] < $par_reg_2_start){
					$total2x += $temp[$log2col];
					$x2count++;
				}
			}
			
			#This subroutine changes the LogR value in males by a factor of $xmed which is the mean of logR for all chrX probes in men
			$x2med=0;
			if ($cnvfind2 == 0 and $sexnow eq "Male"){
				#$xmed = ($totalx / $xcount);	print "X is $totalx from $xcount SNPs\nMean: $xmed\n";
				if ($x2count > 0){$x2med = ($total2x / $x2count);} 
				else {$x2med = 0}
				#print "X is $total2x from $x2count SNPs\nNew Mean: $x2med\n";
				for ($z=0; $z<=$#subdata; $z++){
					@temp= split("\t", $subdata[$z]);
					#if( $temp[$chromocol] eq "X" or $temp[$chromocol] eq "XY"){
					if( $temp[$chromocol] eq "X"
					and $temp[$chromodist] > $par_reg_1_stop
					and ($temp[$chromodist] < $xtr_reg_start or $temp[$chromodist] > $xtr_reg_stop)
					and $temp[$chromodist] < $par_reg_2_start){
						$newlogR = $temp[$log2col] - $x2med;
						#print "the old logR was $temp[$log2col], but adjusted by $xmed, so now $newlogR \n"; 
						$temp[$log2col] = $newlogR;
						$newline='';
						foreach (@temp){$newline = $newline . "$_\t";}
						chomp($newline);
						$subdata[$z] = $newline;
					}
				}
			}
			elsif ($cnvfind2 == 1){
				#$xmed = ($totalx / $xcount);	print "X is $totalx from $xcount SNPs\nMean: $xmed\n";
				if ($x2count > 0){$x2med = ($total2x / $x2count);} 
				else {$x2med = 0;}
				#print "X is $total2x from $x2count SNPs\nNew Mean: $x2med\n";
				for ($z=0; $z<=$#subdata; $z++){
					@temp= split("\t", $subdata[$z]);
					#if( $temp[$chromocol] eq "X" or $temp[$chromocol] eq "XY"){
					if( $temp[$chromocol] eq "X"
					and $temp[$chromodist] > $par_reg_1_stop
					and ($temp[$chromodist] < $xtr_reg_start or $temp[$chromodist] > $xtr_reg_stop)
					and $temp[$chromodist] < $par_reg_2_start){
						$newlogR = $temp[$log2col] - $x2med;
						#print "the old logR was $temp[$log2col], but adjusted by $xmed, so now $newlogR \n"; 
						$temp[$log2col] = $newlogR;
						$newline='';
						foreach (@temp){$newline = $newline . "$_\t";}
						chomp($newline);
						$subdata[$z] = $newline;
					}
				}
			}
			$chrxcorrect{$samplename} = $x2med;
			print "ChrX LogR correction by... $x2med\n";
			print "Checking array qualitimetrics... ";
			$allneghet=0; $allneghomo=0; $allposhet=0; $allposhomo=0; $running_sum=0; $running_total=0; $totalncs=0;
			#this is where the tresholds are determined for the significant log2 ratios
			@cutoffs = &tail();
			$apos_homo_thresh=$cutoffs[0]; $aneg_homo_thresh=$cutoffs[1]; $apos_het_thresh=$cutoffs[2]; $aneg_het_thresh=$cutoffs[3]; $apos_win_thresh=$cutoffs[4]; $aneg_win_thresh=$cutoffs[5]; 
			$quality_score = ($apos_homo_thresh + $apos_het_thresh + abs($aneg_homo_thresh) + abs($aneg_het_thresh) + $apos_win_thresh + abs($aneg_win_thresh)); 
			print THRESH "$samplename\t$subname\t$apos_homo_thresh\t$aneg_homo_thresh\t$apos_het_thresh\t$aneg_het_thresh\t$apos_win_thresh\t$aneg_win_thresh\t";
			#print "$samplename\t$subname\t$apos_homo_thresh\t$aneg_homo_thresh\t$apos_het_thresh\t$aneg_het_thresh\t$apos_win_thresh\t$aneg_win_thresh\t";

			$quality_score= sprintf("%0.3f", $quality_score);
			print "Chip QC: $quality_score\n";

			$pos_homo_thresh=$apos_homo_thresh;
			$pos_het_thresh=$apos_het_thresh;
			$neg_homo_thresh=$aneg_homo_thresh;
			$neg_het_thresh=$aneg_het_thresh;
			$neg_win_thresh=(-(0.1)); #$aneg_win_thresh;
			$pos_win_thresh=((0.01));  #$apos_win_thresh;
			
			#This section normalizes the X and Y chromosome to the mean
			#for ($z=0; $z<=$#subdata; $z++){
			#	@temp= split("\t", $subdata[$z]);
			#	if( $temp[$chromocol] eq "X" or $temp[$chromocol] eq "XY"){
			#		$newlogR = $temp[$log2col] - $xmed;
			#		#print "the old logR was $temp[$log2col], but adjusted by $xmed, so now $newlogR \n"; 
			#		$temp[$log2col] = $newlogR;
			#		$newline='';
			#		foreach (@temp){$newline = $newline . "$_\t";}
			#		chomp($newline);
			#		$subdata[$z] = $newline;
			#	}
			#	elsif($temp[$chromocol] eq "Y"){
			#		$newlogR = $temp[$log2col] - $ymed;
			#		#print "the old logR was $temp[$log2col], but adjusted by $ymed, so now $newlogR \n"; 
			#		$temp[$log2col] = $newlogR;
			#		$newline='';
			#		foreach (@temp){$newline = $newline . "$_\t";}
			#		chomp($newline);
			#		$subdata[$z] = $newline;
			#	}
			#}
					
			#This stores all the data into memory, storing the data that will be used for the distribution function
			foreach $numb (@distribution){
				$running_sum+=$numb;
			}
			$ncrate=(1 -($totalncs/ ($#distribution+1)));
			$ncrate= sprintf("%0.4f", $ncrate);
			$quality_score= sprintf("%0.4f", $quality_score);
			
			if ($quality_score >= $chip_min_quality or $ncrate<$min_call_rate){$chipcall="bad"}
			else{$chipcall="good"}
			$running_avg=($running_sum / $#distribution);
			$running_avg = sprintf("%04d", $running_avg);
			print THRESH "$running_avg\t$ncrate\t$chipcall\t$quality_score\t$sexnow\t$x2med\n";
			$pos_homo_thresh= (0.75);
			$pos_het_thresh= (0.25);
			$neg_homo_thresh= (- 0.9);
			$neg_het_thresh=(- 0.25); 
			$neg_win_thresh=(- 0.01);
			$pos_win_thresh=( 0.01);

			@data = @subdata;
			#print "this is the size of data $#data and $#subdata\n";
			#print "\n";
			
			$linecounthere = 0; $tracker = 0;
			for ($z=0; $z<=$#data; $z++){
				$tracker++;
				$linecounthere++;
				if ($javamethod == 0){
					print "\rDoing line $linecounthere, this will end before $thevalues";
				}
				elsif ($tracker >= 100000){
					print "Doing line $linecounthere, this will end before $thevalues\n";
					$tracker = 0;
				}
				@temp = split("\t", $data[$z]); $CH="$temp[$chromocol]"; 
				@temp2 = split("\t", $data[$z+1]); $CH2="$temp2[$chromocol]";
	
				$l=($z); $h=($z); $lowmark=($z); $highmark=($z); $hets=0; $ncs=0;
				$count2=0; $count3=0; $smallcount=0; $tcount=0;
				for ($w=$h; $w<($h+$window); $w++){
					@here = split("\t", $data[$w-$view-1]);
					$quick[$tcount] = $here[$log2col];
					if ($here[$log2col] <= $neg_het_thresh){ $count2++;}
					elsif($here[$log2col] >= $pos_het_thresh){ $count3++;}
					$tcount++
				}  
				#This is where linux trips up
				@quick = sort {$b<=>$a} (@quick);
				$sumavg= $quick[$view];
				$interdist=($temp2[$chromodist]-$temp[$chromodist]);
				#print "Difference between chr$CH:$temp[$chromodist] and chr$CH2:$temp2[$chromodist] is $interdist\n";
	
				if ($CH eq $CH2 and $interdist < 1000000){
					#################################################################
					# D E L E T I O N      D E T E C T I O N
					#################################################################
					
					if ($temp[$log2col] <= $neg_het_thresh or $sumavg <= $neg_het_thresh and $count2>=($miniview+1)){
						if ($temp[$log2col] <= $neg_homo_thresh){$count++;}
						$smallcount++;
						
						#This is the growing phase of the seed algorithm
						$hmark=1;
						do{
							$h++; 
							@temp2 = split("\t", $data[$h]); $CH2="$temp2[$chromocol]"; 
							$interdist=($temp2[$chromodist]-$pos2);
							$pos2=$temp2[$chromodist];
							if ($CH eq $CH2 and $interdist < 1000000){
								$summary=0; $count2=0; $tcount=0; $temphet=0; $tempmax=0;
								for ($w=$h; $w<($h+$window); $w++){
									@here = split("\t", $data[$w-$view-1]);
									$quick[$tcount] = $here[$log2col];
									if ($here[$log2col] <= $neg_het_thresh){$count2++;}
									if ($cnvsnps{$here[$snpcol]} <= 1){
										$tempmax++;
										if ($here[($gtcol1)] ne $here[($gtcol2)]){$temphet++;}
									}
									$tcount++;
								}  
								@quick = sort {$b<=>$a} (@quick);
								$sumavg= $quick[$view];
								if ($tempmax==0){$hetavg= 1;}
								else{$hetavg= ($temphet/$tempmax);}
								if ( ($temp2[$log2col] <= $neg_het_thresh) or ($sumavg <= $neg_win_thresh and $count2>($miniview+1) ) or ($sumavg <= $neg_het_thresh and $hetavg<= $max_het_in_homodel) ){
									$smallcount++; $highmark++;
									if ($temp2[$log2col] <= $neg_homo_thresh){ $count++;}
								}
								else{$hmark=0}
							}
							else{$hmark=0}
								
						}until ($hmark==0);
						#Now we see our final position and calculate an average log ratio
						$cnvmark=0;
						if($count>=$min_homo_cnv_snps or $smallcount>=$min_het_cnv_snps){
							$start=($lowmark); $stop=($highmark); $aaas=0; $hets=0; $ncs=0; $misin=0; $misposs=0;
							for ($k=$start; $k<=($stop); $k++){
								@temp = split("\t", $data[$k]);
								if ($k==$start){
									$LOWPOSIT=$temp[$chromodist]; $lowsnp=$temp[$snpcol];
								}
								if ($k==$stop){
									$HIGHPOSIT=$temp[$chromodist]; $highsnp=$temp[$snpcol];
								}
								unless($temp[$log2col] eq "NaN" or $temp[$log2col] eq "--" ){$total_ratio+=$temp[$log2col];}
								unless($temp[$ballele] eq "NaN" or $temp[$log2col] eq $temp[$log2col]){
									$old_diff=1;
									foreach $can (@canons){
										$new_diff = abs($temp[$ballele] - $can);
										if ($new_diff < $old_diff){$real_diff=$new_diff}
										$old_diff=$new_diff;
									}
									$total_diff+=$real_diff;
								}	
								$divisor++;
							
								if ($temp[($gtcol1)] ne $temp[($gtcol2)]){$hets++;}
								elsif ($temp[($gtcol1)] eq "NC" or $temp[($gtcol1)] eq "-"){$ncs++;}
								if ($cnvsnps{$temp[$snpcol]} > 0){
									if ($cnvsnps{$temp[$snpcol]} > $cnvmark){
										$cnvmark = $cnvsnps{$temp[$snpcol]};
									}
								}
							}
							if ($misposs>0){$perc_mis= ($misin/$misposs); print "$misin and $misposs\n";}
							$perc_hets=($hets/$divisor);  $perc_ncs=($ncs/$divisor); 
							$avg_ratio = ($total_ratio/$divisor);
							$avg_ratio = sprintf("%0.2f", $avg_ratio);
							$avg_diff = ($total_diff/$divisor);
							$avg_diff = sprintf("%0.2f", $avg_diff);
							$perc_hets = sprintf("%0.2f", $perc_hets);
							$perc_ncs= sprintf("%0.2f", $perc_ncs);
							$CH = "CHR" . $CH;
							$horatio=(abs($avg_ratio));
							
							if ($cnvmark>1 and $smallcount>=$min_het_cnv_snps or $cnvmark>1 and $count >=$min_homo_cnv_snps){
								if($avg_ratio<=$homolow){
									$typed = "HOMODEL";
									$sigval=( ($horatio*$divisor) - ($perc_hets*$divisor) );
								}
								#A different LogR threshold is used for the male chrX except for the PAR1, XTR and PAR2 regions or the XY regions
								elsif ($sexnow eq "Male" and $CH eq "CHRX" and $avg_ratio <= $homolowx){
									if (($LOWPOSIT > $xtr_reg_stop and $HIGHPOSIT < $par_reg_2_start) or ($HIGHPOSIT < $xtr_reg_start and $LOWPOSIT > $par_reg_1_stop)){
										#print "Homodel: Low: $LOWPOSIT (>$xtr_reg_stop), High: $HIGHPOSIT (<$par_reg_2_start) : High: $HIGHPOSIT (<$xtr_reg_start), Low: $LOWPOSIT (>$par_reg_1_stop)\n";
										$typed = "HOMODEL";
										$sigval=( ($horatio*$divisor) - ($perc_hets*$divisor) );
									}
									else {
										#print "Hetdel: Low: $LOWPOSIT (>$xtr_reg_stop), High: $HIGHPOSIT (<$par_reg_2_start) : High: $HIGHPOSIT (<$xtr_reg_start), Low: $LOWPOSIT (>$par_reg_1_stop)\n";
										$typed="HETDEL"; 
										$sigval=( ($horatio*$divisor) - ($perc_hets*$divisor) - ($perc_hets*$divisor));
									}
								}
								else{
									$typed="HETDEL"; 
									$sigval=( ($horatio*$divisor) - ($perc_hets*$divisor) - ($perc_hets*$divisor));
								}
								#if ($CH eq "CHRX"){print "\ncnv HETDEL $CH:$LOWPOSIT %ncs:$perc_ncs, %hets:$perc_hets, $sexnow, $avg_ratio\n"; }
								$sigval = sprintf("%0.3f", $sigval);
								#if ($typed eq "HOMODEL"){ print "HOMODEL1: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
								if ($typed eq "HETDEL" and $perc_ncs <= $max_nc_in_hetdel and $perc_hets <= $max_het_in_hetdel or $typed eq "HOMODEL" and $perc_ncs >= $min_nc_in_homodel and $perc_hets <= $max_het_in_homodel ){
									#if ($typed eq "HOMODEL"){ print "HOMODEL2: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
									#This step excludes all male, chrX hetdel calls that are not in PAR1, XTR or PAR2
									if ($typed eq "HOMODEL" or ($typed eq "HETDEL" and ($sexnow eq "Female" or $CH !~ /x/gi or $LOWPOSIT < $par_reg_1_stop or $HIGHPOSIT > $par_reg_2_start or ($LOWPOSIT < $xtr_reg_stop and $HIGHPOSIT > $xtr_reg_start)))){
										#if ($typed eq "HOMODEL"){ print "HOMODEL3: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
										if ($chipcall eq "bad"){
											print BADCHIP "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
										}
										elsif ($CH ne "CHRY" or ($CH eq "CHRY" and $sexnow eq "Male")){
											#if ($typed eq "HOMODEL"){ print "HOMODEL4: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
											print RESULTS "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
											$cnvsuccess++;
										}
									}
								}
							}
							else{
								if($avg_ratio<=$homolow){
									$typed = "HOMODEL";  
									$sigval=( ($horatio*$perc_ncs*$perc_ncs*$divisor) - ($perc_hets*$divisor) - ($perc_hets*$divisor) );
								}
								#A different LogR threshold is used for the male chrX except for the PAR1, XTR and PAR2 regions or the XY regions
								elsif ($sexnow eq "Male" and $CH eq "CHRX" and $avg_ratio <= $homolowx){
									if (($LOWPOSIT > $xtr_reg_stop and $HIGHPOSIT < $par_reg_2_start) or ($HIGHPOSIT < $xtr_reg_start and $LOWPOSIT > $par_reg_1_stop)){
										#print "Homodel: Low: $LOWPOSIT (>$xtr_reg_stop), High: $HIGHPOSIT (<$par_reg_2_start) : High: $HIGHPOSIT (<$xtr_reg_start), Low: $LOWPOSIT (>$par_reg_1_stop)\n";
										$typed = "HOMODEL";
										$sigval=( ($horatio*$divisor) - ($perc_hets*$divisor) );
									}
									else {
										#print "Hetdel: Low: $LOWPOSIT (>$xtr_reg_stop), High: $HIGHPOSIT (<$par_reg_2_start) : High: $HIGHPOSIT (<$xtr_reg_start), Low: $LOWPOSIT (>$par_reg_1_stop)\n";
										$typed="HETDEL"; 
										$sigval=( ($horatio*$divisor) - ($perc_hets*$divisor) - ($perc_hets*$divisor));
									}
								}
								else{
									$typed="HETDEL"; 
									$sigval=( ($horatio*$divisor) - ($perc_ncs*$divisor) - ($perc_hets*$divisor) );
								}
								$sigval = sprintf("%0.3f", $sigval);
								#if ($CH eq "CHRX"){print "\nHETDEL $CH:$LOWPOSIT %ncs:$perc_ncs, %hets:$perc_hets, $sexnow, $avg_ratio\n"; }
								#if ($typed eq "HOMODEL"){ print "HOMODEL5: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\t$sigval\n";}
								if ($typed eq "HETDEL" and $perc_ncs <= $max_nc_in_hetdel and $perc_hets <= $max_het_in_hetdel or $typed eq "HOMODEL" and $perc_ncs >= $min_nc_in_homodel and $perc_hets <= $max_het_in_homodel ){
									#if ($typed eq "HOMODEL"){ print "HOMODEL6: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
									#This step excludes all male, chrX hetdel calls that are not in PAR1, XTR or PAR2
									if ($typed eq "HOMODEL" or ($typed eq "HETDEL" and ($sexnow eq "Female" or $CH !~ /x/gi or $LOWPOSIT < $par_reg_1_stop or $HIGHPOSIT > $par_reg_2_start or ($LOWPOSIT < $xtr_reg_stop and $HIGHPOSIT > $xtr_reg_start)))){
										if ($sigval>$zero){
											#if ($typed eq "HOMODEL"){ print "HOMODEL7: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
											if ($chipcall eq "bad"){
												print BADCHIP "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
											}
											elsif ($CH ne "CHRY" or ($CH eq "CHRY" and $sexnow eq "Male")){
												$success++; 
												#if ($typed eq "HOMODEL"){ print "HOMODEL8: $samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$perc_ncs (>= $min_nc_in_homodel)\t$perc_hets (<= $max_het_in_homodel)\n";}
												print RESULTS "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
		#										print "$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\n";
											}
										}
									}
								}
							}
							#print "$success CNVs and $cnvsuccess CNV-probe CNVs\n";
						}
					}
				#################################################################
				# D U P L I C A T I O N      D E T E C T I O N
				#################################################################

				elsif ($temp[$log2col] >= $pos_het_thresh or $sumavg >= $pos_het_thresh and $count3>=$miniview){
							$smallcount++;
							if ($temp[$log2col] >= $pos_homo_thresh){$count++;}
							#This is the growing phase of the seed algorithm
							$hmark=1;
							do{
								$h++; 
								@temp2 = split("\t", $data[$h]); $CH2="$temp2[$chromocol]";
								$interdist=($temp2[$chromodist]-$pos2);
								$pos2=$temp2[$chromodist];
								if ($CH eq $CH2 and $interdist < 1000000){
									$summary=0; $count2=0; $tcount=0; @quick=''; $total_diff=0;
									for ($w=$h; $w<($h+$newwindow); $w++){
										@here = split("\t", $data[$w-$newview]);
										$quick[$tcount] = $here[$log2col];
										if ($here[$log2col] >= $pos_het_thresh){ $count2++;}
										unless($here[$ballele] eq "NaN" or $here[$log2col]<0 or $here[$gtcol1] eq $here[$gtcol2] and $here[$gtcol1] ne "-"){		
											$old_diff=1;
											foreach $can (@canons){
												$new_diff = abs($here[$ballele] - $can);
												if ($new_diff < $old_diff){$real_diff=$new_diff}
												$old_diff=$new_diff;
											}
											if ($real_diff>=$min_balleledev){
												$total_diff++;
											}
										}
									}
									@quick = sort {$b<=>$a} (@quick);
									$sumavg= $quick[$newview];
									if ($temp2[$log2col] >= $pos_het_thresh or $sumavg >= $pos_win_thresh and $count2>=$miniview or $sumavg >= $pos_win_thresh and $total_diff>=2){
										$smallcount++; $highmark++;
										if ($temp2[$log2col] >= $pos_homo_thresh){$count++;}
									}
									else{$hmark=0}
								}
								else{$hmark=0}
							}until ($hmark==0);
							#Now we see our final position and calculate an average log ratio
							if($CH eq "X" and $count>=$min_x_cnv_snps or $CH ne "X" and $count>=$min_homo_cnv_snps or $smallcount>=$min_het_dup_snps){
								$cnvmark=0;
								$lastcount=0;
								$sum_diff=0;
								$start=($lowmark); $stop=($highmark);  $aaas=0; $hets=0; $ncs=0; $misin=0; $misposs=0;
								for ($k=$start; $k<=$stop; $k++){
									@temp = split("\t", $data[$k]);
									
									if ($k==$start){
										$LOWPOSIT=$temp[$chromodist]; $lowsnp=$temp[$snpcol];
									}
									if ($k==$stop){
										$HIGHPOSIT=$temp[$chromodist]; $highsnp=$temp[$snpcol];
									}

									unless($temp[$log2col] eq "NaN" or $temp[$log2col] eq "--" ){$total_ratio+=$temp[$log2col];}
		
									unless($temp[$ballele] eq "NaN" or $temp[$log2col] eq "NaN"){
										$old_diff=1;
										foreach $can (@canons){
											$new_diff = abs($temp[$ballele] - $can);
											if ($new_diff < $old_diff){$real_diff=$new_diff}
											$old_diff=$new_diff;
										}
												$sum_diff+=$real_diff;
									}
									$divisor++;
								
									if ($temp[($gtcol1)] ne $temp[($gtcol2)]){$hets++;}
									elsif ($temp[($gtcol1)] eq "NC" or $temp[($gtcol1)] eq "-"){$ncs++;}
									
									if ($cnvsnps{$temp[$snpcol]} > 0){
										if ($cnvsnps{$temp[$snpcol]} > $cnvmark){
											$cnvmark = $cnvsnps{$temp[$snpcol]}; 
										}
									}
									
								}
								$perc_badstd=($badstd/$divisor);  $perc_badlogr=($badlogr/$divisor); 
								$perc_hets=($hets/$divisor);  $perc_ncs=($ncs/$divisor); 
								$avg_ratio = ($total_ratio/$divisor);
								$avg_ratio = sprintf("%0.2f", $avg_ratio);
								$avg_diff = ($sum_diff/$divisor);
								$avg_diff = sprintf("%0.2f", $avg_diff);
								$perc_hets = sprintf("%0.2f", $perc_hets);
								$perc_ncs= sprintf("%0.2f", $perc_ncs);
								$CH = "CHR" . $CH;
		
								if($count==$divisor ){$typed = "AMPLIF";
									$sigval=( ($avg_ratio*(1-$perc_ncs)*(1-$perc_ncs)*$divisor) + ($perc_hets*$divisor) + ($total_diff*$divisor));
								}
								else{$typed="HETDUP";
									$sigval=( ($avg_ratio*(1-$perc_ncs)*(1-$perc_ncs)*$divisor) + ($perc_hets*$divisor) + ($total_diff*$divisor));
								}
		
								$sigval = sprintf("%0.3f", $sigval);
		#						if ($sigval>$zero){

											if ($cnvmark>0 and $smallcount>=$min_het_dup_snps and $avg_ratio >= $pos_het_thresh or $cnvmark>0 and $typed eq "AMPLIF" or $cnvmark==1 and $count >=$min_homo_cnv_snps and $avg_ratio >= $pos_het_thresh){
													if ($chipcall eq "bad"){
														print BADCHIP "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
													}
													else{
														print RESULTS "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
														$cnvsuccess++;
													}
											}
											else{
												if ($typed eq "HETDUP" and $smallcount>=$min_het_dup_snps and $avg_ratio >= $pos_het_thresh or $typed eq "AMPLIF" or $count >=$min_homo_cnv_snps and $avg_ratio >= $pos_het_thresh){
													if ($chipcall eq "bad"){
														print BADCHIP "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
													}
													else{
													$success++; 
														print RESULTS "$samplename\t$CH\t$LOWPOSIT\t$HIGHPOSIT\t$lowsnp\t$highsnp\t$divisor\t$avg_ratio\t$avg_diff\t$perc_hets\t$perc_ncs\t$typed\t$cnvmark\t$sigval\t$perc_mis\n";
													}
												}
											}
									#print "$success CNVs and $cnvsuccess CNV-probe CNVs\n";
		#						}
							}
					}
				$z=($highmark);
				$total_ratio=0; $count=0; $divisor=0; $hets=0; $ncs=0; $count2=0; $total_diff=0; @coding=''; @genes='';
				}
			}
			#	print "Patient $ogtemp[$v]: \n$total_count probes out of $z are beyond thresholds of $pos_thresh and $neg_thresh";
			#	for ($w=0; $w<=$#aggregates; $w++){print  "$w\t$aggregates[$w]\n";}
			print "\nCNVs Detected: $cnvsuccess from CNV probes, $success from SNP probes\n\n";
			#$total_count=0; 
			@aggregates=''; $success=0; $cnvsuccess=0;
			#close(RESULTS);
			#close(THRESH);
			#close(RANDOM);
		}
		$currenttime= &showtime;
	}
}

#Trims the file from the top leaving the bottom (opposite of testfile_maker)
if ($rem_head == 1){
	print "How many lines do you want to remove from the start of the file (eg 10)?\n";
	$user_answer = <STDIN>; chomp $user_answer;
	$lines = $user_answer;

	$inputfile = $ARGV[0];
	$outfile = "Headoff_".$inputfile;
	
	open (IN, "<$inputfile") || die "Could not open input file $inputfile; check it is in the right directory\n";
	open (OUT, ">$outfile") || die "Could not open output file $outfile; may already be open\n";
	print LISTOUT $curr_dir."$outfile\n";
	
	while (<IN>){
		if ($linecount >= $lines){
			print OUT $_;
		}
		$linecount++;
	}
	print "All done!\n";
}

#Trims the file from the bottom leaving the top (opposite of rem_head)
if ($testfile_maker == 1){
	print "How many lines do you want in your testfile (eg 1000)?\n";
	$user_answer = <STDIN>; chomp $user_answer;
	$lines = $user_answer;

	$inputfile = $ARGV[0];
	$outfile = "Test_".$inputfile;
	
	open (IN, "<$inputfile") || die "Could not open input file $inputfile; check it is in the right directory\n";
	open (OUT, ">$outfile") || die "Could not open output file $outfile; may already be open\n";
	print LISTOUT $curr_dir."$outfile\n";
	
	while (<IN>){
		$tracker++;
		if ($javamethod == 0){
			print "\rDoing line $linecount";
		}
		elsif($tracker >= 10000){
			print "Doing line $linecount\n";
			$tracker = 0;
		}
		if ($linecount >= $lines){
			print "\nAll done!\n";
			close IN;
			close OUT;
		}
		else {
			print OUT $_;
		}
		$linecount++;
	}
}

#Removes duplicate lines in a file
if ($rem_dup == 1){

	$inputfile = $ARGV[0];
	$outfile = "Nodup_".$inputfile;
	
	open (IN, "<$inputfile") || die "Could not open input file $inputfile; check it is in the right directory\n";
	open (OUT, ">$outfile") || die "Could not open output file $outfile; may already be open\n";
	print LISTOUT $curr_dir."$outfile\n";
	@input = (<IN>); chomp @input; close IN;
	@input = &uniq(@input);
	
	foreach $line (@input){
		print OUT "$line\n";
	}
	print "All done!\n";
}

#Trims FinalReports to a list of consensus SNPs
if ($fr_prune == 1){
	$chiporder{"1Mv3"}    = 12;
	$chiporder{"650Y"}    = 11;
	$chiporder{"610v1"}   = 10;
	$chiporder{"370v2"}   = 9;
	$chiporder{"1Mv1"}    = 8;
	$chiporder{"550v3"}   = 7;
	$chiporder{"370v1"}   = 6;
	$chiporder{"300v2"}   = 5;
	$chiporder{"550v1"}   = 4;
	$chiporder{"300v1"}   = 3;
	$chiporder{"Unknown"} = 0;
	
	&fr_columns;
	print LISTOUT "$curr_dir\n";
	
	#Uses a subroutine to find the chip name and chooses the most recent chip as a template
	print "Working out the types of chip present\n";
	@output = &fr_name;
	foreach $chip (@output){
		@tab = split ("\t", $chip);
		$littlechip = $tab[3];
		if($chiporder{$littlechip} > $highchip){
			$highchip = $chiporder{$littlechip};
			$chip_to_use = $littlechip;
			$fr_to_open = $tab[0];
			$sample = $tab[1];
		}
	}
	print "\nA template of SNP positions will be made from a $chip_to_use chip\n";
	
	#Opens the consensus list of SNPs and the FinalReport chossen above
	$input = $ARGV[0];
	print "Opening Consensus SNP file: $input\n";
	open (IN1, "<$input") || die "Could not open FinalReport file $input, check it is in the right directory\n";
	print "Opening FinalReport file: $fr_to_open\n";
	open (IN2, "<$fr_to_open") || die "Could not open FinalReport file $fr_to_open\n";
	
	print "Processing the Consensus file\n";
	while (<IN1>){
		$numbersnps++;
		chomp $_;
		$rs{$_} = 1;
	}
	close IN1;
	
	#Makes a hash of SNP positions and the 
	print "Processing the FinalReport file to make a template\n";
	$linecount = 0;
	while (<IN2>){
		if ($javamethod == 0){print "\rDoing line $linecount";}
		chomp $_;
		@tab = split ("\t", $_);
		if ($sample eq $tab[$patname]
		and $rs{$tab[$snpcol]} == 1){
			$posit{$tab[$snpcol]} = "$tab[$chromocol]\t$tab[$chromodist]";
		}
		$linecount++;
	}
	close IN2;
	
	print "\nPruning SNPs from all the FinalReport files present\n";
	@files = <*>;
	foreach (@files){
		if ($_ =~ /$namematch/
		and $_ !~ /Pruned/){
			$filenumber++;
		}
	}
	$filecounter = 0;
	foreach $file (@files){
		if ($file =~ /$namematch/
		and $file !~ /Pruned/){
			$filecounter++;
			print "Doing file $filecounter of $filenumber: $file\n";
			open (FR, "<$file") || die "Could not open FinalReport file $file, check it is in the right directory\n";
			$outputfile = "Pruned_".$file;
			open (OUT, ">$outputfile") || die "Could not open $outputfile, may already be open\n";
			$linecount = 0;
			@sorter = ();
			while (<FR>){
				if ($linecount < 11){
					if ($linecount == 3){
						@tab = split ("\t", $_);
						print OUT "$tab[0]\t\tPruned_"."$tab[2]";
					}
					elsif ($linecount == 4 or $linecount == 5){
						@tab = split ("\t", $_);
						print OUT "$tab[0]\t$numbersnps\n";
					}
					else {print OUT $_;}
				}
				else {
					@tab = split ("\t", $_);
					if ($posit{$tab[$snpcol]} ne ""){
					$output =  "$tab[$patname]\t$tab[$snpcol]\t$posit{$tab[$snpcol]}\t$tab[$gtcol1]\t$tab[$gtcol2]\t$tab[$theta]\t$tab[$rvalue]\t$tab[$ballele]\t$tab[$log2col]";
					push (@sorter, $output);
					}
				}
				$linecount++;
			}
			@sorter = sort { (split '\t', $a)[$snpcol] cmp (split '\t', $b)[$snpcol] } @sorter;
			print OUT @sorter;
		}
	}
	print "\nAll Done\n";
}

#Old option used for making a GNOSIS input file, now obselete
if ($gn_chip_input == 1){
	#1) Creates an 'All_Illumina_SNPs" file from the final reports
	#2) Flags SNPs where there is inconsistentcy in position between the different chips - put these in a separate file
	#3) Outputs the GNOSIS chip specific files (eg 1Mv1)
	#4) Works out the proportions of overlap
	
	&fr_columns;
	
	@files = <*>;
	foreach (@files){
		if ($_ =~ /FinalReport/){
			print "Loading FinalReport $_\n";
			open (FRIN, "<$_") || die "Couldn't open the input file $_.\n";
			@frin = <FRIN>; chomp @frin; close FRIN;
			@here = split ("\t", $frin[20]);
			$sample = $here[$patname];
			@here = split ("\t", $frin[3]);
			$chip = $here[2];
			foreach (@frin){
				@tab = split ("\t", $_);
				if($tab[$patname] eq $sample){
					$snpname = $tab[$snpcol];
					$chromo = $tab[$chromocol];
					$position = $tab[$chromodist];
					$chromo =~ s/T//g;
					$input = "$snpname\tchr$chromo\t$position\t1\thg18\t$chip";
					$mini = "chr$chromo\t$position\t1\thg18";
					if ($snplist{$snpname} =~ /$mini/){}
					elsif ($snplist{$snpname} eq ""){
						$snplist{$snpname} = "$input";
					}
					else{
						if ($snplist{$snpname} eq "Mismatch"){
							$mismatch{$snpname} = "$mismatch{$snpname}\t$input";
						}
						else{
							$mismatch{$snpname} = "$snplist{$snpname}\t$input";
							$snplist{$snpname} = "Mismatch";
						}
					}
					$algolist{$snpname} = "$chip,$algolist{$snpname}";
					$$chip{$snpname} = 1;
					$chips{$chip} = 0;
				}
			}
		}
	}
	
	@chips = keys %chips;
	
	
	@mismatch = keys %mismatch;
	$mismatches = @mismatch;
	print "\n$mismatches SNPs with mismatched positions; these are shown in the file Mismatch.txt\n";
	open (OUT, ">Mismatch.txt")||die "Couldn't open the output file Mismatch.txt.\n";
	foreach (@mismatch){
		@here = split ("\t", $mismatch{$_});
		$input = "$here[0]\t$here[1]\t$here[2]\t$here[3]\t$here[4]\t$here[5]";
		push (@extra, $input);
		print OUT "$mismatch{$_}\n";
	}
	
	print "Working out how many SNPs are shared by each chip\n";
	open (OUT, ">ChipCombo.txt")||die "Couldn't open the output file ChipCombo.txt.\n";
	#open (OUT2, ">550v1_unique.txt")||die "Couldn't open the output file 550v1_unique.txt.\n";
	@snps = keys %algolist;
	foreach (@snps){
		$chipcount{$algolist{$_}} = $chipcount{$algolist{$_}} + 1;
		if ($algolist{$_} eq "BDCHP-1X10-HUMANHAP550_11218540_C.bpm,"){
			print OUT2 "$snplist{$_}\n";
		}
	}
	@chipcombo = keys %chipcount;
	foreach (@chipcombo){
		print OUT "$_\t$chipcount{$_}\n";
	}
	
	print "\nMaking hg18_all_Illum_chip_SNPs.txt file...";
	open (OUT, ">hg18_all_Illum_chip_SNPs.txt")||die "Couldn't open the output file hg18_all_Illum_chip_SNPs.txt.\n";
	@snps = values %snplist;
	$lines = @snps;
	print " $lines SNPs found\n";
	print "Add first column in Mismatch.txt to the all_SNPs file (\'y\' or \'n\')?\n";
	$user_answer = <STDIN>; chomp $user_answer;
	if ($user_answer =~ /y/i){
		print "OK, adding the SNPs and sorting the file, this may take a while...\n";
		@snps = (@snps, @extra);
		@snps = sort { (split '\t', $a)[2] <=> (split '\t', $b)[2] } @snps;
		@snps = sort { (split '\t', $a)[1] cmp (split '\t', $b)[1] } @snps;
		foreach (@snps){
			unless ($_ =~ /Mismatch/){
				@here = split ("\t", $_);
				print OUT "$here[0]\t$here[1]\t$here[2]\t$here[3]\t$here[4]\n";
				foreach $chip (@chips){
					if($$chip{$here[0]} == 1){
						push (@$chip, $here[0])
					}
				}
			}
		}
		print "\nCreating GNOSIS file specific input files\n";
		foreach $chip (@chips){
		open (OUT, ">$chip.txt")||die "Couldn't open the output file $chip.\n";
		$linenum = @$chip;
		print "$linenum SNPs for chip $chip\n";
		foreach (@$chip){
			print OUT "$_\n";
		}
			
	}
		
	}
	else {
		print "OK, printing the unsorted file without the mismatches\n";
		foreach (@snps){
			unless ($_ =~ /Mismatch/){
				@here = split ("\t", $_);
				print OUT "$here[0]\t$here[1]\t$here[2]\t$here[3]\t$here[4]\n";
			}
		}
	}
}

#Sorts a file by up to two columns
if ($sorter == 1){
	#This sorter examines line 11 to determine if the data is numeric or alphanumeric
	$input = $ARGV[0];
	$output = "Sort_$input";
	open (INN, "<$input") || die "Could not open file $input, check it is in the right directory\n";
	open (OUT, ">$output") || die "could not open file $output, it may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	
	@lines = <INN>;
	@here = split ("\t", $lines[0]);
	foreach (@here){
		$count++;
		print "$count) $_\n";
	}
	
	print "Does your data have a header (\'y\' or \'n\')?\n";
	$user_answer = <STDIN>; chomp $answer;
	if ($user_answer =~ /y/i){
		print OUT $lines[0];
		shift @lines;
	}
	print "Which column do you want to sort by first (1, 2, 3 etc)?\n";
	$user_col1 = <STDIN>; chomp $user_col1; $col1 = $user_col1 - 1;
	print "Which column do you want to sort by second (leave blank if none)?\n";
	$user_col2 = <STDIN>; chomp $user_col2; 
	if ($user_col2 ne ""){
		$col2 = $user_col2 - 1;
		print "Which column do you want to sort by third (leave blank if none)?\n";
		$user_col3 = <STDIN>; chomp $user_col3; 
		if ($user_col3 ne ""){
			$col3 = $user_col3 - 1;
			print "Performing tertiary sort\n";
		}
	}
	
	@here = split ("\t", $lines[10]);
	@there = split ("\t", $lines[-1]);
	
	if ($col3 ne ""
	and $here[$col3] =~ /\D/ 
	and $there[$col3] =~ /\D/){
		print "Performing tertiary sort (text)\n";
		@lines = sort { (split '\t', $a)[$col3] cmp (split '\t', $b)[$col3] } @lines;
	}
	elsif ($col3 ne ""){
		print "Performing tertiary sort (numbers)\n";
		@lines = sort { (split '\t', $a)[$col3] <=> (split '\t', $b)[$col3] } @lines;
	}
	if ($col2 ne ""
	and $here[$col2] =~ /\D/
	and $there[$col3] =~ /\D/){
		print "Performing secondary sort (text)\n";
		@lines = sort { (split '\t', $a)[$col2] cmp (split '\t', $b)[$col2] } @lines;
	}
	elsif ($col2 ne ""){
		print "Performing secondary sort (numbers)\n";
		@lines = sort { (split '\t', $a)[$col2] <=> (split '\t', $b)[$col2] } @lines;
	}
	if ($here[$col1] =~ /\D/
	and $there[$col3] =~ /\D/){
		print "Performing primary sort (text)\n";
		@lines = sort { (split '\t', $a)[$col1] cmp (split '\t', $b)[$col1] } @lines;
	}
	else{
		print "Performing primary sort (numbers)\n";
		@lines = sort { (split '\t', $a)[$col1] <=> (split '\t', $b)[$col1] } @lines;
	}
	print "Printing out the sorted data; here are the first 10 lines:\n";
	print "$lines[0]$lines[1]$lines[2]$lines[3]$lines[4]$lines[5]$lines[6]$lines[7]$lines[8]$lines[9]";
	print OUT "@lines";
}

#Looks up the genomic coordinates of genes using UCSC file (quick)
if ($gene_pos2 == 1){
	#$genesfile1 = $ANloc."hg18_UCSC_genes_merged_no_duplicate.txt";
	$genesfile1 = $ANloc."hg18_RefSeq_genes_merged_no_duplicate.txt";
	$inputfile = $ARGV[0];
	$outputfile = "Genes_".$inputfile;
	open (IN, "<$inputfile") || die "Could not open file $inputfile; check it is in the right directory\n";
	open (GENES, "<$genesfile1") || die "Could not open file $genesfile1; check it is in $ANloc\n";
	open (OUT, ">$outputfile") || die "Could not open file $outputfile; may already be open\n";
	print LISTOUT $curr_dir."$outputfile\n";
	#Display the first line of the input file and determine which column has the genename in
	@genes = (<IN>); chomp @genes; close IN;
	@lineshow = split ("\t", $genes[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	print "\nWhich column is the gene name in (1, 2, 3 etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $gene = $user_chromo - 1;
	print "Is there a header ('y' or 'n')?\n";
	$headpres = <STDIN>; chomp $headpres;
	
	#If a header is present then print it out and remove it from the file
	$head = "Gene name\tNumber of locations\tChr\tStart\tStop\tExons\tStrand\tDescription\n";
	if ($headpres =~ /y/i){
		print OUT "$genes[0]\t$head";
		shift @genes;
	}
	else {
		@tab = split (/\t/,$genes[0]);
		foreach (@tab){
			print OUT "\t";
		}
		print OUT "$head";
	}
	
	#Make a hash of genenames and positions from the genesfile
	# print "Establishing positions of known UCSC genes\n";
	# while (<GENES>){
		# @tab = split (/\t/,$_);
		# chomp @tab;
		# $tab[0] =~ s/\s//gi;
		# $tab[0] =~ s/"//gi;
		# if ($genename{uc $tab[0]} eq ""){
			# $genename{uc $tab[0]} = $tab[6];
		# }
		# elsif ($genename{uc $tab[0]} !~ $tab[6]){
			# $genename{uc $tab[0]} = $genename{uc $tab[0]}.", ".$tab[6];
		# }
		# if ($exonmax{$tab[6]} < $tab[7]){
			# $exonmax{$tab[6]} = "$tab[7]";
			# $genepos{$tab[6]} = "chr$tab[1]\t$tab[2]\t$tab[3]\t".$exonmax{$tab[6]}."\t$tab[9]\t$tab[10]";
		# }
		
	# }
	# close GENES;
	
	#Make a hash of genenames and positions from the genesfile
	print "Establishing positions of known RefSeq genes\n";
	while (<GENES>){
		@tab = split (/\t/,$_);
		chomp @tab;
		$tab[0] =~ s/\s//gi;
		$tab[0] =~ s/"//gi;
		if ($genename{uc $tab[0]} eq ""){
			$genename{uc $tab[0]} = $tab[6];
		}
		elsif ($genename{uc $tab[0]} !~ $tab[6]){
			$genename{uc $tab[0]} = $genename{uc $tab[0]}.", ".$tab[6];
		}
		if ($exonmax{$tab[6]} < $tab[7]){
			$exonmax{$tab[6]} = "$tab[7]";
			$genepos{$tab[6]} = "chr$tab[1]\t$tab[2]\t$tab[3]\t".$exonmax{$tab[6]}."\t$tab[9]\t$tab[10]";
		}
		
	}
	close GENES;
	
	
	#Work out the number of genes with that name in each line then print out the result
	print "Now working out where the genes of interest are found\n";
	$numgenes = @genes;
	foreach $line (@genes){
		@tab = split (/\t/,$line);
		$tab[$gene] =~ s/\s//gi;
		$tab[$gene] =~ s/"//gi;
		$genename = $tab[$gene];
		print "Doing gene $genename\n";
		@subgene = split (/, /,$genename{uc $genename});
		$arraysize = @subgene;
		if ($arraysize == 0){
			print OUT "$line\tNo genes with the name $genename found in the UCSC database\n";
			$absent++;
		}
		else {
			foreach $genehere (@subgene){
				#print "Subgenename is $genehere\n";
				print OUT "$line\t".$genehere."\t".$arraysize."\t".$genepos{$genehere}."\n";
			}
		}
	}
	$present = $numgenes - $absent;
	print "$present genes found out of $numgenes\n";
}

#Looks up the genomic coordinates of genes using UCSC (slow)
if ($gene_pos == 1){
	# Loads all important LWP classes, and makes sure your version is reasonably recent.
	use LWP 5.64;
	my $browser = LWP::UserAgent->new;
	
	#Variables
	$clade = "mammal";
	$org = "Human";
	$db = "hg18";
	#$position = "";
	$pix = "800";
	$submitter = "submit";
	
	$input = $ARGV[0];
	$output = "UCSC_$input";
	open (IN, "<$input") || die "Could not open file $input; check it is in the right directory\n";
	print "Opening Genes file $input\n";
	open (OUT, ">$output") || die "Could not open file $output; may already be open\n";
	print "Opening results file $output\n";
	print LISTOUT $curr_dir."$output\n";
	@snps = <IN>; chomp @snps; close IN;
	print OUT "Gene name\tChr\tStart\tEnd";
	$count = 0;
	$total = @snps;
	
	foreach $position (@snps){
		$count++;
		my $url = 'http://genome.ucsc.edu/cgi-bin/hgTracks?hgsid=130590748&clade='.$clade.'&org='.$org.'&db='.$db.'&position='.$position.'&pix='.$pix.'&Submit='.$submitter;
		#print "$url\n";
		my $response = $browser->get($url);
		die "Can't get $url -- ", $response->status_line unless $response->is_success;
		die "Content is not HTML", $response->content_type unless $response->content_type eq 'text/html';
		
		if ($response->is_success == 1){
			print "\n$count of $total) $position\tWebpage downloaded\t";
		}
		else {
			print "\n$count of $total) $position\tWebpage not found\t";
		}
		#print "Success? " . $response->is_success . "\n";
		#print "Status: " . $response->status_line . "\n";
		#print "Type: " . $response->content_type . "\n";
		#print "Content\n" . $response->content;
		$content = $response->content;
		@content = split ("\n", $content);
		
		@isoform = ();
		foreach $line (@content){
			$snpcount = 0;
			if ($line =~ /knownGene/gi
			and $line =~ />$position /gi){
				@bits = split (" ", $line);
				foreach (@bits){
					#print "Bit: $_\n";
					if ($_ =~ /chr/gi
					and $_ !~ /=/gi
					and $_ !~ /_random/gi
					and $_ =~ /:/gi){
						#print "\nisoform: $_\n";
						$_ =~ s/<\/A>//gi;
						$_ =~ s/:/\t/gi;
						$_ =~ s/-/\t/gi;
						#print "Modified isofrom: $_\n";
						push (@isoform, $_)
					}
				}
			}
			elsif ($line =~ /Sorry/) {
				print "No Gene data found";
				print OUT "\n$position\tNo data\tNo data";
			}
		}
		
		foreach (@isoform){
			@tab = split ("\t", $_);
			$chromo{$tab[0]} = $chromo{$tab[0]} + 1;
			$starters{$tab[1]} = 0;
			$stopers{$tab[2]} = 0;
		}
		
		@chromo = keys %chromo;
		@starter = keys %starters;
		@stoper = keys %stopers;
		#print "\nStarters: @starter\n";
		%chromo = ();
		%starters = ();
		%stopers = ();
		
		$regions = @chromo;
		if ($regions > 1){
			print "Chromo: @chromo\n";
			print "ERROR: Gene $position is found on multiple chromosomes\n";
			print OUT "\n$position\tMultiple locations\t@chromo";
		}
		else{
			@starter = sort  { $a <=> $b } @starter;
			@stoper = sort  { $a <=> $b } @stoper;
			#print "Starter2: @starter\n";
			print "$chromo[0]:$starter[0]-$stoper[-1]";
			print OUT "\n$position\t$chromo[0]\t$starter[0]\t$stoper[-1]";
		}
	}
}

#Looks up the genomic coordinates of SNPs in dbsnp (slow)
if ($snp_pos == 1){
	# Loads all important LWP classes, and makes sure your version is reasonably recent.
	use LWP 5.64;
	my $browser = LWP::UserAgent->new;
	
	#Variables
	$clade = "mammal";
	$org = "Human";
	$db = "hg18";
	#$position = "";
	$pix = "800";
	$submitter = "submit";
	
	$snp_build = 129;
	
	$input = $ARGV[0];
	$output = "UCSC_$input";
	print "Using dbSNP Build $snp_build\n";
	open (IN, "<$input") || die "Could not open file $input; check it is in the right directory\n";
	print "Opening SNPs file $input\n";
	open (OUT, ">$output") || die "Could not open file $output; may already be open\n";
	print "Opening results file $output\n";
	print LISTOUT $curr_dir."$output\n";
	@snps = <IN>; chomp @snps; close IN;
	print OUT "SNP name\tChr\tPosition";
	$count = 0;
	$total = @snps;
	
	foreach $snp (@snps){
		$count++;
		$position = $snp;
		if ($snp =~ /rs/){
			my $url = 'http://genome.ucsc.edu/cgi-bin/hgTracks?hgsid=130590748&clade='.$clade.'&org='.$org.'&db='.$db.'&position='.$position.'&pix='.$pix.'&Submit='.$submitter;
			#print "$url\n";
			my $response = $browser->get($url);
			die "Can't get $url -- ", $response->status_line unless $response->is_success;
			die "Content is not HTML", $response->content_type unless $response->content_type eq 'text/html';
			
			if ($response->is_success == 1){
				print "\n$count of $total) $position\tWebpage downloaded\t";
			}
			else {
				print "\n$count of $total) $position\tWebpage not found\t";
			}
			#print "Success? " . $response->is_success . "\n";
			#print "Status: " . $response->status_line . "\n";
			#print "Type: " . $response->content_type . "\n";
			#print "Content\n" . $response->content;
			$content = $response->content;
			@content = split ("\n", $content);
			
			foreach $line (@content){
				$snpcount = 0;
				if ($line =~ /Polymorphism/){
					@build = split (" ", $line);
					$buildnum = $build[-1];
					$buildnum =~ s/\(snp/Build_/gi;
					$buildnum =~ s/\)<\/H2><PRE>//gi;
					
				}
				elsif ($line =~ /hgFind/gi
				and $buildnum =~ /$snp_build/){
					$snpcount++;
					@position2 = split (" ", $line);
					$position2 = $position2[-1];
					$position2 =~ s/:/\t/gi;
					$position2 =~ s/-/\t/gi;
					$position2 =~ s/<\/A>//gi;
					@coord = split ("\t", $position2);
					$coord[1] = $coord[1] + 250;
					if ($snpcount == 1){
						print "$coord[0]:$coord[1]";
						print OUT "\n$position\t$coord[0]\t$coord[1]";
					}
					else {
						print OUT "\t$coord[0]\t$coord[1]";
						print "\t$coord[0]:$coord[1]";
					}
				}
				elsif ($line =~ /Sorry/) {
					print "No SNP data found";
					print OUT "\n$position\tNo data\tNo data";
				}
			}
		}
		else{
			print "\nERROR: $snp does not have rs in its name; it will not be submitted";
			print OUT "\n$position\tNot submitted\tNot submitted";
		}
	}
	print "\nAll done!\n";
	
	# $response->is_success - tells you if it has successfully downloaded the page (1 or nothing)
	# $response->status_line - tells you the status of the request eg 200 OK, 404 Not found
	# $response->content_type - tells you what type of content is present
	# $response->content - the actual HTML content
	
	# Posting to forms:
	#$response = $browser->post($url,
	# [
	#  formkey1 => value1,
	#  formkey2 => value2,
	# ],
	# headerkey1 => value1,
	#);
}

#Quickly identifies the sample ID and chip type for FinalReports
if ($fr_name == 1){
	$output = "Name_ID.txt";
	open (OUT, ">$output")|| die "Could not open $output, may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	print OUT "FinalReport\tSampleID\tSNPs\tChip\tChip\n";
	@output = &fr_name;
	print OUT @output;
	print "\nAll done!";
}

#Looks for large CNVs next to each other and attmempts to join them
if ($bigcnv == 1){
	$threshold = 100000;
	$factor = 2;
	
	$input = $ARGV[0];
	$output = "Big_".$input;
	print "\nOpening CNV file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input; check it is in the right directory\n";
	@CNV = <INPUT>; chomp @CNV; close (INPUT);
	
	print "\nOpening Results file: $output\n";
	open (OUT, ">$output") || die "Cannot open file $output; may already be open\n";
	print OUT "FamilyID\tSampleID\tChr\tStart\tStop\tTypes\tMax number of algorithms\tSize\tPercent covered by CNVs\tNumber of CNVs\n";
	print LISTOUT $curr_dir."$output\n";
	
	@lineshow = split ("\t", $CNV[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	print "Which column is the sampleID in (1, 2, 3 etc)?\n";
	$user_sampleID = <STDIN>; chomp $user_sampleID; $sampleID = $user_sampleID - 1;
	print "Is the file in Merged (M), Summary (S) or Other (O) format?\n";
	$user_filetype = <STDIN>; chomp $user_filetype;
	if ($user_filetype =~ /m/i){
		$familyID = $sampleID - 1;
		$chromo = $sampleID + 1;
		$start = $sampleID + 9;
		$stop = $sampleID + 10;
		$type = $sampleID + 4;
		$snpnum = $sampleID + 6;
		$thres = $sampleID + 13;
		$head = "y";
	}
	elsif ($user_filetype =~ /s/i){
		$familyID = $sampleID - 1;
		$chromo = $sampleID + 1;
		$start = $sampleID + 2;
		$stop = $sampleID + 3;
		$type = $sampleID + 4;
		$snpnum = $sampleID + 6;
		$thres = $sampleID + 8;
		$head = "y";
	}
	else {
		print "Which column is the familyID in (1, 2, 3 etc)?\n";
		$user_familyID = <STDIN>; chomp $user_familyID; $familyID = $user_familyID - 1;
		print "Which column is the chromosome in (1, 2, 3 etc)?\n";
		$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
		print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
		$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
		print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
		$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
		print "Which column shows the CNV type (1, 2, 3 etc)?\n";
		$user_type = <STDIN>; chomp $user_type; $type = $user_type - 1;
		print "Which column shows the number of SNPs (1, 2, 3 etc)?\n";
		$user_snpnum = <STDIN>; chomp $user_snpnum; $snpnum = $user_snpnum - 1;
		print "Which column shows the number of algorithms (1, 2, 3 etc)?\n";
		$user_thres = <STDIN>; chomp $user_thres; $thres = $user_thres - 1;
		print "Is there a header ('y' or 'n')?\n";
		$head = <STDIN>; chomp $head;
	}
	print "How many algorithms do you want to set as the minimum threshold (1, 2 or 3)?\n";
	$user_thresthres = <STDIN>; chomp $user_thresthres; $thresthres = $user_thresthres;
	
	if ($head !~ /n/){
		shift @CNV;
	}
	print "Sorting CNVs by sampleID, chromosome and start position\n";
	#Sort by Start (numbers)
	@CNV = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNV;
	
	#Sort by Chromosome (string)
	@CNV = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNV;
	
	#Sort by SampleID (string)
	@CNV = sort { (split '\t', lc $a)[$sampleID] cmp (split '\t', lc $b)[$sampleID] } @CNV;
	
#	#Removes calls with a number of algorithms less than set threshold
#	print "Removing CNVs with less than the specified number of algorithms\n";
#	foreach $line (@CNV){
#		@nowt = split ("\t", $line);
#		if ($nowt[$thres] < $thresthres){}
#		else {
#			push (@CNV2, $line);
#		}
#	}
#	
	print "Now looking for CNVs that match\n";
	$track = 0;
	#Does each CNV line up with the one below?
	foreach $line (@CNV){
		@before = split ("\t", $CNV[$track - 1]);
		@now = split ("\t", $CNV[$track]);
		@after = split ("\t",$CNV[$track + 1]);
		
		#This adds half the size of the CNV to each side of the CNV prior to comparison
		$beforesize = ($before[$stop]-$before[$start]) / $factor; 
		$before[$start] = $before[$start] - $beforesize;
		$before[$stop] = $before[$stop] + $beforesize;
		$nowsize = ($now[$stop]-$now[$start]) / $factor;
		$now[$start] = $now[$start] - $nowsize;
		$now[$stop] = $now[$stop] + $nowsize;
		$aftersize = ($after[$stop]-$after[$start]) / $factor;
		$after[$start] = $after[$start] - $aftersize;
		$after[$stop] = $after[$stop] + $aftersize;
		
		$track++;
		$sampcnvcount++;
		
		#print "Now: $now[0],$now[1],$now[2],$now[3],$now[4],ST:$now[$start],ST:$now[$stop]\n";
		#print "$sampleID,$chromo,$start,$stop\n";
		
		#Makes sure that no CNVs have a stop site before their start site
		if (($now[$stop] - $now[$start]) < 0){
			print "ERROR: CNV $now[0] has a stop co-ordinate lower than the start co-ordinate\n";
			print LOG1 "$timeanddate\tMerge\tERROR\tCNV $now[0] has a stop co-ordinate lower than the start co-ordinate\n";
			$error++;
		}
		#print "\n\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
		
		
		
		#If this CNV is from a differet sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if ($now[$sampleID] ne $before[$sampleID]
		or lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
		elsif ($now[$sampleID] eq $before[$sampleID] 
		and lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print "Error with the following CNV, its status could not be determined:\n$line\n";
			print LOG1 "$timeanddate\tMerge\tERROR\tThe status of the following CNV could not be determined:\t$line\n";
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if ($now[$sampleID] ne $after[$sampleID]
		or lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			#Process @match
			@returns = &checkmatch(@match);
			push (@results, $returns[0]);
			@match = ();
			$locicounter++;
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		#The numbers of loci with all, 1, 2 or 3 algos are entered into the corresponding sample name
		if ($now[$sampleID] ne $after[$sampleID]){
			$sampcnv{$now[$sampleID]} = $sampcnvcount;
			$sampcnvcount = 0;
		}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	
	foreach $bigone (@results){
		if ($bigone ne ""){
			print OUT "$bigone\n";
			print "Large CNV found: $bigone\n";
		}
	}
}

#Uses the file dbSNP_build130.txt to count the number of known SNPs in a region
if ($dbsnpcounter == 1){
	#DB SNP includes some regions larger than one basepair. For the sake of ease this algorithm will assume that each SNP is only one basepair long and simply count the number of unique start points
	#DB SNP headers
	$bin =       0;
	$chromopos = 1;
	$startpos =  2;
	$stoppos =   3;
	$name =      4;
	
	print "Opening regions file: $ARGV[0]\n";
	open (INPUT2, "<$ARGV[0]") || die "Could not open file $ARGV[0]; check it is in the right directory\n";
	$inputfile = $ANloc."dbSNP_build130.txt";
	#$inputfile = "C:/Users/ACCUMBENS/Downloads/Test_dbSNP_build130.txt";
	open (INPUT, "<$inputfile") || die "Could not open file $inputfile\n";
	print "Opening file: $inputfile\n";
	
	while (<INPUT>){
		$linecounthere++;
		$tracker++;
		if ($javamethod == 0){print "\rDoing line $linecounthere of 18,833,532";}
		elsif($tracker >= 500000){
			print "Doing line $linecounthere of 18,833,532\n";
			$tracker = 0;
		}
		@tab = split ("\t", $_);
		$chrom = $tab[$chromopos];
		$chromlist{$tab[$chromopos]} = 0;
		push (@$chrom, $tab[$startpos]);
		#$$chrom{$tab[$startpos]} = 0;
	}
	
	close (INPUT);
	
	print "\nSorting chromosome arrays\n";
	
	@chromlist = keys %chromlist;
	@chromlist = sort  { $a cmp $b } @chromlist;
	#print @chromlist;
	
	foreach $chro (@chromlist){
		@$chro = sort  { $a <=> $b } @$chro;
		$probes = @$chro;
		#print "Found $probes probes on $chro\n";
		@$chro = &uniq(@$chro);
		$probes = @$chro;
		print "Found $probes probes on $chro\n";
	}
	
	print "Probes entered into chromosome specific arrays\n";
	
	@CNV = <INPUT2>; close (INPUT2); chomp @CNV;
	@lineshow = split ("\t", $CNV[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	print "Which column is the chromosome in (1, 2, 3 etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	print "Is there a header ('y' or 'n')?\n";
	$head = <STDIN>; chomp $head;
	
	$output = "dbSNP_".$ARGV[0];
	open (OUT, ">$output") || die "Could not open output file $output; may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	if ($head =~ /y/i){
		print OUT "$CNV[0]\tSNPs\n";
		shift @CNV;
		$linecount2 = 1;
	}
	
	$tracker = 0;
	foreach $term (@CNV){
		#print "$term\n";
		$linecount2++;
		@tabs = split ("\t", $term);
		$chrm = $tabs[$chromo];
		$chrm =~ s/chr//gi;
		$chrm =~ tr/a-z/A-Z/;
		$chrm = "chr" . $chrm;
		$starth = $tabs[$start];
		$stoph = $tabs[$stop];
		$count = 0;
		$mark = 0;
		
		$arraysize = @$chrm;
		$newspot = sprintf ("%.f",($arraysize / 2));
		$halfspot = $newspot;
		$round = 0;
		$conquer = 0;
		$zero = 0;
		#print "Array size: $arraysize, Round: $round\n";
		&divide;
		#print "Hey I'm back here\n";
		
		$tracker++;
		if ($javamethod == 0){print "\r$linecount2 lines analysed";}
		elsif($tracker >= 100000){
			print "$linecount2 lines analysed\n";
			$tracker = 0;
		}
		print OUT "$term\t$snpcount\n";
		if ($snpcount == 0){
			$errorcount++;
		}
	}
	
	if ($errorcount / $linecount2 > 0.5){
		print "Finished, but check the results carefully - many regions have 0 SNPs\n";
	}
	else {
		print "\nAll done, everything is ok\n";
	}
}

#Uses a FinalReport to work out how many probes are present within each region
if ($snpcounter == 1){
	@files = <*>;
	$firstfile = 0;
	foreach (@files){
		if ($_ =~ /$namematch/ 
		and $firstfile == 0){
			$firstfile = 1;
			open (INPUT, "<$_") || die "Could not open $namematch file $_\n";
			print "Opening $namematch file: $_\n";
			{ local $/=undef;  $input1=<INPUT>; }
			close INPUT;
			($\) = (/(\r\n|\r|\n)/);
			@snp=split (/\r\n|\r|\n/, $input1); chomp @snp; 
			
			@herebit = split ("\t", $snp[20]);
			$samplename = $herebit[0];
	
			foreach $line (@snp){
				@bits = split ("\t", $line);
				chomp @bits;
				if ($bits[0] eq $samplename){
						$chrom = "chr" . $bits[2];
						$chromlist{"chr" . $bits[2]} = 0;
						push (@$chrom, $bits[3]);
				}
			}
	
			@snp = ();
		}
	}
	
	if ($firstfile == 0){
		die "There was no Final Report file to work out how many SNPs there are in each region; please find any Final Report file from the correct type of chip and put it in this folder\n";
	}
	
	print "Sorting chromosome arrays\n";
	
	@chromlist = keys %chromlist;
	@chromlist = sort  { $a cmp $b } @chromlist;
	#print @chromlist;
	
	foreach (@chromlist){
		@$_ = sort  { $a <=> $b } @$_;
		$probes = @$_;
		print "Found $probes probes on $_\n";
	}
	
	print "Probes entered into chromosome specific arrays\n";
	
	print "Opening CNV file: $ARGV[0]\n";
	open (INPUT, "<$ARGV[0]") || die "couldn't open file $ARGV[0]\n";
	@CNV = <INPUT>; close (INPUT); chomp @CNV;
	@lineshow = split ("\t", $CNV[0]);
	foreach (@lineshow){
		$linecount++;
		unless ($linecount >= 15){
			print "$linecount) $_\n";
		}
	}
	
	print "Which column is the chromosome in (1, 2, 3 etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	print "Is there a header ('y' or 'n')?\n";
	$head = <STDIN>; chomp $head;
	
	$output = "SNP_".$ARGV[0];
	open (OUT, ">$output") || die "Could not open output file $output; may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	if ($head =~ /y/i){
		print OUT "$CNV[0]\tSNPs\n";
		shift @CNV;
		$linecount2 = 1;
	}
	
	foreach $term (@CNV){
		#print "$term\n";
		$linecount2++;
		@tabs = split ("\t", $term);
		$chrm = $tabs[$chromo];
		$chrm =~ s/chr//gi;
		$chrm =~ tr/a-z/A-Z/;
		$chrm = "chr" . $chrm;
		$starth = $tabs[$start];
		$stoph = $tabs[$stop];
		$count = 0;
		$mark = 0;
		
		$arraysize = @$chrm;
		$newspot = sprintf ("%.f",($arraysize / 2));
		$halfspot = $newspot;
		$round = 0;
		$conquer = 0;
		$zero = 0;
		#print "Array size: $arraysize, Round: $round\n";
		&divide;
		#print "Hey I'm back here\n";
		
		$tracker++;
		if ($javamethod == 0){print "\r$linecount2 lines analysed";}
		elsif($tracker >= 100000){
			print "$linecount2 lines analysed\n";
		}
		print OUT "$term\t$snpcount\n";
		if ($snpcount == 0){
			$errorcount++;
		}
	}
	
	if ($errorcount / $linecount2 > 0.5){
		print "Finished, but check the results carefully - many regions have 0 SNPs\n";
	}
	else {
		print "\nAll done, everything is ok\n";
	}
}

#Looks for LogR deviation in Final Reports using multiple processors
if ($chiperror_batch == 1){
	print LOG1 "$timeanddate\tQuality\tINFO\tLooking for widened or extreme LogR values in samples\n";
	$progloc = '"'.$JVloc.$progname.'"';
	if ($processors <= 1){
		$command1 = "perl " . $progloc . " --chiperror\n";
		system ($command1);
	}
	else{
		foreach $pro (1..$processors){
			$countfile = $tag . "_Count_qc_" . $pro . ".txt";
			open (OUT, ">$countfile")|| die "Could not open counter file $countfile; may already be open\n";
			close OUT;
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			push (@batchlist, $batchname);
			open (OUTHERE, ">$batchname") || die "Could not open $batchname; may already be open\n";
			print "Opening chiperror window $pro at $timeanddate\n";
			$command1 = "perl " . $progloc . " --chiperror " . $tag . "_Processor_list_" . $pro . ".txt";
			$command2 = "perl " . $progloc . " --delete ".$countfile;
			print OUTHERE "$command1\n$command2\nexit\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0;
			foreach $pro (1..$processors){
				$countfile = $tag . "_Count_qc_" . $pro . ".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (2);
		} until ($marker == 1);
	}
	
	if ($processors > 1){
		$badout = $pro_dir . "CE_" . $tag . "_bad.txt";
		$wideout = $res_dir . $tag . "_Wide_LogR.txt";
		$extremeout = $res_dir . $tag . "_Extreme_LogR.txt";
		open (OUTCHIP1, ">$extremeout") || die "Could not open the results file $extremeout; may already be open\n";
		open (OUTCHIP2, ">$wideout") || die "Could not open the results file $wideout; may already be open\n";
		open (BADCHIP, ">$badout") || die "Could not open the results file $badout; may already be open\n";
		
		foreach $pro (1..$processors){
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			#The separate Name_ID_sex_ files are combined
			$badout = $pro_dir . "CE_" . $tag . "_bad_".$pro.".txt";
			$wideout = $res_dir . $tag . "_Wide_LogR_".$pro.".txt";
			$extremeout = $res_dir . $tag . "_Extreme_LogR_".$pro.".txt";
			open (IN1, "<$extremeout") || die "Could not open the results file $extremeout; may already be open\n";
			open (IN2, "<$wideout") || die "Could not open the results file $wideout; may already be open\n";
			open (IN3, "<$badout") || die "Could not open the results file $badout; may already be open\n";
			@lines = <IN1>; chomp @lines; close IN1; 
			if ($pro == 1){print OUTCHIP1 "$lines[0]\n"; $last1 = "$lines[-1]\n";}
			shift @lines; pop @lines; pop @lines;
			foreach $line (@lines){
				chomp $line;
				@tab = split (/\t/,$line);
				$extlog{$tab[0]} = $tab[26];
				print OUTCHIP1 "$line\n";
			}
			unlink $extremeout;
			@lines = <IN2>; chomp @lines; close IN2; 
			if ($pro == 1){print OUTCHIP2 "$lines[0]\n"; $last2 = "$lines[-1]\n";}
			shift @lines; pop @lines; pop @lines;
			foreach $line (@lines){
				chomp $line;
				@tab = split (/\t/,$line);
				$widlog{$tab[0]} = $tab[26];
				print OUTCHIP2 "$line\n";
			}
			unlink $wideout;
			@lines = <IN3>; chomp @lines; close IN3; 
			foreach $line (@lines){
				chomp $line;
				@tab = split (/\t/,$line);
				print BADCHIP "$line\n";
			}
			unlink $badout;
			unlink $batchname;
		}
		print OUTCHIP1 "\n$last1\n";
		print OUTCHIP2 "\n$last2\n";
		print "\n";
		
		$timeanddate = &showtime;
		@samplespresent = keys %extlog;
		$totalsamp = @samplespresent;
		print LOG1 "$timeanddate\tQuality\tINFO\tExtreme and Wide LogR values calculated for $totalsamp individuals\n";
		print "Extreme and Wide LogR values calculated for $totalsamp individuals\n";
		
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tExtreme LogR\tWide LogR\n";
		shift @samplist;
		$timeanddate = &showtime;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$ext = $extlog{$tab[0]};
			$wid = $widlog{$tab[0]};
			if ($wid > 0 or $ext > 0){
				print SAMP "$_\t$ext\t$wid\n";
			}
			else {
				print SAMP "$_\t0\t0\n";
				print LOG1 "$timeanddate\tQuality\tERROR\tNo extreme or wide LogR values were calculated for sample $_\n";
				print "ERROR: No extreme or wide LogR values were calculated for sample $_\n";
			}
		}
	}
}

#Looks for LogR deviation in Final Reports
if ($chiperror == 1){
	&fr_columns;
	
	$thres1 = -1;
	$thres2 = 0.5;
	
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
		$badout = "Chiperror_Fail.txt";
		$wideout = "Chiperror_Wide.txt";
		$extremeout = "Chiperror_Extreme.txt";
		print LISTOUT $curr_dir."$badout\n".$curr_dir."$wideout\n".$curr_dir."$extremeout\n";
		@files = <*>;
	}
	elsif ($processors <= 1) {
		print LOG1 "$timeanddate\tQuality\tINFO\tChiperror assessment started\n";
		$method = 0;
		$badout = $pro_dir . "CE_" . $tag . "_bad.txt";
		$wideout = $res_dir . $tag . "_Wide_LogR.txt";
		$extremeout = $res_dir . $tag . "_Extreme_LogR.txt";
		@files = <*>;
	}
	else {
		$method = 0;
		@files = ();
		$prolist = $ARGV[0];
		$pro = $prolist;
		$remove1 = $tag . "_Processor_list_";
		$pro =~ s/$remove1//gi;
		$pro =~ s/.txt//gi;
		$badout = $pro_dir . "CE_" . $tag . "_bad_".$pro.".txt";
		$wideout = $res_dir . $tag . "_Wide_LogR_".$pro.".txt";
		$extremeout = $res_dir . $tag . "_Extreme_LogR_".$pro.".txt";
		print "Opening list of files $prolist\n";
		open (IN, "<$prolist") || die "Could not find $prolist; check it has been created\n";
		while (<IN>){
			chomp $_;
			@tab = split (/\t/,$_);
			push (@files, $tab[0]);
		}
	}
	
	open (OUTCHIP1, ">$extremeout") || die "Could not open the results file $extremeout; may already be open\n";
	open (OUTCHIP2, ">$wideout") || die "Could not open the results file $wideout; may already be open\n";
	open (BADCHIP, ">$badout") || die "Could not open the results file $badout; may already be open\n";
	
	&chiperrorsub(@files);
}

#Joins to files with columns in the same order (opposite of spliter)
if ($combo == 1){
	print "What should the output file be called? (eg TSandAU_Merged)\n";
	$user_file = <STDIN>; chomp $user_file; $output = $user_file;
	$output = $output . ".txt";
	open (COUT, ">$output") || die "Cannot open file $output, may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	
	$files = @ARGV;
	$input = $ARGV[0];
	print "\nOpening CNV file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input, check it is in the right directory\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@CNV1=split (/\r\n|\r|\n/, $input1);
	$firsthead = $CNV1[0];
	chomp $firsthead;
	
	foreach (@ARGV){
		$linecount = 0;
		$filecount++;
		#print "Filecount: $filecount, Max: $files\n";
		if ($filecount <= ($files - 1)){
			#print "Count: $filecount\n";
			print "Opening CNV file: $ARGV[$filecount]\n";
			open (INPUT2, "<$ARGV[$filecount]") || die "Cannot open file $ARGV[$filecount], check it is in the right directory\n";
			{ local $/=undef;  $input1=<INPUT2>; }
			close INPUT2;
			($\) = (/(\r\n|\r|\n)/);
			@CNV2=split (/\r\n|\r|\n/, $input1); 
			#@CNV2 = <INPUT2>; close (INPUT2);
			@lineshow1 = split ("\t", $firsthead);
			@lineshow2 = split ("\t", $CNV2[0]);
			$charc = length($input);
			$charc2 = 38 - $charc;
			print "$input";
			$countspace = 0;
			until ($countspace == $charc2
			or $countspace >= 38){
				$countspace++;
				print " "
			}
			print "$ARGV[$filecount]\n";
			$linecount1 = 0; $linecount2 = 0;
			foreach (@lineshow1){
				$linecount1++;
				unless ($linecount1 >= 15){
					$charc = length($_);
					$charc2 = 35 - $charc;
					print "$linecount1) $_";
					$countspace = 0;
					until ($countspace == $charc2
					or $countspace >= 35){
						$countspace++;
						print " "
					}
					print "$linecount1) $lineshow2[$linecount2]   \n";
				}
				$linecount2++;
			}
			
			print "Are these columns in the same order? ('y' or 'n')\n";
			$user_order = <STDIN>; chomp $user_order; $order = $user_order;
			if ($order eq "n"){
				die "Correct the column order using Excel or a perl script first\n";
			}
			print "Is there a header? ('y' or 'n')\n";
			$user_head = <STDIN>; chomp $user_head; $head = $user_head;
			
			if ($head eq "y"){
				shift (@CNV2);
			}
			print "Combining this file with the first file\n";
			push (@CNV1, @CNV2);
		}
	}
	print "Printing out the combined file\n";
	print COUT @CNV1;
	
}

#Splits a file by the features in a specific column (opposite of combo)
if ($spliter == 1){
	$input = $ARGV[0];
	print "Opening CNV file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input, check it is in the right directory\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@CNV=split (/\r\n|\r|\n/, $input1); chomp @CNV;
	
	@lineshow = split ("\t", $CNV[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	open (STATS, ">Split_stats.txt") || die "Cannot open file Split_stats.txt, may already be open\n";
	print LISTOUT $curr_dir."Split_stats.txt\n$curr_dir\n";
	########################################################
	print "\nWhich column do you want to split by? (1, 2, 3 etc)\n";
	$user_split = <STDIN>; chomp $user_split; $splitcol = $user_split - 1;
	print "Is there a header? ('y' or 'n')\n";
	$user_head = <STDIN>; chomp $user_head; $head = $user_head;
	if ($head eq "y"){
		$headline = "$CNV[0]\n";
		shift (@CNV);
	}
	print "Sorting CNVs by the column to be split\n";
	@CNV = sort { (split '\t', $a)[$splitcol] cmp (split '\t', $b)[$splitcol] } @CNV;
	
	foreach (@CNV){
		@tmp = split ("\t", $_);
		unless ($terms{$tmp[$splitcol]} == 1){
			$terms{$tmp[$splitcol]} = 1;
			$output = $tmp[$splitcol] . "_" . $input;
			open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
			print STATS "$line\n$tmp[$splitcol]\t";
			$line = 0;
			print OUT "$headline";
			$splitcount++;
		}
		print OUT "$_\n";
		$line++;
		$tracker++;
		if ($javamethod == 0){
			print "\rDoing line $line  ";
		}
		elsif ($tracker > 9999){
			print "Doing line $line\n";
			$tracker = 0;
		}
	}
	print STATS "$line";
	print "\n$input split into $splitcount files\n";
}

#Compares the frequency of regions between to CNV files
if ($relfreq == 1){
	$input = $ARGV[0];
	print "Opening CNV file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input; check it is in the right directory\n";
	@CNV1 = <INPUT>; chomp @CNV1; close (INPUT);
	
	$input2 = $ARGV[1];
	print "Opening CNV file: $input2\n\n";
	open (INPUT2, "<$input2") || die "Cannot open file $input2; check it is in the right directory\n";
	@CNV2 = <INPUT2>; chomp @CNV2; close (INPUT2);
	
	$inputm = $input;
	$inputm =~ s/.txt//i;
	$inputm2 = $input2;
	$inputm2 =~ s/.txt//i;
	$output = "Relfreq_" . $inputm . "_" . $inputm2 . ".txt";
	open (RFOUT, ">$output") || die "Cannot open file $output; may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	#print RFOUT "UCSC Lookup\tChr\tStart\tStop\t$inputm Freq\t$inputm Calls\t$inputm2 Freq\t$inputm2 Calls\t$inputm Enrichment\t$inputm2 Enrichment\n";
	
	@lineshow = split ("\t", $CNV1[0]);
	@lineshow2 = split ("\t", $CNV2[0]);
	$charc = length($input);
	$charc2 = 38 - $charc;
	print "$input";
	$countspace = 0;
	until ($countspace == $charc2
	or $countspace >= 38){
		$countspace++;
		print " "
	}
	print "$input2\n";
	foreach (@lineshow){
		$linecount++;
		unless ($linecount >= 15){
			$charc = length($_);
			$charc2 = 35 - $charc;
			print "$linecount) $_";
			$countspace = 0;
			until ($countspace == $charc2
			or $countspace >= 35){
				$countspace++;
				print " "
			}
			print "$linecount) $lineshow2[$linecount2]   \n";
		}
		$linecount2++;
	}
	
	print "\nFor frequency comparisons you probably want to use true start and stop\n";
	print "coordinates rather than loci coordinates\n";
	print "Which column is the sampleID information in (1, 2, 3 etc)?\n";
	$user_samp = <STDIN>; chomp $user_samp; $sampleID = $user_samp - 1;
	print "Which column is the chromosome in (1, 2, 3 etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	
	if ($thres == 1){
		$numsnpcol  = $stop + 3;
		$algonumcol = $stop + 5;
		$cnvtypecol = $stop + 1;
		$numsnpcol1  = $numsnpcol + 1;
		$algonumcol1 = $algonumcol + 1;
		$cnvtypecol1 = $cnvtypecol + 1;
		
		print "\nAre these columns correct ('y' or 'n')?\nNumber of SNPs: $numsnpcol1, Type of CNV: $cnvtypecol1, Number of algorithms: $algonumcol1        ";
		$user_col = <STDIN>; chomp $user_col; $colans = $user_col;
		
		if ($colans eq "n"){
			print "Which column is the number of SNPs in (1, 2, 3 etc)?\n";
			$user_samp = <STDIN>; chomp $user_samp; $numsnpcol = $user_samp - 1;
			print "Which column is the number of algorithms in (1, 2, 3 etc)?\n";
			$user_chromo = <STDIN>; chomp $user_chromo; $algonumcol = $user_chromo - 1;
			print "Which column is the type of CNV in (1, 2, 3 etc)?\n";
			$user_start = <STDIN>; chomp $user_start; $cnvtypecol = $user_start - 1; 
		}
	}
	
	print "Is the second file in the same order as the first file? ('y' or 'n')\n";
	$user_head = <STDIN>; chomp $user_head; $head = $user_head;
	
	$columns{1} = "$sampleID\t$chromo\t$start\t$stop\t$numsnpcol\t$algonumcol\t$cnvtypecol";
	if ($head eq "y"){
		$columns{2} = $columns{1};
	}
	else {
		print "Which column is the sampleID information in (1, 2, 3 etc)?\n";
		$user_samp = <STDIN>; chomp $user_samp; $sampleID = $user_samp - 1;
		print "Which column is the chromosome in (1, 2, 3 etc)?\n";
		$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
		print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
		$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
		print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
		$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
		
		if ($thres == 1){
			print "Which column is the number of SNPs in (1, 2, 3 etc)?\n";
			$user_samp = <STDIN>; chomp $user_samp; $numsnpcol = $user_samp - 1;
			print "Which column is the number of algorithms in (1, 2, 3 etc)?\n";
			$user_chromo = <STDIN>; chomp $user_chromo; $algonumcol = $user_chromo - 1;
			print "Which column is the type of CNV in (1, 2, 3 etc)?\n";
			$user_start = <STDIN>; chomp $user_start; $cnvtypecol = $user_start - 1; 
		}
		$columns{2} = "$sampleID\t$chromo\t$start\t$stop\t$numsnpcol\t$algonumcol\t$cnvtypecol";
	}
	shift (@CNV1);
	shift (@CNV2);
	
	if ($thres == 1){
		print "\nYou have selected to filter the input:\n";
		print "What SNP/Probe threshold do you want? (3, 5, 10; put 0 for none)\n";
		$user_snp = <STDIN>; chomp $user_snp; $snpthresnow = $user_snp;
		print "What number of algorithms do you want? (1, 2, 3; put 0 for none)\n";
		$user_algo = <STDIN>; chomp $user_algo; $algothresnow = $user_algo;
		print "Do you want deletions (del), duplications (dup) or both (both)\n";
		$user_type = <STDIN>; chomp $user_type; $typethresnow = $user_type;
		push (@thresscoreout, "The following thresholds were used:\n");
		push (@thresscoreout, "Number of SNPs/Probes: $snpthresnow\n");
		push (@thresscoreout, "Number of algorithms: $algothresnow\n");
		push (@thresscoreout, "Types of CNVs: $typethresnow\n");
	}
	
	print RFOUT "UCSC Lookup\tChr\tStart\tStop\tSize\t$inputm Freq\t$inputm2 Freq\t$inputm Calls\t$inputm No calls\t$inputm2 Calls\t$inputm2 No calls\t$inputm Enrichment\t$inputm2 Enrichment\n";


	@tmp = split ("\t", $columns{1});
	$sampleID = $tmp[0];
	$chromo = $tmp[1];
	$start = $tmp[2];
	$stop = $tmp[3];
	$numsnpcol = $tmp[4];
	$algonumcol = $tmp[5];
	$cnvtypecol = $tmp[6];
	
	$output = "Freq_" . $input;
	open (FOUT, ">$output") || die "Cannot open file $output; may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	
	$firsttotal = &freqsub (@CNV1);
	@first = @merge;
	@merge = ();
	&reformat (@first);
	@firstmerge = @mergein;
	#print @firstmerge;
	
	@tmp = split ("\t", $columns{2});
	$sampleID = $tmp[0];
	$chromo = $tmp[1];
	$start = $tmp[2];
	$stop = $tmp[3];
	$numsnpcol = $tmp[4];
	$algonumcol = $tmp[5];
	$cnvtypecol = $tmp[6];
	
	close FOUT;
	$output2 = "Freq_" . $input2;
	open (FOUT, ">$output2") || die "Cannot open file $output2; may already be open\n";
	print LISTOUT $curr_dir."$output2\n";
	
	$secondtotal = &freqsub (@CNV2);
	@second = @merge;
	@merge = ();
	#print @second;
	&reformat (@second);
	@secondmerge = @mergein;
	#print @secondmerge;
	
	$famID=0;
	$sampleID=1;
	$chromo=2;
	$start=3;
	$stop=4;
	$cnvtype=5;
	$score=6;
	$numsnps=7;
	$size=8;
	
	#Initialise UniqueID counter
	$uidc=0;
	@CNVall = ();
	
	#open input files 
	#Put the CNVs into an array but with each one with an ID made up of the algo and uniqueID counters	
	print "Assessing enrichment between populations\n";
	for ($c=0; $c<=$#firstmerge; $c++){ 
		chomp $firstmerge[$c];
		if($firstmerge[$c] =~ /FID/){}
		else{
			@tmp = split ("\t", $firstmerge[$c]);
			$uidc++; 
			$uidc = sprintf ("%07d", $uidc);
			$firstmerge[$c] = "A1" . $uidc . "\tA1\t" . $firstmerge[$c];
			push (@CNVall, $firstmerge[$c]);
		}
	}
	
	for ($c=0; $c<=$#secondmerge; $c++){ 
		chomp $secondmerge[$c];
		if($secondmerge[$c] =~ /FID/){}
		else{
			@tmp = split ("\t", $secondmerge[$c]);
			$uidc++; 
			$uidc = sprintf ("%07d", $uidc);
			$secondmerge[$c] = "B2" . $uidc . "\tB2\t" . $secondmerge[$c];
			push (@CNVall, $secondmerge[$c]);
		}
	}
	
	#print "CNV\n";
	#print @CNVall;
	
	&merge2;
	#print "\nMerge\n";
	#print @mergeout;

	
	$famID = 0 + $extra;
	$sampleID = 1 + $extra;
	$chromo = 2 + $extra;
	$lstart = 3 + $extra;
	$lstop = 4 + $extra;
	$cnvtype= 5 + $extra;
	$maxsize = 6 + $extra;
	$maxsnp = 7 + $extra;
	$lsize = 8 + $extra;
	$start = 10 + $extra;
	$stop = 11 + $extra;
	$size = 12 + $extra;
	$type = 13 + $extra;
	$algonum = 14 + $extra;
	$algos = 15 + $extra;
	$algo1 = 17 + $extra;
	$type1 = 18 + $extra;
	$probe1 = 19 + $extra;
	$size1 = 20 + $extra;
	$uid1 = 21 + $extra;
	$algo2 = 22 + $extra;
	$type2 = 23 + $extra;
	$probe2 = 24 + $extra;
	$size2 = 25 + $extra;
	$uid2 = 26 + $extra;
	
	foreach (@mergeout){
		#print RFOUT $_;
		@tmp = split ("\t", $_);
		#print RFOUT "Algo1:$tmp[$algo1] Probe1:$tmp[$probe1] Call1:$tmp[$type1]\n";
		$a1freq = 0;
		$a1call = 0;
		$b2freq = 0;
		$b2call = 0;
		$a1nocall = $firsttotal;
		$b2nocall = $secondtotal;
		if ($tmp[$algo1] eq "A1"){
			$a1freq = $tmp[$probe1];
			$a1call = $tmp[$type1];
			$a1nocall = $firsttotal - $a1call;
		}
		elsif ($tmp[$algo1] eq "B2"){
			$b2freq = $tmp[$probe1];
			$b2call = $tmp[$type1];
			$b2nocall = $secondtotal - $b2call;
		}
		if ($tmp[$algo2] eq "A1"){
			$a1freq = $tmp[$probe2];
			$a1call = $tmp[$type2];
			$a1nocall = $firsttotal - $a1call;
		}
		elsif ($tmp[$algo2] eq "B2"){
			$b2freq = $tmp[$probe2];
			$b2call = $tmp[$type2];
			$b2nocall = $secondtotal - $b2call;
		}
		if ($b2freq ne 0){
			$a1enrich = $a1freq/$b2freq
		}
		else {$a1enrich = "specific";}
		if ($a1freq ne 0){
			$b2enrich = $b2freq/$a1freq
		}
		else {$b2enrich = "specific";}
		$thesizehere = $tmp[$stop] - $tmp[$start];
		
		print RFOUT "$tmp[$chromo]:$tmp[$start]-$tmp[$stop]\t$tmp[$chromo]\t$tmp[$start]\t$tmp[$stop]\t$thesizehere\t$a1freq\t$b2freq\t$a1call\t$a1nocall\t$b2call\t$b2nocall\t$a1enrich\t$b2enrich\n";
		#print "OUT\n"."$tmp[$chromo]:$tmp[$start]-$tmp[$stop]\t$tmp[$chromo]\t$tmp[$start]\t$tmp[$stop]\t$a1freq\t$a1call\t$b2freq\t$b2call\t$a1enrich\t$b2enrich\n";
		
	}
	print RFOUT "\n\n@thresscoreout\n";
}

#Cross references a column of infor from one file to another
if ($vlookup == 1){
	$input = $ARGV[0];
	$output = "Ref_" . $input;
	open (OUT, ">$output") || die "Cannot open file $output; may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	print "Opening CNV file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input; check it is in the right directory\n";
	@CNV = <INPUT>; chomp @CNV; close (INPUT);
	
	@lineshow = split ("\t", $CNV[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	########################################################
	print "\nWhich column do you want to look up? (1, 2, 3 etc)\n";
	$user_lookup = <STDIN>; chomp $user_lookup; $lookup = $user_lookup - 1;
	print "Is there a header? ('y' or 'n')\n";
	$user_head = <STDIN>; chomp $user_head; $head = $user_head;
	if ($head eq "y"){
		$firstline = $CNV[0];
		shift (@CNV);
		print "Header noted\n";
	}
	else {print "No header\n";}
	
	$input2 = $ARGV[1];
	print "Opening reference file: $input2\n";
	open (INPUT2, "<$input2") || die "Cannot open file $input2, check it is in the right directory\n";
	@refer = <INPUT2>; chomp @refer; close (INPUT2);
	@lineshow = split ("\t", $refer[0]);
	foreach (@lineshow){
		$linecount2++;
		unless ($linecount2 >= 15){
			print "$linecount2) $_\n";
		}
	}
	
	########################################################
	print "\nWhich column matches the data you are looking up? (1, 2, 3 etc)\n";
	$user_refer = <STDIN>; chomp $user_refer; $refer = $user_refer - 1;
	print "Which column has the new data that you want to add in it? (1, 2, 3 etc)\n";
	$user_add = <STDIN>; chomp $user_add; $add = $user_add - 1;
	print "Is there a header? ('y' or 'n')\n";
	$user_head = <STDIN>; chomp $user_head; $head2 = $user_head;
	if ($head2 eq "y"){
		@tmp = split ("\t", $refer[0]);
		$newterm = $tmp[$add];
		shift (@refer);
		print "Header noted\n";
	}
	else {print "No header\n";}
	
	if ($head eq "y"){
		print OUT "$newterm\t$firstline\n";
	}
	
	foreach (@refer){
		@tmp = split ("\t", $_);
		chomp @tmp;
		if ($refer{$tmp[$refer]} ne ""
		and $refer{$tmp[$refer]} ne $refer{$tmp[$refer]}){
			print "ERROR The reference column is not unique: $tmp[$refer] equals $refer{$tmp[$refer]} and $add\n";
		}
		$refer{$tmp[$refer]} = $tmp[$add];
	}
	
	$noref = 0;
	$track = 0;
	foreach (@CNV){
		$count++;
		if ($javamethod == 0){print "\rDoing line $count";}
		elsif ($count >= $track){$track += 5000; print "Line $count completed\n";}
		@tmp = split ("\t", $_);
		print OUT "$refer{$tmp[$lookup]}\t$_\n";
		if ($refer{$tmp[$lookup]} eq ""){
			$noref++;
		}
	}
	$refpos = $count - $noref;
	print "\nAll $count lines assessed, $refpos terms added, $noref terms blank\n";
}

#Looks for chromosomes with high degree of homozygosity to identify uniparental disomy
if ($homozyg == 1){
	&fr_columns;
	
	@chromo = (1..22);
	$chrolist = "chr" . join ("\tchr", @chromo);
	#print $chrolist;
	
	$outfile = "Homozygosity.txt";
	open (OUT, ">$outfile") || die "Could not open the results file $outfile; may already be open \n";
	print LISTOUT $curr_dir."$outfile\n";
	print OUT "Sample\t$chrolist\tOver 90%\n";
	print "Finding percentage of homozygosity per chromosome in each sample\n";
	@files =<*>;
	foreach $file (@files){
		
		if ($file =~ /$namematch/){
			$filecount++;
			print "Examining file $filecount, $file\n";
			open (FR, "<$file" || die "unable to open $file\n");
			@finrep = <FR>; chomp @finrep; close FR;
			$linecount = 0;
			%het = ();
			%homo = ();
			$remark = 0;
			
			foreach (@finrep){
				$linecount++;
				if ($linecount >= 12){
					@temp = split ("\t", $_);
					if ($temp[$gtcol1] eq $temp[$gtcol2]){
						$homo{$temp[$chromocol]} = $homo{$temp[$chromocol]} + 1;
						#print "$temp[$chromocol]\t$homo{$temp[$chromocol]}\n";
					}
					else {
						$het{$temp[$chromocol]} = $het{$temp[$chromocol]} + 1;
					}
				}
				if ($linecount == 15){
					print OUT "$temp[$patname]";
				}
			}
			
			foreach (@chromo){
				$total = $homo{$_} + $het{$_};
				$homo = $homo{$_};
				if ($total > 0){
					$percent = sprintf ("%.0f", ($homo / $total * 100));
					if ($percent >= 90){
						$remark = 1;
						print "Found a homozygous chromosome in $temp[$patname]\n"
					}
				}
				else {
					$percent = "Error";
				}
				print OUT "\t$percent%";
			}
			print OUT "\t$remark\n";
		}
	}
}

#Merges the badchips files from different algorithms
if ($nastychip == 1){
	
	if($tag eq ""){
		$tag = "Bad";
		$method = 1;
		$pro_dir = "";
		$output = "Badchips.txt";
	}
	else {
		$method = 0;
		$output = $merge_dir.$tag."_Badchips.txt";
		print LOG1 "$timeanddate\tBadchip\tINFO\tLooking for chips that failed quality control\n";
		chdir $pro_dir;
		}
	
	open (BADOUT, ">$output") || die "Cannot open the file $ouput, may already be open";
	
	@files = <*>;
	foreach $file (@files){
		
		if ($file =~ /bad/i){
			$filecount++;
			$firsttwo = substr($file, 0,2);
			open (NEWFILE, "<$file" || die "Cannot open file $file\n");
			print "Opening file $file: ";
			@newie = <NEWFILE>; chomp @newie; close NEWFILE;
			$badcount = 0;
			$algos{$firsttwo} = 0;
			
			foreach (@newie){
				$badcount++;
				chomp $_;
				#print "Firsttwo: $firsttwo, Samp: $_\n";
				$$firsttwo{$_} = 0;
				$bad{$_} = 0;
			}
			print "$badcount bad chips identified\n";
			print LOG1 "$timeanddate\tBadchip\tINFO\t$firsttwo: $badcount samples failed quality control\n";
		}
	}
	
	chdir $curr_dir;
	@badones = keys %bad;
	$total = @badones;
	foreach (@badones){
		print BADOUT "$_\n";
	}
	print "$total bad chips found in total\n";
	print LOG1 "$timeanddate\tBadchip\tINFO\t$total bad chips from $filecount files\n";
	
	if ($method eq 0){
		@algos = keys %algos;
		
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]";
		foreach (@algos){
			print SAMP "\t$_ QC Pass";
		}
		print SAMP "\tOverall QC Pass\n";
		shift @samplist;
		foreach (@samplist){
			print SAMP "$_";
			@tab = split (/\t/,$_);
			$samp = $tab[0];
			#print "SAMP: $samp\n";
			foreach (@algos){
				if ($$_{$samp} eq 0){
					print SAMP "\t0";
				}
				else {
					print SAMP "\t1";
				}
			}
			if ($bad{$samp} eq 0){
				print SAMP "\t0\n";
			}
			else {
				print SAMP "\t1\n";
			}
		}
	}
}

#Modifies the pedfile to remove bad samples and correct if these were parents
if ($pedmodify){
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
	}
	else {
		print LOG1 "$timeanddate\tPedmod\tINFO\tPedfile modification started\n";
		$method = 0;
	}
	
	#Pedfile input file
	$families = 0;
	$samples = 1;
	$fathers = 2;
	$mothers = 3;
	$gender = 4;
	$phenotype = 5;
	
	@files = <*>;
	while (<@files>){
		if($_ =~ /pedfile/){
			$pedfilecount++;
			push (@pedfiles, "$_ ");
		}
	}
	if ($pedfilecount == 0){
		print LOG1 "$timeanddate\tPedmod\tERROR\tNo pedfile was present to modify by removing badchips\n";
		die "ERROR: No pedfile was present to modify by removing badchips\n";
	}
	elsif ($pedfilecount > 1){
		print LOG1 "$timeanddate\tPedmod\tERROR\tMultiple pedfiles were present: @pedfiles\n";
		die "ERROR: Multiple pedfiles were present: @pedfiles\n";
	}
	
	foreach (@files){
		if($_ =~ /pedfile/){
			print "Opening pedfile: $_\n";
			print LOG1 "$timeanddate\tPedmod\tINFO\tModifying pedfile $_\n";
			open (PED, "<$_") || die "Cannot open file $_, check it is in the right directory\n";
			@peds = <PED>; chomp @peds; close (PED);
			if ($method == 0){
				$output = $merge_dir . "Mod_" . $_;
				$badinput = $merge_dir . $tag . "_Badchips.txt";
				print LOG1 "$timeanddate\tPedmod\tINFO\tUsing badchips file $badinput\n";
			}
			elsif ($method == 1){
				$output = "Mod_" . $_;
				$badinput = "Badchips.txt";
			}
			open (PEDMOD, ">$output") || die "Cannot open file $output, may already be open\n";
			
			print "Opening file $badinput\n";
			open (BADLIST, "<$badinput") || die "Cannot open file $badinput, this is needed to identify bad samples\n";
			@badlist = <BADLIST>; chomp @badlist; close BADLIST;
			
			$badcount = 0; $baddadcount = 0; $badmumcount = 0;
			foreach (@badlist){
				@here = split ("\t", $_);
				chomp @here;
				$badchip{$here[0]} = 1;
				$badcount++;
			}
			
			foreach (@peds){
				$linecount++;
				@pedline = split ("\t", $_);
#				if ($badchip{$pedline[$samples]} == 1){
#					$badcount++;
#					#print "$pedline[$samples] is a badchip\n";
#				}
#				else{
					if ($badchip{$pedline[$fathers]} == 1){
						$baddadcount++;
						#print "$pedline[$samples] has a bad father\n";
						$pedline[$fathers] = 0;
					}
					if ($badchip{$pedline[$mothers]} == 1){
						$badmumcount++;
						#print "$pedline[$samples] has a bad mother\n";
						$pedline[$mothers] = 0;
					}
					$printout = join ("\t", @pedline);
					$hashout{$pedline[$samples]} = "$pedline[$families]\t$pedline[$fathers]\t$pedline[$mothers]\t$pedline[$phenotype]";
					#print "$pedline[$samples]\n";
					print PEDMOD "$printout\n";
#				}
			}
		}
	}
	print "$badcount individuals identifed\n";
	print "This caused $baddadcount fathers and $badmumcount mothers to be modified in the pedfile\n";
	
	if ($method eq 0){
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tFamily\tFather\tMother\tPhenotype\n";
		shift @samplist;
		$total = keys %hashout;
		@hashouta = keys %hashout;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$hashout = $hashout{$tab[0]};
			#Checks that the sample was listed in the PennCNV file
			print SAMP "$_";
			if ($hashout ne ""){
				print SAMP "\t$hashout\n";
				$samplecount++;
			}
			else {
				print SAMP "\t0\t0\t0\t0\n";
				print LOG1 "$timeanddate\tPedmod\tERROR\tNo pedfile record of sample $tab[0]\n";
				print "ERROR: No pedfile record of sample $tab[0]\n";
			}
			#Checks that input was created in the QuantiSNP batch file for this sample
		}
		if ($samplecount != $total){
			print LOG1 "$timeanddate\tPedmod\tERROR\tPedfile contains samples that were not listed in the Samplelist file\n";
			print "ERROR: Pedfile contains samples that were not listed in the Samplelist file\n";
		}
		else {
			print LOG1 "$timeanddate\tPedmod\tINFO\t$badcount individuals identifed from $badinput\n";
			print LOG1 "$timeanddate\tPedmod\tINFO\t$baddadcount father IDs and $badmumcount mother IDs modified in the pedfile\n";
			print LOG1 "$timeanddate\tPedmod\tINFO\tPedfile modification complete\n";
		}
	}
}

#This program initiates the sexsiftroutine to find the SampleID, Gender and chip type of every FinalReport present
if ($sexsift == 1){
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
		@files = <*>;
		&sexsiftroutine(@files);
		$outputname = "Name_ID_sex.txt";
		print LISTOUT $curr_dir."$outputname\n";
	}
	elsif ($processors <= 1) {
		$method = 0;
		@files = <*>;
		&sexsiftroutine(@files);
	}
	else {
		@files = ();
		$prolist = $ARGV[0];
		$pro = $prolist;
		$remove1 = $tag . "_Processor_list_";
		$pro =~ s/$remove1//gi;
		$pro =~ s/.txt//gi;
		$sexresults = "Name_ID_sex_".$pro.".txt";
		print "Opening list of files $prolist\n";
		open (IN, "<$prolist") || die "Could not find $prolist; check it has been created\n";
		open (OUT, ">$sexresults") || "Could not open $sexresults; may already be open\n";
		while (<IN>){
			chomp $_;
			@tab = split (/\t/,$_);
			push (@files, $tab[0]);
		}
		&sexsiftroutine(@files);
	}
	
	#Modifies the Samplelist file to add gender
	if ($method eq 0 and $processors <= 1){
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tGender\n";
		shift @samplist;
		$timeanddate = &showtime;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$sex = $sexlog{$tab[0]};
			if ($sex =~ /ale/){
				print SAMP "$_\t$sex\n";
			}
			else {
				print SAMP "$_\tUnknown\n";
				print LOG1 "$timeanddate\tSexsift\tERROR\tSample $_ has no matching gender\n";
				print "ERROR: Sample $_ has no matching gender\n";
			}
		}
	}
}

#Determines the gender, sampleID and chipip type of FinalReports using multiple processors
if ($sexsift_batch == 1){
	print LOG1 "$timeanddate\tSexsift\tINFO\tDetermining gender of samples present\n";
	$progloc = '"'.$JVloc.$progname.'"';
	print "Loc: $progloc\n";
	if ($processors <= 1){
		$command1 = "perl " . $progloc . " --sexsift\n";
		system ($command1);
	}
	else{
		foreach $pro (1..$processors){
			$countfile = $tag . "_Count_sx_".$pro.".txt";
			open (OUT, ">$countfile")|| die "Could not open counter file $countfile; may already be open\n";
			close OUT;
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			push (@batchlist, $batchname);
			open (OUTHERE, ">$batchname") || die "Could not open $batchname; may already be open\n";
			print "Opening sexsift window $pro at $timeanddate\n";
			$command1 = "perl " . $progloc . " --sexsift " . $tag . "_Processor_list_" . $pro . ".txt";
			$command2 = "perl " . $progloc . " --delete ".$countfile;
			print OUTHERE "$command1\n$command2\nexit\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0;
			foreach $pro (1..$processors){
				$countfile = $tag . "_Count_sx_".$pro.".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (2);
		} until ($marker == 1);
	
		$outputname = "Name_ID_sex.txt";
		open (OUT, ">$outputname") || die "Could not open the results file $outputname, may already be open\n";
		
		$mancount = 0;
		$femcount = 0;
	
		foreach $pro (1..$processors){
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			#The separate Name_ID_sex_ files are combined
			$sexfile = "Name_ID_sex_".$pro.".txt";
			open (INNER, $sexfile) || die "Could not find file $sexfile; check the batch file ran correctly\n";
			@lines = <INNER>; chomp @lines; close INNER;
			foreach $line (@lines){
				chomp $line;
				@tab = split (/\t/,$line);
				$sexlog{$tab[1]} = $tab[6];
				$chiplog{$tab[10]} = 0;
				if ($tab[6] eq "Male"){$mancount++;}
				elsif ($tab[6] eq "Female"){$femcount++;}
				push (@outputarray,$line);
			}
			unlink $sexfile;
			unlink $batchname;
		}
		foreach (@outputarray){
			print OUT "$_\n";
		}
		close OUT;
		print "\n";
		
		
		@samplespresent = keys %sexlog;
		$totalsamp = @samplespresent;
		print LOG1 "$timeanddate\tSexsift\tINFO\t$totalsamp individuals, $mancount male and $femcount female\n";
		print "$totalsamp individuals, $mancount male and $femcount female\n";
		@chiplog = keys %chiplog;
		$chiplog = @chiplog;
		if($chiplog > 1){
			print "ERROR: Multiple chip types present. Check Name_Sex_ID.txt.\n";
			print LOG1 "$timeanddate\tSexsift\tERROR\tMultiple chip types present, Check Name_Sex_ID.txt:\t";
			print "Chips present: ";
			foreach (@chiplog){
				print "$_\t";
				print LOG1 "$_\t";
			}
			print "\n";
			print LOG1 "\n";
			if ($method == 0){
				print "\nDo you want to continue with multiple chip types (not recommended)? (\'y\' or \'n\')      ";
				$user_refer = <STDIN>; chomp $user_refer;
				if ($user_refer =~ /n/i){
					print LOG1 "$timedate\tSexsift\tERROR\t$progname will now end\n";
					die "Program terminating due to multiple chip types\n";
				}
			}
		}
		
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tGender\n";
		shift @samplist;
		$timeanddate = &showtime;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$sex = $sexlog{$tab[0]};
			if ($sex =~ /ale/){
				print SAMP "$_\t$sex\n";
			}
			else {
				print SAMP "$_\tUnknown\n";
				print LOG1 "$timeanddate\tSexsift\tERROR\tSample $_ has no matching gender\n";
				print "ERROR: Sample $_ has no matching gender\n";
			}
		}
	}
}

#Creates the input for PennCNV and QuantiSNP
if ($inputer == 1 or $inputer2 == 1 or $inputer3 == 1){
	$progloc = '"'.$JVloc.$progname.'"';
	#print "$processors\n";
	#QuantiSNP settings
	$emiters = 10;
	$Lsetting = 2000000;
	$maxcopy = 4;
	$printrs = " --printRS";
	$doGCcorrect = ""; #" --doGCcorrect";
	
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
	}
	else {
		print LOG1 "$timeanddate\tInput\tINFO\tCNV algorithm input creation started\n";
		$method = 0;
	}
	print "CNV algorithm input creation started\n";
	mkdir ("01_Raw_Results", 0777);
	mkdir ("03_Merged", 0777);
	mkdir ("02_Processed", 0777);
	mkdir ("04_Final_Results", 0777);
	
	
	#Put the list of file into an array and into a hash with the genders
	$inputfile = "Name_ID_sex.txt";
	unless (-e $inputfile){
		print "The --sexsift command has not been run so there is no file to determine sex from; this is not a problem I'll run the --sexsift command now\n";
		@files = <*>;
		&sexsiftroutine(@files);
	}
	open (INPUT, "<Name_ID_sex.txt") || die "Could not open file \'Name_ID_sex.txt\', please run the --sexsift option\n"; 
	foreach $line (<INPUT>){
		@temp = split("\t", $line);
		$sexes{$temp[1]} = $temp[6];
	}
	
	if ($processors > 1){
		$counter = 0;
		foreach $pro (1..$processors){
			@files = ();
			$prolist = $tag . "_Processor_list_" . $pro . ".txt";
			open (PROIN, "<$prolist") || die "Could not open file $prolist; check it is in the right directory/n";
			while (<PROIN>){
				@tab = split (/\t/, $_);
				chomp @tab;
				$QTin = "QT_" . $tab[1] . ".txt";
				$PNin = "PN_" . $tab[1] . ".txt";
				push (@files, $QTin, $PNin);
			}
			
			$results = "$tag" . "_QT_results_".$pro.".txt";
			$outfile1 = "$PNloc"."$tag"."_Penn_list_".$pro.".txt";
			if ($inputer3 == 1){
				$outfile2 = "$QT2loc"."$tag"."_QT2batch_".$pro.".bat";
			}
			else{
				$outfile2 = "$QTloc"."$tag"."_QTbatch_".$pro.".bat";
			}
			
			open (OUT1, ">$outfile1") || die "Could not open file $outfile1; may already be open/n";
			open (OUT2, ">$outfile2") || die "Could not open file $outfile2; may already be open/n";
			#print OUT1 "cd $PNloc\n";
			#print OUT2 "cd $QTloc\n";
			
			foreach $file (@files){
				@under = split(/_/,$file);
				$sampID = $file;
				$samp_remove1 = $under[0] . "_";
				$samp_remove2 = ".txt";
				$sampID =~ s/$samp_remove1//gi;
				$sampID =~ s/$samp_remove2//gi;
				#print "$sampID\n";
				#Print out PennCNV list of Samples
				if ($file =~ /PN_/ and $file !~ /$tag/){
					unless ($chip =~ /Affy/){
						print OUT1 "$curr_dir" . "$file\n";
						$pninput{$sampID} = 1;
					}
				}
				#Prepare QuantiSNP batch file output
				elsif ($file =~ /QT_/ and $file !~ /$tag/){
					if ($chip =~ /Affy/){
						print OUT1 "$curr_dir" . "$file\n";
						$pninput{$sampID} = 1;
					}
					$qtinput{$sampID} = 1;
					$gender = $sexes{$sampID};
					if ($inputer3 == 1){
						$output = "quantisnp2 --config config.dat --outdir $res_dir --levels levels-hd.dat --config params.dat --sampleid $sampID --gender $gender --emiters $emiters --Lsetting $Lsetting --maxcopy $maxcopy".$printrs.$doGCcorrect." --gcdir ".$QT2loc."gc_data_b36 --input-files $curr_dir$file\n";
						#quantisnp2 --config config.dat --outdir C:/Keck_Illumina_July_2010/TS_July2010/test --levels levels-hd.dat --config params.dat --sampleid 5271336041_R04C01 --gender female --emiters 10 --Lsetting 2000000 --maxcopy 4 --gcdir C:/QuantiSNP2/gc_data_b36 --input-files C:/Keck_Illumina_July_2010/TS_July2010/test/QT_5271336041_R04C01.txt
					}
					else {
						$output = "quantisnp --config config.dat --output $res_dir"."$results --sampleid $sampID --gender $gender --emiters $emiters --Lsetting $Lsetting --maxcopy $maxcopy".$printrs.$doGCcorrect." --gcdir C:/QuantiSNPv1.1/gc_data_b36 --input-files $curr_dir$file $chip\n";
					}
					print OUT2 $output;
					$counter++;
					if ($counter == 1){
						print LOG1 "$timeanddate\tInput\tQuantiSNP\t$output";
					}
				}
			}
			$countfile = $tag . "_Count_qt_". $pro .".txt";
			$command2 = "perl " . $progloc . " --delete ".$countfile;
			$drive = substr $curr_dir, 0, 2;
			$curr_dir =~ s/\//\\/gi;
			print OUT2 "cd $curr_dir\n$drive\n$command2\nexit\n";
		}
	}
	
	else {
		@files=<*>;
		$results = "$tag" . "_QT_results.txt";
		$outfile1 = "$PNloc"."$tag"."_Penn_list.txt";
		$outfile2 = "$QTloc"."$tag"."_QTbatch.bat";
		#$outfile4 = "$PNloc"."$tag"."_Penn_list.bat";
		
		open (OUT1, ">$outfile1") || die "Could not open file $outfile1, may already be open/n";
		open (OUT2, ">$outfile2") || die "Could not open file $outfile2, may already be open/n";
		#open (OUT4, ">$outfile4") || die "Could not open file $outfile4, may already be open/n";
		
		$counter = 0;
		
		foreach $file (@files){
			@under = split(/_/,$file);
			$sampID = $file;
			$samp_remove1 = $under[0] . "_";
			$samp_remove2 = ".txt";
			$sampID =~ s/$samp_remove1//gi;
			$sampID =~ s/$samp_remove2//gi;
			#Print out PennCNV list of Samples
			if ($file =~ /PN_/ and $file !~ /$tag/){
				print OUT1 "$curr_dir" . "$file\n";
				$pninput{$sampID} = 1;
			}
			#Prepare QuantiSNP batch file output
			elsif ($file =~ /QT_/ and $file !~ /$tag/){
				$qtinput{$sampID} = 1;
				$gender = $sexes{$sampID};
				$output = "quantisnp --config config.dat --output $res_dir"."$results --sampleid $sampID --gender $gender --emiters $emiters --Lsetting $Lsetting --maxcopy $maxcopy $printrs $doGCcorrect --gcdir C:/QuantiSNPv1.1/gc_data_b36 --input-files $curr_dir$file $chip\n";
				print OUT2 $output;
				$counter++;
				if ($counter == 1){
					print LOG1 "$timeanddate\tInput\tQuantiSNP\t$output";
				}
			}
		}
		#Print out the QuantiSNP batch file
		$qtbatchout = "cd $curr_dir";
		$qtbatchout =~ s/\//\\/gi;
		print OUT2 "$qtbatchout\n";
		if ($inputer2 == 0 or $inputer3 == 0){
			print OUT2 "Batch2.bat\n";
		}
	}
	
	#print OUT4 "perl detect_cnv.pl -test -hmm lib/hhall.hmm -pfb lib/hhall.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_log.txt -output $res_dir"."$tag"."_Penn_results.txt\n";
	#print OUT4 "perl detect_cnv.pl -test -chrx -hmm lib/hhall.hmm -pfb lib/hhall.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_logX.txt -output $res_dir"."$tag"."_Penn_resultsX.txt\n";
	if ($method eq 0){
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tPN input\tQT input\n";
		shift @samplist;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$pninput = $pninput{$tab[0]};
			$pntotal = keys %pninput;
			$qtinput = $qtinput{$tab[0]};
			$qttotal = keys %qtinput;
			#Checks that the sample was listed in the PennCNV file
			print SAMP "$_";
			if ($pninput == 1){
				print SAMP "\tYes";
				$pnsamplecount++;
			}
			else {
				print SAMP "\tNo";
				print LOG1 "$timeanddate\tInput\tERROR\tNo PennCNV input created for sample $tab[0]\n";
				print "ERROR: No PennCNV input created for sample $tab[0]\n";
			}
			#Checks that input was created in the QuantiSNP batch file for this sample
			if ($qtinput == 1){
				print SAMP "\tYes\n";
				$qtsamplecount++;
			}
			else {
				print SAMP "\tNo\n";
				print LOG1 "$timeanddate\tInput\tERROR\tNo QuantiSNP input created for sample $tab[0]\n";
				print "ERROR: No QuantiSNP input created for sample $tab[0]\n";
			}
		}
		if ($pnsamplecount != $pntotal){
			print LOG1 "$timeanddate\tInput\tERROR\tPennCNV files were present that were not listed in the Samplelist file\n";
			print "ERROR: PennCNV files were present that were not listed in the Samplelist file\n";
		}
		if ($qtsamplecount != $qttotal){
			print LOG1 "$timeanddate\tInput\tERROR\tQuantiSNP files were present that were not listed in the Samplelist file\n";
			print "ERROR: QuantiSNP files were present that were not listed in the Samplelist file\n";
		}
		if ($pnsamplecount == $pntotal and $qtsamplecount == $qttotal) {
			print LOG1 "$timeanddate\tInput\tINFO\tInput creation complete for $pntotal samples\n";
			print "Input creation complete for $pntotal samples\n";
		}
	}
}

#Converts a FinalReports to PennCNV and QuantiSNP input files using multiple processors
if ($convert_batch == 1){
	print LOG1 "$timeanddate\tConvert\tINFO\tConversion started\n";
	$progloc = '"'.$JVloc.$progname.'"';
	if ($processors == 1){
		$command1 = "perl " . $progloc . " --convert\n";
		system ($command1);
	}
	else{
		foreach $pro (1..$processors){
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			$countfile = $tag . "_Count_cv_".$pro.".txt";
			open (OUT, ">$countfile")|| die "Could not open counter file $countfile; may already be open\n";
			close OUT;
			push (@batchlist, $batchname);
			open (OUTHERE, ">$batchname") || die "Could not open $batchname; may already be open\n";
			print "Opening conversion window $pro at $timeanddate\n";
			$command1 = "perl " . $progloc . " --convert " . $tag . "_Processor_list_" . $pro . ".txt";
			$command2 = "perl " . $progloc . " --delete ".$countfile;
			print OUTHERE "$command1\n$command2\nexit\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		$countfile = $tag . "_Count.txt";
		do{
			$windowcount = 0;
			foreach $pro (1..$processors){
				$countfile = $tag . "_Count_cv_".$pro.".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (5);
		} until ($marker == 1);
	}
	foreach $pro (1..$processors){
		$batchname = $tag . "_Batch_" . $pro . ".bat";
		unlink $batchname;
	}
	
	print "\n";
	
	@files = <*>;
	foreach $file (@files){
		$samp = $file;
		$samp =~ s/.txt//gi;
		$samp =~ s/PN_//gi;
		$samp =~ s/QT_//gi;
		if ($file =~ /PN_/ and $file =~ /.txt/){
			$pnfilesize = sprintf("%.1f",(-s $file) / 1000000);
			$filesizepn{$samp} = $pnfilesize;
		}
		elsif ($file =~ /QT_/ and $file =~ /.txt/){
			$qtfilesize = sprintf("%.1f",(-s $file) / 1000000);
			$filesizeqt{$samp} = $qtfilesize;
		}
	}
	
	$samplist = $tag. "_Sample_list.txt";
	open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
	@samplist = <SAMP>; chomp @samplist; close SAMP;
	$totalfiles = @samplist;
	open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
	print SAMP "$samplist[0]\tPN filesize (Mb)\tQT filesize (Mb)\n";
	shift @samplist;
	$totalfiles = @samplist;
	foreach (@samplist){
		@tab = split (/\t/,$_);
		$filesizes = $filesizepn{$tab[0]};
		if ($filesizes ne ""){
			print SAMP "$_\t$filesizepn{$tab[0]}\t$filesizeqt{$tab[0]}\n";
			$samplecount++;
		}
		else {
			print SAMP "$_\tUnknown\tUnknown\n";
			print LOG1 "$timeanddate\tConvert\tERROR\tSample $tab[0] did not correspond with a converted FinalReport\n";
			print "ERROR: Sample $tab[0] did not correspond with a converted FinalReport\n";
		}
	}
	if ($samplecount == $totalfiles){
		print LOG1 "$timeanddate\tConvert\tINFO\tConversion complete for $totalfiles samples\n";
	}
	else {
		print LOG1 "$timeanddate\tConvert\tERROR\tConversion complete, however $namematch files were present that were not listed in the Samplelist file\n";
		print "ERROR: Conversion complete, however $namematch files were present that were not listed in the Samplelist file\n";
	}
	$currenttime = &showtime;
	print "Conversion complete for $totalfiles samples at $currenttime\n";
}

#Converts a FinalReport to PennCNV and QuantiSNP input files
if ($convert == 1){
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
	}
	else {
		$method = 0;
	}
	&fr_columns;
	$filecount=0;
	$counter=0;
	$ARGV = @ARGV;
	if ($ARGV < 1){
		@files = <*>;
		#print "$ARGV is zero\n";
	}
	else {
		#print "$ARGV is over zero\n";
		@files = ();
		$filename = $ARGV[0];
		#print "Opening $filename\n";
		open (INHERE, "<$filename") || die "Could not open $filename; check it is in the right directory\n";
		while (<INHERE>){
			@tab = split (/\t/,$_);
			#print "File: $tab[0]\n";
			push (@files, $tab[0]);
		}
	}
	#$output4 = "$MUloc" . "SampleIDs";
	#open (OUTPUT4, ">$output4" || "Could not open $output4\n");
	foreach (@files){
		if ($_ =~ /$namematch/){
			$totalfiles++;
		}
	}
	
	foreach $file (@files){
		if ($file =~ /$namematch/gi){
			$filecount++;
			$currenttime = &showtime;
	
			#The first file is used to make a template and is then processed
			if ($filecount == 1){
				open (NEWFILE, "<$file" || die "Could not open $namematch file $file\n");
				@data='';
				@data=<NEWFILE>; chomp @data; close (NEWFILE);
				print "Sorting first file, which will serve as a template for all who follow...\n";
				@data= sort specificSort @data;
				print "Done, now loading data:\n";
				foreach (@data){
					@temp=split("\t", $_); chomp(@temp);
					$snp=$temp[$snpcol];
					if ($#temp > 5 && $snp !~ /Name/){
						$counter++;
						$orderedsnps{$counter}=$snp;
					}
				}
			}
			
			print "$filecount of $totalfiles) Loading $file at $currenttime\n";
			open (NEWFILE, "<$file" || die "Could not open $namematch file $file\n");
			
			%QT=''; %PN=''; %MU='';
			while (<NEWFILE>){
				chomp;
				@temp=split("\t", $_);
				if ($temp[$gtcol1] eq $temp[$gtcol2] and $temp[$ballele] <= 0.5){$gtype="AA"; $gt1="A"; $gt2="A";}
				elsif ($temp[$gtcol1] eq $temp[$gtcol2] and $temp[$ballele] > 0.5){$gtype="BB"; $gt1="B"; $gt2="B";}
				else {$gtype="AB"; $gt1="A"; $gt2="B";}
				$QT{$temp[$snpcol]} = "$temp[$snpcol]\t$temp[$chromocol]\t$temp[$chromodist]\t$temp[$log2col]\t$temp[$ballele]";
				$PN{$temp[$snpcol]} = "$temp[$snpcol]\t$temp[$chromocol]\t$temp[$chromodist]\t$gtype\t$temp[$log2col]\t$temp[$ballele]";
				#$MU{$temp[$snpcol]} = "$temp[$patname]\t$temp[$snpcol]\t$temp[$chromocol]\t$temp[$chromodist]\t$temp[$gtcol1]\t$temp[$gtcol2]\t$temp[$ballele]\t$temp[$log2col]";
				#This is the sample ID
				$id = $temp[0];
			}
			
			#Open new files
			$output = "PN_" . $id . ".txt";
			$output2 = "QT_" . $id . ".txt";
			#$output3 = "$MUloc"."MU_".$tag."$filecount".".txt";
			open (OUTPUT, ">$output" || die "Could not open $output, may already be open\n");
			open (OUTPUT2, ">$output2" || die "Could not open $output2, may already be open\n");
			#open (OUTPUT3, ">$output3" || "Could not open $output3, may already be open\n");\
			
			#Print headers
			print OUTPUT "Name	Chr	Position	$temp[$patname].GType	$temp[$patname].Log R Ratio	$temp[$patname].B Allele Freq\n";
			print OUTPUT2 "Name\tChr\tPosition\t$temp[$patname].Log R Ratio\t$temp[$patname].B Allele Freq\n";
			#print OUTPUT3 "Name\tSNP\tChr\tPosition\tGT1\tGT2\tBAF\tLog R Ratio\n";
			
			#Goes through the sorted list of SNPs and prints the corresponding LogR and BAF values
			for ($a=1; $a<=$counter; $a++){
				$snpper = $orderedsnps{$a};
				if ($PN{$snpper} ne ""){
					print OUTPUT	"$PN{$snpper}\n";
					print OUTPUT2 "$QT{$snpper}\n";
					#print OUTPUT3 "$MU{$snpper}\n";
				}
			}
			close (OUTPUT);
			close(OUTPUT2);
			#$pnfilesize = sprintf("%.1f",(-s $output) / 1000000);
			#$qtfilesize = sprintf("%.1f",(-s $output2) / 1000000);
			#$filesize{$id} = "$pnfilesize\t$qtfilesize";
			#close(OUTPUT3);
			close(NEWFILE);
		}
	}
	
	
	$currenttime = &showtime;
	print "Conversion complete for $totalfiles samples at $currenttime\n";
}

#Moves the GNOSIS results files into the correct folder
if ($moveGNres == 1){
use File::Copy;
@files=<*>;
$counter = 0;
foreach (@files){
	$findterm = $tag."_GNOSIS";
	#print "The term is $findterm\n";
	if ($_ =~ /$findterm/){
		$counter++;
		$file_location = ($curr_dir . $_);
		$newfile= ($res_dir . $_);
		move ($file_location, $newfile); 
		}
	}
	print "$counter GNOSIS results files moved to $res_dir\n";
	print LOG1 "$timeanddate\tmoveGNresults\tINFO\t$counter GNOSIS results files moved to $res_dir\n";
}

#Removes PennCNV and QuantiSNP formatted files
if ($clean == 1){
	print "\nWould you like to clean up the files (removes PN and QT converted files)? (\'y\' or \'n\')\n";
	$user_refer = <STDIN>; chomp $user_refer;
	
	$user_refer = lc $user_refer;
	if ($user_refer =~ /y/){
		@filelist = <*>;
		foreach (@filelist){
			if ($_ =~ /PN_/
			or $_ =~ /QT_/
			or $_ =~ /hg18_/
			or $_ =~ /_Processor_list_/
			or $_ =~ /.bat/){
				unlink ("$_");
			}
		}
	}
	$mainresult = $fin_dir."Annot_".$tag."_CNV_Results.txt";
	$hdresult = $fin_dir."Annot_HD_".$tag."_Merge.txt";
	$logfile = $curr_dir.$tag."_Logfile.txt";
	$samplist = $curr_dir.$tag."_Sample_List.txt";
	print LISTOUT "$logfile\n$samplist\n$mainresult\n$hdresult\n";
}

#Formats PennCNV files into Merge format
if ($PNformat == 1){
	print "Reformatting PennCNV results\n";
	if ($tag eq ""){
		$tag = "M";
		$method = 1;
		$pro_dir = "";
		$pnres = $ARGV[0];
	}
	else {
		$method = 0;
		print LOG1 "$timeanddate\tPNformat\tINFO\tPennCNV reformatting started\n";
		$pnres = $res_dir.$tag."_Penn_results.txt";
		$pnxres = $res_dir.$tag."_Penn_resultsX.txt";
		$pnlog = $res_dir.$tag."_Penn_log.txt";
	}
	$PNform = "$pro_dir"."PN_" . $tag . "_Formated.txt";
	$PNmerge = "$pro_dir"."PN_" . $tag . "_Merge.txt";
	open (PNRES, ">$PNform" || die "Could not open $PNform; may already be open\n");
	print PNRES "Sample ID\tChr\tStart\tStop\tStartSNP\tStopSNP\tProbes\tSize\tType\n";
	open (PNMER, ">$PNmerge" || die "Could not open $PNmerge; may already be open\n");
	open (PRES, "<$pnres" || die "ERROR: No results file for PennCNV: looking for $pnres\n");
	
	while (<PRES>){
		push (@PN, $_);
	}
	close PRES;
	if ($method == 0){
		open (PXRES, "<$pnxres" || die "Could not open $pnxres, there may have been a problem with running PennCNV\n");
		$counterx=0;
		while (<PXRES>){
			push (@PN, $_);
		}
		close PXRES;
	}
	
	$theM = 'M';
	$theX = 'X';
	$thechr = 'chr';
	$callcount = 0;
	$xcallcount = 0;
	@PN = sort { (split /\s+/, $a)[4] cmp (split /\s+/, $b)[4] } @PN;
	
	foreach $line (@PN){
		$linecount++;
		@tempa = split(/\s+/,$line);
		@nextline = split(/\s+/,@PN[$linecount]);
		chomp($tempa[0]);
		@chromoall = split(":", $tempa[0]);
		$chromo = $chromoall[0];
		#print "$chromo\n";
		if ($chromo =~ /$theX/){
			$xcallcount++;
		}
		else{
			$callcount++;
		}
		$chromo =~ s/MT/$theM/gi;
		$chromo =~ s/$thechr//gi;
		$chromo = "chr".$chromo;
		($CNVstart, $CNVstop) = split("-",  $chromoall[1]);
		$CNVstart =~ s/,//gi;
		$CNVstop =~ s/,//gi;
		$tempa[1] =~ s/numsnp=//i;
		$tempa[2] =~ s/length=//i;
		$tempa[2] =~ s/,//gi;
		$tempa[3] =~ s/state5,cn=//i;
		$tempa[3] =~ s/state6,cn=//i;
		$tempa[3] =~ s/state2,cn=//i;
		$tempa[3] =~ s/state1,cn=//i;
		$tempa[3] =~ s/state3,cn=2/4/i;
		$thisone = $tempa[4];
		@tab = split (/_/,$tempa[4]);
		$samp_remove1 = ".txt";
		#print "$samp_remove1\n";
		$tempa[4] =~ s/$samp_remove1//gi;
		#print "$tempa[4]\n";
		@tab = split (/PN_/,$tempa[4]);
		$tempa[4] = $tab[-1];
		#print "$tempa[4]\n";
		$sample{$tempa[4]} = 0;
		$tempa[5] =~ s/startsnp=//i;
		$tempa[6] =~ s/endsnp=//i;
		$tempb = "$tempa[4]\t$chromo\t$CNVstart\t$CNVstop\t$tempa[5]\t$tempa[6]\t$tempa[1]\t$tempa[2]\t$tempa[3]\n";
		push (@PNF, $tempb);
		$tempc = "NA1000\t$tempa[4]\t$chromo\t$CNVstart\t$CNVstop\t$tempa[3]\t0\t$tempa[1]\t$tempa[2]\n";
		if ($thisone ne $nextline[4]){
			$pncalls{$tempa[4]} = "$callcount\t$xcallcount";
			#print "Sample: $thisone, Next: $nextline[4], Sample:$tempa[4], Calls:$callcount\n";
			$callcount = 0;
			$xcallcount = 0;
		}
		push (@PNM, $tempc);
	}
	
	print PNRES @PNF;
	print PNMER @PNM;
	@samplecount = keys %sample;
	$samplecount = @samplecount;
	if ($samplecount == 0){
		print "ERROR: No samples found in PennCNV results file\n";
		print LOG1 "$timeanddate\tPNformat\tERROR\tNo samples found in PennCNV results file\n";
	}
	
	if ($method == 0){
		open (PLOG, "<$pnlog" || die "Could not open $pnlog\n");
		
		#PennCNV Quality Control Criteria
		$LRR_SDl = 0.28;
		$BAF_Medianl = 0.55;
		$BAF_Medianh = 0.45;
		$BAF_Driftl = 0.01;
		$WFl = 0.05;
		$WFh = -0.05;
		
		$logcount=1;
		$pnbadcount = 0;
		
		while (<PLOG>){
			if ($_ =~ /quality summary/){
				$logcount++;
				$_ =~ s/NOTICE: quality summary for //i;
				$_ =~ s/$curr_dir//i;
				$_ =~ s/PN_//i;
				$_ =~ s/.txt://i;
				(@space) = split(/\s+/,$_);
				$space[1] =~ s/LRR_mean=//i;
				$space[2] =~ s/LRR_median=//i;
				$space[3] =~ s/LRR_SD=//i;
				$space[4] =~ s/BAF_mean=//i;
				$space[5] =~ s/BAF_median=//i;
				$space[6] =~ s/BAF_SD=//i;
				$space[7] =~ s/BAF_DRIFT=//i;
				$space[8] =~ s/WF=//i;
				$space[9] =~ s/GCWF=//i;
				
				$sid = $space[0] . "\n";
				$LRR_SD = $space[3];
				$BAF_Median = $space[5];
				$BAF_Drift = $space[7];
				$WF = $space[8];
				$logsample{$space[0]} = 1;
				
				if($LRR_SD > $LRR_SDl
				or $BAF_Drift > $BAF_Driftl
				or $WF > $WFl
				or $WF < $WFh){
					$pnbadcount++;
					push (@bad, $sid);
				}
				
				$ptemp = $space[0] . "\t" . $space[1] . "\t" . $space[2] . "\t" . $space[3] . "\t" . $space[4] . "\t" . $space[5] . "\t" . $space[6] . "\t" . $space[7] . "\t" . $space[8] . "\t" . $space[9] . "\t=IF(D$logcount>$LRR_SDl,0,1)\t=IF(H$logcount>$BAF_Driftl,0,1)\t=IF(I$logcount>$WFl,0,IF(I$logcount<$WFh,0,1))\t=if(sum(K$logcount:N$logcount)<3,1,0)\n";
				push (@log, $ptemp);
				
				$PNlogout = "$pro_dir"."PN_" . $tag . "_log.txt";
				$PNbad = "$pro_dir"."PN_" . $tag . "_bad.txt";
				open (PNBAD, ">$PNbad" || die "Could not open $PNbad; may already be open\n");
				print PNBAD @bad;
				open (PNLOG, ">$PNlogout" || die "Could not open $PNlogout; may already be open\n");
				print PNLOG "Sample ID\tLRR Mean\tLRR Median\tLRR SD\tBAF Mean\tBAF Median\tBAF SD\tBAF Drift\tWF\tGC WF\tLRR SD fail\tBAF Drift fail\tWF fail\tOverall fail\n";
				print PNLOG @log;
			}
		}
		@logsamp = keys %logsample;
		$logsamp = @logsamp;
		
		print "$pnbadcount bad samples identified out of $logsamp samples by PennCNV\n";
	}
	
	if ($method eq 0){
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tPN calls (autosomes)\tPN calls (ChrX)\n";
		shift @samplist;
		$totalfiles = keys %pncalls;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$filesizes = $pncalls{$tab[0]};
			if ($filesizes ne ""){
				print SAMP "$_\t$filesizes\n";
				$samplecounter++;
			}
			else {
				if ($logsample{$tab[0]} == 1){
					print SAMP "$_\t0\t0\n";
					print LOG1 "$timeanddate\tPNformat\tERROR\tNo CNVs found by PennCNV for $tab[0], however sample has been run\n";
					print "ERROR: No CNVs found by PennCNV for $tab[0], however sample has been run\n";
				}
				else {
					print SAMP "$_\tNot run\tNot run\n";
					print LOG1 "$timeanddate\tPNformat\tERROR\tNo CNVs found by PennCNV for $tab[0]; sample has not been run\n";
					print "ERROR: No CNVs found by PennCNV for $tab[0]; sample has not been run\n";
				}
			}
		}
		if ($samplecounter == $totalfiles){
			print LOG1 "$timeanddate\tPNformat\tINFO\tPennCNV reformatting complete for $totalfiles samples; there were $pnbadcount bad samples\n";
		}
		else {
			print LOG1 "$timeanddate\tPNformat\tERROR\tPennCNV reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
			print "ERROR: PennCNV reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
		}
	}
	print "Results reformatted for $samplecount samples\n";
}

#Runs QuantiSNPv1.1 off multiple processors
if ($qt_batch == 1){
	print LOG1 "$timeanddate\tQuantiSNP\tINFO\tStarting QuantiSNP CNV Prediction Algorithm\n";
	print "QT1: $QTloc\n";
	$QTloc =~ s/\//\\/gi;
	print "QT2: $QTloc\n";
	chdir $QTloc;
	$cdir = getcwd;
	print "Directory is $cdir\n";
	if ($processors <= 1){
		$batchname = $tag . "_QTbatch.bat";
		system ($batchname);
	}
	else {
		foreach $pro (1..$processors){
			$countfile = $curr_dir . $tag . "_Count_qt_".$pro.".txt";
			open (OUTER, ">$countfile") || die "Could not open file $countfile; may already be open\n";
			close OUTER;
			$batchname = $tag . "_QTbatch_" . $pro . ".bat";
			print "Opening QuantiSNP window $pro at $timeanddate\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0; 
			foreach $pro (1..$processors){
				$countfile = $curr_dir . $tag . "_Count_qt_".$pro.".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (5);
		} until ($marker == 1);
		print "\n";
		unlink $countfile;
		foreach $pro (1..$processors){
			$batchname = $tag . "_QTbatch_" . $pro . ".bat";
			unlink $batchname;
		}
		chdir $res_dir;
		foreach $pro (1..$processors){
			$qtres = $tag."_QT_results_".$pro."_output.txt";
			$qtlog = $tag."_QT_results_".$pro."_QC.txt";
			$finalqtres = $tag."_QT_results_output.txt";
			$finalqtlog = $tag."_QT_results_QC.txt";
			open (QTIN, "<$qtres")||die "Could not open $qtres; check the QT file was made\n";
			open (QTQCIN, "<$qtlog")||die "Could not open $qtlog; check the QT file was made\n";
			if ($pro == 1){
				open (QTOUT, ">$finalqtres")||die "Could not open $finalqtres; may already be open\n";
				open (QTQCOUT, ">$finalqtlog")||die "Could not open $finalqtlog; may already be open\n";
				while (<QTIN>){print QTOUT $_}; close QTOUT; close QTIN; unlink $qtres;
				while (<QTQCIN>){print QTQCOUT $_}; close QTQCOUT; close QTQCIN; unlink $qtlog;
			}
			else {
				open (QTOUT, ">>$finalqtres")||die "Could not open $finalqtres; may already be open\n";
				open (QTQCOUT, ">>$finalqtlog")||die "Could not open $finalqtlog; may already be open\n";
				@qtin = (<QTIN>); shift @qtin; foreach (@qtin) {print QTOUT $_;} close QTOUT; close QTIN; unlink $qtres;
				@qtqcin = (<QTQCIN>); shift @qtqcin; foreach (@qtqcin){print QTQCOUT $_;} close QTQCOUT; close QTQCIN; unlink $qtlog;
			}
		}
	}
	$timeanddate = &showtime;
	chdir $curr_dir;
	print "\nQuantiSNP complete for $totalfiles samples at $timeanddate\n";
	print LOG1 "$timeanddate\tQuantiSNP\tINFO\tQuantiSNP CNV Prediction Algorithm complete\n";
}

#Runs QuantiSNPv2.3 off multiple processors
if ($qt2_batch == 1){
	print LOG1 "$timeanddate\tQuantiSNPv2.3\tINFO\tStarting QuantiSNPv2.3 CNV Prediction Algorithm\n";
	#print "QT1: $QT2loc\n";
	$QT2loc =~ s/\//\\/gi;
	#print "QT2: $QT2loc\n";
	chdir $QT2loc;
	$cdir = getcwd;
	#print "Directory is $cdir\n";
	if ($processors <= 1){
		$batchname = $tag . "_QT2batch.bat";
		system ($batchname);
	}
	else {
		foreach $pro (1..$processors){
			$countfile = $curr_dir . $tag . "_Count_qt_".$pro.".txt";
			open (OUTER, ">$countfile") || die "Could not open file $countfile; may already be open\n";
			close OUTER;
			$batchname = $tag . "_QT2batch_" . $pro . ".bat";
			print "Opening QuantiSNPv2.3 window $pro at $timeanddate\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0; 
			foreach $pro (1..$processors){
				$countfile = $curr_dir . $tag . "_Count_qt_".$pro.".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (5);
		} until ($marker == 1);
		print "\n";
		unlink $countfile;
		foreach $pro (1..$processors){
			$batchname = $tag . "_QT2batch_" . $pro . ".bat";
			unlink $batchname;
		}
		chdir $res_dir;
		
		#Find input files and open output files
		@files = <*>;
		$finalqtres = $tag."_QT2_results_output.txt";
		$finalqtlog = $tag."_QT2_results_QC.txt";
		$finalqtloh = $tag."_QT2_results_LOH.txt";
		open (QT2CNV, ">$finalqtres")||die "Could not open $finalqtres; may already be open\n";
		open (QT2QC, ">$finalqtlog")||die "Could not open $finalqtlog; may already be open\n";
		open (QT2LOH, ">$finalqtloh")||die "Could not open $finalqtloh; may already be open\n";
		
		#Go through each sample file and write results to the a common file
		foreach $file (@files){
			#CNV files
			if (substr($file, -4,4) =~ /\.cnv/){
				$qt2cnvfilecount++;
				open (QTIN, "<$file")||die "Could not open $file; check the QT file was made\n";
				if ($qt2cnvfilecount == 1){
					while (<QTIN>){print QT2CNV $_}; close QTIN; #unlink $file;
				}
				else {
					@qtin = (<QTIN>); shift @qtin; foreach (@qtin) {print QT2CNV $_;} close QTIN; #unlink $file;
				}
			}
			#QC files
			elsif (substr($file, -3,3) =~ /\.qc/){
				$qt2qcfilecount++;
				open (QTIN, "<$file")||die "Could not open $file; check the QT file was made\n";
				if ($qt2qcfilecount == 1){
					while (<QTIN>){print QT2QC $_}; close QTIN; #unlink $file;
				}
				else {
					@qtin = (<QTIN>); shift @qtin; foreach (@qtin) {print QT2QC $_;} close QTIN; #unlink $file;
				}
			}
			#LOH files
			elsif (substr($file, -4,4) =~ /\.loh/){
				$qt2lohfilecount++;
				open (QTIN, "<$file")||die "Could not open $file; check the QT file was made\n";
				if ($qt2lohfilecount == 1){
					while (<QTIN>){print QT2LOH $_}; close QTIN; #unlink $file;
				}
				else {
					@qtin = (<QTIN>); shift @qtin; foreach (@qtin) {print QT2LOH $_;} close QTIN; #unlink $file;
				}
			}
		}
	}
	$timeanddate = &showtime;
	chdir $curr_dir;
	print "\nQuantiSNPv2.3 complete for $totalfiles samples at $timeanddate\n";
	print LOG1 "$timeanddate\tQuantiSNPv2.3\tINFO\tQuantiSNPv2.3 CNV Prediction Algorithm complete\n";
}

#Runs PennCNV off multiple processors
if ($pn_batch == 1){
	print LOG1 "$timeanddate\tPennCNV\tINFO\tStarting PennCNV CNV Prediction Algorithm\n";
	$progloc = '"'.$JVloc.$progname.'"';
	chdir $PNloc;
	if ($chip =~ /Omni/){
		unless (-e $PNloc."lib/ho1v1.hg18.pfb"){
			print "ERROR: Omni specific PFB file (ho1v1.hg18.pfb) is not present in $PNloc".'lib/. Please download from the PennCNV website.\n';
			print LOG1 "$timeanddate\tPennCNV\tERROR\tOmni specific PFB file (ho1v1.hg18.pfb) is not present in $PNloc".'lib/. Please download from the PennCNV website.';
		}
		else{
			print "Running in Omni1 mode which uses a different pfb file (ho1v1.hg18.pfb)\n";
		}
	}
	elsif ($chip eq "Affy6.0"){
		unless (-e $PNloc."lib/affygw6.hg18.pfb" and -e $PNloc."lib/affygw6.hmm"){
			print "ERROR: Affy6.0 specific PFB file (affygw6.hg18.pfb) or HMM file (affygw6.hmm) are not present in $PNloc".'lib/. Please download from the PennCNV website.\n';
			print LOG1 "$timeanddate\tPennCNV\tERROR\tAffy6.0 specific PFB file (affygw6.hg18.pfb) or HMM file (affygw6.hmm) are not present in $PNloc".'lib/. Please download from the PennCNV website.';
		}
		else{
			print "Running in Affy6.0 mode which uses a different PFB file (affygw6.hg18.pfb) and HMM file (affygw6.hmm)\n";
		}
	}
	else {
		print "Standard mode\n";
	}
	
	if ($processors <= 1){
		if ($chip =~ /Omni/){
			$command1 = "perl detect_cnv.pl -test -hmm lib/hhall.hmm -pfb lib/ho1v1.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_log.txt -output $res_dir"."$tag"."_Penn_results.txt\n";
			system ($command1);
			$command2 = "perl detect_cnv.pl -test -chrx -hmm lib/hhall.hmm -pfb lib/ho1v1.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_logX.txt -output $res_dir"."$tag"."_Penn_resultsX.txt\n";
			system ($command2);
		}
		elsif ($chip eq "Affy6.0"){
			$command1 = "perl detect_cnv.pl -test -hmm lib/affygw6.hmm -pfb lib/affygw6.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_log.txt -output $res_dir"."$tag"."_Penn_results.txt\n";
			system ($command1);
			$command2 = "perl detect_cnv.pl -test -chrx -hmm lib/affygw6.hmm -pfb lib/affygw6.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_logX.txt -output $res_dir"."$tag"."_Penn_resultsX.txt\n";
			system ($command2);
		}
		else {
			$command1 = "perl detect_cnv.pl -test -hmm lib/hhall.hmm -pfb lib/hhall.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_log.txt -output $res_dir"."$tag"."_Penn_results.txt\n";
			system ($command1);
			$command2 = "perl detect_cnv.pl -test -chrx -hmm lib/hhall.hmm -pfb lib/hhall.hg18.pfb -list "."$tag"."_Penn_list.txt -log $res_dir"."$tag"."_Penn_logX.txt -output $res_dir"."$tag"."_Penn_resultsX.txt\n";
			system ($command2);
		}
		chomp $command1; chomp $command2; 
		print LOG1 "$timeanddate\tPennCNV\tPennCNV\t$command1";
		print LOG1 "$timeanddate\tPennCNV\tPennCNVX\t$command2";
	}
	else {
		foreach $pro (1..$processors){
			#$listname = $tag . "_Penn_list_".$pro.".txt";
			$countfile = $curr_dir . $tag . "_Count_pn_".$pro.".txt";
			open (OUTER, ">$countfile") || die "Could not open file $countfile; may already be open\n";
			close OUTER;
			$batchname = $tag . "_Penn_".$pro.".bat";
			open (BATCH, ">$batchname") || die "Could not open file $batchname; may already be open\n";
			if ($chip =~ /Omni/){
				$command1 = "perl detect_cnv.pl -test -hmm lib/hhall.hmm -pfb lib/ho1v1.hg18.pfb -list "."$tag"."_Penn_list_".$pro.".txt -log $res_dir"."$tag"."_Penn_log_".$pro.".txt -output $res_dir"."$tag"."_Penn_results_".$pro.".txt\n";
				$command2 = "perl detect_cnv.pl -test -chrx -hmm lib/hhall.hmm -pfb lib/ho1v1.hg18.pfb -list "."$tag"."_Penn_list_".$pro.".txt -log $res_dir"."$tag"."_Penn_logX_".$pro.".txt -output $res_dir"."$tag"."_Penn_resultsX_".$pro.".txt\n";
			}
			elsif ($chip eq "Affy6.0"){
				$command1 = "perl detect_cnv.pl -test -hmm lib/affygw6.hmm -pfb lib/affygw6.hg18.pfb -list "."$tag"."_Penn_list_".$pro.".txt -log $res_dir"."$tag"."_Penn_log_".$pro.".txt -output $res_dir"."$tag"."_Penn_results_".$pro.".txt\n";
				$command2 = "perl detect_cnv.pl -test -chrx -hmm lib/affygw6.hmm -pfb lib/affygw6.hg18.pfb -list "."$tag"."_Penn_list_".$pro.".txt -log $res_dir"."$tag"."_Penn_logX_".$pro.".txt -output $res_dir"."$tag"."_Penn_resultsX_".$pro.".txt\n";
			}
			else {
				$command1 = "perl detect_cnv.pl -test -hmm lib/hhall.hmm -pfb lib/hhall.hg18.pfb -list "."$tag"."_Penn_list_".$pro.".txt -log $res_dir"."$tag"."_Penn_log_".$pro.".txt -output $res_dir"."$tag"."_Penn_results_".$pro.".txt\n";
				$command2 = "perl detect_cnv.pl -test -chrx -hmm lib/hhall.hmm -pfb lib/hhall.hg18.pfb -list "."$tag"."_Penn_list_".$pro.".txt -log $res_dir"."$tag"."_Penn_logX_".$pro.".txt -output $res_dir"."$tag"."_Penn_resultsX_".$pro.".txt\n";
			}
			if ($pro == 1){print LOG1 "$timeanddate\tPennCNV\tPennCNV\t$command1";}
			if ($pro == 1){print LOG1 "$timeanddate\tPennCNV\tPennCNV\t$command2";}
			$command3 = "perl " . $progloc . " --delete ".$countfile;
			print BATCH "$command1\n$command2\ncd $curr_dir\n$command3\nexit\n";
			print "Opening PennCNV window $pro at $timeanddate\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0; 
			foreach $pro (1..$processors){
				$countfile = $curr_dir . $tag . "_Count_pn_".$pro.".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (5);
		} until ($marker == 1);
		print "\n";
		unlink $countfile;
		foreach $pro (1..$processors){
			$batchname = $tag . "_Penn_".$pro.".bat";
			$listname = $tag . "_Penn_list_".$pro.".txt";
			unlink $batchname;
			unlink $listname;
		}
		
		chdir $res_dir;
		foreach $pro (1..$processors){
			$pnres = $tag."_Penn_results_".$pro.".txt";
			$pnlog = $tag."_Penn_log_".$pro.".txt";
			$pnresX = $tag."_Penn_resultsX_".$pro.".txt";
			$pnlogX = $tag."_Penn_logX_".$pro.".txt";
			$finalpnres = $tag."_Penn_results.txt";
			$finalpnlog = $tag."_Penn_log.txt";
			$finalpnresX = $tag."_Penn_resultsX.txt";
			$finalpnlogX = $tag."_Penn_logX.txt";
			open (PNIN, "<$pnres")||die "Could not open $pnres; check the PN file was made\n";
			open (PNQCIN, "<$pnlog")||die "Could not open $pnlog; check the PN file was made\n";
			open (PNINX, "<$pnresX")||die "Could not open $pnresX; check the PN file was made\n";
			open (PNQCINX, "<$pnlogX")||die "Could not open $pnlogX; check the PN file was made\n";
			if ($pro == 1){
				open (PNOUT, ">$finalpnres")||die "Could not open $finalpnres; may already be open\n";
				open (PNQCOUT, ">$finalpnlog")||die "Could not open $finalpnlog; may already be open\n";
				open (PNOUTX, ">$finalpnresX")||die "Could not open $finalpnresX; may already be open\n";
				open (PNQCOUTX, ">$finalpnlogX")||die "Could not open $finalpnlogX; may already be open\n";
				while (<PNIN>){print PNOUT $_}; close PNOUT; close PNIN; unlink $pnres;
				while (<PNQCIN>){print PNQCOUT $_}; close PNQCOUT; close PNQCIN; unlink $pnlog;
				while (<PNINX>){print PNOUTX $_}; close PNOUTX; close PNINX; unlink $pnresX;
				while (<PNQCINX>){print PNQCOUTX $_}; close PNQCOUTX; close PNQCINX; unlink $pnlogX;
			}
			else {
				open (PNOUT, ">>$finalpnres")||die "Could not open $finalpnres; may already be open\n";
				open (PNQCOUT, ">>$finalpnlog")||die "Could not open $finalpnlog; may already be open\n";
				open (PNOUTX, ">>$finalpnresX")||die "Could not open $finalpnresX; may already be open\n";
				open (PNQCOUTX, ">>$finalpnlogX")||die "Could not open $finalpnlogX; may already be open\n";
				@pnin = (<PNIN>); shift @pnin; foreach (@pnin) {print PNOUT $_;} close PNOUT; close PNIN; unlink $pnres;
				@pnqcin = (<PNQCIN>); shift @pnqcin; foreach (@pnqcin){print PNQCOUT $_;} close PNQCOUT; close PNQCIN; unlink $pnlog;
				@pninX = (<PNINX>); shift @pninX; foreach (@pninX) {print PNOUTX $_;} close PNOUTX; close PNINX; unlink $pnresX;
				@pnqcinX = (<PNQCINX>); shift @pnqcinX; foreach (@pnqcinX){print PNQCOUTX $_;} close PNQCOUTX; close PNQCINX; unlink $pnlogX;
			}
		}
	}
	$timeanddate = &showtime;
	print "\nPennCNV complete for $totalfiles samples at $timeanddate\n";
	print LOG1 "$timeanddate\tPennCNV\tINFO\tPennCNV CNV Prediction Algorithm complete\n";
}

#Formats GNOSIS files into Merge format
if ($GNformat == 1){
	#GScore criteria
	$gscore = 3;
	print "Reformatting GNOSIS results\n";
	if ($tag eq ""){
		$tag = "M";
		$method = 1;
		$pro_dir = "";
		$filename = $ARGV[0];
	}
	else {
		print LOG1 "$timeanddate\tGNformat\tINFO\tGNOSIS reformatting started; G score threshold is $gscore\n";
		$method = 0;
		chdir $res_dir;
		@resfile = <*>;
		while (<@resfile>){
			#print "Here is: $_\n";
			if ($_ =~ /GNOSIS_CNVs_/){
				$filename = $res_dir.$_;
				#print "$filename\n";
			}
			elsif ($_ =~ /GNOSIS_badchips/){
				$badfile = $_;
			}
			elsif ($_ =~ /GNOSIS_thresholds/){
				$thresfile = $_;
			}
		}
		chomp $badfile;
		open (THRES, "<$thresfile") || die "Could not open file $thresfile\n";
		open (BAD, "<$badfile") || die "Could not open file $badfile\n";
		$badgn = $pro_dir . "GN_" . $tag . "_bad.txt";
		open (BADOUT, ">$badgn") || die "Could not open file $badgn\n";
		chdir $curr_dir;
		$gnbadcount = 0;
		
		while (<THRES>){
			chomp $_;
			@tab = split ("\t", $_);
			$xmed{$tab[0]} = $tab[13];
		}
		close THRES;
		
		while (<BAD>){
			@tab = split ("\t", $_);
			$badlist{$tab[0]} = 0;
			push (@badcalls, $_);
		}
		@id = sort keys %badlist;
		$gnbadcount = @id;
		$badouthere = join ("\n",@id);
		#print "Bad: $gnbadcount, @id\n";
		print BADOUT $badouthere;
	}
	
	chomp $filename; 
	
	open (DATA, "<$filename") || die "Could not open file $filename; check it is in the right directory\n";
	@DATA = <DATA>;
	chomp @DATA;
	shift @DATA;
	close DATA;
	#$datasize = @DATA;
	
	push (@DATA, @badcalls);
	
	if ($method == 0){
		$outfile = $pro_dir . "GN_" . $tag . "_Merge.txt";
	}
	elsif ($method == 1){
		$outfile = "Formated_" . $filename;
	}
	open (OUT, ">$outfile") || die "Could not open file $outfile; may already be open\n";
	
	$theM = 'M';
	$bigchr = 'CHR';
	$littlechr = 'chr';
	$unrawcount = 0;
	$rawcount = 0;
	$xunrawcount = 0;
	$xrawcount = 0;
	
	foreach $line (@DATA) {
		$linecount++;
		@temp = split ("\t", $line);
		@nextline = split ("\t", $DATA[$linecount]);
		#If the sample does not change
		if ($temp[1] =~ /x/i or $temp[1] =~ /y/i){
			$xrawcount++;
		}
		else {
			$rawcount++;
		}
		chomp (@temp);
		if($temp[13] >= $gscore){
			if ($temp[1] =~ /x/i){
				$xunrawcount++;
			}
			else {
				$unrawcount++;
			}
			$temp[1] =~ s/MT/$theM/gi;
			$temp[1] =~ s/$bigchr/$littlechr/gi;
			$temp[11] =~ s/HOMODEL/0/i;
			$temp[11] =~ s/HETDEL/1/i;
			$temp[11] =~ s/HETDUP/3/i;
			$temp[11] =~ s/AMPLIF/4/i;
			$size = $temp[3] - $temp[2];
			if ($size < 0){
				print "ERROR: Line $linecount has a stop co-ordinate that is less than the start co-ordinate\n";
				print LOG1 "$timeanddate\tGNformat\tERROR\tLine $linecount has a stop co-ordinate that is less than the start co-ordinate\n";
			}
			print OUT "NA1000\t$temp[0]\t$temp[1]\t$temp[2]\t$temp[3]\t$temp[11]\t$temp[13]\t$temp[6]\t$size\n";
			$samples{$temp[0]} = 0;
			if($temp[1] eq ""
			or $temp[2] eq ""){
				$blank++;
			}
		}
		if ($temp[0] ne $nextline[0]){
			$gncalls{$temp[0]} = "$rawcount\t$xrawcount\t$unrawcount\t$xunrawcount";
			$unrawcount = 0;
			$rawcount = 0;
			$xunrawcount = 0;
			$xrawcount = 0;
		}
	}
	@gncount = keys %samples;
	$gncount = @gncount;
	$sampcount = $gncount;
	
	if ($blank > 0){
		print "ERROR: $blank lines had blank entries for the start/stop co-ordinates; check GNOSIS results\n";
		print LOG1 "$timeanddate\tGNformat\tERROR\t$blank lines had blank entries for the start/stop co-ordinates; check GNOSIS results\n";
	}
	if ($sampcount == 0){
		print "ERROR: No samples found in GNOSIS results file\n";
		print LOG1 "$timeanddate\tGNformat\tERROR\tNo samples found in GNOSIS results file\n";
	}
	
	if ($method eq 0){
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tGN chrX Correction Factor\tGN Raw calls (autosomes)\tGN Raw calls (chrX, chrY)\tGN Processed calls (autosomes)\tGN Processed calls (chrX, chrY)\n";
		shift @samplist;
		$totalfiles = keys %gncalls;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$filesizes = $gncalls{$tab[0]};
			if ($filesizes ne ""){
				print SAMP "$_\t$xmed{$tab[0]}\t$filesizes\n";
				$samplecount++;
			}
			else {
				print SAMP "$_\tNot run\tNot run\tNot run\tNot run\tNot run\n";
				print LOG1 "$timeanddate\tGNformat\tERROR\tNo CNVs found by GNOSIS for $tab[0]\n";
				print "ERROR: No CNVs found by GNOSIS for $tab[0]\n";
			}
		}
		if ($samplecount == $totalfiles){
			print LOG1 "$timeanddate\tGNformat\tINFO\tGNOSIS reformatting complete for $totalfiles samples, there were $gnbadcount bad samples\n";
		}
		else {
			print LOG1 "$timeanddate\tGNformat\tERROR\tGNOSIS reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
			print "ERROR: GNOSIS reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
		}
	}
	print "Results reformatted for $sampcount samples\n";
}

#Formats QuantiSNP v1.1 files into Merge format
if ($QTformat == 1){
	#Log Bayes Critera and QT Quality Control Criteria
	$logb = 10;
	$BAFout = 0.1;
	$logrout = 0.1;
	$BAFstd = 0.2;
	$logrstd = 0.4;
	$passrange = 90;
	
	print "Reformatting QuantiSNP results\n";
	if ($tag eq ""){
		$tag = "M";
		$method = 1;
		$pro_dir = "";
		$filename2 = $ARGV[0];
		$outfile2 = "Formatted_".$filename2;
	}
	else {
		print LOG1 "$timeanddate\tQTformat\tINFO\tQuantiSNP reformatting started; LogBayes score threshold is $logb\n";
		$method = 0;
		$filename2 =  $res_dir.$tag."_QT_results_output.txt";
		$qcfile = $filename2;
		$qcfile =~ s/output/QC/gi;
		$outfile2 = $pro_dir . "QT_" . $tag . "_Merge.txt";
		chomp $filename2;
	}
	chdir $res_dir;
	open (DATA2, "<$filename2") || die "Could not open file $filename2, check it is spelled correctly\n";
	chdir $curr_dir;
	open (OUT2, ">$outfile2") || die "Could not open file $outfile2; may already be open\n";
	@DATA2 = <DATA2>;
	chomp @DATA2;
	shift @DATA2;
	close DATA2;

	foreach $line (@DATA2) {
		$linecount++;
		@temp = split ("\t", $line);
		@nextline = split ("\t", $DATA2[$linecount]);
		if ($temp[1] =~ /x/i){
			$xrawcount++;
		}
		else{
			$rawcount++;
		}
		chomp (@temp);
		
		$theM = 'M';
		
		if($temp[9] >= $logb){
			if ($temp[1] =~ /x/i){
				$xunrawcount++;
			}
			else{
				$unrawcount++;
			}
			$temp[1] =~ s/MT/$theM/gi;
			$temp[2] =~ s/ //gi;
			$temp[3] =~ s/ //gi;
			$temp[4] =~ s/ //gi;
			$temp[8] =~ s/2/3/gi;
			print OUT2 "NA1000\t$temp[0]\tchr$temp[1]\t$temp[2]\t$temp[3]\t$temp[8]\t$temp[9]\t$temp[7]\t$temp[4]\n";
			$sampcount{$temp[0]} = 0;
		}
		if ($temp[0] ne $nextline[0]){
			$qtcalls{$temp[0]} = "$rawcount\t$xrawcount\t$unrawcount\t$xunrawcount";
			#print "$temp[0]\n";
			$rawcount = 0;
			$unrawcount = 0;
			$xrawcount = 0;
			$xunrawcount = 0;
		}
	}
	$thisid = 0;
	$lastid = 0;
	$goodcount = 0;
	
	if ($method == 0){
		chdir $res_dir;
		open (QC, "<$qcfile") || die "Could not open file $qcfile\n";
		chdir $curr_dir;
		@qc = <QC>; chomp @qc; close QC;
		$badqt = $pro_dir . "QT_" . $tag . "_bad.txt";
		$newlog = $pro_dir . "QT_" . $tag . "_log.txt";
		open (BADQT, ">$badqt") || die "Could not open file $badqt; may already be open\n";
		open (QTLOG, ">$newlog") || die "Could not open file $newlog; may already be open\n";
		
		$qtcounter = 1;
		chomp $qc[0];
		print QTLOG "$qc[0]\tBAF pass\tLogR pass\tBAF StDev Pass\tLogR StDev Pass\tTotal\n";
		$qtbadcount = 0;
		
		foreach (@qc){
			@before = split ("\t", $qc[$qtcounter - 1]);
			@now = split ("\t", $qc[$qtcounter]);
			@after = split ("\t", $qc[$qtcounter + 1]);
			$qtcounter++;
			
			if ($now[3] <= $BAFout) {$goodcount++;}
			if ($now[4] <= $logrout) {$goodcount++;}
			if ($now[5] <= $BAFstd) {$goodcount++;}
			if ($now[6] <= $logrstd) {$goodcount++;}
			
			if ($now[0] ne $after[0]
			or $after[0] eq "") {
				if ($goodcount < $passrange
				and $now[0] ne ""){
					#print "Good: $goodcount, Sample: $now[0], Next: $after[0]\n";
					$qtbadcount++;
					print BADQT "$now[0]\n";
					}
				$goodcount = 0;
			}
			if ($now[0] ne ""){
				print QTLOG "$now[0]\t$now[1]\t$now[2]\t$now[3]\t$now[4]\t$now[5]\t$now[6]\t=IF(D$qtcounter<=$BAFout,1,0)\t=IF(E$qtcounter<=$logrout,1,0)\t=IF(F$qtcounter<=$BAFstd,1,0)\t=IF(G$qtcounter<=$logrstd,1,0)\t=sum(H$qtcounter:K$qtcounter)\n";
				$logcount{$now[0]} = 0;
			}
		}
		
		#Opens the Sample List file to add the QT calls
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tQT Raw calls (autosomes)\tQT Raw calls (chrX)\tQT Processed calls (autosomes)\tQT Processed calls (chrX)\n";
		shift @samplist;
		$totalfiles = keys %qtcalls;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$filesizes = $qtcalls{$tab[0]};
			if ($filesizes ne ""){
				print SAMP "$_\t$filesizes\n";
				$samplecount++;
			}
			else {
				print SAMP "$_\t0\t0\n";
				print LOG1 "$timeanddate\tQTformat\tERROR\tNo CNVs found by QuantiSNP for $tab[0]\n";
				print "ERROR: No CNVs found by QuantiSNP for $tab[0]\n";
			}
		}
		if ($samplecount == $totalfiles){
			print LOG1 "$timeanddate\tQTformat\tINFO\tQuantiSNP reformatting complete for $totalfiles samples; there were $qtbadcount bad samples\n";
			print "$qtbadcount bad samples identified\n";
		}
		else {
			print LOG1 "$timeanddate\tQTformat\tERROR\tQuantiSNP reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
			print "ERROR: QuantiSNP reformatting complete, however CNVs were present for samples\nthat were not listed in the Samplelist file\n";
		}
	}
	
	@sampcount = keys %sampcount;
	$sampcount = @sampcount;

	if ($sampcount == 0){
		print "ERROR: No samples found in QuantiSNP results file\n";
		print LOG1 "$timeanddate\tQTformat\tERROR\tNo samples found in QuantiSNP results file\n";
	}
	print "Results reformatted for $sampcount samples\n";
}

#Formats homozygous deletion files into Merge format
if ($HDformat == 1){
	print "Reformatting Homodel results\n";
	if ($tag eq ""){
		$tag = "M";
		$method = 1;
		$pro_dir = "";
		$hdres = $ARGV[0];
		$HDmerge = "Formatted_".$ARGV[0];
	}
	else {
		$method = 0;
		print LOG1 "$timeanddate\tHDformat\tINFO\tHomodel reformatting started\n";
		$hdres = $res_dir.$tag."_Homozygous_deletions.txt";
		$HDmerge = "$pro_dir"."HD_" . $tag . "_Merge.txt";
	}
	$HDmerge = "$pro_dir"."HD_" . $tag . "_Merge.txt";
	open (HDMER, ">$HDmerge" || die "Could not open $HDmerge; may already be open\n");
	print HDMER "FID\tIID\tChr\tStart\tStop\tType\tLogR\tProbes\tSize\n";
	open (HDRES, "<$hdres" || die "ERROR: No results file for Homodel: looking for $hdres\n");
	@PN = <HDRES>; close HDRES;
	shift @PN;
	
	$theM = 'M';
	$thechr = 'chr';
	@PN = sort { (split /\t/, $a)[0] cmp (split /\t/, $b)[0] } @PN;
	
	foreach $line (@PN){
		$linecount++;
		@tempa = split(/\t/,$line);
		@nextline = split(/\t/,@PN[$linecount]);
		$callcount++;
		$tempc = "NA1000\t$tempa[0]\t$tempa[1]\t$tempa[2]\t$tempa[3]\t0\t$tempa[12]\t$tempa[7]\t$tempa[6]\n";
		if ($tempa[0] ne $nextline[0]){
			$hdcalls{$tempa[0]} = $callcount;
			#print "Sample: $thisone, Next: $nextline[4], Sample:$tempa[4], Calls:$callcount\n";
			$callcount = 0;
			$samplecount++;
		}
		push (@HDM, $tempc);
	}
	
	print HDMER @HDM;
	if ($samplecount == 0){
		print "ERROR: No samples found in Homdel results file\n";
		print LOG1 "$timeanddate\tHDformat\tERROR\tNo samples found in Homodel results file\n";
	}
	
	if ($method eq 0){
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tHD calls\n";
		shift @samplist;
		$totalfiles = keys %hdcalls;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$filesizes = $hdcalls{$tab[0]};
			if ($filesizes ne ""){
				print SAMP "$_\t$filesizes\n";
				$samplecounter++;
			}
			else {
				if ($logsample{$tab[0]} == 1){
					print SAMP "$_\t0\n";
					print LOG1 "$timeanddate\tHDformat\tERROR\tNo CNVs found by Homodel for $tab[0], however sample has been run\n";
					print "ERROR: No CNVs found by Homodel for $tab[0], however sample has been run\n";
				}
				else {
					print SAMP "$_\tNot run\n";
					print LOG1 "$timeanddate\tHDformat\tERROR\tNo CNVs found by homodel for $tab[0]; sample has not been run\n";
					print "ERROR: No CNVs found by Homodel for $tab[0]; sample has not been run\n";
				}
			}
		}
		if ($samplecount == $totalfiles){
			print LOG1 "$timeanddate\tHDformat\tINFO\tHomodel reformatting complete for $totalfiles samples\n";
		}
		else {
			print LOG1 "$timeanddate\tHDformat\tERROR\tHomodel reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
			print "ERROR: Homodel reformatting complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
		}
	}
	print "Results reformatted for $samplecount samples\n";
}

#Merges multiple files of regions 
if ($merge_prog == 1){
	
	$famID =    0;
	$sampleID = 1;
	$chromo =   2;
	$start =    3;
	$stop =     4;
	$cnvtype =  5;
	$score =    6;
	$numsnps =  7;
	$size =     8;
	$rs1 =      9;
	$rs2 =     10;
	
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
	}
	else {
		$method = 0;
		mkdir ("03_Merged", 0777);
		print LOG1 "$timeanddate\tMerge\tINFO\tMerge programe started for: ";
		chdir $pro_dir;
	}
	
	#Initialise UniqueID counter
	$uidc = 0;
	$chrword = "chr";
	
	#open input files 
	#Put the CNVs into an array but with each one with an ID made up of the algo and uniqueID counters	
	
	foreach $file (@ARGV){ 
		print LOG1 "$file, ";
		print "Loading file $file\n";
		open (INPUT, "<$file") || die "Could not open file $file; check it is spelled correctly\n";
		{ local $/=undef;  $input1=<INPUT>; }
		close INPUT;
		($\) = (/(\r\n|\r|\n)/);
		@CNV=split (/\r\n|\r|\n/, $input1); chomp @CNV;
		chomp;
		$file =~ s/\\/\//gi;
		$file =~ s/$pro_dir//;
		$firsttwo = substr($file, 0,2);
		if ($firsttwo{$firsttwo} == 1){
			print "ERROR: The first two letter of the files being merged are the same\n";
			print LOG1 "$timeanddate\tMerge\tERROR\tThe first two letter of the files being merged are the same\n";
		} 
		$firsttwo{$firsttwo} = 1;
		$firstnames = $firstnames.$firsttwo."_";
		for ($c=0; $c<=$#CNV; $c++){
			if($CNV[$c] =~ /FID/i or $CNV[$c] =~ /IID/i or $CNV[$c] =~ /sample/i){}
			else{
				@tab = split (/\t/,$CNV[$c]);
				$tab[$chromo] =~ s/$chrword//gi;
				$tab[$chromo] = "chr".$tab[$chromo];
				$CNV[$c] = join ("\t",@tab);
				$uidc++; 
				$uidc = sprintf ("%07d", $uidc);
				$CNV[$c] = $firsttwo . $uidc . "\t" . $firsttwo . "\t" . $CNV[$c];
				push (@CNVall, $CNV[$c]);
			}
		}
	}
	print LOG1 "\n";
	
	@CNV = ();
	
	#Two columns have been added so numbers are corrected
	$unique   = 0;
	$algo     = 1;
	$famID    = $famID + 2;
	$sampleID = $sampleID + 2;
	$chromo   = $chromo + 2;
	$start    = $start + 2;
	$stop     = $stop + 2;
	$cnvtype  = $cnvtype + 2;
	$score    = $score + 2;
	$numsnps  = $numsnps +2 ;
	$size     = $size + 2;
	
	#I then need to sort the CNVs by sample, chr, start
	print "Sorting CNVs\n";
	
	#Sort by Start (numbers)
	@CNVall = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNVall;
	
	#Sort by Chromosome (string)
	@CNVall = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNVall;
	
	#Sort by SampleID (string)
	@CNVall = sort { (split '\t', lc $a)[$sampleID] cmp (split '\t', lc $b)[$sampleID] } @CNVall;
	
	#Open merged CNV results file
	
	if ($method == 1){
		$output = "Merged_".$firstnames.".txt";
		$UniqueID = "UID_".$firstnames.".txt";
	}
	else {
		$output = $merge_dir . $tag . "_Merged.txt";
		$UniqueID = $merge_dir . $tag . "_UID.txt";
	}
	chdir $curr_dir;
	open (UID, ">$UniqueID") || die "Cannot open the file $UniqueID.txt";
	print UID "UniqueID\tAlgo\tFID\tSampleID\tChr\tStart\tEnd\tType\tScore\tProbes\tSize\tMatch?\n";
	open (MOUT, ">$output") || die "Cannot open the file $output; may already be open";
	print MOUT "FID\tSampleID\tChr\tLoci start\tLoci end\tLoci type\tMaxScore\tMaxSNP\tLoci size\t \tStart\tEnd\tSize\tType\t#Algos\tAlgos\t ";
	
	foreach (@ARGV){
		$splur++;
		print MOUT "\tAlgo$splur\tType$splur\tScore$splur\tProbes$splur\tSize$splur\tUniqueID$splur";
	}
	
	print MOUT "\n";
	
	#print MOUT "@CNVall";
	
	print "Looking for CNVs that match\n";
	
	$error = 0;
	$track = 0;
	
	#Does each CNV line up with the one below?
	foreach $line (@CNVall){
		@before = split ("\t", $CNVall[$track - 1]);
		@now = split ("\t", $CNVall[$track]);
		@after = split ("\t",$CNVall[$track + 1]);
		$track++;
		$sampcnvcount++;
		
		if (($now[$stop] - $now[$start]) < 0){
			print "ERROR: CNV $now[0] has a stop co-ordinate lower than the start co-ordinate\n";
			print LOG1 "$timeanddate\tMerge\tERROR\tCNV $now[0] has a stop co-ordinate lower than the start co-ordinate\n";
			$error++;
		}
		#print "\n\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
			
		#If this CNV is from a differet sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if ($now[$sampleID] ne $before[$sampleID]
		or lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
		elsif ($now[$sampleID] eq $before[$sampleID] 
		and lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print "Error with the following CNV, its status could not be determined:\n$line\n";
			print LOG1 "$timeanddate\tMerge\tERROR\tThe status of the following CNV could not be determined:\t$line\n";
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if ($now[$sampleID] ne $after[$sampleID]
		or lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			#Process @match
			&setexaminem;
			$locicounter++;
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		#The numbers of loci with all, 1, 2 or 3 algos are entered into the corresponding sample name
		if ($now[$sampleID] ne $after[$sampleID]){
			$sampcnv{$now[$sampleID]} = $sampcnvcount;
			$sampcnvcount = 0;
		}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	close MOUT;
	push (@sminq,$output,0);
	&smallinquire(@sminq);
	
	if ($method == 0){
		#Opens the Sample List file to add the QT calls
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tCNVs prior to merge\tTotal Loci after merge (all)\tTotal Loci after merge (chrX)\t3 Algo Loci\t2 Algo Loci\t1 Algo Loci\n";
		shift @samplist;
	}
	$timeanddate = &showtime;
	$totalsamp = keys %sampcnv;
	if ($method == 1){
		@samplist = keys %sampcnv;
	}
	foreach (@samplist){
		@tab = split (/\t/,$_);
		$allcnv = $sampcnv{$tab[0]};
		$allloci = $totalloci{$tab[0]};
		$xallloci = $xtotalloci{$tab[0]};
		$threeloci = $samploci3{$tab[0]};
		$twoloci = $samploci2{$tab[0]};
		$oneloci = $samploci1{$tab[0]};
		
		if ($threeloci == 0){
			print LOG1 "$timeanddate\tMerge\tERROR\tNo 3 algorithm loci found for $tab[0]; check input file format\n";
			print "ERROR: No 3 algorithm loci found for $tab[0]; check input file format\n";
		}
		if ($allloci ne ""){
			print SAMP "$_\t$allcnv\t$allloci\t$xallloci\t$threeloci\t$twoloci\t$oneloci\n";
			$samplecount++;
		}
		else {
			print SAMP "$_\t0\t0\t0\t0\n";
			print LOG1 "$timeanddate\tMerge\tERROR\tNo CNVs found by any algorithms for $tab[0]\n";
			print "ERROR: No CNVs found by any algorithms for $tab[0]\n";
		}
	}
	if ($samplecount == $totalsamp){
		print LOG1 "$timeanddate\tMerge\tINFO\tMerge complete for $totalsamp samples\n";
		print "Merge complete for $totalsamp samples\n";
	}
	else {
		print LOG1 "$timeanddate\tMerge\tERROR\tMerge complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
		print "ERROR: Merge complete, however CNVs were present for samples that were not listed in the Samplelist file\n";
	}
	
	sub setexaminem{
		chomp @match;
		$matches = @match;
		
		#If there is only one CNV it is simply printed out
		if ($matches == 1){
			#print "Single entry\n";
			#print UID "$line\t0\n";
			$now[$chromo] =~ s/chr//i;
			print MOUT "$now[$famID]\t$now[$sampleID]\tchr$now[$chromo]\t$now[$start]\t$now[$stop]\t$now[$cnvtype]\t$now[$score]\t$now[$numsnps]\t$now[$size]\t\t$now[$start]\t$now[$stop]\t$now[$size]\t$now[$cnvtype]\t1\t$now[$algo]\t\t$now[$algo]\t$now[$cnvtype]\t$now[$score]\t$now[$numsnps]\t$now[$size]\t$now[$unique]\n";
			@match = ();
		}
		
		%coord = ();
		%locitype = ();
		%sizes = ();
		%snps = ();
		%scores = ();
		$counter = 0;
		
		#Put the start and stop positions into a new array and determine the characteristics of this group of matches
		foreach (@match){
			#print UID "$line\t1\n";
			@temp = split ("\t", $_);
			$temp[$start] =~ s/^\s+//;
			$temp[$stop] =~ s/^\s+//;
			chomp $temp[$start];
			chomp $temp[$stop];
			$coord{$temp[$start]} = 0;
			$coord{$temp[$stop]} = 0;
			$locitype{$temp[$cnvtype]} = 0;
			$sizes{$temp[$size]} = 0;
			$scores{$temp[$score]} = 0;
			$snps{$temp[$numsnps]} = 0;
			$family = $temp[$famID];
			$sample = $temp[$sampleID];
			$chrm = $temp[$chromo];
		}
		
		$chrm =~ s/chr//i;
		
		#Sort and determine the loci start and end
		@coord = keys %coord;
		@coord = sort {$a <=> $b} @coord;
		$locistart = $coord[0];
		$locistop = $coord[$#coord];
		$locisize = $locistop - $locistart;
		
		#Sort and determine the loci types present
		@locitypes = keys %locitype;
		@locitypes = sort {$a <=> $b} @locitypes;
		$locitypes = $types = join (", ", @locitypes);
		
		
		#Sort and determine the largest sized call
		@sizes = keys %sizes;
		@sizes = sort {$a <=> $b} @sizes;
		$maxsize = $sizes[-1];
		
		#Sort and determine the highest score in a call
		@scores = keys %scores;
		@scores = sort {$a <=> $b} @scores;
		$maxscore = $scores[-1];
		
		#Sort and determine the call with the most snps
		@snps = keys %snps;
		@snps = sort {$a <=> $b} @snps;
		$maxsnp = $snps[-1];
		
		#print "Within this set: Start = $locistart, End = $locistop, Types = $locitypes, Maxsize = $maxsize, MaxSNP = $maxsnp\n";
		
		$algocount = 0;
		%algos = ();
		%uid = ();
		%type = ();
		
		############################################################################################################################
		
		#Look for matches at each line of the co-ord
		foreach $bp (@coord){
			$lastone = $coord[$counter - 1];
			$thisone = $coord[$counter];
			$nextone = $coord[$counter + 1];
			
			$counter++;
			
			##########################################################################################################################
			
			#For each coord look for matches in the start and stop coords
			foreach $line4 (@match){
				@cnvm = split ("\t", $line4);
				#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
				if($thisone == $cnvm[$start]){
					$algocount++;
					chomp $cnvm[$unique];
					$uid{$cnvm[$unique]} = "$cnvm[$algo]\t$cnvm[$cnvtype]\t$cnvm[$score]\t$cnvm[$numsnps]\t$cnvm[$size]\t$cnvm[$unique]";
				}
				
				#If the coord matches the end of a CNV the uid and algo are removed from the hashes
				if ($thisone == $cnvm[$stop]){
					$algocount = $algocount - 1;
					chomp $cnvm[$unique];
					delete ($uid{$cnvm[$unique]});
				}
			}
			
			%type = ();
			%algoname = ();
			@features = ();
			
			@uidkeys = keys %uid;
			
			foreach $key (@uidkeys){
				$temp = $uid{$key};
				chomp $temp;
				push (@features, $temp);
				@bits = split ("\t", $temp);
				$type{$bits[1]} = 1;
				$algoname{$bits[0]} = 1;
			}
			
			#Work out the size of this section
			$musize = $nextone - $thisone;
			
			#Work out the types present
			@type = keys %type; 
			chomp @type;
			@type = sort @type;
			$types = join (", ", @type);
			
			#Work out the algorithms present
			@algoname = keys %algoname;
			@algoname = sort @algoname;
			$algonames = join (", ", @algoname);
			$algonamesize = @algoname;
			if ($algonamesize > 1){
				foreach $key (@uidkeys){
					$uidmatch{$key} = 1;
				}
			}
			
			#Printout this section of the match
			$printout1 = "$family\t$sample\tchr$chrm\t$locistart\t$locistop\t$locitypes\t$maxscore\t$maxsnp\t$locisize\t\t$thisone\t$nextone\t$musize\t$types\t$algocount\t$algonames\t\t";
			chomp @features;
			$printout2 = join ("\t", @features);
			unless ($nextone == ""){
				print MOUT $printout1 . $printout2 . "\n";
			}
		}
	}
	foreach $line (@CNVall){
		chomp $line;
		$line =~ s/\\n//gi;
		@temp = split ("\t", $line);
		print UID "$line\t$uidmatch{$temp[0]}\n";
	}
}

#Compares CNVs to the DGV loci to find rare ones
if ($rarealg == 1){
	#Input file
	$dgv = $ARGV[1];
	$inputfile = $ARGV[0];
	$dgvmod = $dgv;
	$dgvmod =~ s/.txt//;
	$dgvmod =~ s/\\/\//g;
	$dgvmod =~ s/$ANloc//;
	$dchromo = 1;
	$dstart = 2;
	$dstop = 3;
	$common = "Common";
	
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
	}
	else {
		$method = 0;
		mkdir ("03_Merged", 0777);
		print LOG1 "$timeanddate\tRare\tINFO\tStarting comparison with common CNVs from $dgv\n";
		chdir $merge_dir;
	}
	
	print "Loading CNV file $inputfile\n";
	open (INPUT, "<$inputfile") || die "Cannot open file $inputfile; check it is in the \'02_Processed\' directory\n";
	@CNVall = <INPUT>; chomp @CNVall; close (INPUT);
	chdir $curr_dir;
	########################################################
	
	#open output file
	$filename = $inputfile;
	if ($method == 0){
		$filename =~ s/\\/\//gi;
		$filename =~ s/$merge_dir//;
	}
	$output = $merge_dir . "Rare_" . $filename;
	open (ROUT, ">$output") || die "Cannot open the file $output; may already be open\n";
	print ROUT "$dgvmod\t$CNVall[0]\n";
	if ($method == 1){
		print "\nThe start of the file header is:\n";
		@lineshow = split ("\t", $CNVall[0]);
		foreach (@lineshow){
			$linecount++;
			if ($_ =~ /FID/){
				$lineanswer = $linecount - 1;
			}
			print "$linecount) $_\n";
		}
	}
	shift (@CNVall);
	
	
	########################################################
	#Merge prog Output file
	if ($method == 1){
		print "\nHow many columns have been added before the family ID (0, 1, 2, etc)? The answer is probably $lineanswer\t";
		$user_extra = <STDIN>; chomp $user_extra; $extra = $user_extra;
	}
	else {
		$extra = 0;
	}
	
	$famID = 0 + $extra;
	$sampleID = 1 + $extra;
	$chromo = 2 + $extra;
	$lstart = 3 + $extra;
	$lstop = 4 + $extra;
	$cnvtype= 5 + $extra;
	$maxsize = 6 + $extra;
	$maxsnp = 7 + $extra;
	$lsize = 8 + $extra;
	$start = 10 + $extra;
	$stop = 11 + $extra;
	$size = 12 + $extra;
	
	###############################################################################################
	
	#open the reference input file 
	print "Opening DGV file: $dgv\n";
	if ($method == 0){
		chdir $ANloc;
	}
	open (DGV, "<$dgv") || die "Could not open the file $dgv; check is in the $ANloc directory\n";
	chdir $curr_dir;
	@DGVs = <DGV>; chomp @DGVs; close (DGV);
	
	#Sort by Start (numbers)
	@DGVs = sort { (split '\t', $a)[$dstart] <=> (split '\t', $b)[$dstart] } @DGVs;
	
	#Sort by Chromosome (string)
	@DGVs = sort { (split '\t', lc $a)[$dchromo] cmp (split '\t', lc $b)[$dchromo] } @DGVs;
	
	#Make the reference input file into loci
	$tracker = 0;
	foreach (@DGVs){
		$DGVs[$tracker] =~ s/ //;
		@before = split ("\t", $DGVs[$tracker - 1]);
		@now = split ("\t", $DGVs[$tracker]);
		@after = split ("\t", $DGVs[$tracker + 1]);
		$tracker++;
		
		#print "\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
		
		#If this CNV is from a differet chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$dchromo] ne lc $before[$dchromo]
		or ($now[$dstart] > $before[$dstop]
		and $now[$dstart] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New chromosome or set\n";
			push (@match, $_);
		}
		
		#If it is the same chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$dchromo] eq lc $before[$dchromo]
		and ($now[$dstart] <= $before[$dstop]
		or $now[$dstart] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $_);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print "Error with the following CNV, its status could not be determined:\n$line\n";
			print LOG1 "$timeanddate\tRare\tERROR\tThe status of the following locus could not be determined:\t$line\n";
			$error++;
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$dchromo] ne lc $after[$dchromo]
		or ($now[$dstop] < $after[$dstart]
		and $maxbase < $after[$dstart])
		or $after[$dstart] == ""){
			
			#Process @match
			%bps = ();
			foreach (@match){
				@bp = split ("\t", $_);
				$bps{$bp[$dstart]} = 0;
				$bps{$bp[$dstop]} = 0;
				$chrm = $bp[$dchromo];
			}
			@bps = keys %bps;
			@bps = sort {$a <=> $b} @bps;
			$st = $bps[0];
			$en = $bps[-1];
	
			$arrayin = $chrm."\t".$st."\t".$en."\n";
			push (@input, $arrayin);
			#print "Last match in a set, dealing with matches\n@match\nInput: $arrayin\n";
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$dstop] > $maxbase){
			$maxbase = $now[$dstop];
		}
		
	}
	
	@DGVs = ();
	#open (IN, ">Modified_DGV.txt") || die "couldn't open file Modified_DGV.txt\n";
	#print IN @input;
	
	#Modify DGV array to same format as CNV file and make unique sampleID
	foreach $line (@input){
		@essex = split ("\t", $line);
		chomp @essex;
		$dcounter++;
		@newarray[$sampleID] = "$common$dcounter";
		@newarray[$chromo] = $essex[0];
		@newarray[$start] = $essex[1];
		@newarray[$stop] = $essex[2];
		$moddgv = join ("\t", @newarray);
		#print "$moddgv\n";
		push (@DGV2, "$moddgv\n");
	}
	
	#print IN @DGV2;
	
	###########################################################################################################
	
	print "Sorting CNV file by Sample\n";
	#Sort by SampleID (string)
	@CNVall = sort { (split '\t', lc $a, 20)[$sampleID] cmp (split '\t', lc $b, 20)[$sampleID] } @CNVall;
	
	#Split CNV list into samples then process each sample as it ends
	$tracker = 0;
	foreach (@CNVall){
		@before = split ("\t", $CNVall[$tracker - 1]);
		@now = split ("\t", $CNVall[$tracker]);
		@after = split ("\t", $CNVall[$tracker + 1]);
		$tracker++;
		
		#print "\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
		
		push (@sample, $_);
		
		if ($now[$sampleID] ne $after[$sampleID]
		or $after[$sampleID] eq ""){
			#print "Now: $now[$sampleID], After: $after[$sampleID]\n";
			#print "This is the next sample:\n";
			&samplepro2r;
			@sample = ();
		}
	}
	
	##################################################################################################################
	
	sub samplepro2r{
		
		$setcount++;
		if ($javamethod == 0){print "\rExamining sample $setcount";}
		
		#Combine CNVs of each sample with list of DGV
		@combine = ();
		push (@combine, @DGV2);
		push (@combine, @sample);
		
		#Sort by Start (numbers)
		@combine = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @combine;
		
		#Sort by Chromosome (string)
		@combine = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @combine;
		
		#print IN "@combine";
		
		$track = 0;
		#Does each CNV line up with the one below?
		foreach $line (@combine){
			@before = split ("\t", $combine[$track - 1]);
			@now = split ("\t", $combine[$track]);
			@after = split ("\t",$combine[$track + 1]);
			$track++;
			
			#print "\n\nNew CNV:\n";
			#print "Before: @before\n";
			#print "Now: @now\n";
			#print "After: @after\n";
			
			
			#If this CNV is from a differet sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
			if (lc $now[$chromo] ne lc $before[$chromo]
			or ($now[$start] > $before[$stop]
			and $now[$start] > $maxbase)){
				$maxbase = 0;
				@match = ();
				#print "New sample, chromosome or set\n";
				push (@match, $line);
			}
			
			#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
			elsif (lc $now[$chromo] eq lc $before[$chromo]
			and ($now[$start] <= $before[$stop]
			or $now[$start] <= $maxbase)){
				
				#All matches get put into an array called @match
				push (@match, $line);
				#print "This was a match\n";
			}
			
			#If the CNV does not fit into either of the above groups then there is an error
			else{
				print "Error with the following CNV, its status could not be determined:\n$line\n";
				print LOG1 "$timeanddate\tRare\tERROR\tThe status of the following locus could not be determined:\t$line\n";
				$error++;
			}
			
			#If this CNV does not overlap with the next CNV then @match is processed
			if (lc $now[$chromo] ne lc $after[$chromo]
			or ($now[$stop] < $after[$start]
			and $maxbase < $after[$start])
			or $after[$start] == ""){
				#Process @match
				&setexaminer;
				#print "Last match in a set, dealing with matches\n@match\n";
				}
			
			#Makes Maxbase the highest it can be
			if ($now[$stop] > $maxbase){
				$maxbase = $now[$stop];
			}
		}
	}
	
	##################################################################################################################
	
	sub setexaminer{
		
		$matches = @match;
		
		#If there is only one CNV it is simply printed out
		if ($matches == 1){
			if($match[0] !~ /$common/){
				$hashin = $now[$chromo].$now[$lstart].$now[$lstop];
				$thissamp = $now[$sampleID];
				$$thissamp{$hashin} = 0;
				$totalsamp{$now[$sampleID]} = 0;
				print ROUT "0\t$match[0]\n";
			}
			@match = ();
		}
		
		%coord = ();
		%locitype = ();
		%sizes = ();
		%snps = ();
		$counter = 0;
		
		#Put the start and stop positions into a new array and determine the characteristics of this group of matches
		foreach (@match){
			@temp = split ("\t", $_);
			$temp[$start] =~ s/^\s+//;
			$temp[$stop] =~ s/^\s+//;
			chomp $temp[$start];
			chomp $temp[$stop];
			$coord{$temp[$start]} = 0;
			$coord{$temp[$stop]} = 0;
			if ($temp[$sampleID] !~ /$common/){
				$sample = $temp[$sampleID];
				$chrm = $temp[$chromo];
			}
		}
		
		#Sort and determine the loci start and end
		@coord = keys %coord;
		@coord = sort {$a <=> $b} @coord;
		
		$algocount = 0;
		%algos = ();
		%uid = ();
		%type = ();
		$counter = 0;
		
		############################################################################################################################
		
		#Look for matches at each line of the co-ord
		foreach $bp (@coord){
			$lastone = $coord[$counter - 1];
			$thisone = $coord[$counter];
			$nextone = $coord[$counter + 1];
			
			$counter++;
			
			##########################################################################################################################
			
			#For each coord look for matches in the start and stop coords
			foreach $line4 (@match){
				@cnvm = split ("\t", $line4);
				#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
				if($thisone == $cnvm[$start]){
					$algocount++;
					chomp $cnvm[$sampleID];
					$uid{$cnvm[$sampleID]} = "$line4";
				}
				
				#If the coord matches the end of a CNV the uid and algo are removed from the hashes
				if ($thisone == $cnvm[$stop]){
					$algocount = $algocount - 1;
					chomp $cnvm[$sampleID];
					delete ($uid{$cnvm[$sampleID]});
				}
			}
			
			#Work out the size of this section
			$musize = $nextone - $thisone;
			
			@uidkeys = keys %uid;
			$commie = 0;
			$interest = 0;
			
			foreach $key (@uidkeys){
				if ($key =~ /$common/){
					$commie = 1;
					}
				else {
					@resize = split ("\t", $uid{$key});
					$resize[$start] = $thisone;
					$resize[$stop] = $nextone;
					$resize[$size] = $musize;
					$interest = 1;
				}
			}
			
			#Printout this section of the match
			$printout = join ("\t", @resize);
			unless ($nextone eq ""
			or $interest == 0){
				$hashin = $resize[$chromo].$resize[$lstart].$resize[$lstop];
				$thissamp = $resize[$sampleID];
				$$thissamp{$hashin} = $$thissamp{$hashin} + $commie;
				$totalsamp{$resize[$sampleID]} = 0;
				print ROUT $commie . "\t" . $printout . "\n";
			}
		}
	}
	print "\n";
	if ($method == 0){
		#Opens the Sample List file to add the QT calls
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tRare: Total Loci\tRare: Loci marked common ($dgvmod)\n";
		shift @samplist;
	}
	$timeanddate = &showtime;
	$totalsamples = keys %totalsamp;
	#print "Total samples: $totalsamples\n";
	if ($method == 1){
		@samplist = keys %totalsamp;
	}
	foreach $line (@samplist){
		$common = 0;
		@tab = split (/\t/,$line);
		$thissamp = $tab[0];
		#print "Thissamp: $thissamp\n";
		@samploci = keys %$thissamp;
		$numloci = @samploci;
		#print "Numloci: $numloci\n";
		foreach (@samploci){
			#print "Samp: $thissamp, Loci: $_, Common: $$thissamp{$_}\n";
			if ($$thissamp{$_} > 0){
				$common++;
			}
		}
		
		if ($numloci ne ""){
			print SAMP "$line\t$numloci\t$common\n";
			$samplecount++;
		}
		else {
			print SAMP "$line\t0\t0\n";
			print LOG1 "$timeanddate\tRare\tERROR\tNo loci present for $tab[0]\n";
			print "ERROR: No loci present for $tab[0]\n";
		}
	}
	if ($samplecount == $totalsamples){
		print LOG1 "$timeanddate\tRare\tINFO\tComparison with common CNVs complete for $totalsamples samples\n";
		print "Comparison with common CNVs complete for $totalsamples samples\n";
	}
	else {
		print LOG1 "$timeanddate\tRare\tERROR\tComparison with common CNVs complete, however loci were present for samples that were not listed in the Samplelist file\n";
		print "ERROR: Comparison with common CNVs complete, however loci were present for samples that were not listed in the Samplelist file\n";
	}
}

#Identifies differences between two similar files
if ($file_difference == 1){
	#Input: File to be marked, file of samples (with samplID in column 1)
	$input1 = $ARGV[0];
	$input2 = $ARGV[1];
	$output = "Diff_" . $input1 . "_" . $input2;
	open (IN, "<$input1") || die "Cannot open file $input1, check it is in the right directory\n";
	@input = <IN>; chomp @input; close (IN);
	open (PED, "<$input2") || die "Cannot open file $input2, check it is in the right directory\n";
	@peds = <PED>; chomp @peds; close (PED);
	open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
	
	$diffcount = 0;
	
	$inputsize = @input;
	$pedsize = @peds;
	if ($inputsize >= $pedsize){
		$maxsize = $inputsize;
	}
	else {
		$maxsize = $pedsize;
	}
	@arrayhere = (0..$maxsize);
	foreach $number (@arrayhere){
		if ($input[$number] eq $peds[$number]){
			print OUT "\n";
		}
		else {
			@tab1 = split ("\t",$input[$number]);
			@tab2 = split ("\t",$peds[$number]);
			$tab_1 = @tab1;
			$tab_2 = @tab2;
			if ($tab_1 >= $tab_2){
				$maxsize = $tab_1;
			}
			else {
				$maxsize = $tab_2;
			}
			@tabhere = (0..$maxsize);
			foreach $num (@tabhere){
				if ($tab1[$num] eq $tab2[$num]){
					print OUT "\t";
				}
				else {
					$diffcount++;
					print OUT "$tab1[$num],$tab2[$num]\t";
				}
			}
			print OUT "\n";
		}
	}
	print "$diffcount differences found\n";
}

#Marks all samples in a file based on a list of samples
if ($mark_samples == 1){
	#Input: File to be marked, file of samples (with samplID in column 1)
	$input1 = $ARGV[0];
	$input2 = $ARGV[1];
	
	open (IN, "<$input1") || die "Cannot open file $input1, check it is in the right directory\n";
	@input = <IN>; chomp @input; close (IN);
	open (PED, "<$input2") || die "Cannot open sample list file $input2, check it is in the right directory\n";
	@peds = <PED>; chomp @peds; close (PED);
	
	print "Method: $method\n";
	if ($method == 0){
		@tab = split (/\\/,$input1);
		$output = $merge_dir . $tag . "_HD_Results.txt";
		open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
		$headline = $input[0];
		shift @input;
		print OUT "Bad sample\t$headline\n";
		$sampleID = 3;
	}
	else {
		$output = "Mark_" . $input1;
		open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
		print "\n\nThe start of the file header is:\n";
		print LISTOUT $curr_dir.$output;
		@lineshow = split ("\t",$input[0]);
		foreach (@lineshow){
			$linecount++;
			print "$linecount) $_\n";
		}
		
		print "\nWhich column contains the SampleID?\n";
		$user_samp = <STDIN>; chomp $user_samp; $sampleID = $user_samp - 1;
		print "\nIs there a header column? (y or n)\n";
		$user_head = <STDIN>; chomp $user_head;
		if ($user_head =~ /y/){
			$headline = $input[0];
			shift @input;
		}
		print OUT "Marked\t$headline\n";
	}
	
	foreach $samp (@peds){
		@tab = split ("\t",$samp);
		if ($method == 0){
			@tab2 = split (/\\/,$tab[0]);
			$marklist{$tab2[-1]} = 1;
		}
		else {
			$marklist{$tab[0]} = 1;
		}
	}
	
	foreach $line (@input){
		@tab = split ("\t", $line);
		$sample = $tab[$sampleID];
		if ($marklist{$sample} == 1){
			print OUT "1\t$line\n";
		}
		else {
			print OUT "0\t$line\n";
		}
	}
	print "All done!\n";
}

#Changes the order of columns within a file
if ($re_order == 1){
	$input1 = $ARGV[0];
	$output = "Ord_" . $input1;
	open (IN, "<$input1") || die "Cannot open file $input1, check it is in the right directory\n";
	@input = <IN>; chomp @input; close (IN);
	open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
	
	print "\n\nThe first line of the file is:\n";
	print LISTOUT $curr_dir.$output;
	@lineshow = split ("\t",$input[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	$linecount = 0;
	foreach (@lineshow){
		$linecount++;
		print "\n$linecount) $_ Which column should this be in now (eg 3, 5, none)?\n";
		$user_samp = <STDIN>; chomp $user_samp; 
		unless ($user_samp eq "" or $user_samp =~ /n/i){
			$user_samp = $user_samp;
			$neword{$user_samp} = ($linecount - 1);
		}
	}
	
	@list = keys %neword;
	@list = sort {$a <=> $b} @list;
	@collist = ($list[0]..$list[-1]);
	#print @collist;
	
	foreach $line (@input){
		$lineout = "";
		@tab = split ("\t", $line);
		foreach $col (@collist){
			$lineout .= "$tab[$neword{$col}]\t";
			#print "Col: $col, Tab: $neword{$col}, Out: $tab[$neword{$col}]\n";
		}
		chomp $lineout;
		print OUT "$lineout\n";
	}
}

#Annotates a list of samples against the pedfile
if ($ped_detail == 1){
	$input1 = $ARGV[0];
	$input2 = $ARGV[1];
	
	open (IN, "<$input1") || die "Cannot open file $input1, check it is in the right directory\n";
	@input = <IN>; chomp @input; close (IN);
	if ($method == 0){
		@files = <*>;
		@tab = split (/\\/,$input1);
		$output = $merge_dir . "Ped_" . $tab[-1];
		foreach $file (@files){
			if ($file =~ /pedfile/){
				open (PED, "<$file") || die "Cannot open pedfile $file, check it is in the right directory\n";
				@peds = <PED>; chomp @peds; close (PED);
			}
		}
	}
	else {
		open (PED, "<$input2") || die "Cannot open pedfile $input2, check it is in the right directory\n";
		@peds = <PED>; chomp @peds; close (PED);
		$output = "Ped_" . $input1;
	}
	#This subroutine processes a pedfile to make the following hashes:
	#pedfile - key: sampleID, value: all 6 columns of pedfile
	#family_member - key: familyID, value: list of samples in that family
	#which_family - key: sampleID, value: familyID
	#which_mother - key: sampleID, value: mother's sampleID
	#which_father - key: sampleID, value: father's sampleID
	#which_child - key: either parent's sampleID, value: list of children's sampleID
	#which_sibling - key: sampleID, value: list of sibling sampleIDs
	#gender - key: sampleID, value: gender (0, 1, 2)
	#gendername - key: sampleID, value: gender (Unknown, Male, Female)
	#phenotype - key: sampleID, value: phenotype (0, 1, 2)
	#namepheno - key: sampleID, value: Proband, Sibling, Unaffected child, Control, Father, Mother, Unknown
	&propedfile(@peds);
	open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
	
	print LISTOUT $curr_dir.$output;
	
	if ($method == 0){
		$headline = $input[0];
		shift @input;
		$sampleID = 1;
		print OUT "Phenotype\tGender\t$headline\n";
	}
	else {
		print "\n\nThe start of the file header is:\n";
		@lineshow = split ("\t",$input[0]);
		foreach (@lineshow){
			$linecount++;
			print "$linecount) $_\n";
		}
		
		print "\nWhich column contains the SampleID?\n";
		$user_samp = <STDIN>; chomp $user_samp; $sampleID = $user_samp - 1;
		print "\nIs there a header column? (y or n)\n";
		$user_head = <STDIN>; chomp $user_head;
		if ($user_head =~ /y/){
			$headline = $input[0];
			shift @input;
		}
		print OUT "Phenotype\tGender\tFID\t$headline\n";
	}
	
	
	foreach $line (@input){
		@tab = split ("\t", $line);
		$sample = $tab[$sampleID];
		if ($method == 0){
			shift @tab;
			$line = join ("\t", @tab);
		}
		if ($gendername{$sample} eq ""){
			print OUT "No_pedfile\tNo_pedfile\tNo_pedfile\t$line\n";
		}
		else {
			print OUT "$namepheno{$sample}\t$gendername{$sample}\t$which_family{$sample}\t$line\n";
		}
	}
	print "All done!\n";
}

#Finds de novo CNVs my looking for CNVs in the parents
if ($denovoalg == 1){
	
	if($tag eq ""){
		$tag = "M";
		$method = 1;
		$merge_dir = "";
		$input = $ARGV[0];
		$output = "DN_" . $input;
	}
	else {
		mkdir ("03_Merged", 0777);
		#chdir $merge_dir;
		$method = 0;
		print LOG1 "$timeanddate\tDenovo\tINFO\tStarting DeNovo indentification\n";
		$input = $ARGV[0];
		@tab = split (/\\/,$input);
		$output = $merge_dir . "DN_" . $tab[-1];
		#Opens the Sample List file to add the QT calls
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || die "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tPhenotype\tFamily Size\tDe Novo Calculated\n";
		shift @samplist;
		chdir $merge_dir;
	}
	
	########################################################
	#Merge prog Output file
	open (INPUT, "<$input") || die "Cannot open file $input; check it is in the right directory\n";
	chdir $curr_dir;
	print "Opening CNV file: $input\n";
	@CNValls = <INPUT>; close (INPUT);
	@lineshow = split ("\t", $CNValls[0]);
	$pedmodfile = 0;
	
	#open output file
	open (OUT, ">$output") || die "Cannot open the file $output; may already be open\n";
	print OUT "Gender\tPhenotype\tFamily Size\tDe Novo\t$CNValls[0]";
	shift (@CNValls);
	
	#This is all about finding the right pedfile
	chdir $merge_dir;
	@files = <*>;
	foreach (@files){
		if($_ =~ /pedfile/
		and $_ =~ /Mod/){
			print "Opening a modified pedfile: $_\n";
			open (PED, "<$_") || die "Cannot open file $_, check it is in the right directory\n";
			@peds = <PED>; chomp @peds; close (PED);
			$pedmodfile++;
			if ($pedmodfile >= 2){
				print LOG1 "$timeanddate\tDenovo\tERROR\tThere are two modified pedfiles in this directory\n";
				die "ERROR: There are two modified pedfiles in this directory\n";
			}
		}
	}
	
	chdir $curr_dir;
	@files = <*>;
	foreach (@files){
		if($_ =~ /pedfile/
		and $_ =~ /Mod/
		and $pedmodfile == 0){
			print "Opening a modified pedfile: $_\n";
			open (PED, "<$_") || die "Cannot open file $_, check it is in the right directory\n";
			@peds = <PED>; chomp @peds; close (PED);
			$pedmodfile++;
			if ($pedmodfile >= 2){
				print LOG1 "$timeanddate\tDenovo\tERROR\tThere are two modified pedfiles in this directory\n";
				die "ERROR: There are two modified pedfiles in this directory\n";
			}
		}
	}
	
	foreach (@files){
		#print "File: $_\n";
		if($_ =~ /pedfile/
		and $pedmodfile == 0){
			print "Opening an unmodified pedfile: $_\n";
			print LOG1 "$timeanddate\tDenovo\tERROR\tUsing an unmodified pedfile, parents may have failed quality control\n";
			open (PED, "<$_") || die "Cannot open file $_, check it is in the right directory\n";
			@peds = <PED>; chomp @peds; close (PED);
			$pedmodfile++;
			if ($pedmodfile >= 2){
				print LOG1 "$timeanddate\tDenovo\tERROR\tThere are two unmodified pedfiles in this directory\n";
				die "ERROR: There are two unmodified pedfiles in this directory\n";
			}
		}
	}
	
	if ($pedmodfile == 0){
		print LOG1 "$timeanddate\tDenovo\tERROR\tNo pedfile was present\n";
		print "ERROR: No pedfile found\n";
		print LOG1 "$timeanddate\tDenovo\tERROR\tContinuing without a pedfile\n";
		foreach (@CNValls){
			print OUT "Unknown\t1\t\t\t$_";
		}
		$timeanddate = &showtime;
		foreach $line (@samplist){
			print SAMP "$line\t0\t0\t0\n";
		}
		print LOG1 "$timeanddate\tDenovo\tINFO\tDe Novo comparison not run due to missing pedfile; blank columns added to allow pipeline to continue\n";
	}
	
	else {
		if ($method == 1){
			print "\n\nThe start of the file header is:\n";
			foreach (@lineshow){
				$linecount++;
				if ($_ =~ /FID/){
					$lineanswer = $linecount - 1;
				}
				unless ($linecount >= 20){
					print "$linecount) $_\n";
				}
			}
		}
		
		########################################################
		#Merge prog Output file
		
		if ($method == 1){
			print "\nHow many columns have been added before the family ID (0, 1, 2, etc)?\nThe answer is probably $lineanswer\n";
			$user_extra = <STDIN>; chomp $user_extra; $extra = $user_extra;
		}
		else {
			$extra = 1;
		}
		
		$famID = 0 + $extra;
		$sampleID = 1 + $extra;
		$chromo = 2 + $extra;
		$lstart = 3 + $extra;
		$lstop = 4 + $extra;
		$cnvtype= 5 + $extra;
		$maxsize = 6 + $extra;
		$maxsnp = 7 + $extra;
		$lsize = 8 + $extra;
		$start = 10 + $extra;
		$stop = 11 + $extra;
		$size = 12 + $extra;
		
		###############################################################################################
		
		$common = "Common";
		
		###########################################################################################################
		#pedfile - key: sampleID, value: all 6 columns of pedfile
		#family_member - key: familyID, value: list of samples in that family
		#which_family - key: sampleID, value: familyID
		#which_mother - key: sampleID, value: mother's sampleID
		#which_father - key: sampleID, value: father's sampleID
		#which_child - key: either parent's sampleID, value: list of children's sampleID
		#which_sibling - key: sampleID, value: list of sibling sampleIDs
		#gender - key: sampleID, value: gender (0, 1, 2)
		#gendername - key: sampleID, value: gender (Unknown, Male, Female)
		#phenotype - key: sampleID, value: phenotype (0, 1, 2)
		#namepheno - key: sampleID, value: Proband, Sibling, Unaffected child, Control, Father, Mother, Unknown
		
		&propedfile(@peds);
		
		########################################################################################################
		
		#Correct Family IDs in samples using the pedfile
		foreach (@CNValls){
			@here = split ("\t", $_);
			$here[$famID] = $which_family{$here[$sampleID]};
			if ($which_family{$here[$sampleID]} ne ""){
				#print "$which_family{$here[$sampleID]}\n";
				$rejoin = join ("\t", @here);
				push (@CNVall, $rejoin);
			}
			else {
				$missingped{$here[$sampleID]} = 0;
			}
		}
		@CNValls = ();
		
		print "Sorting CNV file by Family\n";
		#Sort by FamID (string)
		@CNVall = sort { (split '\t', lc $a, 20)[$famID] cmp (split '\t', lc $b, 20)[$famID] } @CNVall;
		
		#Split CNV list into families with an array dedicated to each sample and then process each family as it ends
		$tracker2 = 0;
		foreach (@CNVall){
			@before2 = split ("\t", $CNVall[$tracker2 - 1]);
			@now2 = split ("\t", $CNVall[$tracker2]);
			@after2 = split ("\t", $CNVall[$tracker2 + 1]);
			$tracker2++;
			
			#print "\nNew CNV:\n";
			#print "Before: @before2\n";
			#print "Now: @now2\n";
			#print "After: @after2\n";
			
			#Each CNV is put into an array named after the sample ID
			$samphere = $now2[$sampleID];
			push (@$samphere, $_);
			#print "$now2[$sampleID]\n";
			$fammembers{$now2[$sampleID]} = 0;
			
			if ($now2[$famID] ne $after2[$famID]
			or $after2[$famID] eq ""){
				$familiespres++;
				#print "Now: $whichfam{$now2[$sampleID]}, After: $whichfam{$after2[$sampleID]}\n";
				#print "This is the next family:\n";
				$famnow = $now2[$famID];
				@family = keys %fammembers;
				&familyprod(@family);
				%fammembers  = ();
			}
		}
		
		##################################################################################################################
		
		chdir $curr_dir;
		if ($method == 0){
			$timeanddate = &showtime;
			$totalsamples = keys %samplelog;
			#print "Total samples: $totalsamples\n";
			foreach $line (@samplist){
				@tab = split (/\t/,$line);
				$dnout = $samplelog{$tab[0]};
				
				if ($dnout ne ""){
					$samplecounter++;
					print SAMP "$line\t$dnout\n";
				}
				else {
					print SAMP "$line\t0\t0\t0\n";
					print LOG1 "$timeanddate\tDenovo\tERROR\tNo loci present for $tab[0]\n";
					print "ERROR: No loci present for $tab[0]\n";
				}
			}
			if ($samplecounter == $totalsamples){
				print LOG1 "$timeanddate\tDenovo\tINFO\tDe Novo comparison complete for $totalsamples samples from $familiespres families\n";
				print "De Novo comparison complete for $totalsamples samples from $familiespres families\n";
			}
			else {
				print LOG1 "$timeanddate\tDenovo\tERROR\tDe Novo comparison complete, however loci were present for samples that were not listed in the Samplelist file\n";
				print "ERROR: De Novo comparison complete, however loci were present for samples that were not listed in the Samplelist file, $totalsamples, $samplecounter\n";
			}
		}
	}
	print "All done!\n";
}

#Summaries the output of merged data
if ($inquire == 1){
	
	
	
	if($tag eq ""){
		$tag = "$input";
		$method = 1;
		$merge_dir = "";
		$output = "Sum_" . $input;
		$bedin = $input;
		$bedin =~ s/.txt/.bed/gi;
		$bedout1 = "GB_track_Affected_" . $bedin;
		$bedout2 = "GB_track_Unaffected_" . $bedin;
		$badinput = "Badchips.txt";
		
		@lineshow = split ("\t", $CNV[0]);
		foreach (@lineshow){
			$linecount++;
			if ($_ =~ /FID/){
				$lineanswer = $linecount - 1;
			}
			print "$linecount) $_\n";
		}
	}
	else {
		$method = 0;
		mkdir ("04_Final_Results", 0777);
		$pipeline = $curr_dir.'Final_Results/';
		$output = $merge_dir . $tag . "_CNV_Results.txt";
		$bedout1 = $fin_dir . $tag . "_GB_track_Affected.bed";
		$bedout2 = $fin_dir . $tag . "_GB_track_Unaffected.bed";
		$badinput = $merge_dir . $tag . "_Badchips.txt";
		print LOG1 "$timeanddate\tInquire\tINFO\tSummary track creation started\n";
		chdir $merge_dir;
	}
	
	$input = $ARGV[0];
	print "Opening CNV file: $input\n";
	open (INPUT, "<$ARGV[0]") || die "Cannot open file $input; check it is in the \'03_Merged\' directory\n";
	chdir $curr_dir;
	@CNV = <INPUT>; close (INPUT);
	
	shift @CNV;
	open (OUT, ">$output") || die "Cannot open file $output; may already be open\n";
	if ($method == 1){
		print OUT "UCSC lookup\tBad sample\tGender\tPhenotype\tFamily size\tFID\tIID\tChr\tLoci Start\tLoci Stop\tType\tMax Score\tMax SNP\tSize\t#Algos\tAlgos\t%DeNovo1\t%Rare1: $lineshow[$rare1]\t%Rare2: $lineshow[$rare2]\t%Rare3: $lineshow[$rare3]\t%Three Algs\t%Two Algs\t%One Alg\n";
	}
	else {
		print OUT "UCSC lookup\tBad sample\tGender\tPhenotype\tFamily size\tFID\tIID\tChr\tLoci Start\tLoci Stop\tType\tMax Score\tMax SNP\tSize\t#Algos\tAlgos\t%DeNovo1\t%Rare1: $lineshow[$rare1]\t%Three Algs\t%Two Algs\t%One Alg\n";
	}
	open (BEDOUT1, ">$bedout1") || die "Cannot open file $bedout1, may already be open\n";
	print BEDOUT1 "track name=\'" . $tag . "_Proband\' description='CNVs found in $output for the proband' itemRgb=On visibility=3\n";
	open (BEDOUT2, ">$bedout2") || die "Cannot open file $bedout2, may already be open\n";
	print BEDOUT2 "track name=\'" . $tag . "_Other\' description='CNVs found in $output for other family members' itemRgb=On visibility=3\n";

	unless (-e $badinput){
		print LOG1 "$timeanddate\tInquire\tERROR\tNo $badinput file found; samples that fail quality control will not be marked\n";
		$nobad = 1;
	}
	open (BADLIST, "<$badinput") || print "ERROR: There was no $badinput file, badchips will not be marked\n";
	@badlist = <BADLIST>; chomp @badlist; close BADLIST;
	
	foreach (@badlist){
		$badchip{$_} = 1;
	}
	########################################################
	#Merge prog Output file
	if ($method == 1){
		print "\nHow many columns have been added before the family ID (0, 1, 2, etc)?\n";
		print   "The answer is probably $lineanswer.\n";
		$user_extra = <STDIN>; chomp $user_extra; $extra = $user_extra;
		print "\nWhich column is the de novo data in (1, 2, 3, etc)?\n";
		$user_denovo = <STDIN>; chomp $user_denovo; $denovo1 = ($user_denovo - 1);
		print "Which column is the first column of rare data in (1, 2, 3, etc)?\n";
		$user_rare1 = <STDIN>; chomp $user_rare1; $rare1 = ($user_rare1 - 1);
		print "Which column is the second column of rare data in (1, 2, 3, etc)?\n";
		$user_rare2 = <STDIN>; chomp $user_rare2; $rare2 = ($user_rare2 - 1);
		print "Which column is the third column of rare data in (1, 2, 3, etc)?\n";
		$user_rare3 = <STDIN>; chomp $user_rare3; $rare3 = ($user_rare3 - 1);
	}
	else {
		$extra = 5;
		$denovo1 = 3;
		$rare1 = 4;
		$rare2 = 4;
		$rare3 = 4;
	}
	
	$genplace = 0;
	$phenoplace = 1;
	$famsize = 2;
	
	$famID = 0 + $extra;
	$sampleID = 1 + $extra;
	$chromo = 2 + $extra;
	$lstart = 3 + $extra;
	$lstop = 4 + $extra;
	$cnvtype = 5 + $extra;
	$maxsize = 6 + $extra;
	$maxsnp = 7 + $extra;
	$lsize = 8 + $extra;
	$start = 10 + $extra;
	$stop = 11 + $extra;
	$size = 12 + $extra;
	$type = 13 + $extra;
	$algonum = 14 + $extra;
	$algos = 15 + $extra;
	
	###############################################################################################
	
	print "Sorting input file by Sample, Chromosome and Loci Start\n";
	#Sort by Start (numbers)
	@CNV = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNV;
	
	#Sort by Chromosome (string)
	@CNV = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNV;
	
	#Sort by SampleID (string)
	@CNV = sort { (split '\t', lc $a)[$sampleID] cmp (split '\t', lc $b)[$sampleID] } @CNV;
	
	#Split CNV list into loci then process each sample as it ends
	print "CNVs sorted, now examining each loci\n";
	$tracker = 0;
	foreach (@CNV){
		@before = split ("\t", $CNV[$tracker - 1]);
		@now = split ("\t", $CNV[$tracker]);
		@after = split ("\t", $CNV[$tracker + 1]);
		$tracker++;
		
		#print "\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
		
		push (@loci, $_);
		#print "Now: $now[$lstart], After: $after[$lstart]\n";
	
		
		if ($now[$sampleID] ne $after[$sampleID]
		or $now[$chromo] ne $after[$chromo]
		or $now[$lstart] != $after[$lstart]
		or $after[$sampleID] eq ""){
			#print "Now: $now[$sampleID], After: $after[$sampleID]\n";
			#print "This is the next loci:\n";
			&lociproi;
			$locicount++;
			@loci = ();
		}
		
		if ($now[$sampleID] ne $after[$sampleID]){
			$samplelocicount{$now[$sampleID]} = $morelocicount;
			$morelocicount = 0;
		}
	}
	
	sub lociproi{
		$morelocicount++;
		$locicountx++;
		$rare1size = 0;
		$rare2size = 0;
		$rare3size = 0;
		$denovo1size = 0;
		$three = 0;
		$two = 0;
		$one = 0;
		%maxalgo = ();
		$firstthree = 0;
		$threestart = 0;
		$threeend = 0;
		
		foreach $line (@loci){
			@tabs = split ("\t", $line);

			if ($tabs[$rare1] eq 0){
				$rare1size = $rare1size + $tabs[$size];
			}
			if ($tabs[$rare2] eq 0){
				$rare2size = $rare2size + $tabs[$size];
			}
			if ($tabs[$rare3] eq 0){
				$rare3size = $rare3size + $tabs[$size];
			}
			if ($tabs[$denovo1] eq 0){
				$denovo1size = $denovo1size + $tabs[$size];
				#print "Loci Size = $tabs[$lsize], Denovo = $denovo1size, First column = $tabs[$denovo1]\n";
			}
			elsif ($tabs[$denovo1] eq 1){}
			else {
				$denovo1size = "";
			}
			if ($tabs[$algonum] == 3){
				$three = $three + $tabs[$size];
				if ($firstthree == 0){
					$threestart = $tabs[$start];
					$firstthree = 1;
				}
				$threeend = $tabs[$stop];
			}
			if ($tabs[$algonum] == 2){
				$two = $two + $tabs[$size];
			}
			if ($tabs[$algonum] == 1){
				$one = $one + $tabs[$size];
			}
			$maxalgo{$tabs[$algonum]} = $tabs[$algos];
		}
		
		@tab = split ("\t", $loci[0]);
		
		#Percent rare1, de novo and number of algos
		
		if ($tab[$lsize] > 0){
			$prare1 = sprintf ("%0d", $rare1size / $tab[$lsize] * 100);
			$prare2 = sprintf ("%0d", $rare2size / $tab[$lsize] * 100);
			$prare3 = sprintf ("%0d", $rare3size / $tab[$lsize] * 100);
			if ($denovo1size ne "") {
				$pdenovo1 = sprintf ("%0d", $denovo1size / $tab[$lsize] * 100);
				#print "Denovo percentage = $pdenovo1\n";
			}
			else {
				$pdenovo1 = "";
			}
			$pthree = sprintf ("%0d", $three / $tab[$lsize] * 100);
			$ptwo = sprintf ("%0d", $two / $tab[$lsize] * 100);
			$pone = sprintf ("%0d", $one / $tab[$lsize] * 100);
		}
		
		@algo = keys %maxalgo;
		@algo = sort {$a <=> $b} @algo;
		$maxal = $algo[-1];
		$maxalgs = $maxalgo{$maxal};
		
		
		if ($firstthree == 1){
			$thickstart = $threestart;
			$thickend = $threeend;
		}
		else {
			$thickstart = $tab[$lstart];
			$thickend = $tab[$lstart];
		}
		
		if ($tab[$cnvtype] eq "0") {
			$colour = "255,0,0";
		}
		elsif ($tab[$cnvtype] eq "1"
		or $tab[$cnvtype] eq "0, 1") {
			$colour = "100,0,0";
		}
		elsif ($tab[$cnvtype] eq "3"
		or $tab[$cnvtype] =~ /3/
		and $tab[$cnvtype] !~ /0/
		and $tab[$cnvtype] !~ /1/) {
			$colour = "0,150,0";
		}
		elsif ($tab[$cnvtype] eq "4"){
			$colour = "0,255,0";
		}
		else {$colour = "0,0,0";}
		
		if ($tab[$phenoplace] =~ /Father/gi){
			$smallpheno = "Dad";
		}
		elsif ($tab[$phenoplace] =~ /Mother/gi){
			$smallpheno = "Mom";
		}
		elsif ($tab[$phenoplace] =~ /Sibling/gi or $tab[$phenoplace] =~ /Control/gi or $tab[$phenoplace] =~ /Unaffected/gi){
			$smallpheno = "CT";
		}
		elsif ($tab[$phenoplace] =~ /Proband/gi){
			$smallpheno = "Case";
		}
		else {
			$smallpheno = "???";
		}
		
		if ($pdenovo1 >= 90){
			$strand = "+";
			#print "Strand =$strand\n"
		}
		else {$strand = ".";}
		
		if ($nobad == 1){
			$badchip = "?";
		}
		elsif ($badchip{$tab[$sampleID]} == 1){
			$badchip = 1;
		}
		else {$badchip = 0;}
		
		if (($threeend - $threestart)>0){
			$multthree = $three / ($threeend - $threestart);
		}
		else {$multthree = 0;}
		
		if ($tab[$chromo] eq "chrXY"){
			if ($smallpheno eq "Case"
			and $badchip != 1){
				print BEDOUT1 "chrX\t$tab[$lstart]\t$tab[$lstop]\t$tab[$sampleID]_$tab[$famID]_$tab[$maxsnp]_" . $maxal . "_" . $smallpheno . "\t$maxal\t$strand\t$thickstart\t$thickend\t$colour\n";
				print BEDOUT1 "chrY\t$tab[$lstart]\t$tab[$lstop]\t$tab[$sampleID]_$tab[$famID]_$tab[$maxsnp]_" . $maxal . "_" . $smallpheno . "\t$maxal\t$strand\t$thickstart\t$thickend\t$colour\n";
			}
			elsif ($badchip != 1){
				print BEDOUT2 "chrX\t$tab[$lstart]\t$tab[$lstop]\t$tab[$sampleID]_$tab[$famID]_$tab[$maxsnp]_" . $maxal . "_" . $smallpheno . "\t$maxal\t$strand\t$thickstart\t$thickend\t$colour\n";
				print BEDOUT2 "chrY\t$tab[$lstart]\t$tab[$lstop]\t$tab[$sampleID]_$tab[$famID]_$tab[$maxsnp]_" . $maxal . "_" . $smallpheno . "\t$maxal\t$strand\t$thickstart\t$thickend\t$colour\n";
			}
			if ($method == 1){
				print OUT "=HYPERLINK(\"http://genome.ucsc.edu/cgi-bin/hgTracks?clade=mammal&org=Human&db=hg18&position=$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\", \"$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\")\t$badchip\t$tab[$genplace]\t$tab[$phenoplace]\t$tab[$famsize]\t$tab[$famID]\t$tab[$sampleID]\t$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$cnvtype]\t$tab[$maxsize]\t$tab[$maxsnp]\t$tab[$lsize]\t$maxal\t$maxalgs\t$pdenovo1%\t$prare1%\t$prare2%\t$prare3%\t$pthree%\t$ptwo%\t$pone%\n";
			}
			else {
				print OUT "=HYPERLINK(\"http://genome.ucsc.edu/cgi-bin/hgTracks?clade=mammal&org=Human&db=hg18&position=$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\", \"$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\")\t$badchip\t$tab[$genplace]\t$tab[$phenoplace]\t$tab[$famsize]\t$tab[$famID]\t$tab[$sampleID]\t$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$cnvtype]\t$tab[$maxsize]\t$tab[$maxsnp]\t$tab[$lsize]\t$maxal\t$maxalgs\t$pdenovo1%\t$prare1%\t$pthree%\t$ptwo%\t$pone%\n";
			}
		}
		else{
			if ($smallpheno eq "Case"
			and $badchip != 1){
				print BEDOUT1 "$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$sampleID]_$tab[$famID]_$tab[$maxsnp]_" . $maxal . "_" . $smallpheno . "\t$maxal\t$strand\t$thickstart\t$thickend\t$colour\n";
			}
			elsif ($badchip != 1){
				print BEDOUT2 "$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$sampleID]_$tab[$famID]_$tab[$maxsnp]_" . $maxal . "_" . $smallpheno . "\t$maxal\t$strand\t$thickstart\t$thickend\t$colour\n";
			}
			if ($method == 1){
				print OUT "=HYPERLINK(\"http://genome.ucsc.edu/cgi-bin/hgTracks?clade=mammal&org=Human&db=hg18&position=$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\", \"$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\")\t$badchip\t$tab[$genplace]\t$tab[$phenoplace]\t$tab[$famsize]\t$tab[$famID]\t$tab[$sampleID]\t$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$cnvtype]\t$tab[$maxsize]\t$tab[$maxsnp]\t$tab[$lsize]\t$maxal\t$maxalgs\t$pdenovo1%\t$prare1%\t$prare2%\t$prare3%\t$pthree%\t$ptwo%\t$pone%\n";
			}
			else {
				print OUT "=HYPERLINK(\"http://genome.ucsc.edu/cgi-bin/hgTracks?clade=mammal&org=Human&db=hg18&position=$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\", \"$tab[$chromo]:$tab[$lstart]-$tab[$lstop]\")\t$badchip\t$tab[$genplace]\t$tab[$phenoplace]\t$tab[$famsize]\t$tab[$famID]\t$tab[$sampleID]\t$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$cnvtype]\t$tab[$maxsize]\t$tab[$maxsnp]\t$tab[$lsize]\t$maxal\t$maxalgs\t$pdenovo1%\t$prare1%\t$pthree%\t$ptwo%\t$pone%\n";
			}
		}
		if ($javamethod == 0){print "\rDoing number $locicountx";}
	}
	print "\n";
	if ($method == 0){
		#Opens the Sample List file to add the QT calls
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tInquire Loci\n";
		shift @samplist;
	
		$timeanddate = &showtime;
		$totalsamples = keys %samplelocicount;
		#print "Total samples: $totalsamples\n";

		foreach $line (@samplist){
			@tab = split (/\t/,$line);
			$lociout = $samplelocicount{$tab[0]};
			
			if ($lociout ne ""){
				$samplecounter++;
				print SAMP "$line\t$lociout\n";
			}
			else {
				print SAMP "$line\t0\n";
				print LOG1 "$timeanddate\tInquire\tERROR\tNo loci present for $tab[0]\n";
				print "ERROR: No loci present for $tab[0]\n";
			}
		}
		if ($samplecounter == $totalsamples){
			print LOG1 "$timeanddate\tInquire\tINFO\tSummary track complete for $totalsamples samples\n";
			print "Summary track complete for $totalsamples samples\n";
		}
		else {
			print LOG1 "$timeanddate\tInquire\tERROR\tSummary track complete, however loci were present for samples that were not listed in the Samplelist file\n";
			print "ERROR: Summary track complete, however loci were present for samples that were not listed in the Samplelist file, $totalsamples, $samplecounter\n";
		}
	}
	else {
		print "Summary track complete\n";
	}
}

#Makes easily readable summaries of the merge file output: submit @_ with name of merge file, number of extra columns
if ($smallinquire == 1){
	@input = ($ARGV[0],0);
	&smallinquire(@input);
}

#Move Murim's results file to the correct folder
if ($moveMU == 1){
	use File::Copy;
	chdir ("$MUloc");
	$counter = 0;
	@files=<*>;
	foreach (@files){
		if(($_ =~ /DELs/ and /CNVlist/)
		or ($_ =~ /DUPs/ and /CNVlist/)){
			$counter++;
			$file_location = ($MUloc . $_);
			$newfile= ($res_dir . $_);
			move ($file_location, $newfile); 
		}
	}
	print "$counter results files moved from $MUloc to $res_dir\n";
	print LOG1 "moveMU\t$counter results files moved from $MUloc to $res_dir\n";
	if ($counter == 0){
		print "ERROR: No results files moved from Murim algorithm\n";
		print LOG1 "moveMU\tERROR: No results files moved from Murim algorithm\n";
	}
}

#Formats the output of Murim's CNV algorithm
if ($MUformat == 1){
	use File::Copy;
	chdir $res_dir;
	
	# List of Murim columns
	my $id_number_col = 0;
	my $cnv_id_col = 2;
	my $num_snps_col = 3;
	my $chr_col = 4;
	my $start_col = 5;
	my $end_col  = 6;
	my $length_col = 7;
	my $pvalue_col = 9;
	
	# This number describes the number of extra rows (the first 3 right now) in Murim's data file that need to be cut out before data processing.
	my $num_extra_rows = 3;
	
	# Main program loop = opens all files and processes each using subroutines
	my @files = <*>;
	
	$output = $pro_dir . "MU_" . $tag . "_Merge.txt";
	open (OUTPUT, ">$output" || die "couldn't open $output\n");
	
	foreach (@files){
		if ($_ =~ /DEL/ and /CNVlist/) {
			process_file($_);
			print "processed $_\n";
		} elsif ($_ =~ /DUP/ and /CNVlist/){
			process_file($_);
			print "processed $_\n";
		}
	}
	
	
	# This subroutine processes a single file
	sub process_file {
		my($filename) = @_;
		
		# Open file and put contents into an array - each row is a line in the array
		open(FILENAME, "<$filename" || die "couldn't open $filename\n");
		my(@full_file) = <FILENAME>; close FILENAME;
		
		# Eliminate useless rows of Murim's output file.
		splice(@full_file,0,$num_extra_rows);
		
		# This code loops through each line of the old file.  It takes each line and creates an array with individual elements for each column.
		my $line;
		foreach $line (@full_file) {
			my @one_line = split ("\t", $line);
			chomp @one_line;
			process_line(@one_line);
		}
	}
	
	# This subroutine processes a single line of a file and inserts a newline character at the end.
	sub process_line {
		my(@oldline) = @_;
		$oldline[$chr_col] =~ s/\s//g; #removes white space from chromosome column
		
		# This set of code changes the CNV id to a number
		if ( $oldline[$cnv_id_col] =~ /Het/ ) {
			$oldline[$cnv_id_col] = 1;
		} elsif ( $oldline[$cnv_id_col] =~ /Hom/ ) {
				$oldline[$cnv_id_col] = 0;
		} elsif ( $oldline[$cnv_id_col] =~ /Dup/ ) {
				$oldline[$cnv_id_col] = 3;
		} else {
			print "error: $oldline[$cnv_id_col] is not a valid CNV id\n";
		}
		# This code prints the single line back to the OUTPUT file and moves it to the next line
		print OUTPUT "NA1000\t$oldline[$id_number_col]\t$oldline[$chr_col]\t$oldline[$start_col]\t$oldline[$end_col]\t$oldline[$cnv_id_col]\t$oldline[$pvalue_col]\t$oldline[$num_snps_col]\t$oldline[$length_col]\n";
		$sampcount{$oldline[$id_number_col]} = 0;
	}
	chdir $curr_dir;
	@sampcount = keys %sampcount;
	$sampcount = @sampcount;
	print "$sampcount samples reformatted from Murim results\n";
	print LOG1 "MUformat\t$sampcount samples reformatted from Murim results\n";
	if ($samplecount == 0){
		print "ERROR: No samples found in Murim results file\n";
		print LOG1 "MUformat\tERROR: No samples found in Murim results file\n";
	}
}

#Finds homozygous deletions in Final Reports
if ($homodelfind == 1){
	if ($tag eq ""){
		$tag = "M";
		$method = 1;
		$pro_dir = "";
		$this = "Homozygous_deletions.txt";
		open (OUT, ">$this") || die "Could not open the results file $this; it may be open\n";
		@files = <*>;
		print LISTOUT $curr_dir."$this\n";
	}
	elsif ($processors <= 1) {
		$method = 0;
		$this = $res_dir.$tag."_Homozygous_deletions.txt";
		open (OUT, ">$this") || die "Could not open the results file $this; it may be open\n";
		print LOG1 "$timeanddate\tHomodel\tINFO\tHomodel detection started\n";
		@files = <*>;
	}
	else {
		@files = ();
		$prolist = $ARGV[0];
		$pro = $prolist;
		$remove1 = $tag . "_Processor_list_";
		$pro =~ s/$remove1//gi;
		$pro =~ s/.txt//gi;
		$hdresults = $res_dir.$tag."_Homozygous_deletions_".$pro.".txt";
		print "Opening list of files $prolist\n";
		open (IN, "<$prolist") || die "Could not find $prolist; check it has been created\n";
		open (OUT, ">$hdresults") || "Could not open $hdresults; may already be open\n";
		while (<IN>){
			chomp $_;
			@tab = split (/\t/,$_);
			push (@files, $tab[0]);
		}
	}
	print OUT "Sample ID\tChr\tStart\tStop\trsStart\trsStop\tSize\tProbes\tFlank start\tFlank stop\tFlank rsStart\tFlank rsStop\tMean LogR\tLogRs\n";
	&homodelsub(@files);
}

#Finds homozygous deletions in Final Reports using multiple processors
if ($homodelfind_batch == 1){
	print LOG1 "$timeanddate\tHomodel\tINFO\tFinding Homozygous deletions in samples\n";
	$progloc = '"'.$JVloc.$progname.'"';
	if ($processors <= 1){
		$command1 = "perl " . $progloc . " --homodel\n";
		system ($command1);
	}
	else{
		foreach $pro (1..$processors){
			$countfile = $tag . "_Count_hd_" . $pro . ".txt";
			open (OUT, ">$countfile")|| die "Could not open counter file $countfile; may already be open\n";
			close OUT;
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			push (@batchlist, $batchname);
			open (OUTHERE, ">$batchname") || die "Could not open $batchname; may already be open\n";
			print "Opening homodel window $pro at $timeanddate\n";
			$command1 = "perl " . $progloc . " --homodel " . $tag . "_Processor_list_" . $pro . ".txt";
			$command2 = "perl " . $progloc . " --delete ".$countfile;
			print OUTHERE "$command1\n$command2\nexit\n";
			system ("start $batchname");
			#sleep (1);
		}
		print "Waiting for multiple windows to complete, this may take some time...\n";
		$marker = 0; $lastwindowcount = -1;
		do{
			$windowcount = 0; 
			foreach $pro (1..$processors){
				$countfile = $tag . "_Count_hd_" . $pro . ".txt";
				#print "Looking for $countfile\n";
				unless (-e $countfile){
					#print "Found that $countfile is not present\n";
					$windowcount++;
				}
			}
			if ($lastwindowcount != $windowcount){
				print "$windowcount windows complete\n";
			}
			if ($windowcount == $processors){
				$marker = 1;
			}
			$lastwindowcount = $windowcount;
			sleep (2);
		} until ($marker == 1);
		
		$outputname = $res_dir.$tag."_Homozygous_deletions.txt";
		open (OUT, ">$outputname") || die "Could not open the results file $outputname, may already be open\n";
		print OUT "Sample ID\tChr\tStart\tStop\trsStart\trsStop\tSize\tProbes\tFlank start\tFlank stop\tFlank rsStart\tFlank rsStop\tMean LogR\tLogRs\n";
		
		foreach $pro (1..$processors){
			$batchname = $tag . "_Batch_" . $pro . ".bat";
			#The separate Name_ID_sex_ files are combined
			$hodelfile = $res_dir.$tag."_Homozygous_deletions_".$pro.".txt";
			open (INNER, $hodelfile) || die "Could not find file $hodelfile; check the batch file ran correctly\n";
			@lines = <INNER>; chomp @lines; close INNER; shift @lines;
			foreach $line (@lines){
				chomp $line;
				@tab = split (/\t/,$line);
				$homolog{$tab[0]} = $homolog{$tab[0]} + 1;
				push (@outputarray,$line);
			}
			unlink $hodelfile;
			unlink $batchname;
		}
		foreach (@outputarray){
			print OUT "$_\n";
		}
		close OUT;
		print "\n";
		
		$timeanddate = &showtime;
		@samplespresent = keys %homolog;
		$totalsamp = @samplespresent;
		print LOG1 "$timeanddate\tHomodel\tINFO\tHomozygous deletions detected for $totalsamp individuals\n";
		print "Homozygous deletions detected for $totalsamp individuals\n";
		
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tHomodels\n";
		shift @samplist;
		$timeanddate = &showtime;
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$hd = $homolog{$tab[0]};
			if ($hd > 0){
				print SAMP "$_\t$hd\n";
			}
			else {
				print SAMP "$_\t0\n";
				print LOG1 "$timeanddate\tHomodel\tERROR\tNo homozygous deletions were detected for sample $_\n";
				print "ERROR: No homozygous deletions were detected for sample $_\n";
			}
		}
	}
}

#This algorithm processes the variation file downloaded from the DGV website (http://projects.tcag.ca/variation/tableview.asp?table=DGV_Content_Summary.txt); it makes several output files: A summary of the DGV and controls, a frequency file for all CNVs, dels and dups and a Genome Browser file for these three combined together.
if ($freq_dgv == 1){
	#perl $progname --freqdgv Name of DGV file (standard DGV download format)
	print "Loading input file $ARGV[0]\n";
	open (INPUT, "<$ARGV[0]") || die "couldn't open file $ARGV[0]\n";
	print "Uploading target regions\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@CNVall=split (/\r\n|\r|\n/, $input1); chomp @CNVall;
	#@CNVall = <INPUT>; chomp @CNVall; close (INPUT);
	
	#Open merged CNV results file
	$output = "Freq_All_DGV_" . "$ARGV[0]";
	open (FOUT, ">$output") || die "Cannot open the file $output; may already be open";
	
	#Open DGVsum file
	$output2 = "DGV_summary_"  . "$ARGV[0]";
	open (DGVSUM, ">$output2") || die "Cannot open the file $output2; may already be open";
	print DGVSUM "Paper\tMethod\tControls\tNumber of controls\tHapmap?\tTotal Dups\tTotal Dels\tTotal All\tDistinct\tInclude for freq?\tDups in freq\tDels in freq\tAll in freq\tDistinct in freq\tInclude for Dups?\tDups\tDistinct Dups\tInclude for Dels?\tDels\tDistinct Dels\n";
	print LISTOUT $curr_dir."$output2\n".$curr_dir."$output\n";
	
	#Open BEDgraph file
	$output = "BED_"  . "$ARGV[0]";
	$output =~ s/.txt/.bed/gi;
	open (BED, ">$output") || die "Cannot open the file $output; may already be open";
	
	shift (@CNVall);
	
	$sampleID = 0;
	$chromo = 2;
	$start = 3;
	$stop = 4;
	$paper = 9;
	$method = 11;
	$dupinput = 12;
	$delinput = 13;
	$dgvinput = 14;
	$controls = 16;
	
	#A hash is made looking for distinct papers, methods and control populations
	foreach (@CNVall){
		@tab = split (/\t/,$_);
		$studyuid = "$tab[$paper]\t$tab[$method]\t$tab[$controls]";
		$studyid{$studyuid} = $tab[$controls];
		$distinctcnv{$studyuid} = $distinctcnv{$studyuid} + 1;
		$totalcnv{$studyuid} = $totalcnv{$studyuid} + $tab[$dgvinput];
		$delcnv{$studyuid} = $delcnv{$studyuid} + $tab[$delinput];
		$dupcnv{$studyuid} = $dupcnv{$studyuid} + $tab[$dupinput];
		if ($tab[$dgvinput] > 0){
			push (@cnvs, $_);
			$distinctcnvu{$studyuid} = $distinctcnvu{$studyuid} + 1;
			$totalcnvu{$studyuid} = $totalcnvu{$studyuid} + $tab[$dgvinput];
			$delcnvu{$studyuid} = $delcnvu{$studyuid} + $tab[$delinput];
			$dupcnvu{$studyuid} = $dupcnvu{$studyuid} + $tab[$dupinput];
			$include{$studyuid} = "Yes";
			if ($tab[$dupinput] > 0){
				push (@dupcnvs, $_);
				$dupinclude{$studyuid} = "Yes";
				$dupcnvdu{$studyuid} = $dupcnvdu{$studyuid} + $tab[$dupinput];
				$distinctcnvdu{$studyuid} = $distinctcnvdu{$studyuid} + 1;
			}
			if ($tab[$delinput] > 0){
				push (@delcnvs, $_);
				$delinclude{$studyuid} = "Yes";
				$delcnvde{$studyuid} = $delcnvde{$studyuid} + $tab[$delinput];
				$distinctcnvde{$studyuid} = $distinctcnvde{$studyuid} + 1;
			}
		}
	}
	@studies = keys %studyid;
	
	foreach (@studies){
		@tab = split (/ /,$studyid{$_});
		#There is one study where the controls column does not start with a 1, instead it is ABC
		if ($tab[0] =~ /ABC10_Yoruba/){
			$count = 1;
			$total++;
		}
		else {
			$count = $tab[0];
			$total += $count;
		}
		
		#Here the DGV papers are split into those that do not have Hapmap samples and those that do
		if ($_ =~ /Hapmap/i or $_ =~ /\(NA/ or $_ =~ /ABC10_Yoruba/){
			$hap = "Yes";
			$hapmap{$_} = 1;
		}
		else {
			$hap = "No";
			$hapmap{$_} = 0;
			$nohap += $count;
		}
		#Is there data on numbers of people with the CNV
		if ($include{$_} eq "Yes"){
			$include = "Yes";
			$allinc += $count;
		}
		else {
			$include = "No";
		}
		#Is there data on numbers of people with the duplication
		if ($dupinclude{$_} eq "Yes"){
			$dupinclude = "Yes";
			$dupinc += $count;
		}
		else {
			$dupinclude = "No";
		}
		#Is there data on numbers of people with the deletion
		if ($delinclude{$_} eq "Yes"){
			$delinclude = "Yes";
			$delinc += $count;
		}
		else {
			$delinclude = "No";
		}
		
		print DGVSUM "$_\t$count\t$hap\t$dupcnv{$_}\t$delcnv{$_}\t$totalcnv{$_}\t$distinctcnv{$_}\t$include\t$dupcnvu{$_}\t$delcnvu{$_}\t$totalcnvu{$_}\t$distinctcnvu{$_}\t$dupinclude\t$dupcnvdu{$_}\t$distinctcnvdu{$_}\t$delinclude\t$delcnvde{$_}\t$distinctcnvde{$_}\n";
		$duptot += $dupcnv{$_}; $deltot += $delcnv{$_}; $tottot += $totalcnv{$_}; $distot += $distinctcnv{$_}; 
		$duptotu += $dupcnvu{$_}; $deltotu += $delcnvu{$_}; $tottotu += $totalcnvu{$_}; $distotu += $distinctcnvu{$_};
		$duptotdu += $dupcnvdu{$_}; $distotdu += $distinctcnvdu{$_}; $deltotde += $delcnvde{$_}; $distotde += $distinctcnvde{$_};
	}
	print DGVSUM "\t\t\t$total\t\t$duptot\t$deltot\t$tottot\t$distot\t\t$duptotu\t$deltotu\t$tottotu\t$distotu\t\t$duptotdu\t$distotdu\t\t$deltotde\t$distotde\n\n\n";
	print DGVSUM "Numbers of controls\nAll\t$total\nAll for Freq\t$allinc\nDups for Freq\t$dupinc\nDels for Freq\t$delinc\nNo Hapmap\t$nohap\n";
	$total = $allinc;
	print BED "track type=bedGraph name=\"DGV_All\" description='Population Frequency plot of CNVs in the DGV from $total controls' visibility=full autoScale=off yLineMark=0 yLineOnOff=on color=0,0,150 priority=10 viewLimits=0:5\n";
	print "\nChecking frequency of all CNVs in the DGV\n";
	&dgvfreqsub (@cnvs);
	
	#The whole subroutine is repeated for the duplications (modifying the input array, column to examine and number of controls
	close FOUT;
	print "\nChecking frequency of all duplications in the DGV\n";
	$output = "Freq_Dups_DGV_" . "$ARGV[0]";
	open (FOUT, ">$output") || die "Cannot open the file $output; may already be open";
	print LISTOUT $curr_dir."$output\n";
	$dgvinput = $dupinput;
	$total = $dupinc;
	print BED "track type=bedGraph name=\"DGV_Dups\" description='Population Frequency plot of Duplications in the DGV from $total controls' visibility=full autoScale=off yLineMark=0 yLineOnOff=on color=0,150,0 priority=10 viewLimits=0:5\n";
	&dgvfreqsub (@dupcnvs);
	
	#The whole subroutine is repeated for the deletions (modifying the input array, column to examine and number of controls
	close FOUT;
	print "\nChecking frequency of all deletions in the DGV\n";
	$output = "Freq_Dels_DGV_" . "$ARGV[0]";
	open (FOUT, ">$output") || die "Cannot open the file $output; may already be open";
	print LISTOUT $curr_dir."$output\n";
	$dgvinput = $delinput;
	$total = $delinc;
	print BED "track type=bedGraph name=\"DGV_Dels\" description='Population Frequency plot of Deletions in the DGV from $total controls' visibility=full autoScale=off yLineMark=0 yLineOnOff=on color=150,0,0 priority=10 viewLimits=0:5\n";
	&dgvfreqsub (@delcnvs);
	
	print "Frequency calculation complete\n";
}

#Determines the frequency of CNVs at each loci in a file
if ($freqfind == 1){
	print "Loading input file $ARGV[0]\n\nThe start of the file header is:\n";
	open (INPUT, "<$ARGV[0]") || die "couldn't open file $ARGV[0]\n";
	print "Uploading target regions\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@CNVall=split (/\r\n|\r|\n/, $input1); chomp @CNVall;
	#@CNVall = <INPUT>; chomp @CNVall; close (INPUT);
	
	#Open merged CNV results file
	$output = "Freq" . "_" . "$ARGV[0]";
	open (FOUT, ">$output") || die "Cannot open the file $output; may already be open";
	print LISTOUT $curr_dir."$output\n";
	
	@lineshow = split ("\t", $CNVall[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	shift (@CNVall);
	
	print "Which column is the sampleID information in (1, 2, 3 etc)?\n";
	$user_samp = <STDIN>; chomp $user_samp; $sampleID = $user_samp - 1;
	print "Which column is the chromosome in (1, 2, 3 etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3 etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3 etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	
	if ($thres == 1){
		$numsnpcol  = $stop + 3;
		$algonumcol = $stop + 5;
		$cnvtypecol = $stop + 1;
		$numsnpcol1  = $numsnpcol + 1;
		$algonumcol1 = $algonumcol + 1;
		$cnvtypecol1 = $cnvtypecol + 1;
		
		print "\nAre these columns correct ('y' or 'n')?\nNumber of SNPs: $numsnpcol1, Type of CNV: $cnvtypecol1, Number of algorithms: $algonumcol1\n";
		$user_col = <STDIN>; chomp $user_col; $colans = $user_col;
		
		if ($colans eq "n"){
			print "Which column is the number of SNPs in (1, 2, 3 etc)?\n";
			$user_samp = <STDIN>; chomp $user_samp; $numsnpcol = $user_samp - 1;
			print "Which column is the number of algorithms in (1, 2, 3 etc)?\n";
			$user_chromo = <STDIN>; chomp $user_chromo; $algonumcol = $user_chromo - 1;
			print "Which column is the type of CNV in (1, 2, 3 etc)?\n";
			$user_start = <STDIN>; chomp $user_start; $cnvtypecol = $user_start - 1; 
		}
	}
	
	&freqsub (@CNVall);
	
	#print "\n$error errors found\n";
	print "Frequency calculation complete\n";
}

#Converts genomic regions to non-overlapping loci
if ($locifind == 1){

	#open the reference input file
	$input = $ARGV[0];
	print "Opening input file: $input\n";
	open (LOCI, "<$input") || die "Could not open file $input; check it is in the right directory\n";
	@loci = <LOCI>; chomp @loci; close (LOCI);
	@output = split (/\\/,$input);
	@output2 = split (/\//,$output[-1]);
	#$output =~ s/$curr_dir//gi;
	$output = "Loci_".$output2[-1];
	open (OUT, ">$output") || die "Could not open file $output; may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	
	@lineshow = split ("\t", $loci[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	#shift (@loci);
	
	########################################################
	#Merge prog Output file
	print "\nWhich column is the chromosome number in (0, 1, 2, etc)?\n";
	$user_chrome = <STDIN>; chomp $user_chrome; $chromo = $user_chrome - 1;
	print "\nWhich column is the region start co-ordinate in (0, 1, 2, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	print "\nWhich column is the region end co-ordinate in (0, 1, 2, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	
	print "$lineshow[$chromo]\t$lineshow[$start]\t$lineshow[$stop]\n";
	
	print "Sorting file by chromosome and number\n";
	#Sort by Start (numbers)
	@loci = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @loci;
	
	#Sort by Chromosome (string)
	@loci = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @loci;
	
	#print @loci;
	print "Looking for loci\n";
	$tracker = 0;
	
	foreach (@loci){
		@before = split ("\t", $loci[$tracker - 1]);
		@now = split ("\t", $loci[$tracker]);
		@after = split ("\t", $loci[$tracker + 1]);
		$tracker++;
		
		#If this CNV is from a differet chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			push (@match, $_);
		}
		
		#If it is the same chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $_);
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			
			#Process @match
			$count++;
			%bps = ();
			@bps = ();
			foreach (@match){
				@bp = split ("\t", $_);
				$bps{$bp[$start]} = 0;
				$bps{$bp[$stop]} = 0;
				$chrm = $bp[$chromo];
			}
			@bps = keys %bps;
			@bps = sort {$a <=> $b} @bps;
			$st = $bps[0];
			$en = $bps[-1];
			
			$out = $chrm."\t".$st."\t".$en."\n";
			if ($javamethod == 0){
				print "\rFound $count loci";
			}
			print OUT $out;
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	print "\nEverything is ok\n";
}

#Converts genomic regions to non-overlapping loci
if ($loci_annotate == 1){

	#open the reference input file
	$input = $ARGV[0];
	print "Opening input file: $input\n";
	open (LOCI, "<$input") || die "Could not open file $input; check it is in the right directory\n";
	@loci = <LOCI>; chomp @loci; close (LOCI);
	@output = split (/\\/,$input);
	@output2 = split (/\//,$output[-1]);
	#$output =~ s/$curr_dir//gi;
	$output = "LociA_".$output2[-1];
	open (OUT, ">$output") || die "Could not open file $output; may already be open\n";
	print OUT "Region\tChr\tStart\tStop\n";
	print LISTOUT $curr_dir."$output\n";
	
	@lineshow = split ("\t", $loci[0]);
	# foreach (@lineshow){
		# $linecount++;
		# print "$linecount) $_\n";
	# }
	shift (@loci);
	
	########################################################
	#Merge prog Output file
	# print "\nWhich column is the chromosome number in (0, 1, 2, etc)?\n";
	# $user_chrome = <STDIN>; chomp $user_chrome; $chromo = $user_chrome - 1;
	# print "\nWhich column is the region start co-ordinate in (0, 1, 2, etc)?\n";
	# $user_start = <STDIN>; chomp $user_start; $start = $user_start - 1;
	# print "\nWhich column is the region end co-ordinate in (0, 1, 2, etc)?\n";
	# $user_stop = <STDIN>; chomp $user_stop; $stop = $user_stop - 1;
	
	$chromo = 0; $start = 1; $stop = 2;
	
	print "$lineshow[$chromo]\t$lineshow[$start]\t$lineshow[$stop]\n";
	
	print "Sorting file by chromosome and number\n";
	#Sort by Start (numbers)
	@loci = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @loci;
	
	#Sort by Chromosome (string)
	@loci = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @loci;
	
	#print @loci;
	print "Looking for loci\n";
	$tracker = 0;
	
	foreach (@loci){
		@before = split ("\t", $loci[$tracker - 1]);
		@now = split ("\t", $loci[$tracker]);
		@after = split ("\t", $loci[$tracker + 1]);
		$tracker++;
		
		#If this CNV is from a differet chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			push (@match, $_);
		}
		
		#If it is the same chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $_);
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			
			#Process @match
			$count++;
			%bps = ();
			@bps = ();
			foreach (@match){
				@bp = split ("\t", $_);
				$bps{$bp[$start]} = 0;
				$bps{$bp[$stop]} = 0;
				$chrm = $bp[$chromo];
			}
			@bps = keys %bps;
			@bps = sort {$a <=> $b} @bps;
			$st = $bps[0];
			$en = $bps[-1];
			
			$out = $chrm."\t".$st."\t".$en."\n";
			if ($javamethod == 0){
				print "\rFound $count loci";
			}
			$printcount++;
			print OUT "Region".$printcount."\t".$out;
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	print "\nEverything is ok\n";
}


#Makes a list of every file or directory in the directory
if ($filelistprog == 1){
	@files=<*>;
	#print "Filelist\n@files\n";
	$outfile = "File_list.txt";
	open (OUT, ">$outfile") || die "Could not open file $outfile\n";
	print LISTOUT $curr_dir."$outfile\n";
	
	foreach $file (@files){
	print OUT "$file\n";
	}
	$filecount = @files;
	print "\n$filecount files listed in File_list.txt\n";
}

#Moves files on a list from one directory to another
if ($movestuff == 1){
	use Cwd;
	use File::Copy;
	
	$file1 = "$ARGV[0]";
	open (LIST, "<$file1" || die "Cannot open $file, may be in the wrong directory\n");
	@filelist = <LIST>; chomp @filelist; close LIST;
	print "\nWhere do you want to move the files? Either enter a folder within this folder:\ne.g. Moved_files\nOr enter a competely different folder:\ne.g. C:/Users/ACCUMBENS/Documents/Moved_files\nIf the folder is not already there it will be created\n";
	$dir = <STDIN>; chomp $dir;
	$dir = $dir . "/";
	if ($dir !~ /:/){
		$dir = $curr_dir . $dir;
	}
	mkdir "$dir";
	
	foreach $file (@filelist){
	$list{$file} = 1;
	}
	
	$number = keys %list;
	print "List contains $number files\n";
	$filecount = 0;
	
	@files=<*>;
	foreach $file2 (@files){
		if ($list{$file2} == 1){
			$filecount++;
			$newfile = $dir . "/" . $file2;
			move ($file2, $newfile);
			if (($filecount - $lastfilecount) >= 50){
				$timeanddate = &showtime;
				print "Moved $filecount files by $timeanddate\n";
				$lastfilecount = $filecount;
			}
		}
	}
	$missed = $number - $filecount;
	print "Moved $filecount out of $number files so $missed were not present\n";
	
	print "\nAll done\nLook in the folder at $dir\n";
	print LISTOUT "$dir\n";
}

#Copies files on a list from one directory to another
if ($copystuff == 1){
	use Cwd;
	use File::Copy;
	
	$file1 = "$ARGV[0]";
	open (LIST, "<$file1" || die "Cannot open $file, may be in the wrong directory\n");
	@filelist = <LIST>; chomp @filelist; close LIST;
	print "\nWhere do you want to copy the files? Either enter a folder within this folder:\ne.g. Copied_files\nOr enter a competely different folder:\ne.g. C:/Users/ACCUMBENS/Documents/Moved_files\nIf the folder is not already there it will be created\n";
	$dir = <STDIN>; chomp $dir;
	$dir = $dir . "/";
	if ($dir !~ /:/){
		$dir = $curr_dir . $dir;
	}
	mkdir "$dir";
	
	foreach $file (@filelist){
		chomp $file;
		$list{$file} = 1;
	}
	
	$number = keys %list;
	print "List contains $number files\n";
	$filecount = 0;
	
	@files=<*>;
	foreach $file2 (@files){
		chomp $file2;
		if ($list{$file2} == 1){
			$filecount++;
			$newfile = $dir . "/" . $file2;
			#print "Newfile: $newfile\n";
			copy ($file2, $newfile);
			if (($filecount - $lastfilecount) >= 50){
				$timeanddate = &showtime;
				print "Copied $filecount files by $timeanddate\n";
				$lastfilecount = $filecount;
			}
		}
	}
	$missed = $number - $filecount;
	print "Copied $filecount out of $number files so $missed were not present\n";
	
	print "\nAll done\nLook in the folder at $dir\n";
	print LISTOUT "$dir\n";
}

#Looks for SNPs that are present in two or more FinalReports
if ($fr_consensus == 1){
	#This program works out the consensus SNPs from a group of FinalReports
	#perl Combined_CNV.pl --fr_consensus
	#The output is a list of SNP names found in all the FinalReports in a file called Consensus.txt
	
	#FinalReport file
	&fr_columns;
	
	$filecount = 0;
	
	@files = <*>;
	
	foreach $file (@files){
		$count = 0;
		if ($file =~ /$namematch/){
			$filecount++;
			open (NEWFILE, "<$file" || die "Could not open $namematch file $file\n");
			@data = <NEWFILE>; chomp @data; close (NEWFILE);
			@temp = split("\t", $data[4]);
			$chip = &chiper($temp[1]);
			$chipout{$chip} = 0;
			#print "Chip is $chip, Chipout is $chipout\n";
			@random = split ("\t", $data[30]);
			chomp @random;
			$sample = $random[$patname];
			print "Sample: $sample\n";
			
			foreach $line (@data){
				@bits = split ("\t", $line);
				chomp @bits;
				#If it is a line with the sample name in then the snpname is put in as the key of the hash %rs with the value increased by one
				if ($bits[$patname] == $sample){
					#print "RS: $bits[$snpcol]: $rs{$bits[$snpcol]}\n";
					$rs{$bits[$snpcol]} = $rs{$bits[$snpcol]} + 1;
					#print "RS: $bits[$snpcol]: $rs{$bits[$snpcol]}\n";
					$count++;
				}
			}
			print "File: $file, SNPs: $count\n";
		}
	}
	@allchips = keys %chipout;
	$chipout = join ("_",@allchips);
	$namehere = $chipout."_Consensus.txt";
	open (OUT, ">$namehere") || die "Could not open the results file $namehere; it may be open\n";
	print LISTOUT $curr_dir."$namehere\n";
	
	@rsname = keys %rs;
	
	#If the value of the snpname is the same as the number of FinalReports it is put into the output file
	foreach (@rsname){
		if ($javamethod == 0){print "\r$count2";}
		if ($rs{$_} == $filecount){
			print OUT "$_\n";
			$consensus++;
		}
	}
	print "Consensus file has $consensus SNPs\n";
	print "All done\n";
}

#Finds CNVs in other FinalReports that have already been found in cases
if ($e_taqman == 1){
	#FinalReport file
	&fr_columns;
	
	use Cwd;
	$cdir = getcwd;
	$curr_dir = $cdir . "/";
	#$timeanddate = &showtime;
	#print "Algo has started at $timeanddate\n";
	#Checks that there is just one pedfile present
	@files = <*>;
	$input2 = "pedfile";

	#$timeanddate = &showtime;
	#print "Pedfile detected at $timeanddate\n";
	$input = $ARGV[0];
	$output = "DN2_" . $input;
	$output2 = "eqPCR_" . $input;
	$output3 = "eqPCR_Raw_" . $input;
	$routput = $curr_dir."06_Plots/".$input."_Copy_into_R.txt";
	print LISTOUT $curr_dir."$output2\n";
	print LISTOUT $curr_dir."$output3\n";
	print "Opening co-ordinates file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input, check it is in the right directory\n";
	if ($verbose == 1){
		open (OUTPUT, ">$output") || die "Cannot open file $output; it may already be open\n";
		print LISTOUT $curr_dir."$output\n";
		print "Opening output file: $output\n";
	}
	open (OUTPUT2, ">$output2") || die "Cannot open file $output2; it may already be open\n";
	open (OUTPUT3, ">$output3") || die "Cannot open file $output3; it may already be open\n";
	if ($plot == 1){
		mkdir ("06_Plots", 0777);
		mkdir ("05_Data", 0777);
		open (ROUT, ">$routput") || die "Cannot open file $routput; it may already be open\n";
		print LISTOUT "$routput\n";
	}
	print "Output files and folders created\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@input=split (/\r\n|\r|\n/, $input1); chomp @input;
	$regions = @input;
	#$timeanddate = &showtime;
	#print "Input file loaded at $timeanddate\n";
	$linecount = 0;
	@lineshow = split ("\t", $input[0]);
	foreach (@lineshow){
		$linecount++;
		unless ($linecount >= 20){
			print "$linecount) $_\n";
		}
	}
	
#	print "\nWhich column is the sampleID data in (1, 2, 3, etc)?\n";
#	$user_sid = <STDIN>; chomp $user_sid; $sampleID = ($user_sid - 1);
	print "Which column is the chromosome data in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = ($user_chromo - 1);
	print "Which column is the start co-ordinate data in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = ($user_start - 1);
	print "Which column is the stop co-ordinate data in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = ($user_stop - 1);
#	print "Which column is the CNV type data in (1, 2, 3, etc)?\n";
#	$user_type = <STDIN>; chomp $user_type; $typecol = ($user_type - 1);
	print "Is there a header ('y' or 'n')?\n";
	$head = <STDIN>; chomp $head;
	
	@colour = (blank, forestgreen, green, darkgreen);
	
	#Adds headers depending on the output to the user quesiton above
	$headers = "Sample\tTypeP\tMean LogRP\tSD LogRP\tMin LogRP\tMax LogRP\tMedian LogRP\tHomoP\tHetP\tDupP\tOtherP";
	if ($head =~ /y/i){
		print OUTPUT "$input[0]\t$headers\n";
		print OUTPUT2 "$input[0]\t";
		print OUTPUT3 "$input[0]\t$headers\n";
		shift @input;
		$regions = $regions - 1;
	}
	else {
		@nohead = split ("\t", $input[0]);
		$tabnum = @nohead;
		@headline = (1..$tabnum);
		$headline = join ("\t", @headline);
		print OUTPUT "$headline\t$headers\n";
		print OUTPUT2 "$headline\t";
		print OUTPUT3 "$headline\t$headers\n";
	}
	
	print "\n$regions regions detected in input file\n";
	
	#Works out family relationships from the pedfile, the corresponding FinalReport for each chip ID and builds chromosome specific arrays of SNPs
	$firstfile = 0;
	foreach (@files){
		$file = $_;
		#Opens the first FinalReport only
		if ($_ =~ /$namematch/ 
		and $firstfile == 0){
			$firstfile = 1;
			open (INPUT, "<$_") || die "Could not open $namematch file $_\n";
			print "Building a SNP template from the $namematch file: $_\n";
			@snp = <INPUT>; chomp @snp; close (INPUT);
			
			@herebit = split ("\t", $snp[20]);
			$samplename = $herebit[0];
	
			#Makes a hash of the chromosome names and puts the SNP positions and oringinal array positions into chromosome specific arrays
			$counter = 0;
			foreach $line (@snp){
				@bits = split ("\t", $line);
				chomp @bits;
				if ($bits[0] eq $samplename){
						$chrom = "chr" . $bits[2];
						$chromlist{"chr" . $bits[2]} = 0;
						$input = "$bits[3]\t$counter";
						push (@$chrom, $input);
				}
				$counter++;
			}
			@snp = ();
		}
		
		#Works out which FinalReport corresponds to which sample
		if ($file =~ /$namematch/){
			open (IN2, "<$file")|| die "Could not open file $file";
			$count2 = 0;
			while (<IN2>){
				$count2++;
				if($count2 == 20){
					@tab = split ("\t", $_);
					$finalrep{$tab[0]} = $file;
					#print "$file\t$tab[0]\n";
					close IN2;
				}
			}
		}
	}
	
	@finalreps = keys %finalrep;
	$finalrepsize = @finalreps;
	print "Worked out FinalReport names for $finalrepsize samples\n";
	
	#Error message if there are no FinalReports present
	if ($firstfile == 0){
		die "There was no FinalReport file to work out how many SNPs there are in each region; please find any Final Report file from the correct type of chip and put it in this folder\n";
	}
	
	print "Sorting chromosome arrays, this may take a while...\n";
	
	#The list of chromosomes is sorted (for neatness...)
	@chromlist = keys %chromlist;
	@chromlist = sort  { $a cmp $b } @chromlist;
	#print @chromlist;
	
	#Then each specific array is sorted by position
	foreach (@chromlist){
		@$_ = sort { (split '\t', $a)[0] <=> (split '\t', $b)[0] } @$_;
		$probes = @$_;
		#print "$_, ";
	}
	print "\nProbes entered into chromosome specific arrays\n";
	
	@sample = @input;
	
	#Goes through each FinalReport in turn
	foreach $member (@finalreps){
		#print "Looking for $member from family $family\n";
		if ($member eq 0){}
		elsif ($member eq ""){}
		else {
			$samplecount++;
			print "$samplecount) $finalrep{$member} $member\n";
			print OUTPUT2 "$member\t";
			$frname = $finalrep{$member};
			open (FR, "<$frname")|| print "FinalReport $frname is missing from this directory\n";
			@fr = <FR>; close FR;
			$regcount = 0;
			foreach $region (@sample){
				#print "New Region\n";
				$wide = 0;
				@region = split ("\t", $region);
				@lines = ();
				@lines = &linefind(@region);
				@poslogbaf = (); 
				#The data from the Final Report is put into the arrays logr and baf with numbering according to region and family member
				$total = 0;
				$totallines = @lines;
				foreach $line (@lines){
					@tab = split ("\t", $fr[$line]); chomp @tab;
					$poslogbaf = "$tab[$chromodist]\t$tab[$log2col]\t$tab[$ballele]";
					#print "$poslogbaf\n";
					push (@poslogbaf, $poslogbaf);
				}
				#Sends an array in the format Position\tLogR\tBAF and gets back:
				#Type\tMeanLogR\tSTdev\tMin\tMax\tMedian\tHomo\tHet\tDup\tOther
				$output_for_reg = &taqprocess(@poslogbaf);
				if ($verbose == 1){
					print OUTPUT "$region\t$member\t$output_for_reg\n";
				}
				@tab = split (/\t/,$output_for_reg);
				$typeonly = $tab[0];
				$keyin = $member."\t".$regcount;
				$finalresult{$keyin} = $typeonly;
				if ($typeonly !~ /2/){
					print OUTPUT3 "$region\t$member\t$output_for_reg\n";
				}
				
				#If the plot setting is enabled then the wider regions are put into the corresponding logrw and bafw arrays
				if ($plot == 1 and $typeonly !~ /2/){
					#print "Making a plot\n";
					$wide = 1;
					@lines = ();
					@lines = &linefind(@region);
					@poslogbaf = (); 
					#The data from the Final Report is put into the array poslogbaf
					$total = 0;
					$totallines = @lines;
					foreach $line (@lines){
						@tab = split ("\t", $fr[$line]); chomp @tab;
						$poslogbaf = "$tab[$chromodist]\t$tab[$log2col]\t$tab[$ballele]\t";
						#print "$poslogbaf\n";
						push (@poslogbaf, $poslogbaf);
					}
					&plotpro(@poslogbaf);
					@lines = ();
				}
				$regcount++;
			}
		}
	}
	
	#The concise output table is written here using a hash called %finalresult with 'sampleID\tRegion count' as a key
	print OUTPUT2 "\n";
	$regcount = 0;
	foreach $region (@sample){
		@resultout = ();
		foreach $sample (@finalreps){
			$keyin = $sample."\t".$regcount;
			push (@resultout, $finalresult{$keyin});
		}
		$outhere = join ("\t",@resultout);
		print OUTPUT2 "$region\t$outhere\n";
		$regcount++;
	}
	print "All done\n";
}

#Determines the parent of origin of de novo CNVs
#1) ARGV[0] = list of de novo regions (same as denovo2 input). Needs to be in the same folder as the pedfile and FinalReports. It extracts BAF data from the trio (only) and makes a file per region in the '05_Data' folder.
#The file 'DeNovo_CNV_list.txt' is written to the same folder as the input file - this will be the input for the R script
#2) Open the 'parent_of_origin.R' script in Notepad and make sure the first three lines point to the right places
#3) Open R, file 'Open script', open 'parent_of_origin.R', Edit 'Run all'
#4) Results should appear in the original folder in a space delimited file called 'Results_deno_orig.txt'. 'm' is mother, 'f' is father
if ($parent_origin == 1){
	#FinalReport file
	&fr_columns;
	
	use Cwd;
	$cdir = getcwd;
	$curr_dir = $cdir . "/";
	#$timeanddate = &showtime;
	#print "Algo has started at $timeanddate\n";
	#Checks that there is just one pedfile present
	@files = <*>;
	$input2 = "pedfile";
	foreach (@files){
		if ($_ =~ /$input2/){
			$pedfilecount++;
			push (@pedfilelist, "$_\n");
		}
	}
	if ($pedfilecount == 0){
		print "ERROR: There is no file with the word 'pedfile' in the filename.\n";
		die "Please correct and restart\n";
	}
	elsif ($pedfilecount >= 2){
		print "ERROR: There is more than one file with the word 'pedfile' in its name.\nPlease remove all but the most recent. Files affected:\n";
		print @pedfilelist;
		die "\nPlease correct and restart\n";
	}
	#$timeanddate = &showtime;
	#print "Pedfile detected at $timeanddate\n";
	$input = $ARGV[0];
	$output = "DeNovo_CNV_list.txt";
	print "Opening co-ordinates file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input, check it is in the right directory\n";
	$output2 = "Parent_of_origin_results.txt";
	open (OUTHERE, ">$output2") or die "Could not open $output2; may already be open\n";
	print "Opening output file: $output\n";
	open (OUTPUT, ">$output") || die "Cannot open file $output; it may already be open\n";
	mkdir ("05_Data", 0777);
	print "Output files and folders created\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@input=split (/\r\n|\r|\n/, $input1); chomp @input; 
	$regions = @input;
	print "There are $regions regions\n";
	#$timeanddate = &showtime;
	#print "Input file loaded at $timeanddate\n";
	$linecount = 0;
	@lineshow = split ("\t", $input[0]);
	foreach (@lineshow){
		$linecount++;
		unless ($linecount >= 20){
			print "$linecount) $_\n";
		}
	}
	print OUTHERE "$input[0]\tLines\tDups\tDels\tHets\tOther\tDupMother\tDupFather\tDelMother\tDelFather\tType\tParentOfOrigin\tConfidence\n";

	#$sampleID = 2; $chromo = 3; $start = 4; $stop = 5; $typecol = 6; $head = 'y';
	
	print "\nWhich column is the sampleID data in (1, 2, 3, etc)?\n";
	$user_sid = <STDIN>; chomp $user_sid; $sampleID = ($user_sid - 1);
	print "Which column is the chromosome data in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = ($user_chromo - 1);
	print "Which column is the start co-ordinate data in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = ($user_start - 1);
	print "Which column is the stop co-ordinate data in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = ($user_stop - 1);
	print "Which column is the CNV type data in (1, 2, 3, etc)?\n";
	$user_type = <STDIN>; chomp $user_type; $typecol = ($user_type - 1);
	print "Is there a header ('y' or 'n')?\n";
	$head = <STDIN>; chomp $head;
	
	
	if ($head =~ /y/i){
		shift @input;
		$regions = $regions - 1;
	}
	
	print "\n$regions regions detected in input file\n";
	
	#Works out family relationships from the pedfile, the corresponding FinalReport for each chip ID and builds chromosome specific arrays of SNPs
	$firstfile = 0;
	$prechromlist = "Pre_sorted_chromo_array.txt";
	if (-e $prechromlist){
		#print "I've got a prechromlist\n";
		$prechrom = 1;
		$firstfile = 1;
		open (PRECHROM, "$prechromlist") or $firstfile = 0;
		while ($linehere = <PRECHROM>){
			chomp $linehere;
			if ($linehere =~ /%%/){
				$linehere =~ s/%%//gi;
				$thechromhere = $linehere;
				$chromlist{$thechromhere} = 0;
			}
			else {
				push (@$thechromhere, $linehere);
			}
		}
		close PRECHROM;
	}
	
	foreach (@files){
		$file = $_;
		#Opens the first FinalReport only
		if ($_ =~ /$namematch/ 
		and $firstfile == 0){
			$firstfile = 1;
			open (INPUT, "<$_") || die "Could not open $namematch file $_\n";
			print "Building a SNP template from the $namematch file: $_\n";
			@snp = <INPUT>; chomp @snp; close (INPUT);
			
			@herebit = split ("\t", $snp[20]);
			$samplename = $herebit[0];
	
			#Makes a hash of the chromosome names and puts the SNP positions and oringinal array positions into chromosome specific arrays
			$counter = 0;
			foreach $line (@snp){
				@bits = split ("\t", $line);
				chomp @bits;
				if ($bits[0] eq $samplename){
						$chrom = "chr" . $bits[2];
						$chromlist{"chr" . $bits[2]} = 0;
						$input = "$bits[3]\t$counter";
						push (@$chrom, $input);
				}
				$counter++;
			}
			@snp = ();
		}
		
		#Opens the pedfile
		elsif ($_ =~ /$input2/){
			open (IN2, "<$_")|| die "Could not open pedfile $_";
			print "Opening pedfile $_\n";
			{ local $/=undef;  $input3=<IN2>; }
			close IN2;
			($\) = (/(\r\n|\r|\n)/);
			@input2=split (/\r\n|\r|\n/, $input3); chomp @input2; 
			
			#Makes four hashes: %family (family ID of sample), %members (Samples in family), %pheno (phenotype of sample), %gender (gender of sample)
			foreach (@input2){
				@tab = split ("\t", $_);
				$family{$tab[1]} = $tab[0];
				$members{$tab[0]} = $members{$tab[0]} . "$tab[1]\t";
				$familysize{$tab[0]} = $familysize{$tab[0]} + 1;
				# if($tab[4] == 2){
					# $gender{$tab[1]} = "Female";
				# }
				# elsif($tab[4] == 1){
					# $gender{$tab[1]} = "Male";
				# }
				if($tab[5] == 2){
					$pheno{$tab[1]} = "Proband";
					$parents{$tab[1]} = "$tab[2]\t$tab[3]";
				}
				elsif($pheno{$tab[1]} eq ""){
					$pheno{$tab[1]} = "Sibling";
					$parents{$tab[1]} = "$tab[2]\t$tab[3]";
				}
				unless ($tab[2] == 0){$pheno{$tab[2]} = "Father";}
				unless ($tab[3] == 0){$pheno{$tab[3]} = "Mother";}
				#print "Sample: $tab[1], Gender: $gender{$tab[1]}, Family: $family{$tab[1]} Parents: $parents{$tab[1]} Familysize: $familysize{$family{$tab[1]}}\n";
			}
			@phenos = keys %pheno;
			$phenosize = @phenos;
			print "Loaded phenotypic and family data for $phenosize samples\n";
		}
		
		#Works out which FinalReport corresponds to which sample
		if ($file =~ /$namematch/){
			open (IN2, "<$file")|| die "Could not open file $file";
			$count2 = 0;
			while (<IN2>){
				$count2++;
				if($count2 == 20){
					@tab = split ("\t", $_);
					$finalrep{$tab[0]} = $file;
					#print "$file\t$tab[0]\n";
					close IN2;
				}
			}
		}
	}
	
	@finalreps = keys %finalrep;
	$finalrepsize = @finalreps;
	print "Worked out FinalReport names for $finalrepsize samples\n";
	
	#Error message if there are no FinalReports present
	if ($firstfile == 0){
		die "There was no Final Report file to work out how many SNPs there are in each region; please find any Final Report file from the correct type of chip and put it in this folder\n";
	}
	
	#The list of chromosomes is sorted (for neatness...)
	@chromlist = keys %chromlist;
	@chromlist = sort  { $a cmp $b } @chromlist;
	#print @chromlist;
	if ($prechrom != 1){
		print "Sorting chromosome arrays\n";
		open (PRECHROM, ">$prechromlist");
		#Then each specific array is sorted by position
		foreach (@chromlist){
			@$_ = sort { (split '\t', $a)[0] <=> (split '\t', $b)[0] } @$_;
			$probes = @$_;
			#print "$_, ";
			print PRECHROM "%%".$_."\n";
			foreach $line (@$_){
				print PRECHROM "$line\n";
			}
		}
		print "\nProbes entered into chromosome specific arrays, this may take a while...\n";
	}
		
	#Sort the input file by sample 
	print "Sorting the CNV file by sample ID\n";
	@input = sort { (split '\t', $a)[$sampleID] cmp (split '\t', $b)[$sampleID] } @input;
	$counter = 0;
	
	#Splits the file into samples and sends an array per sample to the sampledeal subroutine
	foreach $region (@input){
		@thisone = split ("\t", $input[$counter]);
		@nextone = split ("\t", $input[$counter + 1]);
		push (@sample, $region);
		#print "Reg: $region: This: $thisone[$sampleID], Next: $nextone[$sampleID]\n"; sleep (1);
		if ($thisone[$sampleID] ne $nextone[$sampleID]){
			&sampledeal_parent(@sample);
			@sample = ();
		}
		$counter++;
	}
	print "\nAll done\n";
}

#Finds de novo CNVs, or visualises CNVs or finds further CNVs in the same area for other CNVs
if ($denovo2 == 1 or $plot == 1 and $e_taqman == 0){
	@colours = ("darkgreen", "blue", "hotpink", "darkolivegreen", "forestgreen", "green", "seagreen", "green", "green", "green");
	
	#FinalReport file
	&fr_columns;
	
	use Cwd;
	$cdir = getcwd;
	$curr_dir = $cdir . "/";
	#$timeanddate = &showtime;
	#print "Algo has started at $timeanddate\n";
	#Checks that there is just one pedfile present
	@files = <*>;
	$input2 = "pedfile";
	foreach (@files){
		if ($_ =~ /$input2/){
			$pedfilecount++;
			push (@pedfilelist, "$_\n");
		}
	}
	if ($pedfilecount == 0){
		print "ERROR: There is no file with the word 'pedfile' in the filename.\n";
		die "Please correct and restart\n";
	}
	elsif ($pedfilecount >= 2){
		print "ERROR: There is more than one file with the word 'pedfile' in its name.\nPlease remove all but the most recent. Files affected:\n";
		print @pedfilelist;
		die "\nPlease correct and restart\n";
	}
	#$timeanddate = &showtime;
	#print "Pedfile detected at $timeanddate\n";
	$input = $ARGV[0];
	$output = "DN2_" . $input;
	$routput = $curr_dir."06_Plots/".$input."_Copy_into_R.txt";
	print LISTOUT $curr_dir."$output\n";
	print "Opening co-ordinates file: $input\n";
	open (INPUT, "<$input") || die "Cannot open file $input, check it is in the right directory\n";
	print "Opening output file: $output\n";
	open (OUTPUT, ">$output") || die "Cannot open file $output; it may already be open\n";
	if ($plot == 1){
		mkdir ("06_Plots", 0777);
		mkdir ("05_Data", 0777);
		open (ROUT, ">$routput") || die "Cannot open file $routput; it may already be open\n";
		print LISTOUT "$routput\n";
	}
	print "Output files and folders created\n";
	{ local $/=undef;  $input1=<INPUT>; }
	close INPUT;
	($\) = (/(\r\n|\r|\n)/);
	@input=split (/\r\n|\r|\n/, $input1); chomp @input;
	$regions = @input;
	#$timeanddate = &showtime;
	#print "Input file loaded at $timeanddate\n";
	$linecount = 0;
	@lineshow = split ("\t", $input[0]);
	foreach (@lineshow){
		$linecount++;
		unless ($linecount >= 20){
			print "$linecount) $_\n";
		}
	}
	
	print "\nWhich column is the sampleID data in (1, 2, 3, etc)?\n";
	$user_sid = <STDIN>; chomp $user_sid; $sampleID = ($user_sid - 1);
	print "Which column is the chromosome data in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromo = ($user_chromo - 1);
	print "Which column is the start co-ordinate data in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $start = ($user_start - 1);
	print "Which column is the stop co-ordinate data in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stop = ($user_stop - 1);
	print "Which column is the CNV type data in (1, 2, 3, etc)?\n";
	$user_type = <STDIN>; chomp $user_type; $typecol = ($user_type - 1);
	print "Is there a header ('y' or 'n')?\n";
	$head = <STDIN>; chomp $head;
	
	@colour = (blank, forestgreen, green, darkgreen);
	
	#Adds headers depending on the output to the user quesiton above
	$headers = "Result\tTypeP\tTypeD\tTypeM\tTypeS\t\t%probes similar to sample in Dad\t%probes similar to sample in Mom\t%probes misinherited\t\tMean LogRP\tSD LogRP\tMax LogRP\tMin LogRP\tMedian LogRP\tHomoP\tHetP\tDupP\tOtherP\t\tMean LogRD\tSD LogRD\tMax LogRD\tMin LogRD\tMedian LogRD\tHomoD\tHetD\tDupD\tOtherD\t\tMean LogRM\tSD LogRM\tMax LogRM\tMin LogRM\tMedian LogRM\tHomoM\tHetM\tDupM\tOtherM\t\tMean LogRS\tSD LogRS\tMax LogRS\tMin LogRS\tMedian LogRS\tHomoS\tHetS\tDupS\tOtherS";
	if ($head =~ /y/i){
		print OUTPUT "$input[0]\t$headers\n";
		shift @input;
		$regions = $regions - 1;
	}
	else {
		@nohead = split ("\t", $input[0]);
		$tabnum = @nohead;
		@headline = (1..$tabnum);
		$headline = join ("\t", @headline);
		print OUTPUT "$headline\t$headers\n";
	}
	
	print "\n$regions regions detected in input file\n";
	
	#Works out family relationships from the pedfile, the corresponding FinalReport for each chip ID and builds chromosome specific arrays of SNPs
	$firstfile = 0;
	foreach (@files){
		$file = $_;
		#Opens the first FinalReport only
		if ($_ =~ /$namematch/ 
		and $firstfile == 0){
			$firstfile = 1;
			open (INPUT, "<$_") || die "Could not open $namematch file $_\n";
			print "Building a SNP template from the $namematch file: $_\n";
			@snp = <INPUT>; chomp @snp; close (INPUT);
			
			@herebit = split ("\t", $snp[20]);
			$samplename = $herebit[0];
	
			#Makes a hash of the chromosome names and puts the SNP positions and oringinal array positions into chromosome specific arrays
			$counter = 0;
			foreach $line (@snp){
				@bits = split ("\t", $line);
				chomp @bits;
				if ($bits[0] eq $samplename){
						$chrom = "chr" . $bits[2];
						$chromlist{"chr" . $bits[2]} = 0;
						$input = "$bits[3]\t$counter";
						push (@$chrom, $input);
				}
				$counter++;
			}
			@snp = ();
		}
		
		#Opens the pedfile
		elsif ($_ =~ /$input2/){
			open (IN2, "<$_")|| die "Could not open pedfile $_";
			print "Opening pedfile $_\n";
			@input2 = <IN2>; chomp @input2; close IN2;
			
			#Makes four hashes: %family (family ID of sample), %members (Samples in family), %pheno (phenotype of sample), %gender
			foreach (@input2){
				@tab = split ("\t", $_);
				$family{$tab[1]} = $tab[0];
				$members{$tab[0]} = $members{$tab[0]} . "$tab[1]\t";
				$familysize{$tab[0]} = $familysize{$tab[0]} + 1;
				if($tab[4] == 2){
					$gender{$tab[1]} = "Female";
				}
				elsif($tab[4] == 1){
					$gender{$tab[1]} = "Male";
				}
				if($tab[5] == 2){
					$pheno{$tab[1]} = "Proband";
					$parents{$tab[1]} = "$tab[2]\t$tab[3]";
				}
				elsif($pheno{$tab[1]} eq ""){
					$pheno{$tab[1]} = "Sibling";
					$parents{$tab[1]} = "$tab[2]\t$tab[3]";
				}
				unless ($tab[2] == 0){$pheno{$tab[2]} = "Father";}
				unless ($tab[3] == 0){$pheno{$tab[3]} = "Mother";}
				#print "Sample: $tab[1] Family: $family{$tab[1]} Parents: $parents{$tab[1]} Familysize: $familysize{$family{$tab[1]}}\n";
			}
			@phenos = keys %pheno;
			$phenosize = @phenos;
			print "Loaded phenotypic and family data for $phenosize samples\n";
		}
		
		#Works out which FinalReport corresponds to which sample
		if ($file =~ /$namematch/){
			open (IN2, "<$file")|| die "Could not open file $file";
			$count2 = 0;
			while (<IN2>){
				$count2++;
				if($count2 == 20){
					@tab = split ("\t", $_);
					$finalrep{$tab[0]} = $file;
					#print "$file\t$tab[0]\n";
					close IN2;
				}
			}
		}
	}
	
	@finalreps = keys %finalrep;
	$finalrepsize = @finalreps;
	print "Worked out FinalReport names for $finalrepsize samples\n";
	
	#Error message if there are no FinalReports present
	if ($firstfile == 0){
		die "There was no Final Report file to work out how many SNPs there are in each region; please find any Final Report file from the correct type of chip and put it in this folder\n";
	}
	
	print "Sorting chromosome arrays\n";
	
	#The list of chromosomes is sorted (for neatness...)
	@chromlist = keys %chromlist;
	@chromlist = sort  { $a cmp $b } @chromlist;
	#print @chromlist;
	
	#Then each specific array is sorted by position
	foreach (@chromlist){
		@$_ = sort { (split '\t', $a)[0] <=> (split '\t', $b)[0] } @$_;
		$probes = @$_;
		#print "$_, ";
	}
	print "\nProbes entered into chromosome specific arrays, this may take a while...\n";
	
	#Sort the input file by sample 
	print "Sorting the CNV file by sample ID\n";
	@input = sort { (split '\t', $a)[$sampleID] cmp (split '\t', $b)[$sampleID] } @input;
	$counter = 0;
	
	#Splits the file into samples and sends an array per sample to the sampledeal subroutine
	foreach $region (@input){
		@thisone = split ("\t", $input[$counter]);
		@nextone = split ("\t", $input[$counter + 1]);
		push (@sample, $region);
		#print "This: $thisone[$sampleID], Next: $nextone[$sampleID]\n";
		if ($thisone[$sampleID] ne $nextone[$sampleID]){
			&sampledeal(@sample);
			@sample = ();
		}
		$counter++;
	}
	print "\nAll done\n";
}

#Checks the first allele genotype in each file and compares it to the first file - distinguishes different stranded FinalReports
if ($plink_check == 1){
	&fr_columns;
	&complement;
	$outfile = "Plink_check.txt";
	open (OUT, ">$outfile") || die "Could not open $outfile; may already be open\n";
	print OUT "File\tSNPs checked\tFile1 Allele 1 Match\tFile1 Allele 1 Mismatch\tResult\n";
	@files = <*>;
	foreach $file (@files){
		if ($file =~ /$namematch/){
			$filecount++;
			print "$filecount) Checking file $file..."; 
			open (IN, "<$file") || die "Could not open file $file\n";
			if ($filecount == 1){
				while (<IN>) {
					if ($_ =~ /rs/){
						@tab = split (/\t/,$_);
						$gt1{$tab[$snpcol]} = $tab[$gtcol1];
						$type1{$file} += 1;
						$total{$file} += 1;
					}
				}
			}
			else {
				while (<IN>) {
					if ($_ =~ /rs/){
						@tab = split (/\t/,$_);
						if ($tab[$gtcol1] eq $gt1{$tab[$snpcol]}){
							$type1{$file} += 1;
							$total{$file} += 1;
						}
						elsif ($comp{$tab[$gtcol1]} eq $gt1{$tab[$snpcol]}){
							$type2{$file} += 1;
							$total{$file} += 1;
						}
					}
				}
			}
			$type1per = $type1{$file} / $total{$file} * 100;
			$type2per = $type2{$file} / $total{$file} * 100;
			if ($type2per >= 20) {$result = "Strand type 2";}
			else {$result = "Strand type 1";}
			print OUT "$file\t$total{$file}\t".$type1per."%\t".$type2per."%\t$result\n";
			print "$result\n";
		}
	}
}


#Designs PCR Primers
if ($design_pcr_primers == 1){
	$inputfile = $ARGV[0]; #Format: chr (0), start (1), +/- stop (2)
	$thechrom = 0;
	$thestart = 1;
	$thestop = 2;
	open (IN, $inputfile) or die "Could not open $inputfile\n";
	{ local $/=undef;  $input1=<IN>; }
	close IN;
	($\) = (/(\r\n|\r|\n)/);
	@input=split (/\r\n|\r|\n/, $input1); chomp @input;
	
	foreach $line (@input){
		@tab = split (/\t/, $line);
		if ($tab[$thestop] =~ /\d+/){
			$stoppos = $tab[$thestart + 1];
		}
		else {
			$stoppos = $tab[$thestop];
		}
		$regstart = $stoppos - 600;
		$regstop = $tab[$thestart] + 600;
		$regdiff = $regstop - $regstart;
		$chromofile = $ANloc."Genome/"."Fix_".$tab[$thechrom].".fa";
		close CHR;
		open (CHR, $chromofile) or print "Could not find the chromosome reference file: $chromofile\n";
		seek (CHR, $regstart, 0);
		read (CHR, $dna, $regdiff);
		print $dna."\n";
	}
	
	
}


#Designs primers
if ($design_primers == 1){
	#Format: Name, Chr, Start, Stop
	use LWP::UserAgent;
	use HTTP::Request::Common;
	$desprimers = $ARGV[0];
	open (SEQS, "<$desprimers") || die "----------\nCannot open the input file $desprimers.  The file may be:\n(1)  Spelled incorrectly, or\n(2)  In a different directory\n----------\nPlease check and try again.\n";
	print "Loading file $desprimers...\n";
	@seqs = <SEQS>; chomp @seqs; close (SEQS);
	
	#Is this qPCR or PCR (this can be chossen using the input commands --qpcr or --pcr
	if ($qpcr_settings == 0 and $pcr_settings == 0){
		print "Do you want to design primers for PCR (type \'pcr\') or qPCR (type \'qpcr\')?\n";
		$user_pcrtype = <STDIN>; chomp $user_pcrtype;
		if ($user_pcrtype =~ /qpcr/gi){
			$qpcr_settings = 1;
		}
		elsif ($user_pcrtype =~ /pcr/gi){
			$pcr_settings = 1;
		}
		else {
			die "ERROR: Response was not recgonised please rerun and type \'pcr\' or \'qpcr\'\n";
		}
	}
	#Is this file a list of regions or sequence
	print "\nIs this file a list of regions (Name,chr,start,stop) or a list of sequences (Name,seq), enter (\'reg\' or \'seq\')?\n";
	$user_response = <STDIN>; chomp $user_response;
	
	if ($user_response =~ /reg/gi){
		$filetype = "reg";
		$desprimers =~ s/.txt//gi;
		$resultsfile = "$desprimers" . "_primer3_output.txt";
		$extractsfile = "$desprimers" . "_seqs.fasta";
		$toblast= "$desprimers" . "_blastme.fasta";
		open (RESULTS, ">$resultsfile") || die "Could not open file $resultsfile; may already be open\n";
		open (EXTRACTS, ">$extractsfile") || die "Could not open file $extractsfile; may already be open\n";
		open (BLASTME, ">$toblast") || die "Could not open file $toblast; may already be open\n";
		print LISTOUT $curr_dir."$resultsfile\n".$curr_dir."$extractsfile\n".$curr_dir."$toblast\n";
		@headerline = split ("\t", $seqs[0]);
		foreach (@headerline){
			$headlinecount++;
			print RESULTS "$headlinecount\t";
		}
		print RESULTS "Name\tStatus\tPrimer\tOrientation\tStart\tLength\tTm\tGC%\t2nd_structure\tPrimer-dimer?\tUniqueeness\tSequence\t\tPrimer\tOrientation\tStart\tLength\tTm\tGC%\t2nd_structure\tPrimer-dimer?\tUniqueeness\tSequence\t\tProduct\tSize(bp)\tChr\tStart\tStop\n";
		@Holder='';
		$name='';
		print "Primer hunting...\n";
		&getDNA(@seqs);
	
		$leftover = @leftover;
		print "\nThere are $leftover regions to redo\n";
		if ($leftover > 0){
			$qPCRthreshold++;
			$PCRthreshold++;
			&getDNA(@leftover);
		}
		
		#$leftover = @leftover;
		#print "\nThere are $leftover regions to redo\n";
		#if ($leftover > 0){
		#	$qPCRthreshold++;
		#	&getDNA(@leftover);
		#}
		
		$leftover = @leftover;
		print "\nThere are $leftover regions that no suitable primers have been made for\n";
	}
	
	elsif ($user_response =~ /seq/gi) {
		$filetype = "seq";
		$seqname = 0;
		$seqDNA = 1;
		foreach $line (@seqs){
			@tab = split (/\t/, $line);
			push (@data, $tab[0], $tab[1]);
			&sortDNA(@data);
		}
			
		$leftover = @leftover;
		print "\nThere are $leftover regions to redo\n";
		if ($leftover > 0){
			$PCRthreshold++;
			$qPCRthreshold++;
			&sortDNA(@leftover);
		}
		$leftover = @leftover;
		print "\nThere are $leftover regions that no suitable primers have been made for\n";
	}
	
	else {
		print "ERROR: Response was not recgonised please rerun and type \'reg\' or \'seq\'\n";
	}
	#Each line of @data is a line of DNA sequence from UCSC. $array[0] is the name of the region with a > in front.
	
}

#Wiggle file maker
if ($wiggle == 1) {
	if (!(defined @ARGV)) { die "Please put the name of an input file after \'--wiggle\' \n";}

	# List of threshold variables
	my $input = $ARGV[0];  	# Input file is the first argument of the program
	my $output = "Wiggle_".$input;
	$output =~ s/.txt/.bed/gi;
	
	
	print "Converting $input to a Wiggle track.\n\n";
	
	open (DATA, "<$input") || die "Cannot open the input file $input; check it is in the right directory\n";
	open (OUTPUT, ">$output") || die "Cannot open the output file $output; may already be open\n";
	my @file = <DATA>; chomp @file;									# Load the entire file into an array
	
	@lineshow = split ("\t", $file[0]);
	print "The first line of the input file reads:\n";
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	print "Chromosome: What column is the chromosome in (e.g. 3)?\n";
	$user_chrom= <STDIN>; chomp $user_chrom; $chrom = ($user_chrom-1);
	
	print "Start: What column is the start position in (e.g. 4)?\n";
	$user_chromo2= <STDIN>; chomp $user_chromo2; $start = ($user_chromo2-1);
	
	print "Stop: What column is the stop position stop in (e.g. 5)?\n";
	$user_chromo3= <STDIN>; chomp $user_chromo3; $stop = ($user_chromo3-1);
	
	print "Value: What column is the value in (e.g. 6)?\n";
	$user_chromo4= <STDIN>; chomp $user_chromo4; $score = ($user_chromo4-1);
	
	print "Header: Do you have a header row?  Type Y(yes) or N(no), then press Enter\n";
	$head= <STDIN>; chomp $head;
	
	
	if ($head =~ /y/gi) { shift @file; } 	# If there's a header, delete the first line

	
	print OUTPUT "track type=wiggle_0 name=Wiggle description='track generated from $input' visibility=full autoScale=off viewLimits=0.0:50.0 yLineMark=0 yLineOnOff=on color=0,0,128 priority=10\n";
	my $line;
	foreach $line (@file) {
		my @tab = split(/\t/, $line);
		if ($tab[$chrom] !~ /chr/){
			$tab[$chrom] = "chr" . $tab[$chrom];
		}
		if ($tab[$chrom] =~ /XY/){
			print OUTPUT "chrX\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
			print OUTPUT "chrY\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
		}
		elsif ($tab[$chrom] =~ /MT/){
			print OUTPUT "chrM\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
		}
		else{
			#print "Chr: $tab[$chrom], Start: $tab[$start], Stop: $tab[$stop], Score: $tab[$score]\n";sleep(1);
			print OUTPUT "$tab[$chrom]\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
		}
	}
	print "Conversion complete\n";
}

#bedGraph maker
if ($bedGraph == 1) {
	if (!(defined @ARGV)) { die "Please put the name of an input file after \'--bedgraph\' \n";}

	# List of threshold variables
	my $input = $ARGV[0];  	# Input file is the first argument of the program
	my $output = "BedGraph_".$input;
	$output =~ s/.txt/.bed/gi;
	
	
	print "Converting $input to a bedGraph track.\n\n";
	
	open (DATA, "<$input") || die "Cannot open the input file $input; check it is in the right directory\n";
	open (OUTPUT, ">$output") || die "Cannot open the output file $output; may already be open\n";
	{ local $/=undef;  $input1=<DATA>; }
	close DATA;
	($\) = (/(\r\n|\r|\n)/);
	@file=split (/\r\n|\r|\n/, $input1); chomp @file;
	#my @file = <DATA>; chomp @file;									# Load the entire file into an array
	
	@lineshow = split ("\t", $file[0]);
	print "The first line of the input file reads:\n";
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	print "Chromosome: What column is the chromosome in (e.g. 3)?\n";
	$user_chrom= <STDIN>; chomp $user_chrom; $chrom = ($user_chrom-1);
	
	print "Start: What column is the start position in (e.g. 4)?\n";
	$user_chromo2= <STDIN>; chomp $user_chromo2; $start = ($user_chromo2-1);
	
	print "Stop: What column is the stop position stop in (e.g. 5)?\n";
	$user_chromo3= <STDIN>; chomp $user_chromo3; $stop = ($user_chromo3-1);
	
	print "Value: What column is the value in (e.g. 6)?\n";
	$user_chromo4= <STDIN>; chomp $user_chromo4; $score = ($user_chromo4-1);
	
	print "Header: Do you have a header row?  Type Y(yes) or N(no), then press Enter\n";
	$head= <STDIN>; chomp $head;
	
	
	if ($head =~ /y/gi) { shift @file; } 	# If there's a header, delete the first line

	
	print OUTPUT "track type=bedGraph name=bedGraph description='track generated from $input' visibility=full autoScale=off viewLimits=0.0:50.0 yLineMark=0 yLineOnOff=on color=0,0,128 priority=10\n";
	my $line;
	foreach $line (@file) {
		my @tab = split(/\t/, $line);
		if ($tab[$chrom] !~ /chr/){
			$tab[$chrom] = "chr" . $tab[$chrom];
		}
		if ($tab[$chrom] =~ /XY/){
			print OUTPUT "chrX\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
			print OUTPUT "chrY\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
		}
		elsif ($tab[$chrom] =~ /MT/){
			print OUTPUT "chrM\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
		}
		else{
			#print "Chr: $tab[$chrom], Start: $tab[$start], Stop: $tab[$stop], Score: $tab[$score]\n";sleep(1);
			print OUTPUT "$tab[$chrom]\t$tab[$start]\t$tab[$stop]\t$tab[$score]\n";
		}
	}
	print "Conversion complete\n";
}

#Final Report as a Genome Browser track
if ($fr_gb == 1) {
	&fr_columns;
	$extra = 1;
	if (!(defined @ARGV)) { die "Please put the name of a finalreport file after \'--fr_gb\' \n";}

	# List of threshold variables
	my $input = $ARGV[0];  	# Input file is the first argument of the program
	my $output = "FR_BedGraph_".$input;
	$output =~ s/.txt/.bed/gi;
	
	
	print "Converting $input to a bedGraph track.\n\n";
	
	open (DATA, "<$input") || die "Cannot open the input file $input; check it is in the right directory\n";
	open (OUTPUT, ">$output") || die "Cannot open the output file $output; may already be open\n";
	my @file = <DATA>; chomp @file;									# Load the entire file into an array
	
	$count2 = 0;
	seek (DATA,0,0);
	while (<DATA>){
		$count2++;
		if($count2 == 4){
			@tab = split ("\t", $_);
			$bigchip = $tab[2];
			chomp $bigchip;
		}
		if($count2 == 5){
			@tab = split ("\t", $_);
			$snps = $tab[1];
			chomp $snps;
			$chip = &chiper($snps);
		}
		if($count2 == 20){
			@tab = split ("\t", $_);
			$sample = $tab[$patname];
			print "$file\t$sample\n";
			close DATA;
		}
	}
	
	print OUTPUT "track type=bedGraph name=\'LogR_".$sample."\' description=\'LogR of ".$sample." on a ".$chip."chip\' visibility=full color=0,150,0 altColor=150,0,0 autoScale=off gridDefault=on graphType=points viewLimits=-4:2 yLineMark=0 yLineOnOff=on\n";
	$outline = "track type=bedGraph name=\'BAF_".$sample."\' description=\'B Allele Frequency of ".$sample." on a ".$chip."chip\' visibility=full color=0,0,150 autoScale=off gridDefault=on graphType=points viewLimits=0:1 yLineMark=1 yLineOnOff=on\n";
	push (@output, $outline);
	my $line;
	foreach $line (@file) {
		$counthere++;
		$tracker++;
		if ($javamethod == 0){
			print "\rLooking at line $counthere";
		}
		elsif ($tracker >= 50000) {
			print "Looking at line $counthere\n";
			$tracker = 0;
		}
		my @tab = split(/\t/, $line);
		$secondcoord = $tab[$chromodist] + $extra;
		if ($counthere < 12 or $tab[$log2col] eq "" or $tab[$log2col] eq "NaN"){}
		elsif ($tab[$chromocol] =~ /XY/){
			print OUTPUT "chrX\t$tab[$chromodist]\t$secondcoord\t$tab[$log2col]\n";
			print OUTPUT "chrY\t$tab[$chromodist]\t$secondcoord\t$tab[$log2col]\n";
			$outline1 = "chrX\t$tab[$chromodist]\t$secondcoord\t$tab[$ballele]\n";
			$outline2 = "chrY\t$tab[$chromodist]\t$secondcoord\t$tab[$ballele]\n";
			push (@output, $outline1, $outline2);
		}
		elsif ($tab[$chromocol] =~ /MT/){
			print OUTPUT "chrM\t$tab[$chromodist]\t$secondcoord\t$tab[$log2col]\n";
			$outline1 = "chrM\t$tab[$chromodist]\t$secondcoord\t$tab[$ballele]\n";
			push (@output, $outline1);
		}
		else{
			#print "Chr: $tab[$chrom], Start: $tab[$start], Stop: $tab[$stop], Score: $tab[$score]\n";sleep(1);
			print OUTPUT "chr$tab[$chromocol]\t$tab[$chromodist]\t$secondcoord\t$tab[$log2col]\n";
			$outline1 = "chr$tab[$chromocol]\t$tab[$chromodist]\t$secondcoord\t$tab[$ballele]\n";
			push (@output, $outline1);
		}
	}
	print OUTPUT @output;
	print "\nConversion complete\n";
}

#BEDfile maker
if ($bedfile == 1) {
	if (!(defined @ARGV)) { die "Please specify an input file as an argument.\n";}

	# List of threshold variables
	my $input = $ARGV[0];  	# Input file is the first argument of the program
	my $output = "BED_".$input;
	$output =~ s/.txt/.bed/gi;	
	
	print "Converting to $input to a BED track\n\n";
	
	open (DATA, "<$input") || die "Cannot open the input file $input; check it is in the right directory\n";
	open (OUTPUT, ">$output") || die "Cannot open the output file $output; may already be open\n";
	print LISTOUT $curr_dir.$output;
	{ local $/=undef;  $input1=<DATA>; }
	close DATA;
	($\) = (/(\r\n|\r|\n)/);
	@file=split (/\r\n|\r|\n/, $input1); chomp @file;

	# Load the entire file into an array
	
	@lineshow = split ("\t", $file[0]);
	print "The first line of the input file reads:\n";
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	print "Name: What column is the name in (e.g. 2)?\n";
	$user_chrom0= <STDIN>; chomp $user_chrom0; $name = ($user_chrom0-1);
	
	print "Chromosome: What column is the chromosome in (e.g. 3)?\n";
	$user_chrom= <STDIN>; chomp $user_chrom; $chrom = ($user_chrom-1);
	
	print "Start: What column is the start position in (e.g. 4)?\n";
	$user_chromo2= <STDIN>; chomp $user_chromo2; $start = ($user_chromo2-1);
	
	print "Stop: What column is the stop position stop in (e.g. 5)?\n";
	$user_chromo3= <STDIN>; chomp $user_chromo3; $stop = ($user_chromo3-1);
	
	print "Type: What column is the CNV Type in (e.g. 6)?\n";
	$user_chromo4= <STDIN>; chomp $user_chromo4; $type = ($user_chromo4-1);
	
	print "Score: What column is the Score in (e.g. 7)?\n";
	$user_chromo5= <STDIN>; chomp $user_chromo5; $score = ($user_chromo5-1);
	
	print "Header: Do you have a header row?  Type Y(yes) or N(no), then press Enter\n";
	$head= <STDIN>; chomp $head;
	
	
	if ($head =~ /y/gi) { shift @file; } 	# If there's a header, delete the first line
	$chrsign = 'chr';
	$anx = "X";
	$anm = "M";
	print OUTPUT "track name='$input\_track' description='track generated from $input' itemRgb=On visibility=3\n";
	my $line;
	foreach $line (@file) {
		$linehere++;
		my @tab = split(/\t/, $line);
		chomp @tab;
		my $color = "0,0,0";
		$tab[$type - 1] =~ s/\"//g;
		my @CNVtype = split(/,/, $tab[$type]);
		if (@CNVtype > 2) {
			# Do nothing
		} elsif (@CNVtype == 2) {
			# Test both
			if ($CNVtype[0] == 0 && $CNVtype[1] == 1) { $color = "100,0,0"; }
			if ($CNVtype[0] == 3 && $CNVtype[1] == 4) { $color = "0,150,0"; }
		} elsif (@CNVtype == 1) {
			if ($CNVtype[0] == 0) { $color = "255,0,0"; }
			if ($CNVtype[0] == 1) { $color = "100,0,0"; }
			if ($CNVtype[0] == 3) { $color = "0,150,0"; }
			if ($CNVtype[0] == 4) { $color = "0,255,0"; }
		}
		unless ($tab[$chrom] =~ /$chrsign/){
			$tab[$chrom] = "chr".$tab[$chrom];
		}
		if ($tab[$chrom] =~ /XY/i){
			$tab[$chrom] =~ s/XY/$anx/i;
		}
		if ($tab[$chrom] =~ /MT/i){
			$tab[$chrom] =~ s/MT/$anm/i;
		}
		if ($score eq ""){
			$scoreout = 0;
		}
		elsif ($tab[$score] < 0){
			$scoreout = 0 - $tab[$score];
		}
		else {
			$scoreout = $tab[$score];
		}
		
		$output = "$tab[$chrom]\t$tab[$start]\t$tab[$stop]\t$tab[$name]\t$scoreout\t.\t$tab[$start]\t$tab[$stop]\t$color\n";
		$output =~ s/ //g;
		
		if ($tab[$name] ne ""){
			print OUTPUT $output;
		}
		else {
			print "Error: line $linehere:\n";
			print "$line\n";
		}
	}
	print "Conversion complete\n";
}

if ($linkage == 1) {
	use File::Copy;
	$linkage_program = "allegro-1.2.exe";
	$root_dir = getcwd; chomp ($root_dir); $program_location = ($root_dir . "\/" . $linkage_program);
	
	# notes: ask the user if mis-inheritance should be checked
	# error -checking, if someone puts a bad decimal, prompt user
	# the ped file should have 
	
	###############################################################################################################
	# There can only be one column at the end of the file, and this is where the allele frequencies are expected
	###############################################################################################################
	
	print "\n******************\nPedigree File:\n******************\nPlease enter the name of the pedigree file\nThen press Enter\n";
	$input2= <STDIN>; chomp $input2; open (PED, "<$input2") || die "----------\nCannot open the input file.  The file may be:\n(1)  Already open,\n(2)  Spelled incorrectly, or\n(3)  In a different directory\n----------\nPlease check and try again.\n"; print "loading file...\n";
	@ped=<PED>; chomp @ped; close (PED); print "finished\n";
	print "\n******************\nChromosome Column :\n******************\nPlease enter the column of the file where the chromosome value is located (e.g. 1,2,3,4...x)\nThen press Enter\n";
	$user_chromo= <STDIN>; chomp $user_chromo; $chromoloc = ($user_chromo-1);
	print "\n******************\nPhysical Distance :\n******************\nPlease enter the column of the file where the chromosome distances are located (in bp, e.g. 112487, 134721,...x)\nThen press Enter\n";
	$user_dist= <STDIN>; chomp $user_dist; $chromodist = ($user_dist-1);
	print "processing file\n";
	print "\n******************\nFirst Genotype Column:\n******************\nPlease enter the column of the file where the first genotype values are located(e.g. 1,2,3,4...x)\nThen press Enter\n";
	$user_geno= <STDIN>; chomp $user_geno; $genoloc = ($user_geno-1);
	print "\n******************\nSNP ID Column:\n******************\nPlease enter the column of the file where the SNP IDs are (e.g. rs63234, or SNP_A-1749385\nThen press Enter\n";
	$SNPloc= <STDIN>; chomp $SNPloc; $snploc = ($SNPloc-1);
	print "\n******************\nGenetic Distance Column:\n******************\nPlease enter the column of the file where the genetic distances are located (e.g. 1,2,3,4...x)\nThen press Enter\n";
	$gend= <STDIN>; chomp $gend; $gendist= ($gend-1);
	
	$root_dir = getcwd; 
	print "$root_dir\n";
	print "\n******************\nArray File:\n******************\nPlease enter the name of the file that you want to split apart\nThen press Enter\n";
	$input= <STDIN>; chomp $input; 
	&misinherit($input);
	$newname = $input . "_clean.txt";
	
	open (OG, "<$newname") || die "----------\nCannot open the input file.  The file may be:\n(1)  Already open,\n(2)  Spelled incorrectly, or\n(3)  In a different directory\n----------\nPlease check and try again.\n"; print "loading file...\n";
	@data=<OG>; chomp @data; close (OG); print "finished\n";
	mkdir ("linkage", 0777); chdir("linkage");
	#chdir(".."); 
	mkdir ($input, 0777); chdir($input);
	$root_dir = getcwd; 
	print "$root_dir\n";
	
	
	print "\n******************\nNumber of Un-Genotyped:\n******************\nPlease enter the number of individuals in your pedigree who have not been genotyped\nThen press Enter\n";
	$num_ungenotyped= <STDIN>; chomp $num_ungenotyped;
	print "\n******************\nFrequency of Disease Causing Allele:\n******************\nPlease enter the frequency of disease causing allele (in decimal format (e.g., 98% is 0.98, 1% is 0.01)\nThen press Enter\n";
	$freq_dis_allele= <STDIN>;
	chomp $freq_dis_allele;  $freq_other_dis_allele = (1-$freq_dis_allele);
	$freq_dis_allele = sprintf "%.6f", $freq_dis_allele;
	$freq_other_dis_allele = sprintf "%.6f", $freq_other_dis_allele;
	print "\n******************\nPhenocopy Rate:\n******************\nPlease enter the Phenocopy rate (in decimal format (e.g., 98% is 0.98, 1% is 0.01)\nThen press Enter\n";
	$phenocopy_rate= <STDIN>;
	chomp $phenocopy_rate;  
	print "\n******************\nPenetrance of Heterozygous Condition:\n******************\nPlease enter the Penetrance of the Heterozygous Condition(in decimal format (e.g., 98% is 0.98, 1% is 0.01)\nThen press Enter\n";
	$penet_hetero= <STDIN>;
	chomp $penet_hetero;  
	print "\n******************\nPenetrance of Homozygous Condition:\n******************\nPlease enter the Penetrance of the Homozygous Condition(in decimal format (e.g., 98% is 0.98, 1% is 0.01)\nThen press Enter\n";
	$penet_homo= <STDIN>;
	chomp $penet_homo;
	print "\n******************\nNumber of Lines:\n******************\nPlease enter the number of lines you want in each file\nThen press Enter\n";
	$lines= <STDIN>;
	chomp $lines;
	print "\n******************\nOverlap:\n******************\nPlease enter the range of overlap between the files which are output (If you enter the number 500, it separates the file's lines like so: file1(0-6500), file2 (6000-12500), etc...)\nThen press Enter\n";
	$overlap= <STDIN>;
	chomp $overlap;
	#print "\n******************\nemail notifcation:\n******************\nPlease enter your email for server notification of job completion\nThen press Enter\n";
	#$email= <STDIN>; chomp $email;
	print "processing file\n";
	shift(@data);
	@data = sort specificSort @data;
	foreach $linez (@data){$line =~ s/NC/NoCall/gi;}
	
	open (PBS, ">pbs_script_$input.sh") || die "Could not open shell script file pbs_script_$input.sh\n";
	open (BATCH, ">allegro_all_chrs_batch.bat") || die "Could not open shell script file pbs_script_$input.sh\n";
	open (PLOTS, ">to_paste_into_R.txt") || die "Could not open shell script file pbs_script_$input.sh\n";
	
	print PBS "#!/bin/bash\n";
	open (COLLECT, ">collector.pl") || die "Could not open shell script file collector.pl\n";
	print COLLECT "#usr/bin/perl\nuse Cwd;\nuse File::Copy;\n\$herenow = getcwd;\n";
	
	
	#Then we find the number of chromosomes in the input file
	@zygosity=''; $replace="X"; $replace2="XY"; $replace3="Y";
	for ($z=0; $z<=$#data; $z++){$data[$z] =~ s/$replace/23/; $data[$z] =~ s/$replace2/23/; $data[$z] =~ s/$replace3/24/;}
	
	$count=0; @chromosomes='';
	@ogman = split("\t", $data[0]); $old=$ogman[$chromoloc]; $chromosomes[$count]=$old;
	foreach (@data){@temp=split("\t", $_);
		chomp($temp[$chromoloc]);
		if ($temp[$chromoloc] eq $old){}
		else{
			if ($temp[$chromoloc]>0 and $temp[$chromoloc] ne ''){
				$count++; $chromosomes[$count]=$temp[$chromoloc];
				}
		}
		$old=$temp[$chromoloc];
	}
	
	$total=($#chromosomes +1);
	print "\n$total chromosomes were detected.\n"; 
	foreach $dude (@chromosomes){print "chr:$dude,";}print "\n";
	print "Now creating directories and map files\n";
	
	$chromo_dir=0; $counter=1; $marker=0; 
	$filename = $chromosomes[$chromo_dir]. "_" . "$counter" . ".txt";
	
	$chromo_name = "chr" . $chromosomes[$chromo_dir] . "_" . $counter;
	mkdir($chromo_name, "0777"); chdir($chromo_name);
	open (OUTPUT, ">$filename") || die "Cannot open the file of the genome sequence of $filename !";
	
	for ($z=1; $z<$#data; $z++){
		@temp=split("\t", $data[$z]);
		@temp2=split("\t", $data[$z+1]);
	
		if ($temp[$chromoloc] eq $temp2[$chromoloc]){
	
			if ($marker<$lines){
				print OUTPUT "$data[$z]\n";
				$marker++;
			}
			else{
				close (OUTPUT);
				&chunky($filename, $chromosomes[$chromo_dir], $counter);
				&filegen ($chromo_name);
				#We've reached the end of the line limit
				$marker=0;
				$z= ($z - $overlap);
				$counter++;
							
				print COLLECT "\$newfile= (\$herenow . \"\/\" . \"$chromo_name/param\")\;\n";
				print COLLECT "\$newspot = \$herenow . \"\/param_$chromo_name.txt\";\n";
				print COLLECT "copy (\$newfile, \$newspot);\n";
				#Directory Change
				$chromo_name = "chr" . $chromosomes[$chromo_dir] . "_" . $counter;
				
				chdir(".."); mkdir ($chromo_name, 0777); chdir ($chromo_name); 
	
				#Filename change
				$filename = $chromosomes[$chromo_dir]. "_" . "$counter" . ".txt";
				open (OUTPUT, ">$filename") || die "Cannot open the file of the genome sequence of $filename !";
				print OUTPUT "$data[$z]\n";
			}
		}
		else{
			if ($counter>1){
				print OUTPUT "$data[$z]\n";
				$a=$z-$lines; 
				close (OUTPUT);
				open (OUTPUT, ">$filename") || die "Cannot open the file of the genome sequence of $filename !";
				for ($b=$a; $b<=$z; $b++){
					print OUTPUT "$data[$b]\n"; 
				}
			}
			else{
				print OUTPUT "$data[$z]\n";
			}
			close (OUTPUT);
			$file=$filename; $chromo=$chromosomes[$chromo_dir]; $c=$counter;
			&chunky($filename, $chromo, $c);
			&filegen ($chromo_name);
			
			$marker=0; $counter=0; $chromo_dir++; $counter++;
			print COLLECT "\$newfile= (\$herenow . \"\/\" . \"$chromo_name/param\")\;\n";
			print COLLECT "\$newspot = \$herenow . \"\/param_$chromo_name.txt\";\n";
			print COLLECT "copy (\$newfile, \$newspot);\n";
	
			$chromo_name = "chr" . $chromosomes[$chromo_dir] . "_" . $counter;
			chdir(".."); mkdir ($chromo_name, 0777); chdir ($chromo_name);
	
			$filename = $chromosomes[$chromo_dir]. "_" . "$counter" . ".txt";
			open (OUTPUT, ">$filename") || die "Cannot open the file of the genome sequence of $hyb !";
		}
	}
	#this is for when we get to the end of the file
	close (OUTPUT);
			$file=$filename; $chromo=$chromosomes[$chromo_dir]; $c=$counter;
			&chunky($filename, $chromo, $c);
			&filegen ($chromo_name);
		
	
	
	#########################
	#Integration of Chunky
	#########################
	sub chunky
	{
		@inter_marker='';
		$datname = @_[1] . "_" . @_[2] . ".dat";
		$prename = "a" . @_[1] . "_" . @_[2] . ".pre";
		open (DAT, ">$datname") || die "Cannot open the file of the genome sequence of $filename !";
		open (PRE, ">$prename") || die "Cannot open the file of the genome sequence of $filename !";
		
		%data = (); @input='';
		$untypedN = 0;
		$inputfile = @_[0];
		open (INPUT, "<$inputfile") || die "Cannot open the file of the genome sequence of $filename !";
		@input = <INPUT>; chomp (@input);
		$col=0;; 
		#Create a 2D array with all the data.
		for ($g=0; $g<=$#input; $g++){
			@tempee= split ("\t", $input[$g]);
			for ($f=$genoloc; $f<=$#tempee; $f++){
				$predata[$g][$f]= $tempee[$f];
			}	
		}
		$inputsize=($#input+2);
		#now we make the .dat file
			print DAT "$inputsize 0 0 5 \n";
			print DAT "0 0.000 0.000 0 \n";
			for ($b=0; $b<=$#input; $b++){($tempb=$b+1); print DAT "$tempb ";} print DAT "$inputsize";
			print DAT "\n1 2 # \<\< \n";
			print DAT " $freq_other_dis_allele $freq_dis_allele\n";
			print DAT "1\n";
			$phenocopy_rate = sprintf "%.4f", $phenocopy_rate; 	
			$penet_hetero = sprintf "%.4f", $penet_hetero;
			$penet_homo = sprintf "%.4f", $penet_homo;
			print DAT " $phenocopy_rate $penet_hetero $penet_homo\n";
			
			for ($w=0; $w<=($#input); $w++){
				@temp = split ("\t", $input[$w]);
				@temp2 = split ("\t", $input[$w+1]);
				
						$diff = ($temp2[$gendist] - $temp[$gendist]);
						$new_diff =($diff);
						if ($new_diff >= 0.5){ $new_diff=0.49};
						$new_diff = sprintf "%.5f", $new_diff; 	
						if ($new_diff < .00001){
							$new_diff = .00001;
							$new_diff = sprintf "%.5f", $new_diff;
							$inter_marker[$w] = ($new_diff);
						}
						else{
						$inter_marker[$w] = ($new_diff);
						}
				print DAT "3 2 \# $temp[$snploc]\n";
				$bFreq = 1 - $temp[$#temp];
				$aFreq = $temp[$#temp];
				
				#this bit of code is to make the allele frequencies readable later on
				################
				if ($bFreq == 1){$bFreq=0.99; $aFreq=0.01;}
				elsif ($aFreq == 1){$aFreq=0.99; $bFreq=0.01;}
				################
				
				$R_bFreq = sprintf "%.6f", $bFreq; 	
				$R_aFreq = sprintf "%.6f", $aFreq;
					
				print DAT " $R_aFreq $R_bFreq\n";
				
			}
			print DAT "0 0\n 0.00000 "; $cnt=1;
			#Can only have 512 listings per line
			for ($d=0; $d<$#input; $d++){
				print DAT "$inter_marker[$d] ";
			}
			print DAT "Haldane\n1 0.10000 0.45000";
	
	##LET'S MAKE A PRE FILE!!!
	$person=0;
	
		for ($f=$genoloc; $f<$#tempee; $f++){
			print PRE "$ped[$person]\t";
			for ($g=0; $g<=$#input; $g++){
			
				$chunk = $predata[$g][$f];
				$chunk =~ s/AA/1 1/;
				$chunk =~ s/AB/1 2/;
				$chunk =~ s/BA/2 1/;
				$chunk =~ s/BB/2 2/;
				$chunk =~ s/NoCall/0 0/;
				$chunk =~ s/NC/0 0/;
				print PRE "$chunk ";
			}
			print PRE "\n";
			$person++;
		}
		###########################################################################
		#this is for the individuals with no genotype
		for ($l=1; $l<=$num_ungenotyped; $l++){
			print PRE "$ped[$person]\t";
			for ($g=0; $g<=$#input; $g++){
				print PRE "0 0 ";
			}
			print PRE "\n";
			$person++;
		}
		close(DAT); close (INPUT); close(PRE);
		#End of chunky
	}
	
	#########################
	#Creation of Other Files
	#########################
	sub filegen
	{
		$name= @_[0];
		$optname = $name . ".opt";	
		$pbsname = $name . ".sh";
		open (OPT, ">$optname") || die "Cannot open the file of the genome sequence of $optname!";
		print OPT "% Read input in linkage analysis format:\nPREFILE $prename\nDATFILE $datname\n\n";
		print OPT "% Linkage analysis to be for performed:\nMODEL mpt par het param fparam\nMODEL mpt exp pairs equal\n\n";
		print OPT "% other statistical analysis to be performed:\nHAPLOTYPE\nCROSSOVERRATE\n\n";
		print OPT "% other options\nUNINFORMATIVE	% Write uninformative markers to uninformative.out\nMAXMEMORY 4000	% Maximum memory set to 4000 Mb\n"; 
		print OPT "% Units specification\nUNIT centimorgan";
			
	
	
		#prepare PBS scripts
		open (PBSNEW, ">..\/$pbsname") || die "Cannot open the file of the genome sequence of $pbsname!";
		print PBSNEW "cd /lustre0/scratch/kb374/$input/$name/\n";
		print PBSNEW "/bulldogc_homes/home3/njc2/bioInformatics/MuratGunel/051013/allegro-1.2b/bin/allegro $optname\n";
	
		
		print PBS "qsub -q sandbox -W group_list=sand -l nodes=1:ncpus=2 -A Mason -m abe ./$pbsname\n";
		
		$curr_dir=getcwd; 
		$newfile= ($curr_dir . "\/" . $linkage_program);
		copy ($program_location, $newfile); 
	
		$newspot = ($curr_dir . "\/" . $name);
		print BATCH "cd $name\n";
		print BATCH "allegro-1.2 $optname\n";
		print BATCH "cd ..\n";
	
		$rget = "param_" . $name . ".txt";
		$rout =  $name . ".jpeg";
		
		print PLOTS "data <- read.table(\"$rget\", header=TRUE)\n";
		print PLOTS "LOD_score <- data[,2]\n";
		print PLOTS "plot(LOD_score)\n";
		print PLOTS "dev.print(file=\"$rout\", device=jpeg, width=600)\n";	
	
		close (PBSNEW);
	}
	
	print "now collecting all param files\n";
	print "perl collector.pl\n";
	
	
	#########################
	# Mis-inheritance Check
	#########################
	# usage &misinherit($filename, $filename2);
	sub misinherit
	{
		$inputfile = @_[0];
		open (GENOTYPES, "<$inputfile") || die "Cannot open the genotypes file $inputfile!"; 
		open (MISIN, ">misinheritance.txt") || die "Cannot open output file!";
		@genotypes = <GENOTYPES>; chomp (@genotypes); close (GENOTYPES);
		$namer= "$inputfile" . "_clean.txt";
		open (OUTPUT, ">$namer") || die "Cannot open the genotypes file $inputfile!"; 
		
		@tarray=split("\t", $genotypes[$#genotypes-1]);
		$people_genotyped = ($#tarray - $genoloc);
		#let's see who we can check for mis-inheritance - build family relationships [$row][$column]
		for ($h=0; $h<=$#ped; $h++){
			@kid=split("\t", $ped[$h]);
			$pcounter=0;
			$file_start=(0);
			
			for ($i=0; $i<=$#ped; $i++){
				@possible_parent=split("\t", $ped[$i]);
				#we make a 2D array, rows are two parents, columns are kids
				$fileline=($i+1);  #This places the actual line number for the family member into the array
				if ($possible_parent[1] == $kid[2]){$parent[$h][$pcounter]=$fileline; $pcounter++;}	
				elsif ($possible_parent[1] == $kid[3]){$parent[$h][$pcounter]=$fileline; $pcounter++;}
			}
		}
		print "$#genotypes SNPs on the array, $people_genotyped people genotyped, pedigree built\nNow checking for misinheritance\n";
	
		for ($n=0; $n<=$#genotypes; $n++){
			
			@ttype=split("\t", $genotypes[$n]);
			
			for ($p=0; $p<$people_genotyped; $p++){
				$ktype=$ttype[$p+$genoloc];
	
				if ($parent[$p][0] > 0 and $parent[$p][1] > 0){
		#			print "parents of $ktype[1] are $parent[$p][1] and $parent[$p][0], next is $parent[$p+1][0]\n";
					$ptype1= $ttype[($genoloc + ($parent[$p][0])-1)];
					$ptype2= $ttype[($genoloc + ($parent[$p][1])-1)];
		
					#If either parent is BB, the kid can't be AA
					if ($ptype1 eq "BB" and $ktype eq "AA" or $ptype2 eq "BB" and $ktype eq "AA"){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#If either parent is AA, the kid can't be BB
					elsif ($ptype1 eq "AA" and $ktype eq "BB" or $ptype2 eq "AA" and $ktype eq "BB"){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#If both parents are BB, the kid can't be A anything
					elsif ($ptype1 eq "BB" and $ptype2 eq "BB" and $ktype =~ m/A/){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#If both parents are AA, the kid can't be B anything
					elsif ($ptype1 eq "AA" and $ptype2 eq "AA" and $ktype =~ m/B/){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#If parents are homozygous AA,BB, the kid must be heterozygous
					elsif ($ptype1 eq "AA" and $ptype2 eq "BB" and $ktype eq "AA" or $ptype1 eq "AA" and $ptype2 eq "BB" and $ktype eq "BB" ){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#If parents are homozygous BB,AA, the kid must be heterozygous
					elsif ($ptype1 eq "BB" and $ptype2 eq "AA" and $ktype eq "AA" or $ptype1 eq "BB" and $ptype2 eq "AA" and $ktype eq "BB" ){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#No call and BB means the kid can't be AA
					elsif ($ptype1 eq "NoCall" and $ptype2 eq "BB" and $ktype eq "AA" or $ptype1 eq "NoCall" and $ptype2 eq "AA" and $ktype eq "BB" ){$ttype[$p+$genoloc] = "NoCall";print MISIN"$n\n";} 
					#No call and AA means the kid can't be BB
					elsif ($ptype2 eq "NoCall" and $ptype1 eq "BB" and $ktype eq "AA" or $ptype2 eq "NoCall" and $ptype1 eq "AA" and $ktype eq "BB" ){$ttype[$p+$genoloc] = "NoCall";print MISIN"$n\n";}
				}			
				elsif ($parent[$p][0] > 0){
					$ptype= $ttype[($genoloc + ($parent[$p][0] - 1))];
					#BB means the kid can't be AA
					if ($ptype eq "BB" and $ktype eq "AA"){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#AA means the kid can't be BB
					if ($ptype eq "AA" and $ktype eq "BB"){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
				
				}
				elsif ($parent[$p][1] > 0){
					$ptype= $ttype[($genoloc + ($parent[$p][1] - 1))];
					#BB means the kid can't be AA
					if ($ptype eq "BB" and $ktype eq "AA"){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
					#AA means the kid can't be BB
					if ($ptype eq "AA" and $ktype eq "BB"){$ttype[$p+$genoloc]= "NoCall";print MISIN"$n\n";} 
				
				}
			}
			foreach $part (@ttype){print OUTPUT "$part\t";}print OUTPUT"\n";
			
		}
		close(OUTPUT);
	}
	print "\nFINISHED!.  Look into the directory for your new files.\n";

}

if ($compare_alg == 1){

	######################################################
	#
	#	Call Matches Program
	# Call_matches.v2.pl
	# VFS 6/9/09
	#
	# Assumptions: For measuring switches, any switch of 
	# direction in the second file is counted, regardless 
	# of what percent overlap the call had with the baseline
	# call. 2s from QuantiSNP are treated as duplications.
	# Note that this is different from the allow_switches
	# option, which determines whether bi-directional calls
	# are even included in the comparison set.  If you include
	# only one direction of cals (dups or dels), you should not
	# get any switches registered by the program, regardless of
	# whether allow_switches is enabled.
	# For the overlap calculation, the program
	# calculates the total coverage of calls by 1, 2, or 3
	# algorithms.  If the coverage by 3 algorithms meets the
	# minimum, then it reports a 3 algorithm confirmation.
	# If not, it checks whether 2 algorithm calls covered
	# minimum percent coverage. Then 1 algorithm calls.  An
	# original call can only be confirmed once.
	# The master sample list should just be two columns of
	# corresponding sample IDs, without a header.  You can also
	# just change the headers in the comparison files and omit
	# a sample ID file.
	#
	#####################################################
	
	#use strict;
	
	########################################################
	# OPTIONS
	########################################################
	my $type_option = 0;
	my $allow_switches = 0;
	my $onlyX = 0;
	my $include_XYM = 0;
	use Getopt::Long;
	GetOptions(  'type' => \$type_option, "allow_switches" => \$allow_switches, "onlyX" => \$onlyX, "include_XYM" => \$include_XYM);
	
	########################################################
	# DEFAULT VARIABLES
	########################################################
	# List of threshold variables
	my $percent_overlap_threshold = .0; 			# This number is inclusive - CNVs with percent overlap that is >= the number are counted
	my $SNPs_threshold_orig = 0;							# SNP threshold for the original file. This number is inclusive - CNVs with SNPs >= the number are counted
	my $SNPs_threshold_comp = 0;							# SNP threshold for the confirmation file. This number is inclusive - CNVs with SNPs >= the number are counted
	my $SNPs_max_orig = 10000000;							# SNP threshold for the original file. This number is inclusive - CNVs with SNPs <= the number are counted
	my $SNPs_max_comp = 10000000;							# SNP threshold for the confirmation file. This number is inclusive - CNVs with SNPs <= the number are counted
	
	
	# List of filenames
	my $first_file = "$ARGV[0]"; 										# INPUT 1 - Comparison file number 1
	my $second_file = "$ARGV[1]"; 									# INPUT 2 - Comparison file number 2
	my $ids_file;
	if (defined $ARGV[2]) {
		$ids_file = "$ARGV[2]"; 											# INPUT 3 - Master sample IDs file
	}
	
	# List of Important columns - The columns must be the same for the both input files
	my $sample_id_col = 5;
	my $chr_col = 6;
	my $start_col = 7;
	my $end_col = 8;
	my $type_col = 9;
	my $SNPs_col = 11;
	my $num_algs_col = 13;
	
	########################################################
	# OPTION DEFINITIONS
	########################################################
	
	# TYPE_OPTION - Allow the user to set the type of CNVs considered
	my $input = "01234";
	if ($type_option == 1) {
		print "Please specify the types of CNV calls you would like considered (you can enter more than 1):\n";
		print "0 - Homozygous deletion\n1 - Heterozygous deletion\n2 - QuantiSNP duplication in male X Chromosome\n3 - Heterozygous Duplication\n4 - Homozygous Duplication\n";
		print "Enter numbers: ";
		$input = <STDIN>;
		print "\nOnly including:\n";
		if ($input =~ /0/i) {print "Homozygous deletions\n";}
		if ($input =~ /1/i) {print "Heterozygous deletions\n";}
		if ($input =~ /2/i) {print "QuantiSNP duplications in male X chromosome\n";}
		if ($input =~ /3/i) {print "Heterozygous Duplications\n";}
		if ($input =~ /4/i) {print "Homozygous Duplications\n";}
	}
	
	# ALLOW_SWITCHES - Allows the user to include calls that have switches
	my $dont_allow_switches = 1;
	if ($allow_switches == 1) {
		$dont_allow_switches = 0;
		print "\nAllow_switches option turned on - Calls in both directions will be included\n"
	} else {
		print "Allow_switches option off (default) - Calls in both directions will not be included\n";
	}
	
	# ONLY_X - Allows the user to only test X chromosome calls - implemented below
	# INCLUDE_XYM - Allows the user to include the XYM chromosomes in the analysis - implemented below
	
	########################################################
	#
	# MAIN BODY OF PROGRAM
	#
	########################################################
	
	########################################################
	# ERROR CHECKING
	########################################################
	if (!defined $ARGV[2]) {
		print "\nBaseline file: $first_file\nComparison file: $second_file\nMaster ID list: none provided\n\n";
	}
	else {print "\nBaseline file: $first_file\nComparison file: $second_file\nMaster ID list: $ids_file.\n\n";}
	
	########################################################
	# INITIAL DATA PROCESSING - LOADING FILES
	########################################################
	
	print "Loading file $first_file\n";
	# Open file and put contents into an array - each row is a line in the array
	open(INPUT_ONE, "<$first_file" || die "couldn't open $first_file\n");
	my @full_file_one = <INPUT_ONE>;
	close INPUT_ONE;
	
	print "Loading file $second_file\n";
	# Open file and put contents into an array - each row is a line in the array
	open(INPUT_TWO, "<$second_file" || die "couldn't open $second_file\n");
	my @full_file_two = <INPUT_TWO>;
	close INPUT_TWO;
	
	# MASTER SAMPLE ID FILE
	my %sample_id_hash = ();
	# If a master sample ID file was provided, format, otherwise go to the else statement
	if (defined $ARGV[2]) {
		# Convert sample ID numbers in both files to one final list
		print "Loading sample ID master list in file $ids_file\n";
		open(IDS, "<$ids_file" || die "couldn't open $ids_file\n");
		my @id_master_list = <IDS>;
		close IDS;
		# Limited header detection for Master ID 
		if ($id_master_list[0] =~ /"1M"/i or $id_master_list[0] =~ /duo/i) { print "Header detected in master sample ID file\n"; shift @id_master_list; }
		# Create master ID number hash
		foreach (@id_master_list) {
			chomp $_;
			my @id_line = split(/\t/);
			$sample_id_hash{ $id_line[0] } = $id_line[1];
		}
	}
	else {
		print "No master sample ID file provided, assuming that sample IDs in the two files correspond already.\n";
	}
	
	print "\n";
	
	# shift @full_file_one;
	# shift @full_file_two;
	
	#############################################
	# Delete newline characters from each line and change chromosome data to numbers to make sorting effective
	#############################################
	
	# Format first file
	print "Processing and formatting $first_file\n";
	my @temp_array = ();
	my $changes_made = 0;
	foreach (@full_file_one) { 
		chomp;
		my @line = split (/\t/, $_);
		$line[$chr_col] =~ s/chr//i; 												# Remove "chr" tag from chromosome column and format X Y M
		if ($line[$chr_col] eq "X") { $line[$chr_col] = 23 };
		if ($line[$chr_col] eq "Y") { $line[$chr_col] = 24 };
		if ($line[$chr_col] eq "M" || $line[$chr_col] eq "MT") { $line[$chr_col] = 25 };
		if ($line[$chr_col] eq "XY") { $line[$chr_col] = 27 };
		if (defined $sample_id_hash{ $line[$sample_id_col] }) { 
			$line[$sample_id_col] = $sample_id_hash{ $line[$sample_id_col] }; 
			$changes_made = 1;
		}
		
		# This sequence excludes calls that go in both directions, and only includes calls that the user has specified
		my $include_this_line = 0;
		my $type_var = $line[$type_col];
		if ($line[$type_col] =~ /1/ || $line[$type_col] =~ /0/) { $type_var = "del";}
		if ($line[$type_col] =~ /2/ || $line[$type_col] =~ /3/ || $line[$type_col] =~ /4/) { 
				if ($type_var eq "del") { $type_var = "both"; }
				else {$type_var = "dup";}
		}
		
		
		my $type_line = $line[$type_col]; 	
		$type_line =~ s/\"//ig;			
		my @types = split(/,/, $type_line);   		# Array of the types of call
		foreach (@types) {
			if ($input =~ /$_/i) { $include_this_line = 1;}																		# Tests to see if the target call contains one of the types allowed by $input
		}
		
		if ($dont_allow_switches && $type_var eq "both") {$include_this_line = 0;}					# Remove bi-directional calls
		# if ($include_this_line == 0) { print "type_var = $type_var, $line[$type_col]\n";} # DEBUGGING TOOL
		
		if ($onlyX == 1 && !($line[$start_col] =~ /start/i) && $line[$chr_col] != 23) {$include_this_line = 0;}														# Implement ONLY_X option
		if ($onlyX == 0 && $include_XYM == 0 && !($line[$start_col] =~ /start/i) && $line[$chr_col] >= 23) {$include_this_line = 0;}			# Implement INCLUDE_XYM option
		
		if (!($line[$start_col] =~ /start/i)) {																							# Remove headers
			if ($line[$SNPs_col] >= $SNPs_threshold_orig && $line[$SNPs_col] <= $SNPs_max_orig) {		# Implement size threshold
				if ($include_this_line) {																												# Implement type control and don't allow switches control
					push ( @temp_array , join("\t", @line) );
				}
			}
		}
		else {
			print "Header detected in $first_file\n";
		}
	}
	if ($changes_made == 1) { print "Sample IDs converted in $first_file\n"; $changes_made = 0;}
	
	# I put things back into the full_file_one array to avoid yet another confusing name change for the same data
	@full_file_one = @temp_array;
	
	# Format second file
	print "Processing and formatting $second_file\n";	
	@temp_array = ();
	foreach (@full_file_two) { 
		chomp;
		my @line = split (/\t/, $_);
		$line[$chr_col] =~ s/chr//i; 												# Remove "chr" tag from chromosome column and format X Y M
		if ($line[$chr_col] eq "X") { $line[$chr_col] = 23 };
		if ($line[$chr_col] eq "Y") { $line[$chr_col] = 24 };
		if ($line[$chr_col] eq "M" || $line[$chr_col] eq "MT") { $line[$chr_col] = 25 };
		if ($line[$chr_col] eq "XY") { $line[$chr_col] = 27 };
		if (defined $sample_id_hash{ $line[$sample_id_col] }) { 
			$line[$sample_id_col] = $sample_id_hash{ $line[$sample_id_col] }; 
			$changes_made = 1;
		} 
		
		# This sequence excludes calls that go in both directions, and only includes calls that the user has specified
		my $include_this_line = 0;
		my $type_var = $line[$type_col];
		if ($line[$type_col] =~ /1/ || $line[$type_col] =~ /0/) { $type_var = "del";}
		if ($line[$type_col] =~ /2/ || $line[$type_col] =~ /3/ || $line[$type_col] =~ /4/) { 
				if ($type_var eq "del") { $type_var = "both"; }
				else {$type_var = "dup";}
		}
		
		
		my $type_line = $line[$type_col]; 	
		$type_line =~ s/\"//ig;			
		my @types = split(/,/, $type_line);   		# Array of the types of call
		foreach (@types) {
			if ($input =~ /$_/i) { $include_this_line = 1;}																		# Tests to see if the target call contains one of the types allowed by $input
		}
		
		if ($dont_allow_switches && $type_var eq "both") {$include_this_line = 0;}					# Remove bi-directional calls
		# if ($include_this_line == 0) { print "type_var = $type_var, $line[$type_col]\n";} # DEBUGGING TOOL
	
		if ($onlyX == 1 && !($line[$start_col] =~ /start/i) && $line[$chr_col] != 23) {$include_this_line = 0;}														# Implement ONLY_X option
		if ($onlyX == 0 && $include_XYM == 0 && !($line[$start_col] =~ /start/i) && $line[$chr_col] >= 23) {$include_this_line = 0;}			# Implement INCLUDE_XYM option
			
		if (!($line[$start_col] =~ /start/i)) {																							# Remove headers
			if ($line[$SNPs_col] >= $SNPs_threshold_comp && $line[$SNPs_col] <= $SNPs_max_comp) {	# Implement size threshold
				if ($include_this_line) {																												# Implement type control and don't allow switches control
					push ( @temp_array , join("\t", @line) );
				}
			}
		}
		else {
			print "Header detected in $second_file\n";
		}
	}
	if ($changes_made == 1) { print "Sample IDs converted in $second_file\n";}
	print "\n";
	
	# I put things back into the full_file_two array to avoid yet another confusing name change for the same data
	@full_file_two = @temp_array;
	
	# Sort by start location, then by chromosome number, then by sample id
	@full_file_one = sort { (split /\t/, $a)[$start_col] <=> (split /\t/, $b)[$start_col] } @full_file_one;
	@full_file_one = sort { (split /\t/, $a)[$chr_col] <=> (split /\t/, $b)[$chr_col] } @full_file_one;
	@full_file_one = sort { (split /\t/, $a)[$sample_id_col] cmp (split /\t/, $b)[$sample_id_col] } @full_file_one;
	
	# Sort by start location, then by chromosome number, then by sample id
	@full_file_two = sort { (split /\t/, $a)[$start_col] <=> (split /\t/, $b)[$start_col] } @full_file_two;
	@full_file_two = sort { (split /\t/, $a)[$chr_col] <=> (split /\t/, $b)[$chr_col] } @full_file_two;
	@full_file_two = sort { (split /\t/, $a)[$sample_id_col] cmp (split /\t/, $b)[$sample_id_col] } @full_file_two;
	
	
	# Post sorting and editing, place the data into array of arrays for analysis purposes
	my (@original_array, @compare_array , @tmp) = ();
	my $line;
	foreach $line (@full_file_one) {
		      @tmp = split(/\t/ , $line);
	        push @original_array, [ @tmp ];
	}
	foreach $line (@full_file_two) {
		      @tmp = split(/\t/ , $line);
	        push @compare_array, [ @tmp ];
	}
	
	##########################################################################
	#
	# DATA PROCESSING
	#
	##########################################################################
	
	print "Comparing $first_file with $second_file\n\n";
	
	# VARIABLES
	my $track_one = 0;												# Master counter for moving through the original file
	my @hits_counter_array = ([0,0,0,0], [0,0,0,0], [0,0,0,0]);  # Master array to count confirmed calls
	my $switches = 0;													# Counter the number of times that the direction of a call changed in the comparison file
	# Note: the bp_overlap variables are global so they can be manipulated by subroutines
	my $bp_overlap1 = 0;									# Tracks the number of basepairs of the original CNV call confirmed by 1 algorithm calls
	my $bp_overlap2 = 0;									# Tracks the number of basepairs of the original CNV call confirmed by 2 algorithm calls
	my $bp_overlap3 = 0;									# Tracks the number of basepairs of the original CNV call confirmed by 3 algorithm calls
	
	
	my @total_size = (0,0,0);
	my @overall_bp_overlap = ([0,0,0], [0,0,0], [0,0,0]);
	
	# While loop that cycles through the original file, one CNV call at a time.
	while ($track_one < @original_array) {
			my $track_two = 0;										# Counter for the comparison file, resets with each new CNV call from the original file.
			
			# Advance in comparison file until sample IDs match, chromosome numbers are correct, and the start location for 
			# the loci of original chip is before the end location of the comparison loci.  Also checks that the end of the
			# comparison file has not been reached.
			while ($track_two < @compare_array && $original_array[$track_one][$sample_id_col] ne $compare_array[$track_two][$sample_id_col]) {$track_two++;}
			while ($track_two < @compare_array 
						 && $original_array[$track_one][$sample_id_col] eq $compare_array[$track_two][$sample_id_col] 
						 && $original_array[$track_one][$chr_col] > $compare_array[$track_two][$chr_col]) {
				$track_two++;
			}
			while ($track_two < @compare_array 
			&& $original_array[$track_one][$sample_id_col] eq $compare_array[$track_two][$sample_id_col] 
						 && $original_array[$track_one][$chr_col] == $compare_array[$track_two][$chr_col] 
						 && $original_array[$track_one][$start_col] > $compare_array[$track_two][$end_col])  {
					$track_two++;
			}
			# print "first set of while loops completed, track two at $track_two\n";
			
			# PRIMARY ANALYSIS LOOP - continues tracking switches and overap until it reaches a comparison CNV that
			# starts after the end of the original CNV being confirmed) or the end of the comparison file).
			while ($track_two < @compare_array
			&& $original_array[$track_one][$sample_id_col] eq $compare_array[$track_two][$sample_id_col]
			&& $original_array[$track_one][$chr_col] == $compare_array[$track_two][$chr_col]
			&& $original_array[$track_one][$end_col] > $compare_array[$track_two][$start_col])  {
						eval_overlap($original_array[$track_one][$start_col], $original_array[$track_one][$end_col],$compare_array[$track_two][$start_col],$compare_array[$track_two][$end_col],$compare_array[$track_two][$num_algs_col]);
						eval_del_dup($original_array[$track_one][$type_col], $compare_array[$track_two][$type_col]);
						$track_two++;
				}
	
			# print "finished processing line $track_one, track two is $track_two\n";
			# print "number of algorithm calls is $original_array[$track_one][13]\n\n";
			
			# Send the size to the subroutine eval_hit, which basically decides how many algorithms (if any) confirmed the CNV call
			my $size = ($original_array[$track_one][$end_col]-$original_array[$track_one][$start_col]);
			eval_hit($size);
			
			# Add to total counter variables, move to the next CNV call, and reset variables
			$total_size[$original_array[$track_one][$num_algs_col]-1] += $size;
			$overall_bp_overlap[$original_array[$track_one][$num_algs_col]-1][0] += $bp_overlap1;
			$overall_bp_overlap[$original_array[$track_one][$num_algs_col]-1][1] += $bp_overlap2;
			$overall_bp_overlap[$original_array[$track_one][$num_algs_col]-1][2] += $bp_overlap3;
			
			$track_one++;
			$bp_overlap1 = 0;									# Reset bp_overlap variables
			$bp_overlap2 = 0;
			$bp_overlap3 = 0;
			
			# When finished with a sample, print the sample ID and the spreadsheet line.  Also, note when the last sample ID has been reached.
			my $prev_track = $track_one-1;
			if ($prev_track == @original_array - 1) {
				print "Processed last CNV call for $original_array[$prev_track][$sample_id_col] at spreadsheet line $track_one\nLast sample ID detected.\n\n";
			}
			elsif ($prev_track < @original_array - 1 && ($original_array[$prev_track][$sample_id_col] ne $original_array[$track_one][$sample_id_col])) {
						print "Processed last CNV call for $original_array[$prev_track][$sample_id_col] at spreadsheet line $track_one\n";
				} 
	}
	
	##########################################################################
	#
	# OUTPUT FILES
	#
	##########################################################################
	
	# Output file
	my $first_file_abbrev = $first_file;
	$first_file_abbrev =~ s/.txt//ig;
	my $second_file_abbrev = $second_file;
	$second_file_abbrev =~ s/.txt//ig;
	my $output_file = "baseline_" . $first_file_abbrev . "_comp_to_" . $second_file_abbrev . ".txt";
	
	print "Outputting results to $output_file\n";
	open(OUTPUT, ">$output_file") || die "Could not open $output_file; may already be open\n";
	print LISTOUT $curr_dir."$output_file\n";
	print OUTPUT "Baseline file: $first_file\nCompared to: $second_file\n\n";
	print OUTPUT "Minimum number of SNPs in baseline file for the CNV to be tested: $SNPs_threshold_orig\n";
	print OUTPUT "Maximum number of SNPs in baseline file for the CNV to be tested: $SNPs_max_orig\n";
	print OUTPUT "Minimum number of SNPs in comparison file for the CNV to be tested: $SNPs_threshold_comp\n";
	print OUTPUT "Maximum number of SNPs in comparison file for the CNV to be tested: $SNPs_max_comp\n";
	print OUTPUT "Minimum base pair percent overlap to be counted as confirmation: $percent_overlap_threshold\n";
	print OUTPUT "Types of call's included: ";
	if ($input =~ /0/i) {print OUTPUT "Homozygous deletions, ";}
	if ($input =~ /1/i) {print OUTPUT "Heterozygous deletions, ";}
	if ($input =~ /2/i) {print OUTPUT "QuantiSNP duplications in male X chromosome, ";}
	if ($input =~ /3/i) {print OUTPUT "Heterozygous Duplications, ";}
	if ($input =~ /4/i) {print OUTPUT "Homozygous Duplications ";}
	
	if ($include_XYM == 1) {
		print OUTPUT "\nXYM Chromosomes included: yes\n"
	} else {
		print OUTPUT "\nXYM Chromosomes included: no (default)\n";
	}
	
	if ($onlyX == 1) {
		print OUTPUT "X Chromosome only option activated\n"
	}
	
	if ($allow_switches == 1) {
		print OUTPUT "Switches allowed: yes\n"
	} else {
		print OUTPUT "Switches allowed: no (default)\n";
	}
	print OUTPUT "\n";
	
	# OUTPUT TABLE BASED ON NUMBER OF CALLS
	print OUTPUT "NUMBER OF CALLS CONFIRMED\n";
	print OUTPUT "Number of Calls Confirmed\t1 Alg Confirmed\t2 Alg Confirmed\t3 Alg Confirmed\t\tTotal calls confirmed\tUnconfirmed\tTotal Calls\n";
	print "\nNumber of Calls Confirmed\n";
	print "\t\t1\t2\t3\t\tTConf\tTUnconf\tTCalls\n";
	
	my $t_confirmed = 0;
	my $t_unconfirmed = 0;
	my $t_calls = 0;
	my $c=1;
	my @percent_calls_confirmed = ([0,0,0,0,0],[0,0,0,0,0],[0,0,0,0,0]);
	foreach (@hits_counter_array) {
		# Some final analysis
		my $total_confirmed = @$_[1] + @$_[2] + @$_[3];
		my $total_calls = $total_confirmed + @$_[0]; 
		my $total_unconfirmed = shift @$_;
		
		$t_confirmed += $total_confirmed;
		$t_unconfirmed += $total_unconfirmed;
		$t_calls += $total_calls;
		
		push (@$_, "", $total_confirmed, $total_unconfirmed, $total_calls);
	
	 	# Do calculations for the next table	
		if ($total_calls != 0) {
			$percent_calls_confirmed[$c - 1][0] = @$_[0] / $total_calls;
			$percent_calls_confirmed[$c - 1][1] = @$_[1] / $total_calls;
			$percent_calls_confirmed[$c - 1][2] = @$_[2] / $total_calls;
			$percent_calls_confirmed[$c - 1][3] = $total_unconfirmed / $total_calls;
			$percent_calls_confirmed[$c - 1][4] = $total_confirmed / $total_calls;
			}
		
		my $l = join ("\t", @$_);
		print OUTPUT "$c Alg Calls\t$l\n";
		print "$c Alg Calls\t$l\n";
		
		$c++;
	}
	print OUTPUT "\t\t\t\tTotals\t$t_confirmed\t$t_unconfirmed\t$t_calls\n";
	print "\t\t\t\t\tTotals\t$t_confirmed\t$t_unconfirmed\t$t_calls\n";
	
	# OUTPUT TABLE THAT SUMMARIZES PERCENT OF CALLS CONFIRMED
	print OUTPUT "\nPERCENTAGE OF CALLS CONFIRMED\n";
	print OUTPUT "Number of Calls Confirmed\t1 Alg Confirmed\t2 Alg Confirmed\t3 Alg Confirmed\t\tTotal % Confirmed\t% Calls Unconfirmed\n";
	print "\nPercentage of Calls Confirmed\n";
	print "\t\t1\t2\t3\t\tT%Conf\t%Unconf\n";
	
	$c = 1;
	foreach (@percent_calls_confirmed) {
		foreach (@$_) {
			$_ = sprintf("%.1f", $_ * 100) . "%";
		}
		my $confirmed_percent = pop @$_;
		my $unconfirmed_percent = pop @$_;
		push (@$_, "", $confirmed_percent, $unconfirmed_percent);
		
		my $l = join ("\t", @$_);
		print OUTPUT "$c Alg Calls\t$l\n";
		print "$c Alg Calls\t$l\n";
		
		$c++;
	}
	
	
	# OUTPUT TABLE BASED ON PERCENT BASE PAIRS CONFIRMED
	print OUTPUT "\nPERCENTAGE OF BASE PAIRS CONFIRMED\n";
	print OUTPUT "Original Calls\t1 Alg Confirmed\t2 Alg Confirmed\t3 Alg Confirmed\tTotal Percent Confirmed\n";
	print "\nPercentage of Base Pairs Confirmed\n";
	print "\t\t1\t2\t3\tT BP Conf\n";
	
	$c = 1;
	foreach (@overall_bp_overlap) {
		if ($total_size[$c - 1] != 0) { @$_[0] /= $total_size[$c - 1]; }
		if ($total_size[$c - 1] != 0) { @$_[1] /= $total_size[$c - 1]; }
		if ($total_size[$c - 1] != 0) { @$_[2] /= $total_size[$c - 1]; }
		
		my $total_percent = sprintf("%.1f", (@$_[0] + @$_[1] + @$_[2]) * 100) . "%";
		push (@$_, $total_percent);
		
		@$_[0] = sprintf("%.1f", @$_[0] * 100) . "%";
		@$_[1] = sprintf("%.1f", @$_[1] * 100) . "%";	
		@$_[2] = sprintf("%.1f", @$_[2] * 100) . "%";
			
		my $l = join ("\t", @$_);
		print OUTPUT "$c Alg Calls\t$l\n";
		print "$c Alg Calls\t$l\n";
		
		$c++;
	}
	
	my $file_two_size = $#full_file_two + 1;
	
	print OUTPUT "\nNumber of switches detected: $switches out of $file_two_size\n";
	print "\nNumber of switches detected: $switches out of $file_two_size\n";
	close OUTPUT;
	
	##########################################################################
	#
	# SUBROUTINES
	#
	##########################################################################
	
	# Eval_hit looks at the overlap by 1,2,and 3 algorithm calls, and determines whether the original call was confirmed
	sub eval_hit {
			my $sz = $_[0];
			# print "size: $sz\n";
			# print "1: $bp_overlap1\n";
			# print "2: $bp_overlap2\n";
			# print "3: $bp_overlap3\n";
			my $overlap1 = $bp_overlap1 / $sz;
			my $overlap2 = $bp_overlap2 / $sz;
			my $overlap3 = $bp_overlap3 / $sz;
			# These if statements move from 3 to 2 to 1 to 0 to make sure that an original call can only be confirmed once.
			# It knows the number of calls for the original CNV by referencing the original array number of calls column (minus 1
			# to match up with the array, and it knows the number of calls and percent coverage based on the bp_overlap variable
			# it is considering.
			if ($overlap3 > $percent_overlap_threshold) {
				$hits_counter_array[$original_array[$track_one][$num_algs_col]-1][3]++;
				# print "3 counter to $hits_counter_array[$original_array[$track_one][$num_algs_col]-1][3]\n";
			}
			elsif ($overlap2 > $percent_overlap_threshold) {
				$hits_counter_array[$original_array[$track_one][$num_algs_col]-1][2]++;
			}
			elsif ($overlap1 > $percent_overlap_threshold) {
				$hits_counter_array[$original_array[$track_one][$num_algs_col]-1][1]++;
			}
			else { $hits_counter_array[$original_array[$track_one][$num_algs_col]-1][0]++; }
	}
	
	# eval_overlap takes the original start, original end, comparison start, comparison end, 
	# and number of calls of the comparison algorithm as arguments. It calculates the number
	# of base pairs overlapping and adds that number to the correct variable, depending on how
	# many algorithms predicted the comparison call.
	sub eval_overlap {
		my $orig_start = $_[0];
		my $orig_end = $_[1];
		my $size = $orig_end - $orig_start;
		my $comp_start = $_[2];
		my $comp_end = $_[3];
		my $calls = $_[4];
		if ($calls == 1) {
			if ($orig_start <= $comp_start && $orig_end <= $comp_end) { $bp_overlap1 += ( $orig_end - $comp_start ); }
			elsif ($orig_start >= $comp_start && $orig_end <= $comp_end) { $bp_overlap1 += $size; }
			elsif ($orig_start >= $comp_start && $orig_end >= $comp_end) { $bp_overlap1 += ( $comp_end - $orig_start ); }
			elsif ($orig_start <= $comp_start && $orig_end >= $comp_end) { $bp_overlap1 += ( $comp_end - $comp_start ); }
		}
			if ($calls == 2) {
			if ($orig_start <= $comp_start && $orig_end <= $comp_end) { $bp_overlap2 += ( $orig_end - $comp_start ); }
			elsif ($orig_start >= $comp_start && $orig_end <= $comp_end) { $bp_overlap2 += $size; }
			elsif ($orig_start >= $comp_start && $orig_end >= $comp_end) { $bp_overlap2 += ( $comp_end - $orig_start ); }
			elsif ($orig_start <= $comp_start && $orig_end >= $comp_end) { $bp_overlap2 += ( $comp_end - $comp_start ); }
		}
			if ($calls == 3) {
			if ($orig_start <= $comp_start && $orig_end <= $comp_end) { $bp_overlap3 += ( $orig_end - $comp_start ); }
			elsif ($orig_start >= $comp_start && $orig_end <= $comp_end) { $bp_overlap3 += $size; }
			elsif ($orig_start >= $comp_start && $orig_end >= $comp_end) { $bp_overlap3 += ( $comp_end - $orig_start ); }
			elsif ($orig_start <= $comp_start && $orig_end >= $comp_end) { $bp_overlap3 += ( $comp_end - $comp_start ); }
		}
	}
	# eval_del_dup checks for directional switches in the type of call.  If there has been a switch, it adds one to $switches.
	sub eval_del_dup {
		my $orig_type = $_[0];
		my $comp_type = $_[1]; 
		if ($_[0] =~ /1/ || $_[0] =~ /0/) { $orig_type = "del";}
		if ($_[0] =~ /2/ || $_[0] =~ /3/ || $_[0] =~ /4/) { 
				if ($orig_type eq "del") { $orig_type = "both"; }
				else {$orig_type = "dup";}
			}
		if ($_[1] =~ /1/ || $_[1] =~ /0/) { $comp_type = "del";}
		if ($_[1] =~ /2/ || $_[1] =~ /3/ || $_[1] =~ /4/) { 
				if ($comp_type eq "del") { $comp_type = "both"; }
				else {$comp_type = "dup";}
			}
		if ($comp_type ne $orig_type) { 
			# print "orig: $orig_type, comp: $comp_type\n";					# DEBUGGING TOOL
			$switches++;
		}
	}

}

#Makes a plink input file from the FinalReports listed in a pedfile
if ($plinkmaker == 1){
	mkdir ("07_Plink", 0777);
	&fr_columns;
	$filemarker = $namematch;
	print LISTOUT "$plink_dir\n";

	print "=============================================================================\n                             PLINK file maker\n=============================================================================\n";
	print "Loading pedfile: $ARGV[0]\n";
	
	###################################################################################################################################################################
	###################################################################################################################################################################
	
	$input = $ARGV[0];
	#first put the ped file
	open (PEDS, "<$input") || die "Cannot open the file of the input, six-column ped file, $input.  Please check to be sure it isn't open\n";
	@ped=<PEDS>; chomp (@ped);
	$extraout = $plink_dir."Alleles_per_SNP.txt";
	open (EXTRA, ">$extraout") || die "Could not open $extraout; may already be open\n";
	print "Loading family structure...";
	foreach(@ped){
		chomp;
		@here = split("\t", $_);
		chomp(@here);
		$pedfile{$here[1]} = $_;
	}
	@pedfile = keys %pedfile;
	$pedfilesize = @pedfile;
	print "$pedfilesize individuals found\n";
	
	$filecount=0;
	@files =<*>;
	$input =~ s/.txt//gi;
	$input = $plink_dir.$input;
	
	open (PEDIND, ">$input.ind") || die "Could not open the file of the output file $input.ind, may already be open";
	open (PED, ">$input.ped") || die "Could not open the file of the output file $input.ped, may already be open";
	open (MAPFILE, ">$input.map") || die "Could not open the file of the output file $input.map, may already be open";
	$xchr= "	X	";
	$xychr= "	XY	";
	$ychr= "	Y	";
	$xrep= "	23	";
	$yrep= "	24	";
	$xyrep= "	25	";


	unless ($only_these_snps eq ''){
		open (THESE, "<$only_these_snps") || "Could not open the list of restricted SNPs, $only_these_snps\n";
		while (<THESE>){
			chomp;
			$goodsnps{$_}="yes";
		}
	}
	if ($largefile eq ''){
		
		###########################
		# Analysis of the cases
		###########################
		foreach (@files){
			if ($_ =~ /$filemarker/){
				$frcount++;
			}
		}
		$totalfr = $frcount;
		$snperror = 0;
		print "Loading first FinalReport file to act as a map file template\n";
		foreach $file (@files){
			if ($file =~ /$filemarker/){
				#print "I'm here\n";
				$filecount++;
				$nocalls=0;
				open (NEWFILE, "<$file" || die "Could not open FinalReport file $file\n");
				
				#Check the call rate for the chip
				$snpcount=0; 
				%snphash='';
				
				while(<NEWFILE>){
					@temp=split("\t", $_); chomp(@temp);
					if ($temp[$snpcol] =~ /rs/){
						$patname = $temp[$patientid];
						last;
					} 
				}
				@counter = split ("\t", $pedfile{$patname});
				#If the pedfile for this patient is longer than two columns...
				if ($#counter > 2){
					#print "Do I ever get here?\n";
					$cases++;
					print PEDIND "$pedfile{$patname}\t$file\n";
					
					chomp($pedfile{$patname});
					
					print PED "$pedfile{$patname} ";
					
					seek(NEWFILE, 0, 0);
					while(<NEWFILE>){
						@temp=split("\t", $_); chomp(@temp);
						$snp=$temp[$snpcol];
						$temp[$gtcol1] =~ s/-/0/gi;
						$temp[$gtcol2] =~ s/-/0/gi;
						if ($gtpresent{$snp} =~ $temp[$gtcol1]
						and $gtpresent{$snp} =~ $temp[$gtcol2]){
							$snphash{$snp} = "$temp[$gtcol1] $temp[$gtcol2] ";
						}
						elsif ($temp[$gtcol1] eq 0){
							$snphash{$snp} = "0 0 ";
						}
						elsif ($gtpresent{$snp} =~ /Three/i){
							$snphash{$snp} = "0 0 ";
						}
						else {
							@gtpresent = split (//,$gtpresent{$snp});
							$sizehere = @gtpresent;
							#If there are already two alleles present
							if ($sizehere >= 2){
								$gtpresent{$snp} = "Three".$gtpresent{$snp}.$temp[$gtcol1].$temp[$gtcol2];
								$snphash{$snp} = "0 0 ";
								$snperror++;
							}
							#If this is the first time and the hash is empty
							elsif ($gtpresent{$snp} !~ $temp[$gtcol1]
							and $gtpresent{$snp} !~ $temp[$gtcol2]
							and $sizehere == 0){
								$snphash{$snp} = "$temp[$gtcol1] $temp[$gtcol2] ";
								if ($temp[$gtcol1] eq $temp[$gtcol2]){
									$gtpresent{$snp} = $temp[$gtcol1];
								}
								else{
									$gtpresent{$snp} = $temp[$gtcol1].$temp[$gtcol2];
								}
							}
							#If there is only one allele present then another can be added
							elsif ($sizehere <= 1){
								if ($temp[$gtcol1] eq $temp[$gtcol2]){
									$snphash{$snp} = "$temp[$gtcol1] $temp[$gtcol2] ";
									$gtpresent{$snp} = $gtpresent{$snp}.$temp[$gtcol1];
								}
								elsif ($gtpresent{$snp} =~ $temp[$gtcol1]
								and $gtpresent{$snp} !~ $temp[$gtcol2]){
									$snphash{$snp} = "$temp[$gtcol1] $temp[$gtcol2] ";
									$gtpresent{$snp} = $gtpresent{$snp}.$temp[$gtcol2];
								}
								elsif ($gtpresent{$snp} !~ $temp[$gtcol1]
								and $gtpresent{$snp} =~ $temp[$gtcol2]){
									$snphash{$snp} = "$temp[$gtcol1] $temp[$gtcol2] ";
									$gtpresent{$snp} = $gtpresent{$snp}.$temp[$gtcol1];
								}
							}
							#Otherwise the new alleles will not fit
							else {
								$gtpresent{$snp} = "Three".$gtpresent{$snp}.$temp[$gtcol1].$temp[$gtcol2];
								$snphash{$snp} = "0 0 ";
								$snperror++;
							}
						}
					}
					#This is where the map file is made
					if ($cases == 1){
						#print "Now I'm here\n";
						seek(NEWFILE, 0, 0);
						print "Initializing map file for other files, this may take a while...\n";
						@data=<NEWFILE>; chomp(@data);
						print "1 of 3) Changing chromosomes from letters to numbers\n";
						foreach $line (@data){
							$line =~ s/$xchr/$xrep/g;
							$line =~ s/$xychr/$xyrep/g;
							$line =~ s/$ychr/$yrep/g;
#							print "$line\n";
						}
						print "2 of 3) Sorting file by chromosome and position\n";
						@data= sort specificSortPLINK @data;
						print "3 of 3) Writing map file\n";
						foreach $line (@data){
							@newtemp=split("\t", $line);
							$snp=$newtemp[$snpcol];
							if ($only_these_snps eq ''){
								if ($snp =~ /rs/){
									print MAPFILE "$newtemp[$chromocol]\t$snp\t0\t$newtemp[$chromodist]\n";
									$allsnps[$snpcount] = $snp;
									$snpcount++;
								}
							}
							else{
								if ($snp =~ /rs/ and $goodsnps{$snp} eq "yes"){
									print MAPFILE "$newtemp[$chromocol]\t$snp\t0\t$newtemp[$chromodist]\n";
									$allsnps[$snpcount] = $snp;
									$snpcount++;
								}
							}
						}
						print "map file has been created, now making the ped file\n";
					}
					for ($z=0; $z<=$#allsnps; $z++){
						print PED "$snphash{$allsnps[$z]}";
					}
					print PED "\n";
					print "$filecount of $totalfr) $file, SNPs: $z\n";
					print "There are $snperror SNPs with over two alleles being removed\n";
				}
			}
		}
	}
	
	@snplist = keys %gtpresent;
	foreach $snp (@snplist){
		if ($gtpresent{$snp} =~ /Three/i){
			print EXTRA "$snp\t$gtpresent{$snp}\n";
		}
	}
	
}

#Uses UCSC to determine the GC content of regions
if ($find_gc == 1){
	$input = $ARGV[0];
	$output = "GC_".$input;
	open (IN, "<$input") || die "Cannot open $input; check it is spelled correctly\n";
	@regions = <IN>; chomp (@regions); close IN;
	open (OUT, ">$output") || die "Cannot open $output; may already be open\n";
	
	#Show the first line of the regions file
	@lineshow = split ("\t", $regions[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	#Ask user which column the name, chr, start, stop are in and if there is a header
	print "\nWhich column has the region name (1, 2, 3, etc)?\n";
	$user_region = <STDIN>; chomp $user_region; $regh = $user_region - 1;
	print "Which column is the chromosome in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromoh = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $starth = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stoph = $user_stop - 1;
	print "Is there a header (Y or N)?\n\n";
	$user_head = <STDIN>; chomp $user_head;
	
	#Remove header if present
	if ($user_head =~ /y/i){
		$theheader = $regions[0];
		print OUT $theheader."\tGC\tSize\tGC content (%)\n";
		shift @regions;
	}
	
	$totalcount = @regions;
	chomp @regions;
	
	#Get DNA sequence for each region
	foreach $region (@regions){
		$regioncount++;
		@tab = split ("\t", $region);
		print "Downloading sequence for region $regioncount of $totalcount: $tab[$regh]\n";
		$regname = $tab[$regh];
		print OUT "$region\t";
		$size{$regname} = ($tab[$stoph] - $tab[$starth] + 1);
		@regsubmit = ($tab[$chromoh], $tab[$starth], $tab[$stoph]);
		@page = &extract(@regsubmit);
#		foreach $line (@page){
#			print $line;
#		}
#		print "\n";
		
		#Process page to remove everything except sequence
		@sequence = &dna_process(@page);
		$theseq = "";
		foreach $line (@sequence){
			chomp $line;
			$line =~ s/\s//g;
			$theseq .= $line;
		}
		#print "SEQ: $theseq\n";
		$thelength  = length ($theseq);
		$theg = ($theseq =~ tr/g//);
		$theG = ($theseq =~ tr/G//);
		$thec = ($theseq =~ tr/c//);
		$theC = ($theseq =~ tr/C//);
		$thegc = $theg + $theG + $thec + $theC;
		
		$gccontent = sprintf ("%.4f", (eval {$thegc / $thelength} * 100))."%";
		
		print "GC is $thegc, seqeuence length is $thelength, GC content is $gccontent\n";
		print OUT "$thegc\t$thelength\t$gccontent\n";
	}
	#close OUT;
}

#Uses UCSC to get the FASTA format sequence of regions
if ($get_dna == 1){
	$input = $ARGV[0];
	$output = "FASTA_".$input;
	open (IN, "<$input") || die "Cannot open $input; check it is spelled correctly\n";
	@regions = <IN>; chomp (@regions); close IN;
	open (OUT, ">$output") || die "Cannot open $output; may already be open\n";
	
	#Show the first line of the regions file
	@lineshow = split ("\t", $regions[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	#Ask user which column the name, chr, start, stop are in and if there is a header
	print "\nWhich column has the region name (1, 2, 3, etc)?\n";
	$user_region = <STDIN>; chomp $user_region; $regh = $user_region - 1;
	print "Which column is the chromosome in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromoh = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $starth = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stoph = $user_stop - 1;
	print "Is there a header (Y or N)?\n\n";
	$user_head = <STDIN>; chomp $user_head;
	
	#Remove header if present
	if ($user_head =~ /y/i){
		shift @regions;
	}
	
	$totalcount = @regions;
	chomp @regions;
	
	#Get DNA sequence for each region
	foreach $region (@regions){
		$regioncount++;
		@tab = split ("\t", $region);
		print "Downloading sequence for region $regioncount of $totalcount: $tab[$regh]\n";
		$regname = '>'.$tab[$regh];
		print OUT "$regname\n";
		$size{$regname} = ($tab[$stoph] - $tab[$starth] + 1);
		@regsubmit = ($tab[$chromoh], $tab[$starth], $tab[$stoph]);
		@page = &extract(@regsubmit);
#		foreach $line (@page){
#			print $line;
#		}
#		print "\n";
		
		#Process page to remove everything except sequence
		@sequence = &dna_process(@page);
		
		print OUT @sequence;
		#print @sequence;
	}
	close OUT;
	
	#Check the results to make sure that the number of basepairs equals the expected amount
	print "\nChecking DNA sequence results\n\n";
	open (IN2, "$output") || die "Cannot open $output\n";
	$output2 = "Lines_".$output;
	open (OUT2, ">$output2") || die "Cannot open $output2; may already be open\n";
	print LISTOUT $curr_dir.$output2."\n";
	print OUT2 "Region\tSize\tLines\tN's\tA's\tG's\tC's\tT's\tTotal\n";
	$errorcount = 0;
	
	#Go through each line counting the basepairs and lines, print result if line contains '>'
	while (<IN2>){
		#print "Line: $_\n";
		#print "Count: $linecount, N: $ncount\n";
		if ($_ =~ />/){
			chomp;
			$regcount++;
			print "Checking sequence for region $regcount of $totalcount: $_\n";
			$totcount = $ncount + $acount + $gcount + $ccount + $tcount;
			if ($regcount > 1){
				if ($size != $totcount) {$error = "\tERROR"; $errorcount++;}
				else {$error = "";}
				print OUT2 $lastreg."\t".$size."\t".$linecount."\t".$ncount."\t".$acount."\t".$gcount."\t".$ccount."\t".$tcount."\t".$totcount.$error."\n";
			}
			#Reset variable and 
			$linecount = 0; $ncount = 0; $acount = 0; $gcount = 0; $ccount = 0; $tcount = 0; 
			$lastreg = $_;
			$size = $size{$_};
		}
		else {
			$ncount += (uc $_ =~ tr/N//);
			$acount += (uc $_ =~ tr/A//);
			$gcount += (uc $_ =~ tr/G//);
			$ccount += (uc $_ =~ tr/C//);
			$tcount += (uc $_ =~ tr/T//);
		}
		$linecount++;
	}
	$totcount = $ncount + $acount + $gcount + $ccount + $tcount;
	if ($size != $totcount) {$error = "\tERROR"; $errorcount++;}
	else {$error = "";}
	print OUT2 $lastreg."\t".$size."\t".$linecount."\t".$ncount."\t".$acount."\t".$gcount."\t".$ccount."\t".$tcount."\t".$totcount.$error."\n";
	print "\nThere were $errorcount errors detected\n";
	print "All done!\n";
}

#Uses UCSC to get the FASTA format sequence of regions version 2
if ($get_dna2 == 1){
	#use WWW::Mechanize;
	$input = $ARGV[0];
	$output = "FASTA_".$input;
	open (IN, "<$input") || die "Cannot open $input; check it is spelled correctly\n";
	@regions = <IN>; chomp (@regions); close IN;
	open (OUT, ">>$output") || die "Cannot open $output; may already be open\n";
	
	#Show the first line of the regions file
	@lineshow = split ("\t", $regions[0]);
	foreach (@lineshow){
		$linecount++;
		print "$linecount) $_\n";
	}
	
	#Ask user which column the name, chr, start, stop are in and if there is a header
	print "\nWhich column has the region name (1, 2, 3, etc)?\n";
	$user_region = <STDIN>; chomp $user_region; $regh = $user_region - 1;
	print "Which column is the chromosome in (1, 2, 3, etc)?\n";
	$user_chromo = <STDIN>; chomp $user_chromo; $chromoh = $user_chromo - 1;
	print "Which column are the start co-ordinates in (1, 2, 3, etc)?\n";
	$user_start = <STDIN>; chomp $user_start; $starth = $user_start - 1;
	print "Which column are the stop co-ordinates in (1, 2, 3, etc)?\n";
	$user_stop = <STDIN>; chomp $user_stop; $stoph = $user_stop - 1;
	print "Is there a header (Y or N)?\n\n";
	$user_head = <STDIN>; chomp $user_head;
	
	#Remove header if present
	if ($user_head =~ /y/i){
		shift @regions;
	}
	
	$totalcount = @regions;
	chomp @regions;
	
	#Get DNA sequence for each region
	foreach $region (@regions){
		$regioncount++;
		@tab = split ("\t", $region);
		print "Downloading sequence for region $regioncount of $totalcount: $tab[$regh]\n";
		$regname = '>'.$tab[$regh];
		print OUT "$regname\n";
		$size{$regname} = ($tab[$stoph] - $tab[$starth] + 1);
		@regsubmit = ($tab[$chromoh], $tab[$starth], $tab[$stoph]);
		@page = &extract2(@regsubmit);
#		foreach $line (@page){
#			print $line;
#		}
#		print "\n";
		
		#Process page to remove everything except sequence
		@sequence = &dna_process(@page);
		
		print OUT @sequence;
	}
	close OUT;
	
	#Check the results to make sure that the number of basepairs equals the expected amount
	print "\nChecking DNA sequence results\n\n";
	open (IN2, "$output") || die "Cannot open $output\n";
	$output2 = "Lines_".$output;
	open (OUT2, ">$output2") || die "Cannot open $output2; may already be open\n";
	print LISTOUT $curr_dir.$output2."\n";
	print OUT2 "Region\tSize\tLines\tN's\tA's\tG's\tC's\tT's\tTotal\n";
	$errorcount = 0;
	
	#Go through each line counting the basepairs and lines, print result if line contains '>'
	while (<IN2>){
		#print "Line: $_\n";
		#print "Count: $linecount, N: $ncount\n";
		if ($_ =~ />/){
			chomp;
			$regcount++;
			print "Checking sequence for region $regcount of $totalcount: $_\n";
			$totcount = $ncount + $acount + $gcount + $ccount + $tcount;
			if ($regcount > 1){
				if ($size != $totcount) {$error = "\tERROR"; $errorcount++;}
				else {$error = "";}
				print OUT2 $lastreg."\t".$size."\t".$linecount."\t".$ncount."\t".$acount."\t".$gcount."\t".$ccount."\t".$tcount."\t".$totcount.$error."\n";
			}
			#Reset variable and 
			$linecount = 0; $ncount = 0; $acount = 0; $gcount = 0; $ccount = 0; $tcount = 0; 
			$lastreg = $_;
			$size = $size{$_};
		}
		else {
			$ncount += (uc $_ =~ tr/N//);
			$acount += (uc $_ =~ tr/A//);
			$gcount += (uc $_ =~ tr/G//);
			$ccount += (uc $_ =~ tr/C//);
			$tcount += (uc $_ =~ tr/T//);
		}
		$linecount++;
	}
	$totcount = $ncount + $acount + $gcount + $ccount + $tcount;
	if ($size != $totcount) {$error = "\tERROR"; $errorcount++;}
	else {$error = "";}
	print OUT2 $lastreg."\t".$size."\t".$linecount."\t".$ncount."\t".$acount."\t".$gcount."\t".$ccount."\t".$tcount."\t".$totcount.$error."\n";
	print "\nThere were $errorcount errors detected\n";
	print "All done!\n";
}

#Returns the DNA for a region if given an array with chr in 0, start in 1 and stop in 2; does not extract any further info from the UCSC DNA webpage
sub extract {
	$chromo=$_[0]; $start=$_[1]; $stop=$_[2];
	$combo = ($chromo . ':' . $start . '-' . $stop);
	$browser = LWP::UserAgent -> new() unless $browser;
	
	$db = "hg18"; #db
	$cdsexon = 1; #hgSeq.cdsExon
	$pad5 = 0; #hgSeq.padding5
	$pad3 = 0; #hgSeq.padding3
	$casing = "upper"; #hgSeq.casing (upper, lower)
	$maskrepeat = "on"; #hgSeq.maskRepeats (on, off)
	$boolmaskrepeats = 0; #boolshad.hgSeq.maskRepeats (1)
	$repmask = "lower"; #hgSeq.repMasking (N, lower)
	#$revcomp = '&hgSeq.revComp=on';
	$boolrevcomp = 0; #boolshad.hgSeq.revComp (1)
	
	$seqpage = 'http://yalegb.microangelo.com/cgi-bin/hgc?hgsid=8435&g=htcGetDna2&table=&i=mixed&o='.$start.'&l='.$start.'&r='.$stop.'&getDnaPos='.$combo.'&db='.$db.'&hgSeq.cdsExon='.$cdsexon.'&hgSeq.padding5='.$pad5.'&hgSeq.padding3='.$pad3.'&hgSeq.casing='.$casing.'&hgSeq.maskRepeats='.$maskrepeat.'&boolshad.hgSeq.maskRepeats='.$boolmaskrepeats.'&hgSeq.repMasking='.$repmask.$revcomp.'&boolshad.hgSeq.revComp='.$boolrevcomp.'&submit=get+DNA';
	
	#print "$seqpage\n";
	#sleep (2);
	
	my $resp = $browser -> get($seqpage);
	
	return $resp->content;
}

#Removes all lines except those containing sequence from FASTA format files
sub dna_process {
	#print "Purifying DNA...\n";
	@puredna = ();
	@lines = split ("\n",@_[0]);
	foreach $line (@lines){
		if ($line ne "" and $line ne"\t" and $line !~ />/ and $line !~ /</){
			$newline = $line."\n";
			#$newline =~ tr/[a-z]/N/;
			push (@puredna, $newline);
		}
	}
	return (@puredna);
}

#Returns the DNA for a region if given an array with chr in 0, start in 1 and stop in 2; does not extract any further info from the UCSC DNA webpage
sub extract2 {
	$chromo=$_[0]; $start=$_[1]; $stop=$_[2];
	$combo = ($chromo . ':' . $start . '-' . $stop);
	my $mech = WWW::Mechanize->new( autocheck => 1 );
	
	$db = "hg18"; #db
	$cdsexon = 1; #hgSeq.cdsExon
	$pad5 = 0; #hgSeq.padding5
	$pad3 = 0; #hgSeq.padding3
	$casing = "upper"; #hgSeq.casing (upper, lower)
	$maskrepeat = "on"; #hgSeq.maskRepeats (on, off)
	$boolmaskrepeats = 1; #boolshad.hgSeq.maskRepeats (1)
	$repmask = "N"; #hgSeq.repMasking (N, lower)
	#$revcomp = '&hgSeq.revComp=on';
	$boolrevcomp = 0; #boolshad.hgSeq.revComp (1)
	
	$seqpage = 'http://yalegb.microangelo.com/cgi-bin/hgc?hgsid=8467&g=htcGetDna2&table=&i=mixed&o='.$start.'&l='.$start.'&r='.$stop.'&getDnaPos='.$combo.'&db='.$db.'&hgSeq.cdsExon='.$cdsexon.'&hgSeq.padding5='.$pad5.'&hgSeq.padding3='.$pad3.'&hgSeq.casing='.$casing.'&hgSeq.maskRepeats='.$maskrepeat.'&boolshad.hgSeq.maskRepeats='.$boolmaskrepeats.'&hgSeq.repMasking='.$repmask.$revcomp.'&boolshad.hgSeq.revComp='.$boolrevcomp.'&submit=extended+case%2Fcolor+options';
	$mech->get( "$seqpage" );
	#print $mech->content;
	
	$mech->forms;
	$mech->set_fields(
	   'case' => 'upper',
	   'snp129_case' => 'on',
	   'rmsk_case' => 'on',
	   );
	$response = $mech->submit();
	return $response->content();
	
}

#Gives the varibles required by primer3 for qPCR primers and returns the Primer 3 results page
sub qPCR_primers {
	$input=$_[0];
	# This is the URL of the page to go after.
	
	my $URLtoPostTo = 'http://frodo.wi.mit.edu/cgi-bin/primer3/primer3_www_results.cgi';
	
	#Key variables:
	if ($qPCRthreshold == 0){
		$PRIMER_PRODUCT_SIZE_RANGE = "110-160";
		$PRIMER_MIN_SIZE = "21";
		$PRIMER_OPT_SIZE = "23";
		$PRIMER_MAX_SIZE = "25";
		$PRIMER_MIN_TM = "58.0";
		$PRIMER_OPT_TM = "60.0";
		$PRIMER_MAX_TM = "61.0";
		$PRIMER_PRODUCT_MIN_TM = "75";
		$PRIMER_PRODUCT_OPT_TM = "80";
		$PRIMER_PRODUCT_MAX_TM = "88";
		$PRIMER_GC_CLAMP = "2";
	}
	
	elsif ($qPCRthreshold >= 1){
		$PRIMER_PRODUCT_SIZE_RANGE = "110-170";
		$PRIMER_MIN_SIZE = "20";
		$PRIMER_OPT_SIZE = "23";
		$PRIMER_MAX_SIZE = "25";
		$PRIMER_MIN_TM = "58.0";
		$PRIMER_OPT_TM = "60.0";
		$PRIMER_MAX_TM = "62.0";
		$PRIMER_PRODUCT_MIN_TM = "75";
		$PRIMER_PRODUCT_OPT_TM = "80";
		$PRIMER_PRODUCT_MAX_TM = "88";
		$PRIMER_GC_CLAMP = "1";
	}

	
	my %Fields = (
		"SEQUENCE" => $input,
		"MUST_XLATE_PICK_LEFT" => "1",
		"MUST_XLATE_PICK_RIGHT" => "1",
		"PRIMER_MISPRIMING_LIBRARY" => "HUMAN",
		"PRIMER_PRODUCT_SIZE_RANGE" => "$PRIMER_PRODUCT_SIZE_RANGE",
		"PRIMER_NUM_RETURN" => "8",
		"PRIMER_MAX_END_STABILITY" => "9.0",
		"PRIMER_MAX_MISPRIMING" => "12.00",
		"PRIMER_PAIR_MAX_MISPRIMING" => "24.00",
		"Pick Primers" => "Pick Primers",
		
		"PRIMER_MIN_SIZE" => "$PRIMER_MIN_SIZE",
		"PRIMER_OPT_SIZE" => "$PRIMER_OPT_SIZE",
		"PRIMER_MAX_SIZE" => "$PRIMER_MAX_SIZE",
		"PRIMER_MIN_TM" => "$PRIMER_MIN_TM",
		"PRIMER_OPT_TM" => "$PRIMER_OPT_TM",
		"PRIMER_MAX_TM" => "$PRIMER_MAX_TM",
		"PRIMER_MAX_DIFF_TM" => "100.0",
		"PRIMER_PRODUCT_MIN_TM" => "$PRIMER_PRODUCT_MIN_TM",
		"PRIMER_PRODUCT_OPT_TM" => "$PRIMER_PRODUCT_OPT_TM",
		"PRIMER_PRODUCT_MAX_TM" => "$PRIMER_PRODUCT_MAX_TM",
		"PRIMER_MIN_GC" => "20.0",
		"PRIMER_OPT_GC_PERCENT" => '',
		"PRIMER_MAX_GC" => "80.0",
		"PRIMER_SELF_ANY" => "8.00",
		"PRIMER_SELF_END" => "3.00",
		"PRIMER_NUM_NS_ACCEPTED" => "0",
		"PRIMER_MAX_POLY_X" => "5",
		"PRIMER_INSIDE_PENALTY" => '',
		"PRIMER_OUTSIDE_PENALTY" => "0",
		"PRIMER_FIRST_BASE_INDEX" => "1",
		"PRIMER_GC_CLAMP" => "$PRIMER_GC_CLAMP",
		"PRIMER_SALT_CONC" => "50",
		"PRIMER_DNA_CONC" => "50",
		"PRIMER_LIBERAL_BASE" => "1",
		"MUST_XLATE_PRINT_INPUT" => "1",
		"PRIMER_LIB_AMBIGUITY_CODES_CONSENSUS" => "0",
		"INCLUDED_REGION" => "",
		"PRIMER_START_CODON_POSITION" => "",
		"PRIMER_SEQUENCE_QUALITY" => "",
		"PRIMER_MIN_QUALITY" => "0",
		"PRIMER_MIN_END_QUALITY" => "0",
		"PRIMER_QUALITY_RANGE_MIN" => "0",
		"PRIMER_QUALITY_RANGE_MAX" => "100",
		"PRIMER_WT_TM_LT" => "1.0",
		"PRIMER_WT_TM_GT" => "1.0",
		"PRIMER_WT_SIZE_LT" => "1.0",
		"PRIMER_WT_SIZE_GT" => "1.0",
		"PRIMER_WT_GC_PERCENT_LT" => "0.0",
		"PRIMER_WT_GC_PERCENT_GT" => "0.0",
		"PRIMER_WT_COMPL_ANY" => "0.0",
		"PRIMER_WT_COMPL_END" => "0.0",
		"PRIMER_WT_NUM_NS" => "0.0",
		"PRIMER_WT_REP_SIM" => "0.0",
		"PRIMER_WT_SEQ_QUAL" => "0.0",
		"PRIMER_WT_END_QUAL" => "0.0",
		"PRIMER_WT_POS_PENALTY" => "0.0",
		"PRIMER_WT_END_STABILITY" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_SIZE_LT" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_SIZE_GT" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_TM_LT" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_TM_GT" => "0.0",
		"PRIMER_PAIR_WT_DIFF_TM" => "0.0",
		"PRIMER_PAIR_WT_COMPL_ANY" => "0.0",
		"PRIMER_PAIR_WT_COMPL_END" => "0.0",
		"PRIMER_PAIR_WT_REP_SIM" => "0.0",
		"PRIMER_PAIR_WT_PR_PENALTY" => "1.0",
		"PRIMER_PAIR_WT_IO_PENALTY" => "0.0",
		"PRIMER_INTERNAL_OLIGO_EXCLUDED_REGION" => "",
		"PRIMER_INTERNAL_OLIGO_MIN_SIZE" => "18",
		"PRIMER_INTERNAL_OLIGO_OPT_SIZE" => "20",
		"PRIMER_INTERNAL_OLIGO_MAX_SIZE" => "27",
		"PRIMER_INTERNAL_OLIGO_MIN_TM" => "57.0",
		"PRIMER_INTERNAL_OLIGO_OPT_TM" => "60.0",
		"PRIMER_INTERNAL_OLIGO_MAX_TM" => "63.0",
		"PRIMER_INTERNAL_OLIGO_MIN_GC" => "20.0",
		"PRIMER_INTERNAL_OLIGO_OPT_GC_PERCENT" => "",
		"PRIMER_INTERNAL_OLIGO_MAX_GC" => "80.0",
		"PRIMER_INTERNAL_OLIGO_SELF_ANY" => "",
		"PRIMER_INTERNAL_OLIGO_SELF_END" => "",
		"PRIMER_INTERNAL_OLIGO_NUM_NS" => '',
		"PRIMER_INTERNAL_OLIGO_MAX_POLY_X" => "5",
		"PRIMER_INTERNAL_OLIGO_MISHYB_LIBRARY" => "HUMAN",
		"PRIMER_INTERNAL_OLIGO_MAX_MISHYB" => "",
		"PRIMER_INTERNAL_OLIGO_MIN_QUALITY" => "0",
		"PRIMER_INTERNAL_OLIGO_SALT_CONC" => "50.0",
		"PRIMER_INTERNAL_OLIGO_DNA_CONC" => "50.0",
		"PRIMER_IO_WT_TM_LT" => "1.0",
		"PRIMER_IO_WT_TM_GT" => "1.0",
		"PRIMER_IO_WT_SIZE_LT" => "1.0",
		"PRIMER_IO_WT_SIZE_GT" => "1.0",
		"PRIMER_IO_WT_GC_PERCENT_LT" => "0.0",
		"PRIMER_IO_WT_GC_PERCENT_GT" => "0.0",
		"PRIMER_IO_WT_COMPL_ANY" => "0.0",
		"PRIMER_IO_WT_NUM_NS" => "0.0",
		"PRIMER_IO_WT_REP_SIM" => "0.0",
		"PRIMER_IO_WT_SEQ_QUAL" => "0.0"
	);
	my $BrowserName = "General Pseudo-Browser";
	my $Browser = new LWP::UserAgent;
	if($BrowserName) { $Browser->agent($BrowserName); }
	my $Page = $Browser->request(POST $URLtoPostTo,\%Fields);
	if ($Page->is_success) {
		return $Page->content;
		print $Page;
		print RESULTS $Page->content; 
	}
	else { print $Page->message; }
}

#Gives the varibles required by primer3 for qPCR primers and returns the Primer 3 results page
sub PCR_primers {
	$input=$_[0];
	# This is the URL of the page to go after.
	
	my $URLtoPostTo = 'http://frodo.wi.mit.edu/cgi-bin/primer3/primer3_www_results.cgi';
	
	#Key variables:
	if ($PCRthreshold == 0){
		$PRIMER_MISPRIMING_LIBRARY = "HUMAN";
		$PRIMER_PRODUCT_SIZE_RANGE = "110-600";
		$PRIMER_MIN_SIZE = "22";
		$PRIMER_OPT_SIZE = "25";
		$PRIMER_MAX_SIZE = "27";
		$PRIMER_MIN_TM = "58.0";
		$PRIMER_OPT_TM = "60.0";
		$PRIMER_MAX_TM = "61.0";
		$PRIMER_PRODUCT_MIN_TM = "75";
		$PRIMER_PRODUCT_OPT_TM = "80";
		$PRIMER_PRODUCT_MAX_TM = "88";
		$PRIMER_GC_CLAMP = "2";
	}
	
	elsif ($PCRthreshold >= 1){
		$PRIMER_MISPRIMING_LIBRARY = "NONE";
		$PRIMER_PRODUCT_SIZE_RANGE = "110-600";
		$PRIMER_MIN_SIZE = "22";
		$PRIMER_OPT_SIZE = "25";
		$PRIMER_MAX_SIZE = "27";
		$PRIMER_MIN_TM = "58.0";
		$PRIMER_OPT_TM = "60.0";
		$PRIMER_MAX_TM = "62.0";
		$PRIMER_PRODUCT_MIN_TM = "75";
		$PRIMER_PRODUCT_OPT_TM = "80";
		$PRIMER_PRODUCT_MAX_TM = "88";
		$PRIMER_GC_CLAMP = "1";
	}

	
	my %Fields = (
		"SEQUENCE" => $input,
		"MUST_XLATE_PICK_LEFT" => "1",
		"MUST_XLATE_PICK_RIGHT" => "1",
		"PRIMER_MISPRIMING_LIBRARY" => "$PRIMER_MISPRIMING_LIBRARY",
		"PRIMER_PRODUCT_SIZE_RANGE" => "$PRIMER_PRODUCT_SIZE_RANGE",
		"PRIMER_NUM_RETURN" => "8",
		"PRIMER_MAX_END_STABILITY" => "9.0",
		"PRIMER_MAX_MISPRIMING" => "12.00",
		"PRIMER_PAIR_MAX_MISPRIMING" => "24.00",
		"Pick Primers" => "Pick Primers",
		
		"PRIMER_MIN_SIZE" => "$PRIMER_MIN_SIZE",
		"PRIMER_OPT_SIZE" => "$PRIMER_OPT_SIZE",
		"PRIMER_MAX_SIZE" => "$PRIMER_MAX_SIZE",
		"PRIMER_MIN_TM" => "$PRIMER_MIN_TM",
		"PRIMER_OPT_TM" => "$PRIMER_OPT_TM",
		"PRIMER_MAX_TM" => "$PRIMER_MAX_TM",
		"PRIMER_MAX_DIFF_TM" => "100.0",
		"PRIMER_PRODUCT_MIN_TM" => "$PRIMER_PRODUCT_MIN_TM",
		"PRIMER_PRODUCT_OPT_TM" => "$PRIMER_PRODUCT_OPT_TM",
		"PRIMER_PRODUCT_MAX_TM" => "$PRIMER_PRODUCT_MAX_TM",
		"PRIMER_MIN_GC" => "20.0",
		"PRIMER_OPT_GC_PERCENT" => '',
		"PRIMER_MAX_GC" => "80.0",
		"PRIMER_SELF_ANY" => "8.00",
		"PRIMER_SELF_END" => "3.00",
		"PRIMER_NUM_NS_ACCEPTED" => "0",
		"PRIMER_MAX_POLY_X" => "5",
		"PRIMER_INSIDE_PENALTY" => '',
		"PRIMER_OUTSIDE_PENALTY" => "0",
		"PRIMER_FIRST_BASE_INDEX" => "1",
		"PRIMER_GC_CLAMP" => "$PRIMER_GC_CLAMP",
		"PRIMER_SALT_CONC" => "50",
		"PRIMER_DNA_CONC" => "50",
		"PRIMER_LIBERAL_BASE" => "1",
		"MUST_XLATE_PRINT_INPUT" => "1",
		"PRIMER_LIB_AMBIGUITY_CODES_CONSENSUS" => "0",
		"INCLUDED_REGION" => "",
		"PRIMER_START_CODON_POSITION" => "",
		"PRIMER_SEQUENCE_QUALITY" => "",
		"PRIMER_MIN_QUALITY" => "0",
		"PRIMER_MIN_END_QUALITY" => "0",
		"PRIMER_QUALITY_RANGE_MIN" => "0",
		"PRIMER_QUALITY_RANGE_MAX" => "100",
		"PRIMER_WT_TM_LT" => "1.0",
		"PRIMER_WT_TM_GT" => "1.0",
		"PRIMER_WT_SIZE_LT" => "1.0",
		"PRIMER_WT_SIZE_GT" => "1.0",
		"PRIMER_WT_GC_PERCENT_LT" => "0.0",
		"PRIMER_WT_GC_PERCENT_GT" => "0.0",
		"PRIMER_WT_COMPL_ANY" => "0.0",
		"PRIMER_WT_COMPL_END" => "0.0",
		"PRIMER_WT_NUM_NS" => "0.0",
		"PRIMER_WT_REP_SIM" => "0.0",
		"PRIMER_WT_SEQ_QUAL" => "0.0",
		"PRIMER_WT_END_QUAL" => "0.0",
		"PRIMER_WT_POS_PENALTY" => "0.0",
		"PRIMER_WT_END_STABILITY" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_SIZE_LT" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_SIZE_GT" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_TM_LT" => "0.0",
		"PRIMER_PAIR_WT_PRODUCT_TM_GT" => "0.0",
		"PRIMER_PAIR_WT_DIFF_TM" => "0.0",
		"PRIMER_PAIR_WT_COMPL_ANY" => "0.0",
		"PRIMER_PAIR_WT_COMPL_END" => "0.0",
		"PRIMER_PAIR_WT_REP_SIM" => "0.0",
		"PRIMER_PAIR_WT_PR_PENALTY" => "1.0",
		"PRIMER_PAIR_WT_IO_PENALTY" => "0.0",
		"PRIMER_INTERNAL_OLIGO_EXCLUDED_REGION" => "",
		"PRIMER_INTERNAL_OLIGO_MIN_SIZE" => "18",
		"PRIMER_INTERNAL_OLIGO_OPT_SIZE" => "20",
		"PRIMER_INTERNAL_OLIGO_MAX_SIZE" => "27",
		"PRIMER_INTERNAL_OLIGO_MIN_TM" => "57.0",
		"PRIMER_INTERNAL_OLIGO_OPT_TM" => "60.0",
		"PRIMER_INTERNAL_OLIGO_MAX_TM" => "63.0",
		"PRIMER_INTERNAL_OLIGO_MIN_GC" => "20.0",
		"PRIMER_INTERNAL_OLIGO_OPT_GC_PERCENT" => "",
		"PRIMER_INTERNAL_OLIGO_MAX_GC" => "80.0",
		"PRIMER_INTERNAL_OLIGO_SELF_ANY" => "",
		"PRIMER_INTERNAL_OLIGO_SELF_END" => "",
		"PRIMER_INTERNAL_OLIGO_NUM_NS" => '',
		"PRIMER_INTERNAL_OLIGO_MAX_POLY_X" => "5",
		"PRIMER_INTERNAL_OLIGO_MISHYB_LIBRARY" => "HUMAN",
		"PRIMER_INTERNAL_OLIGO_MAX_MISHYB" => "",
		"PRIMER_INTERNAL_OLIGO_MIN_QUALITY" => "0",
		"PRIMER_INTERNAL_OLIGO_SALT_CONC" => "50.0",
		"PRIMER_INTERNAL_OLIGO_DNA_CONC" => "50.0",
		"PRIMER_IO_WT_TM_LT" => "1.0",
		"PRIMER_IO_WT_TM_GT" => "1.0",
		"PRIMER_IO_WT_SIZE_LT" => "1.0",
		"PRIMER_IO_WT_SIZE_GT" => "1.0",
		"PRIMER_IO_WT_GC_PERCENT_LT" => "0.0",
		"PRIMER_IO_WT_GC_PERCENT_GT" => "0.0",
		"PRIMER_IO_WT_COMPL_ANY" => "0.0",
		"PRIMER_IO_WT_NUM_NS" => "0.0",
		"PRIMER_IO_WT_REP_SIM" => "0.0",
		"PRIMER_IO_WT_SEQ_QUAL" => "0.0"
	);
	my $BrowserName = "General Pseudo-Browser";
	my $Browser = new LWP::UserAgent;
	if($BrowserName) { $Browser->agent($BrowserName); }
	my $Page = $Browser->request(POST $URLtoPostTo,\%Fields);
	if ($Page->is_success) {
		return $Page->content;
		print $Page;
		print RESULTS $Page->content; 
	}
	else { print $Page->message; }
}

#Returns the ePCR product for a pair of oligos if given an array with the left oligo in 0 and the right oligo in 1; does not extract any further info from the UCSC ePCR webpage
sub ePCR {
	$input1=$_[0]; $input1=$_[1];
	# This is the URL of the page to go after.
	$browser = LWP::UserAgent -> new() unless $browser;
	$epcrpage = "http://genome.ucsc.edu/cgi-bin/hgPcr";
	$flipper = 0;
	if ($flipper == 1){print "ERROR: the second oligo has been reversed, change the $flipper command to 0 to correct this\n";}

	my $resp = $browser -> post($epcrpage,[
		"org" => "Human",
		"db" => "hg18",
		"wp_f" => "$leftee",
		"wp_r" => "$rightee",
		"Submit" => "submit",
		"wp_size" => "4000",
		"wp_perfect" => "15",
		"wp_good" => "15",
		"boolshad.wp_flipReverse" => "$flipper"
		]
	);
	return($resp->content,$resp->status_line,$resp->is_success,$resp)
	if wantarray;
	return unless $resp->is_success;
	return $resp->content;

}

sub getDNA {
	@data = ();
	@seqs = @_;
	@leftover=();

	foreach $line (@seqs){
		#a1 = name, a2 = chr, a3 = start, a4 = stop
		($a1, $a2, $a3, $a4) = split ("\t", $line);
		print "Asking UCSC browser for sequence of $a1\n";
		if ($qPCRthreshold == 0 and $PCRthreshold == 0){
			$regiondivide = ($a4 - $a3)/4;
		}
		elsif ($qPCRthreshold == 1 or $PCRthreshold == 1){
			$regiondivide = ($a4 - $a3)/8;
		}
		#elsif ($qPCRthreshold >= 2){
		#	$regiondivide = 0;
		#}
		$startpcr = &round($a3 + $regiondivide);
		$stoppcr = &round($a4 - $regiondivide);
		
		$takeout = &extract($a2, $startpcr, $stoppcr);
		@temp = split(/\n/,$takeout); 
		$marker=0;
		#takes the output from UCSC DNA and treats it line by line, creates an array called @data with the name of the region in 0 and each line of DNA in the next place in the array
		foreach $guy (@temp) {
			if ($guy =~ /<PRE>/){$marker=2;}
			if ($guy =~ /<\/PRE>/){$marker=0;}
			if ($marker==1){
				#This is the first line of the sequence with the name submitted
				if ($guy=~ />/){
					$newname = (">" . $a1);
					push (@data, $newname); print EXTRACTS "$newname\n"; 
				}
				else{	push (@data, $guy); print EXTRACTS "$guy\n";
				}
			}
			if ($marker==2){$marker--;}
		}
	}
	close (EXTRACTS);
	&divideDNA(@data);
}

sub divideDNA {
	$regioncounthere = 0;
	@data = @_;
	$holdcount = 0;
	foreach $line (@data){
		$thisline = $data[$holdcount];
		$nextline = $data[$holdcount + 1];
		
		if ($thisline =~ />/){$name = $thisline;}
		else {push (@Holder1, $thisline);}
		
		#If there is a > in the next line or it has reached the end of the array
		if (($nextline =~ />/ or $holdcount >= $#data) and $thisline ne ""){
			#print "Here: $name, Array: $#data, Place: $holdcount\n";
			&sortDNA(@Holder1);
			@Holder1 = ();
		}
		$holdcount++;
	}
}

sub sortDNA {
	@Holder = @_;
	$namenocarrot = $name;
	$namenocarrot =~ s/>//gi;
	#@Holder is the array with all the lines of the sequence, at the first line of the next sequence all the variables are reset
	$num=0; $Seq=''; $output=''; @temp2=''; @temp=''; $found=0; $minpcrs=10; $pcrs=0; $storage=''; $regioncounthere++;
	chomp (@Holder); $Seq = join('', @Holder);  $Seq =~ s/\s+//g; @Holder='';
	#print "$name\n"; print RESULTS "$name\t"; 
	#Sends the sequence to Primer 3 and gives the qPCR variables then returns the Primer 3 results
	print "Submitting to Primer3...";
	if ($qpcr_settings == 1){
		$output = &qPCR_primers($Seq);
	}
	elsif ($pcr_settings == 1){
		$output = &PCR_primers($Seq);
	}
	print "processing Primer3 data...";
	
	#These will show the Primer3 output
	#$htmlpage = $regioncounthere."_".$namenocarrot."_primer3.html";
	#open (OUTHTML, ">$htmlpage")||die "Cannot open $htmlpage\n";
	#print OUTHTML $output;
	
	@temp = split(/\n/,$output); 
	foreach (@temp) {
		chomp;
		$pcrs=0;
		#This is a progression down the file, assuming the primers are listed in order from left, right, then the product size.  Sequence checks are done when the product size is reported
		if ($_ =~ /LEFT PRIMER/ or $_ =~ /RIGHT PRIMER/ ){
			@extracts = split;
			if ($_ =~ /LEFT PRIMER/){$leftee=$extracts[$#extracts]; $num++;}
			if ($_ =~ /RIGHT PRIMER/){$rightee=$extracts[$#extracts];}
			#formatting the output
			if ($extracts[0] =~ /\d/){}else{$storage.="\t";}
			foreach $part (@extracts){unless ($part eq ''){$storage .= "$part\t";}}
		}
		elsif ($_ =~ /PRODUCT SIZE/){
			print "checking ePCR for set ";
			$comeback = &ePCR($leftee, $rightee);
			
			#These will show the ePCR output
			#$htmlpage = $regioncounthere."_".$namenocarrot."_ePCR.html";
			#open (OUTHTML, ">$htmlpage")||die "Cannot open $htmlpage\n";
			#print OUTHTML $comeback;
			
			@temp2 = split(/\n/,$comeback);
			foreach $dude (@temp2){if ($dude =~ /bp/){$pcrs++;}}
			print "\rSubmitting to Primer3...processing Primer3 data...checking ePCR for set $num: $pcrs";
			if ($pcrs<$minpcrs){$minpcrs=$pcrs};
			$startofamp = 0; $endofamp = 0;
			if ($pcrs == 1){
				foreach $guy (@temp2){
					chomp;
					if ($guy =~ /HREF/ and $guy =~ /bp/ and $endofamp == 0) {$startofamp = 1; $sizeline = $guy;}
					elsif ($startofamp == 1 and $endofamp == 0 and $guy =~ /<\/PRE>/) {$endofamp = 1;}
					elsif ($startofamp == 1 and $endofamp == 0){push (@amplicon, $guy);}
				}
				
				$finalamplicon = join ("", @amplicon);
				@amplicon = ();
				print "\n$regioncounthere) $name\tSuccess!\n";
				
				@points = split (">", $sizeline);
				@sizebit = split (" ", $points[-1]);
				$sizebit = $sizebit[0];
				$sizebit =~ s/bp//gi;
				$sizebit =~ s/ //gi;
				$locationbit = $points[-2];
				$locationbit =~ s/<\/A//gi;
				($chromoprod, $posse) = split (':', $locationbit);
				($startprod, $stopprod) = split ('\+', $posse);
				
				if ($qpcr_settings == 1 and $filetype eq "reg"){
					print BLASTME "$name" . "_L\n$leftee\n";
					print BLASTME "$name" . "_R\n$rightee\n";
					print RESULTS "$seqs[$regioncounthere - 1]\t$name\t$storage\t$finalamplicon\t$sizebit\t$chromoprod\t$startprod\t$stopprod\n";
				}
				elsif ($pcr_settings == 1 and $filetype eq "reg"){
					print RESULTS "$seqs[$regioncounthere - 1]\t$name\t$storage\t$finalamplicon\t$sizebit\t$chromoprod\t$startprod\t$stopprod\n";
				}
				elsif ($qpcr_settings == 1 and $filetype eq "seq"){
					print BLASTME "$name" . "_L\n$leftee\n";
					print BLASTME "$name" . "_R\n$rightee\n";
					#Need to put results into an array to bae added to the end of the file
				}
				elsif ($pcr_settings == 1 and $filetype eq "seq"){
					#Need to put results into an array to bae added to the end of the file
				}
#					print "$storage\t$pcrs\n";
				@temp='';
			}
			else{$storage='';}
		}
		elsif ($_ =~ /No Acceptable Primers Were Found/){
			print "\n$regioncounthere) $name\tNone found\n";
			if ($qpcr_settings == 1 and $filetype eq "reg"){
				print RESULTS "$seqs[$regioncounthere - 1]\t$name\tNone found\n";
				$leftinput = $seqs[$regioncounthere - 1];
				#print "Left input: $leftinput\n";
				push (@leftover, $leftinput);
			}
			elsif ($pcr_settings == 1 and $filetype eq "reg"){
				print RESULTS "$seqs[$regioncounthere - 1]\t$name\tNone found\n";
				$leftinput = $seqs[$regioncounthere - 1];
				#print "Left input: $leftinput\n";
				push (@leftover, $leftinput);
			}
			elsif ($qpcr_settings == 1 and $filetype eq "seq"){
				$leftinput = $Seq;
				push (@leftover, $leftinput);
			}
			elsif ($pcr_settings == 1 and $filetype eq "seq"){
				$leftinput = $Seq;
				push (@leftover, $leftinput);
			}
			$found == 1;
			$pcrs = 0;
			$minpcrs = 0;
		}
	}
	if ($minpcrs > 1){
		print "\n$regioncounthere) $name\tToo many products ($minpcrs)\n";
		if ($qpcr_settings == 1 and $filetype eq "reg"){
			print RESULTS "$seqs[$regioncounthere - 1]\t$name\tToo many found ($minpcrs)\n";
			$leftinput = $seqs[$regioncounthere - 1];
			#print "Left input: $leftinput\n";
			push (@leftover, $leftinput);
		}
		elsif ($pcr_settings == 1 and $filetype eq "reg"){
			print RESULTS "$seqs[$regioncounthere - 1]\t$name\tToo many found ($minpcrs)\n";
			$leftinput = $seqs[$regioncounthere - 1];
			#print "Left input: $leftinput\n";
			push (@leftover, $leftinput);
		}
		elsif ($qpcr_settings == 1 and $filetype eq "seq"){
			$leftinput = $Seq;
			push (@leftover, $leftinput);
		}
		elsif ($pcr_settings == 1 and $filetype eq "seq"){
			$leftinput = $Seq;
			push (@leftover, $leftinput);
		}
	}
}

#Subroutine to determine the LogR deviation of FinalReports
sub chiperrorsub {
	@files = @_;
	$filenum = 0;
	foreach (@files){
		if ($_ =~ /$namematch/){
			$filenum++;
			#print "\rDoing file number $filenum";
			open (FR, "<$_") || die "Could not open the $namematch file $_\n";
			@snps = <FR>; chomp @snps; close FR;
			@temp=split("\t", $snps[4]);
			$probes = $temp[1];
			$chip = &chiper($probes);
			#print "Probes: $probes\tChip: $chip\n";
			
			#These have not been assessed, the 1Mv1 numbers have simply been copied
			if ($chip eq "1Mv3"){
				$extreme = 6000;
				$wide = 42000;
			}
			
			#These have not been assessed, the 1Mv1 numbers have simply been copied
			elsif ($chip eq "Omni1vB"){
				$extreme = 6000;
				$wide = 42000;
			}
			
			#These have not been assessed, the 1Mv1 numbers have simply been copied
			elsif ($chip eq "Omni1vC"){
				$extreme = 6000;
				$wide = 42000;
			}
			
			#These were worked out by assessing 130 sample images in Beadstudio
			elsif ($chip eq "1Mv1"){
				$extreme = 6000;
				$wide = 42000;
			}
			
			#These have not been assessed, the 1Mv1 numbers have simply been divded by 2/3
			elsif ($chip eq "660W"){
				$extreme = 4000;
				$wide = 28000;
			}
			
			#These have not been assessed, the 1Mv1 numbers have simply been divded by 2/3
			elsif ($chip eq "660Q"){
				$extreme = 4000;
				$wide = 28000;
			}
			
			#Assessed by looking at distribution in 1200 samples (0.2% and 1%) 
			elsif ($chip eq "610v1"){
				$extreme = 1240;
				$wide = 6210;
			}
			
			#These are simple duplicates of the 370v1 numbers, they need to be checked
			elsif ($chip eq "370v3"){
				$extreme = 2500;
				$wide = 22000;
			}
			
			#These were assessed by looking at the samples that fail PN and QT. They are the mean + 2SD tested against number of calls and PN, QT failures
			elsif ($chip eq "370v1"){
				$extreme = 2500;
				$wide = 22000;
			}
			#These were assessed by looking at the mean + 2SD for 30 samples on 300v2
			elsif ($chip eq "300v2"){
				$extreme = 1100;
				$wide = 15500;
			}
			
			else {
				$extreme = 100000000;
				$wide = 100000000;
				print "ERROR: Thresholds have not been set for this chip type. Look at the output to determine thresholds; they are usually about the mean plus two standard deviations to each variable\n";
			}
			
			@minitab = split ("\t", $snps[20]);
			$sample = $minitab[$patname];
			
			#Counts the number of SNPs per chromosome and the number that are 'bad'
			foreach (@snps){
				@tabs = split ("\t", $_, 10);
				if ($tabs[$patname] eq $sample){
					$chromo{$tabs[$chromocol]} = $chromo{$tabs[$chromocol]} + 1;
					#Thres1 - counts the number of SNPs below it for Extreme LogR
					if ($tabs[$log2col] <= $thres1){
						$below1{$tabs[$chromocol]} = $below1{$tabs[$chromocol]} + 1;
						$below1total++;
					}
					#Thres2 - counts the number of SNPs above or below it for Wide/Wavy LogR
					if ($tabs[$log2col] >= $thres2
					or $tabs[$log2col] <= (0 - $thres2)){
						$below2{$tabs[$chromocol]} = $below2{$tabs[$chromocol]} + 1;
						$below2total++;
					}
				}
			}
			
			#Identifies the chromosomes present and sorts them numerically
			@chromo = keys %chromo;
			@chromo = sort {$a <=> $b} @chromo;
			
			#Prints out the chromosome names in the output files
			if ($filenum == 1){
				print OUTCHIP1 "Sample\t";
				print OUTCHIP2 "Sample\t";
				foreach (@chromo){
					print OUTCHIP1 "chr$_\t";
					print OUTCHIP2 "chr$_\t";
				}
				print OUTCHIP1 "Total\n";
				print OUTCHIP2 "Total\n";
			}
				
			print OUTCHIP1 "$sample\t";
			print OUTCHIP2 "$sample\t";
			
			#Resest the variables
			$max1 = 0;
			$total1 = 0;
			$max2 = 0;
			$total2 = 0;
			$n = 0;
			$fail = 0;
			$tot1 = 0;
			$tot2 = 0;
			foreach (@chromo){
				$percent1 = $below1{$_} / $chromo{$_} * 100;
				$percent2 = $below2{$_} / $chromo{$_} * 100;
				#This should only identify max for autosomes (letters are not > 0)
				if ($percent1 >= $max1
				and $_ > 0){
					$max1 = $percent1;
					$maxreport1 = $_;
				}
				if ($percent2 >= $max2
				and $_ > 0){
					$max2 = $percent2;
					$maxreport2 = $_;
				}
				if ($_ > 0){
					$total1 += $percent1;
					$tot1 += $below1{$_};
					$total2 += $percent2;
					$tot2 += $below2{$_};
					$n++;
				}
				$out1 = sprintf("%.2f",$percent1);
				$out2 = sprintf("%.2f",$percent2);
				
				print OUTCHIP1 "$out1\%\t";
				print OUTCHIP2 "$out2\%\t";
			}
			
			$onechro = 0;
			$average1 = $total1 / $n;
			$average2 = $total2 / $n;
			
			if ($tot1 >= $extreme){
				$fail = 1;
			}
			if ($tot2 >= $wide){
				$fail = 1;
			}
			$sampleout{$sample} = "$tot1\t$tot2";
			
			#If all the errors are on a single chromosome (max chr percentage three times the mean chr percentage) then it is flagged
			if ($fail == 1){
				$badchip{$sample} = 0;
				if ($max1 >= ($average1*3) and $max2 < ($average2*3)){
					$flag{$sample} = "Extreme: $maxreport1";
				}
				elsif ($max2 >= ($average2*3) and $max1 < ($average1*3)){
					$flag{$sample} = "Wide: $maxreport2";
				}
				elsif ($max2 >= ($average2*3) and $max1 >= ($average1*3) and $maxreport2 == $maxreport1){
					$flag{$sample} = "Extreme and Wide: $maxreport2";
				}
			}
			
			
			print OUTCHIP1 "$tot1\t$fail\t$onechro\t$maxrep1\n";
			print OUTCHIP2 "$tot2\t$fail\t$onechro\t$maxrep2\n";
			%below1 = ();
			%chromo = ();
			%below2 = ();
			print "$filenum) $sample Extreme: $tot1, Wide: $tot2\n";
		}
	}
	print OUTCHIP1 "\nExtreme LogR threshold used was $thres1";
	print OUTCHIP2 "\nWide LogR threshold used was $thres2";
	close OUTCHIP1;
	close OUTCHIP2;
	
	@badchips = keys %badchip;
	foreach (@badchips){
		print BADCHIP "$_\n";
	}
	
	
	
	if ($method == 0 and $processors <= 1){
		#Opens the Sample List file to add the QT calls
		$samplist = $tag. "_Sample_list.txt";
		open (SAMP, "<$samplist") || print "Could not find $samplist file\n";
		@samplist = <SAMP>; chomp @samplist; close SAMP;
		open (SAMP, ">$samplist") || print "Could not find $samplist file\n";
		print SAMP "$samplist[0]\tExtreme LogR Total\tWide LogR Total\tFlagged?\n";
		shift @samplist;
		$timeanddate = &showtime;
		$totalsamp = keys %sampleout;
	
		foreach (@samplist){
			@tab = split (/\t/,$_);
			$logrvalues = $sampleout{$tab[0]};
			$flag = $flag{$tab[0]};
			if ($flag !~ /:/){
				$flag = "None";
			}
			if ($logrvalues ne ""){
				print SAMP "$_\t$logrvalues\t$flag\n";
				$samplecount++;
			}
			else {
				print SAMP "$_\t0\t0\t0\n";
				print LOG1 "$timeanddate\tQuality\tERROR\tNo chiperror result for $tab[0]\n";
				print "ERROR: No chiperror result for $tab[0]\n";
			}
		}
		if ($samplecount == $totalsamp){
			print LOG1 "$timeanddate\tQuality\tINFO\tChiperror assessment complete for $totalsamp samples\n";
			print "Chiperror assessment complete for $totalsamp samples\n";
		}
		else {
			print LOG1 "$timeanddate\tQuality\tERROR\tChiperror assessment complete, however samples were present that were not listed in the Samplelist file\n";
			print "ERROR: Chiperror assessment complete, however values were calculated for samples that were not listed in the Samplelist file\n";
		}
	}
	else {print "\nError check complete\n";}
}

#Main subroutine of Proportion program
sub questprocess {
	#Questions in the format Sample_col at [0] then column\tfunction\tterm
	@questions = @_;
	@tab = split (/\t/, $questions[0]);
	$sample_col = $tab[0];
	shift @questions;
	foreach (@questions){
		@tab = split (/\t/, $_);
		$tab[1] = &functionconv($tab[1]);
		$headername = substr($head[$tab[0]],0,25);
		$gaps = 25 - length ($headername);
		$spaces = "";
		foreach (1..$gaps){
			$spaces = $spaces." ";
		}
		print "$headername".$spaces."   $tab[1]    $tab[2]\n";
	}
	
	$answer = 0; %sample = (); %samplegene = (); %samplecnvgene = (); @genes = (); @newgenes = (); $arraycount = 0; @results2 = (); @results3 = (); @results4 = (); $thesamplelist = "";
	$genelist = ""; 
	#For each line of the CNV file...
	foreach $line (@propinput){
		@linetab = split (/\t/, $line);
		chomp @linetab;
		$result = "Pass1";
		chomp $linetab[$sample_col];
		$linetab[$sample_col] =~ s/^\s+|\s+$//g;
		#This gives the total number of samples
		if ($result =~ /Pass/ and $linetab[$sample_col] ne ""){
			chomp $linetab[$sample_col];
			$totalsamp{$linetab[$sample_col]} = 1;
		}
		#For each question in the input file
		foreach $quest (@questions){
			if ($result =~ /Pass/){
				@tab = split (/\t/, $quest);
				chomp @tab;
				#print "Tab: $tab[0],$tab[1],$tab[2]\n";
				$tab[1] = &functionconv($tab[1]);
				if ($tab[1] eq '=='){&equals(@tab);}
				elsif ($tab[1] eq '>=' or $tab[1] eq '=>'){&greatequal(@tab);}
				elsif ($tab[1] eq '>'){&great(@tab);}
				elsif ($tab[1] eq '<=' or $tab[1] eq '=<'){&lessequal(@tab);}
				elsif ($tab[1] eq '<'){&lessr(@tab);}
				elsif ($tab[1] eq '!=' or $tab[1] eq '=!'){&notequal(@tab);}
				elsif ($tab[1] eq '=~' or $tab[1] eq '~='){&contains(@tab);}
				elsif ($tab[1] eq '!~' or $tab[1] eq '~!'){&notcontain(@tab);}
				elsif ($tab[1] eq '<>' or $tab[1] eq '><'){&between(@tab);}
				else {print "ERROR: Function $tab[1] is not recognised\n"; sleep (2);}
			}
		}
		if ($result =~ /Pass/ and $linetab[$sample_col] ne ""){
			#print "Sample: $linetab[$sample_col]\n";
			$answer++;
			$sample{$linetab[$sample_col]} += 1;
			print CNVOUT $line."\n";
			if ($linetab[$genecol] ne "0" and $linetab[$genecol] ne ""){
				$linetab[$genecol] =~ s/\"//g;
				$genecount = ($linetab[$genecol] =~ tr/,//) + 1;
				$samplegene{$linetab[$sample_col]} += $genecount;
				$samplecnvgene{$linetab[$sample_col]} += 1;
				$genelist .= $linetab[$genecol].", ";
			}
			#push (@genes, $linetab[$genecol]);
			#print "Genes: $linetab[$genecol]\n"; sleep(1);
		}
	}
	
	#Put the list of genes into a hash (key is analysis number, value is list of genes)
	$geneperanalysis{$thiscount} = $genelist;
	$totgenecount = ($genelist =~ tr/,//);
	#$analysiscount++;
	if ($maxgenecount < $totgenecount){
		$maxgenecount = $totgenecount;
	}
	
	@totsampkeys = keys %totalsamp;
	$total = @totsampkeys;
	
	@samplekeys = keys %sample;
	$prop = @samplekeys;
	
	#Work out the output for the ttest file
	foreach $samp (@totsampkeys){
		$ttestcnvout = $sample{$samp} + 0;
		$ttestgeneout = $samplegene{$samp} + 0;
		$ttestgeniccnvout = $samplecnvgene{$samp} + 0;
		$thesamplelist .= $samp."\t";
		push (@results2, $ttestcnvout);
		push (@results3, $ttestgeneout);
		push (@results4, $ttestgeniccnvout);
	}
	# foreach (@genes){
		# #print "Genes: $_\n";
		# @genelist = split (/, /,$_);
		# foreach (@genelist){
			# $_ =~ s/\"//g;
			# push (@newgenes,$_);
			# #print "Genelist: $_\n"; sleep(1);
		# }
	# }
	# $finalgene = @finalgene;
	# $newgene = @newgenes;
	# if ($finalgene >= $newgene){
		# foreach (@finalgene){
			# $finalgene[$arraycount] = $finalgene[$arraycount].$newgenes[$arraycount]."\t";
			# $arraycount++;
		# }
	# }
	# else {
		# foreach (@newgenes){
			# $finalgene[$arraycount] = $finalgene[$arraycount].$newgenes[$arraycount]."\t";
			# #print "Final array: $finalgene[$arraycount]\n"; sleep (1);
			# $arraycount++;
		# }
	# }
	$perprop = sprintf("%.1f",$prop / $total * 100);
	$notgot = $total - $prop;
	print "There are $answer CNVs, $totgenecount genes and $prop people out of $total which is $perprop\n\n";
	$resultin = "$answer\t$totgenecount\t$prop\t$notgot\t$total\t$perprop%";
	push (@results, $resultin);
	#print @samplekeys;
}

#Subroutine of Proportion program
sub askinput {
	print "Which column do you want to filter by next (leave blank if none)?\n";
	$user_col = <STDIN>; chomp $user_col; $thiscol = ($user_col - 1);
	if ($user_col eq ""){
		$nomore = 1;
	}
	else {
		print "What function do you want to filter by (=, !=, >, >=, <, <=, =~, !~, <>)?\n";
		$user_func = <STDIN>; chomp $user_func;
		print "What term do you want to filter by (eg 3, ZNF, 0.9)?\n";
		$user_term = <STDIN>; chomp $user_term;
		if ($user_term =~ /%/){$user_term = $user_term / 100;}
		$arrayin = "$thiscol\t$user_func\t$user_term";
		push (@questions, $arrayin);
	}
}

#Subroutine of Proportion program
sub functionconv {
	$here = $_[0];
	if ($here =~ /equal/i){
		if ($here =~ /not/i){
			$thefunc = '!=';
		}
		else {$thefunc = '==';}
	}
	elsif ($here =~ /contain/i){
		if ($here =~ /not/i){
			$thefunc = '!~';
		}
		else {$thefunc = '=~';}
	}
	elsif ($here =~ /between/i){
		$thefunc = '<>';
	}
	else {$thefunc = $here;}
	return ($thefunc);
}

#Subroutine of Proportion program
sub equals {
	@tab = @_;
	if (lc $linetab[$tab[0]] eq lc $tab[2]){$result = "Pass2";}
	else {$result = "Fail2";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub greatequal {
	@tab = @_;
	#print "$linetab[$tab[0]] vs. $tab[2]          ";
	if ($linetab[$tab[0]] >= $tab[2]){$result = "Pass3";}
	else {$result = "Fail3";}
	#print "$result\n"; sleep (1);
}

#Subroutine of Proportion program
sub great {
	@tab = @_;
	if ($linetab[$tab[0]] > $tab[2]){$result = "Pass4";}
	else {$result = "Fail4";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub lessequal {
	@tab = @_;
	if ($linetab[$tab[0]] <= $tab[2]){$result = "Pass5";}
	else {$result = "Fail5";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub lessr {
	@tab = @_;
	if ($linetab[$tab[0]] < $tab[2]){$result = "Pass6";}
	else {$result = "Fail6";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub notequal {
	@tab = @_;
	if (lc $linetab[$tab[0]] ne lc $tab[2]){$result = "Pass7";}
	else {$result = "Fail7";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub contains {
	@tab = @_;
	if ($linetab[$tab[0]] =~ /$tab[2]/i){$result = "Pass8";}
	else {$result = "Fail8";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub notcontain {
	@tab = @_;
	if ($linetab[$tab[0]] !~ /$tab[2]/i){$result = "Pass9";}
	else {$result = "Fail9";}
	#print "$result\n";
}

#Subroutine of Proportion program
sub between {
	@tab = @_;
	$tab[2] =~ s/ //gi;
	$tab[2] =~ s/\"//gi;
	@reghere = split (",", $tab[2]);
	#print "$reghere[0], $reghere[1], $linetab[$tab[0]]\n";
	if ($linetab[$tab[0]] >= $reghere[0] and $linetab[$tab[0]] <= $reghere[1]){$result = "Pass9";}
	else {$result = "Fail9";}
	#print "$result\n";
}

#Sorting subroutine
sub specificSort{
    @first = split( '\t', $a );
    @second = split( '\t', $b );

    $compare = ( $first[$chromocol] cmp $second[$chromocol] );           # order within gene
    if ( $compare != 0 ) { return ( $compare ); }

    $compare = ( $first[$chromodist] <=> $second[$chromodist] );           # then, order by position 
    return ( $compare );
}

#Sorting subroutine
sub specificSort2 {
    @first = split( '\t', $a );
    @second = split( '\t', $b );

    $compare = ( $first[1] cmp $second[1] );           # chromosome
    if ( $compare != 0 ) { return ( $compare ); }

    $compare = ( $first[2] <=> $second[2] );           # position1
    if ( $compare != 0 ) { return ( $compare ); }

    $compare = ( $first[3] <=> $second[3] );           # position2
    return ( $compare );
}

#Sorting subroutine
sub specificSortPLINK {
    @first = split( /\t/, $a );
    @second = split( /\t/, $b );

    $compare = ( $first[$chromocol] <=> $second[$chromocol] );           # job number
    if ( $compare != 0 ) { return ( $compare ); }

    $compare = ( $first[$chromodist] <=> $second[$chromodist] );           # job number
    { return ( $compare ); }

}

#Subroutine to give the current time
sub showtime {
@months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
@weekDays = qw(Sun Mon Tue Wed Thu Fri Sat Sun);
($second, $minute, $hour, $dayOfMonth, $month, $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime();
$year = 1900 + $yearOffset;
$hour= sprintf("%02d", $hour);
$minute= sprintf("%02d", $minute);
$second= sprintf("%02d", $second);
$smallTime = "h$hour" . "m$minute";
$theTime = "$hour:$minute:$second, $weekDays[$dayOfWeek] $months[$month] $dayOfMonth, $year";
$timeanddate = "$hour:$minute $dayOfMonth/$months[$month]/$year";
$stamp = "$months[$month]$dayOfMonth,$year";
return $timeanddate;
}

sub chiper{
	my ($probes) = @_;
	#print "Probes_sub: $probes\n";
	if ($probes == 317503){
		return "300v1";
	}
	if ($probes == 318237){
		return "300v2";
	}
	elsif ($probes == 370404){
		return "370v1";
	}
	elsif ($probes == 373397){
		return "370v3";
	}
	elsif ($probes == 545080){
		return "550v3vs550v1";
	}
	elsif ($probes == 551767){
		return "550v3vs610v1";
	}
	elsif ($probes == 555352){
		return "550v1";
	}
	elsif ($probes == 561466){
		return "550v3";
	}
	elsif ($probes == 620901){
		return "610v1";
	}
	elsif ($probes == 588690){
		return "610v1vs1Mv1";
	}
	elsif ($probes == 660918){
		return "650Y";
	}
	elsif ($probes == 657366){
		return "660Q";
	}
	elsif ($probes == 655216){
		return "660W";
	}
	elsif ($probes == 733212){
		return "OmniExp";
	}
	elsif ($probes == 1072820){
		return "1Mv1";
	}
	elsif ($probes == 1040853){
		return "1Mv1vs1Mv3";
	}
	elsif ($probes == 1140419){
		return "Omni1vB";
	}
	elsif ($probes == 1138747){
		return "Omni1vC";
	}
	elsif ($probes == 1199187){
		return "1Mv3";
	}
	
	elsif ($probes == 1386386){
		return "Affy6.0";
	}
	else {
		return "Unknown";
	}
}

#This is a key subroutine for the Annotation program
sub newannotate{
	use Getopt::Long;
	use LWP::Simple;

	######################################################
	# Now we filter for known variants and add annotation
	######################################################
	print "\nChecking Toronto Database of Genomic Variants for latest file...\n";
	$fail=0;
	$testsite=get("http://www.google.com") or ($fail=1);
	if ($fail==1){print "Could not connect to the access server, as you appear to be offline.\nWe will not check the site\n";}
	else{
		$content = get('http://projects.tcag.ca/variation/tableview.asp?table=DGV_Content_Summary.txt');
		@page = split("\n", $content);
		
		foreach $guy (@page){
			if ($guy =~ /current/i){
				$uptodate="yes";
			}
			elsif ($guy =~ /older/i){
				$uptodate="no";
			}
				
			if ($guy =~ /variation.hg18/ and $uptodate eq "yes"){
				@bits = split("'>", $guy);
				$bits[1] =~ s/<\/a>\&nbsp;<\/font\>\<\/td>//;
				print "Current version is $bits[1]\n";
				#print "We are looking for $bits[1] and we have $tdgvfile\n";
				if ($bits[1] eq $tdgvfile){
					print "We are up to date\n";
				}
				else{
					print "ERROR: You're version is $tdgvfile\nYou need to download the latest version and place it into this directory\n";
					print LOG1 "$timeanddate\tAnnotation\tERROR\tCurrent DGV file is $bits[1] and we are using $tdgvfile\n";
				}
			}
		}
	}
	$filename2 = @_[0];
	$tchromocol = (@_[1]);
	$tchrspot1 = (@_[2]);
	$tchrspot2 = (@_[3]);
	$headblam = (@_[4]);
	#open (FINALDATA, "<$filename2") || die "Could not open the file $filename2, may already be open\n";

	if ($overlap ne ''){ $sampleoverlap=($overlap / 100); 
	}
	else {$sampleoverlap=(0);}
	$bigguy=($sampleoverlap*100);


	if ($bigguy > 0){
		print "Checking for regions $bigguy% overlap with known CNVs ($sampleoverlap)\n";
	}
	$genome = "hg18";
	@chrs = (CHR1, CHR2, CHR3, CHR4, CHR5, CHR6, CHR7, CHR8, CHR9, CHR10, CHR11, CHR12, CHR13, CHR14, CHR15, CHR16, CHR17, CHR18, CHR19, CHR20, CHR21, CHR22, CHRX, CHRY);

	$range=0; $countah=0;
	@datah = @finaldata; chomp @datah;
	$tracker=0;


	#The gene files are loaded separately into chromosome specific arrays called CHR.chr number eg CHR1
	$inputhere = $ANloc.$genesfile1;
	$inputhere2 = $ANloc."PreMerged_".$genesfile1;
	print "\nLoading genes and exons from $genesfile1";
	if (-e $inputhere2){
		open (INBOUND, "<$inputhere2") || die "Could not open $inputfiles[$ct]; check that the annotation file directory is correct by running --setup\n";
		{ local $/=undef;  $input1=<INBOUND>; }
		close INBOUND;
		($\) = (/(\r\n|\r|\n)/);
		@input2=split (/\r\n|\r|\n/, $input1); chomp @input2;
	}
	
	open (GENES, "<$inputhere") || die "Could not the genes file $genesfile1; check it is in the annotation folder: $ANloc\n";
	{ local $/=undef;  $input1=<GENES>; }
	close GENES;
	($\) = (/(\r\n|\r|\n)/);
	@genes=split (/\r\n|\r|\n/, $input1); chomp @genes;
	
	
	for ($z=0; $z<=$#genes; $z++){
		chomp;
		($ENSG, $chromo, $start_gene, $stop_gene, $start_temp, $stop_temp, $name, $exon, $size) = split("\t", $genes[$z]);
		$countah++;
		if ($countah >= $tracker and $javamethod == 0){print "\rLoading genes and exons from $genesfile1...$countah"; $tracker+=50;}
		#This submits the gene positions to a hash that will become the first file in @inputfiles
		$phangene = "$ENSG\t$chromo\t$start_gene\t$stop_gene";
		$phanlist{$phangene} = 0;
		#This is the original that makes an array of exon starts and stops
		$newexon = "$ENSG\t$chromo\t$start_temp\t$stop_temp\t$name\t$exon\t$size";
		$chromo = "CHR" . $chromo;
		push (@$chromo, $newexon);
	}
	$tracker=0; #print "\n";
	@input3 = keys %phanlist;

	#Here the annotation files are loaded into chromosome specific arrays called $ct.CHR.chr number eg 0CHR1

	for ($ct=0; $ct<=$#inputfiles; $ct++){
		$countah=0;
		@input='';
		if ($ct == 0){
			if ($#input2 > $#input3){
				@input = @input2;
			}
			else{
				@input = @input3;
			}
			$inputhere2 = $ANloc."PreMerged_".$genesfile1;
		}
		else {
			print "Loading file $inputfiles[$ct]";
			$inputhere = $ANloc.$inputfiles[$ct];
			$inputhere2 = $ANloc."PreMerged_".$inputfiles[$ct];
			if (-e $inputhere2){
				open (INBOUND, "<$inputhere2") || die "Could not open $inputfiles[$ct]; check that the annotation file directory is correct by running --setup\n";
				{ local $/=undef;  $input1=<INBOUND>; }
				close INBOUND;
				($\) = (/(\r\n|\r|\n)/);
				@input=split (/\r\n|\r|\n/, $input1); chomp @input;
				$premerged = 1;
			}
			else {
				open (INBOUND, "<$inputhere") || die "Could not open $inputfiles[$ct]; check that the annotation file directory is correct by running --setup\n";
				{ local $/=undef;  $input1=<INBOUND>; }
				close INBOUND;
				($\) = (/(\r\n|\r|\n)/);
				@input=split (/\r\n|\r|\n/, $input1); chomp @input;
				$premerged = 0;
			}
		}
		#This makes a hash of fileheaders
		$fileheaders{$inputfiles[$ct]}=$input[0];
		#Now sort the custom file
		$flag=0;
		if ($inputhere =~ /variation.hg/ and $inputhere !~ /McC/ and $premerged == 0){
			foreach (@input){
				@tab = split (/\t/,$_);
				#DGV will become: Name, Chr, Start, Stop, Papers, Gain, Loss, Total
				$dgvcorrect = "$tab[0]\t$tab[2]\t$tab[3]\t$tab[4]\t$tab[9]\t$tab[12]\t$tab[13]\t$tab[14]";
				#print "$dgvcorrect\n";
				push (@dgvcorrect, $dgvcorrect);
			}
			@input = @dgvcorrect;
			@dgvcorrect = ();
		}
		
		@input = sort { (split '\t', $a, 4)[2] <=> (split '\t', $b, 4)[2] } @input;
		@input = sort { (split '\t', $a, 4)[1] cmp (split '\t', $b, 4)[1] } @input;
		
		$sampleID = 0;
		$chromo = 1;
		$start = 2;
		$stop = 3;
		
		#This looks for overlap in the reference file; if found flag = 1
		$pchromo=''; $pstop=0; $flag=0;
		foreach $piece (@input){
			@pnow = split (/\t/, $piece);
			if ($pnow[$chromo] eq $pchromo && $pnow[$start] <= $pstop){
				#print "$pnow[$chromo] eq $pchromo && $pnow[$start] <= $pstop\n";
				$flag=1;
				last;
			}
			$pchromo=$pnow[$chromo]; $pstop=$pnow[$stop];
		}
		
		if ($flag==1){
			print "\nOverlapping regions detected, merging regions to correct\n";
			
			############################################
			## Start of Merge Function for Custom Files
			############################################
			@input = &MergeAnnotation;
		}
			##########################################
			## End of Merge Function for Custom Files
			##########################################
		foreach (@input){
			chomp;
			#This is the dgv file - note numpaps vs nums
			if ($inputfiles[$ct] eq $tdgvfile){
				($name, $chromo, $start_temp, $stop_temp, $na, $g1, $l1, $t, $freq, ) = split ("\t", $_);
				$chromo = uc $chromo;
				@gainarray = split (/, /,$g1); chomp @gainarray; $g12 = 0; foreach (@gainarray){$g12 += $_;}
				@lossarray = split (/, /,$l1); chomp @lossarray; $l12 = 0; foreach (@lossarray){$l12 += $_;}
				@totarray = split (/, /,$t); chomp @totarray; $t2 = 0; foreach (@totarray){$t2 += $_;}
				$pusher = "$name\t$chromo\t$start_temp\t$stop_temp\t$na\t$g12\t$l12\t$t2\t";
				#print MOUT "$pusher\n";
				#print "Paper: $na, Gain: $g12, Loss: $l12, Tot: $t2\n";
			}
			else{
				($name, $chromo, $start_temp, $stop_temp, $na, $g1, $l1, $t, $freq, $nums) = split ("\t", $_);
				$chromo = uc $chromo;
				$pusher = "$name\t$chromo\t$start_temp\t$stop_temp\t$na\t$g1\t$l1\t$t\t$freq\t";
				#unless ($inputfiles[$ct] =~ $genesfile1){print "Name: $name, Chr: $chromo, Start: $start_temp, Stop: $stop_temp,\nFive: $na, Six: $g1, Seven: $l1, Eight: $t, Nine: $freq\n";}
			}
			$countah++; if ($countah >= $tracker and $javamethod == 0){print "\rLoading file $inputfiles[$ct]...$countah"; $tracker+=25;} 
			$chromo =~ s/chr//i;
			$chromo = "$ct" . "CHR" . $chromo;
			push(@$chromo, $pusher);
		}

		print "\n"; $tracker=0;
	}


	chomp($datah[0]);
	@headtemp=split("\t", $datah[0]);
	if ($headblam =~ /N/i){foreach $man (@headtemp){print OUT "\t";} }
	else{print OUT "$datah[0]\t";shift @datah;}
	

	$dtotal = ($#datah +1); $estimate = ($dtotal / 400); $estimate = sprintf("%0.2f", $estimate);
	print "\nNow annotating your $dtotal lines, should take $estimate seconds\n";
	print LOG1 "$timeanddate\tAnnotation\tINFO\t$dtotal lines to be annotated\n";
	
	print OUT "Gene\tLocation\t#Genes\t#Exons\t5'gene\t5'dist(kb)\t3'gene\t3'dist(kb)\t";

	#This is for the headers of the custom files
	for ($ct=1; $ct<=$#inputfiles; $ct++){
		if ($inputfiles[$ct] =~ /variation.hg18/ && $inputfiles[$ct] !~ /McC/i){print OUT "dups(DGV)\tdels(DGV)\ttotal_known_CNVs(DGV)\tnum_papers(DGV)\tpapers\t";}
		elsif ($inputfiles[$ct] =~ /McCarroll/i){print OUT "CNP_dups(McCarroll)\tCNP_dels(McCarroll)\tCNPsall(McCarroll)\t";}
		else{
			$filehere=$inputfiles[$ct];
			if ($firstcolhash{$filehere} =~ /y/i){
				print OUT "$filehere\t";
			}
			if ($perhash{$filehere} !~ /n/i){
				print OUT "$filehere %overlap\t";
			}
			if ($disthash{$filehere} !~ /n/i){
				print OUT "$filehere 5'feature\t$filehere 5'dist(kb)\t$filehere 3'feature\t$filehere 3'dist(kb)\t";
			}
			@extracolumns='';
			@extracolumns = sort {$a <=> $b}(keys %$filehere);
			#@extracolumns = split(",", $colhash{$filehere});
			@newhead = split (/\t/, $fileheaders{$filehere});
			foreach $edge (@extracolumns){
				#print "File: $filehere, Edge: $edge, Hash: $$filehere{$edge}\n";
				#$edge =~ s/\"//g;
				#the user puts a column, but Perl starts counting at zero, so
				$newedge = ($edge - 1);
				#print "for $inputfiles[$ct], we have $colhash{$filehere}, $newhead[$newedge]\t";
				if ($$filehere{$edge} =~ /y/i or $$filehere{$edge} =~ /Name/i){
					print OUT $filehere."_".$newhead[$newedge]."\t";
				}
				elsif ($$filehere{$edge} =~ /Num/i){
					print OUT $filehere."_".$newhead[$newedge]."_".Number."\t";
				}
				elsif ($$filehere{$edge} =~ /Total/i){
					print OUT $filehere."_".$newhead[$newedge]."_".Total."\t";
				}
				elsif ($$filehere{$edge} =~ /Max/i){
					print OUT $filehere."_".$newhead[$newedge]."_".Max."\t";
				}
				elsif ($$filehere{$edge} =~ /Min/i){
					#print "I'm here\n";
					print OUT $filehere."_".$newhead[$newedge]."_".Min."\t";
				}
			}
		}
	}
	print OUT "\n";
	$countah=0;
	#Here the input file is processed, each line is a region in @datah
	foreach (@datah){
		$countah++; 
		if ($countah >= $tracker and $javamethod == 0){$tracker+=5; print "\r$tracker completed...";}
		elsif ($countah >= $tracker and $javamethod == 1){$tracker+=5000; print "\r$tracker completed...";}
		chomp;
	#	print "$countah\n"; 
		@fornow = split("\t", $_);
		$chrom = $fornow[$tchromocol];
		$head = $fornow[$tchrspot1];
		$tail = $fornow[$tchrspot2];
		$head =~ s/\,|\"//gi;
		$tail =~ s/\,|\"//gi;
		$headtotail= ($tail - $head +1);
		$chrom =~ s/chr//i;
		$chrom =~ tr/[a-z]/[A-Z]/;
		$chrom = "CHR" . (uc $chrom);
	#	print "here is $_\n and we have $chrom\n";
		
		$endspot = $#$chrom;
		$begspot = 0;
		$size = $#$chrom;
		$newspot = ($size / 2);
		#Of note the $newspot is rounded using a subroutine here
		$newspot = &round($newspot);
		$found=0;
		@genes=''; @exons=''; @introns=''; $type=''; $fivegene='in'; $fivedist=0; $threegene='in'; $threedist=0; @names=''; $oldspot='';
		#print "examining $gene, $chrom, $head, $tail\n";
		print OUT "$_\t";
		if($#$chrom<0){
			#print "$_, note: chromosome $#chrom is empty\n";
			$found=1;
		}
	
		#Here it deals with genes and exons from the genes file that was loaded spearately (format CHR.chr number)
		do {
			($ENSG0, $chromo0, $start_temp0, $stop_temp0, $name0, $exon0, $size0) = split("\t", $$chrom[$newspot - 1]);
			($ENSG, $chromo, $start_temp, $stop_temp, $name, $exon, $size) = split("\t", $$chrom[$newspot]);
			($ENSG2, $chromo2, $start_temp2, $stop_temp2, $name2, $exon2, $size2) = split("\t", $$chrom[$newspot + 1]);
			#print "$newspot now $ENSG, $chromo, $start_temp <= $head && $tail >= $stop_temp\n"; 
			#If the head and tail lie within an exon then the region is full within and exon
			if ($start_temp <= $head && $tail <= $stop_temp){ #region inside of exon
				#print "Region within Exon\n";
				#print "Exon: 1\n";
				#print "found one inside! $start_temp <= $position && $position <= $stop_temp\n"; sleep(1);
				push (@genes, $name);
				$found=1; $type="Exon";
				push (@exons, $exon);
			}
			#If the head and tail lie either side of an exon then there is an exon inside
			elsif ($start_temp >= $head && $tail >= $stop_temp){ #exon inside of region
				#print "Exon within Region\n";
				#print "Exon: 2\n";
				#print "found an exon inside of it! $start_temp <= $head && $tail >= $stop_temp\n";
				#print "Adding 1: $name\n";
				push (@genes, $name);
				$found=1; $type="Exon";
				push (@exons, $exon);
					
					#Here the algorithm looks in the 5' direction to find other exons that might be in the region
					#print "5' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n";sleep(1);
					$inspot = $newspot;
					$incheck=0;
					do{
						if ($inspot <= 0){}
						else{
							$inspot--;
							($ENSG, $chromo, $start_temp, $stop_temp, $name, $exon, $size) = split("\t", $$chrom[$inspot]);
							#print "decrement to $inspot\n";
							if ($head < $start_temp){
								#print "Adding 2: $name\n";
								push (@genes, $name);
								push (@exons, $exon);
								push (@introns, $exon);
							}
							elsif($head >= $start_temp && $head <= $stop_temp){
								$incheck=1;
								#print "Adding 3: $name\n";
								push (@genes, $name);
								push (@exons, $exon);
								push (@introns, $exon);
							}
							else{
								$incheck=1;
								if ($ENSG eq $oldgene){
									push (@introns, $ENSG);
								}
							}
							$oldgene = $ENSG;
						}
					}
					until($incheck==1 || $inspot==0);
					#print "3' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n"; sleep(1);
					$inspot = $newspot;
					$incheck=0;
					#Here the algorithm looks in the 3' direction to find other exons that might be in the region
					do{
						if ($inspot >= ($#$chrom)){}
						else{
							$inspot++;
							#print "increment to $inspot\n";
							($ENSG, $chromo, $start_temp, $stop_temp, $name, $exon, $size) = split("\t", $$chrom[$inspot]);
							if ($tail > $stop_temp){
								#print "Adding 4: $name\n";
								push (@genes, $name);
								push (@exons, $exon);
								push (@introns, $exon);
							}
							elsif($tail >= $start_temp && $tail <= $stop_temp){
								$incheck=1;
								#print "Adding 5: $name\n";
								push (@genes, $name);
								push (@exons, $exon);
								push (@introns, $exon);
							}
							else{
								$incheck=1;
								if ($ENSG eq $oldgene){
									push (@introns, $exon);
								}
								else{
								}
							}
							$oldgene = $ENSG;
						}
					}
					until($incheck==1 || $inspot>=$#$chrom);
					$found=1;
			}
			#Newspot has hit a region where the exon extends in the 5' direction
			elsif ($start_temp <= $tail && $tail <= $stop_temp){  #exon 5' overlap
				#print "Exon extends 5'\n";
				#print "Exon: 3\n";
				#print "$newspot 5' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n"; sleep(1);
				push (@genes, $name);
				push (@exons, $exon);
				$type="Exon";
				$inspot = $newspot;
				$incheck=0;
				#Here the algorithm looks in the 5' direction to find other exons that might be in the region
				do{
					unless ($newspot<=0){$inspot--;}
					($ENSG, $chromo, $start_temp, $stop_temp, $name, $exon, $size) = split("\t", $$chrom[$inspot]);
					if ($head < $start_temp){
						push (@genes, $name);
						push (@exons, $exon);
						push (@introns, $exon);
					}
					elsif($head >= $start_temp && $head <= $stop_temp){
						$incheck=1;
						push (@genes, $name);
						push (@exons, $exon);
						push (@introns, $exon);
					}
					else{
						$incheck=1;
						if ($ENSG eq $oldgene){
							push (@introns, $ENSG);
						}
						else{
						}
					}
					$oldgene = $ENSG;	
				}
				until($incheck==1 or $inspot<=0);
				$found=1;
			}
			#Newspot has hit a region where the exon extends in the 3' direction
			elsif ($start_temp <= $head && $stop_temp >= $head){  #exon 3' overlap
				#print "Exon extends 3'\n";
				#print "Exon: 4\n";
				#print "3' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n"; sleep(1);
				push (@genes, $name);
				push (@exons, $exon);
				$type="Exon";
				$inspot = $newspot;
				$incheck=0;
				#Here the algorithm looks in the 3' direction to find other exons that might be in the region
				do{
					$inspot++;
					($ENSG, $chromo, $start_temp, $stop_temp, $name, $exon, $size) = split("\t", $$chrom[$inspot]);
					if ($tail > $stop_temp){
						push (@genes, $name);
						push (@exons, $exon);
						push (@introns, $exon);
					}
					elsif($tail >= $start_temp && $tail <= $stop_temp){
						$incheck=1;
						push (@genes, $name);
						push (@exons, $exon);
						push (@introns, $exon);
					}
					else{
						$incheck=1;
						if ($ENSG eq $oldgene){
							push (@introns, $exon);
						}
						else{
						}
					}
					$oldgene = $ENSG;	
				}
				until($incheck==1 || $inspot>= $#$chrom);
				$found=1;
			}
			#The region is between two exons (before and current) without touching either 5'
			elsif($head > $stop_temp0 && $tail< $start_temp){
				#print "IG: 5\n";
				push (@intergenic, $_);
				$type="Intergenic";
				$fivegene = $name0;
				$threegene = $name;
				$fivedist = ($head - $stop_temp0) / 1000;
				$threedist = ($start_temp - $tail) / 1000;
				$found=1;
			}
			#The region is between two exons (current and next) without touching either 3'
			elsif($head > $stop_temp && $tail< $start_temp2 ){
				#print "IG: 6\n";
				push (@intergenic, $_);
				$type="Intergenic";
				$fivegene = $name;
				$threegene = $name2;
				$fivedist = ($head - $stop_temp) / 1000;
				$threedist = ($start_temp2 - $tail) / 1000;
				$found=1;
			}
			#The region is before the first exon at the 5' end of the chromosome
			elsif($tail < $start_temp && $tail< $stop_temp && $newspot==0){ 
				#print "IG: 7\n";
				#print "Region before first 5' exon\n";
				#print "yut\n"; sleep(1);
				push (@intergenic, $_);
				$type="Intergenic";
				$fivegene = $name;
				$threegene = "StartChr";
				$fivedist = ($start_temp - $tail) / 1000;
				$threedist = "StartChr";
				$found=1;
			}
			#The region is after the last exon at the 3' end of the chromosome
			elsif($head > $start_temp && $head > $stop_temp && $newspot>=$#$chrom){
				#print "IG: 8\n";
				#print "Region after last 3' exon\n";
				#print " i \n";
				$type="Intergenic";
				$fivegene = $name;
				$threegene = "EndChr";
				$fivedist = ($head - $stop_temp) / 1000;
				$threedist = "EndChr";
				$found=1;
			}
			#This is to stop the binary search algorithm getting stuck (it gives it a shake if newspot has not changed since the last round
			elsif($lastspot == $newspot){
				#print "Stuck: 9\n";
				#print "I'm landing here\n";
				#print "later\n"; sleep(1);
				if ($newspot == 1){
					$newspot--;
				}
				elsif($newspot >= $#$chrom){
					$newspot++;
				}
				else{
				push (@nonintrons, $_);
					$found=1;
				}
			}
			#If newspot is not hitting any of the scenarios above then it is repeated
			else{
				#print "Other: 10\n";
				if($tail < $start_temp){
					#print "test: $head is less than $stop_temp\n";
					#then we want the first half
					$oldspot=$newspot;
					$endspot = $newspot;
					#print "switching, so $endspot - $begspot is the new one\n";
					$newspot = ( $begspot + (($endspot - $begspot) / 2));
					$newspot = &round($newspot);
					#print "$newspot lower array  $head $tail vs. $start_temp $stop_temp\n"; sleep(1);
				}
				else{
					$oldspot=$newspot;
					#print "test: $tail is greater than $start_temp\n";
					$begspot = $newspot;
					#print "we are making $endspot vs. $begspot\n";
					$newspot = (  ($begspot + ($endspot - $begspot) / 2 ));
					if ($newspot > $#$chrom){$newspot=$#$chrom; }
					$newspot = &round($newspot);
					#print "$newspot upper array $head $tail vs. $start_temp $stop_temp\n"; sleep(1);
				}
			}
			if ($oldguy ==1 ){$newspot=0;}
			$oldguy = $newspot;
		}until($found==1);
		
		@ordered_genes = sort @genes; 
		$old=''; @hitgenes=''; $genelist=''; $numgenes=0;
		#find out how many unique functions are involved
		foreach(@ordered_genes){
			if ($_ eq $old){}
			else {push (@hitgenes, $_);}
			$old=$_;
		}
		shift(@hitgenes); $genecount=0;
		foreach $gn (@hitgenes){
			if($genecount==0){$genelist=$gn; $allhits{$gn}++;}
			else{$genelist = "$genelist, " . $gn; $allhits{$gn}++;}
			$genecount++;
		}
		if ($type eq "Intergenic"){$genelist="NA"}
		shift(@exons);
		$numgenes=($#hitgenes+1);
		$numexons=($#exons+1);
		#$threedist = ($threedist / 1000);
		#$fivedist = ($fivedist / 1000);
		$allthrees{$threegene}++;
		$allfives{$fivegene}++;
		
		$exonlistouttype = "$type";
		#print "Out: $genelist; $type\n";
		$exonlistout = "$numexons\t$fivegene\t$fivedist\t$threegene\t$threedist";
		$ogchrom=$chrom;
	
		#This is where the inputfiles are used (ie after the genes)
		for ($ct=0; $ct<=$#inputfiles; $ct++){
			$oldspotcount = 0;
			#print "$ct\n";
			$chrom = "$ct" . $ogchrom;
			$totaldups=0; $totaldels=0; $totalcnvs=0; @known=''; $begspot = 0; $found=0; @names=''; @dups=''; @dels=''; @tots=''; @covers=''; @nibs=''; @dist3=''; @dist5='';
			$endspot = $#$chrom;
			$begspot = 0;
			$size = $#$chrom;
			$truesize = @$chrom;
			#print "Size: $size, Truesize: $truesize from $chrom and $ogchrom\n"; #sleep(1);
			$newspot = ($size / 2);
			#Again a rounding subroutine
			$newspot = &round($newspot);
			$found=0;
			#All this is if there is only one term in the chromosome specific array relating to that input file 
			
			if($truesize == 0){
				$found=1;
			}
			
			elsif($truesize==0){
			#elsif($#$chrom==0){
				#print "I get here\n";
				#This is where the percentage cover is calculated ($percover)
				($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$newspot]);
				if ($start_temp <= $head && $tail <= $stop_temp){$percover=1; }
				elsif ($start_temp > $head && $stop_temp < $tail){$percover= ( ($stop_temp - $start_temp) / $headtotail ); }
				elsif ($start_temp < $head && $stop_temp < $tail){$percover= ( ($stop_temp - $head) / $headtotail ); }
				elsif ($start_temp > $head && $stop_temp > $tail){$percover= ( ($tail - $start_temp) / $headtotail ); }
				elsif ($start_temp > $head && $start_temp < $tail){$percover= ( ($tail - $stop_temp) / $headtotail ); }
				
				#If percover was pushed into the @names array rather than $name then it would give the percentage cover as an output...
				#This is where to add arrays if you want over 9 columns
				if ($start_temp <= $head && $tail <= $stop_temp){ #region inside of exon
					#print "yes1\n";
					push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
				}
				elsif ($start_temp >= $head && $tail >= $stop_temp){ #exon inside of region
						if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
				}
				elsif ($start_temp <= $tail && $tail <= $stop_temp){ #exon 5' overlap
						if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
				}
				elsif ($start_temp <= $head && $head <= $stop_temp){ #exon 3' overlap
						if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
				}
					$found=1;
			}
			
			else{
				#print "Region: $chrom:$head-$tail\n";
				#sleep (1);
				$homeincount = 0;
				do {
					#$homeincount++;
					#if($homeincount >= 30){
					#	sleep(1);
					#}
					($name0, $chromo0, $start_temp0, $stop_temp0, $na0, $g0, $l0, $t0, $nib0) = split("\t", $$chrom[$newspot - 1]);
					($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$newspot]);
					($name2, $chromo2, $start_temp2, $stop_temp2, $na2, $g2, $l2, $t2, $nib2) = split("\t", $$chrom[$newspot + 1]);
					#print "CT: $ct, File: $inputfiles[$ct], Newspot: $newspot, St: $start_temp, En: $stop_temp and , $la, $ta, $nib\n"; sleep(1);
					if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
					elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
					elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
					elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
					elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
					elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
					elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
					elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
					else {$percover = 0;} #Need to go through binary search to find this one
					
					#Exon start is less than or equal to the region start and exon end is greater than or equal to the region end
					# --------------------------     exon
					#      ----------------          region
					if ($start_temp <= $head && $tail <= $stop_temp){ #region inside of exon
						#print "found one inside! $start_temp <= $head && $tail <= $stop_temp\n";
						$percover=1;
						$found=1;
							push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
							#print "5' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n";
							$inspot = $newspot;
							$incheck=0;
							do{
								unless ($newspot==0){$inspot--;
									($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
									#let's figure out the overlap
									if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
									elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
									elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
									elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
									elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
									elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
									elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
									elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
									else {$percover = 0;} #Need to go through binary search to find this one
									
									if ($start_temp <= $head && $tail <= $stop_temp){ #region inside of query
										push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
									}
									elsif ($head < $start_temp && $tail > $stop_temp){
										push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
									}
									elsif($head >= $start_temp && $head <= $stop_temp){
										$incheck=1;
										if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
										$inspot--;
									}
									else{$incheck=1;}
								}
								#else {print "This is your problem!\n";}
							}
							until($incheck==1 || $inspot==0);
							#$inspot--;
							($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
							if ($inspot <= 0){$dist5in = "StartChr\tStartChr";}
							else{
								$dist5 = ($head - $stop_temp)/1000;
								$dist5in = "$name\t$dist5";
							}
							#print "1, 5) $percover, $name\n";
							push (@dist5, $dist5in);
							
							
							#print "3' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n";
							$inspot = $newspot;
							$incheck=0;
							do{
								unless ($newspot>=$#$chrom){$inspot++;
									($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
									if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
									elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
									elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
									elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
									elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
									elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
									elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
									elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
									else {$percover = 0;} #Need to go through binary search to find this one
									
									if ($start_temp <= $head && $tail <= $stop_temp){
										push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
									}
									elsif ($tail > $stop_temp){
										push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
									}
									elsif($tail >= $start_temp && $tail <= $stop_temp){
										$incheck=1;
										if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
										$inspot++;
									}
									else{
										$incheck=1;
									}
								}
							}
							until($incheck==1 || $inspot>=$#$chrom);
							#$inspot++;
							($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
							if ($inspot >= $#$chrom){$dist3in = "EndChr\tEndChr";}
							else{
								$dist3 = ($start_temp - $tail)/1000;
								$dist3in = "$name\t$dist3";
							}
							push (@dist3, $dist3in);
							#print "1, 3) $$percover, $name\n";
							$found=1;
					}
					#Exon start is greater than or equal to the region start and the exon end is less than or equal to the region end
					#          ---------------     exon
					# --------------------------   region
					elsif ($start_temp >= $head && $tail >= $stop_temp){ #exon inside of region
						#print "found an exon inside of it! $start_temp <= $head && $tail >= $stop_temp\n";
						if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
						$found=1;
							
							#print "5' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n";
							$inspot = $newspot;
							$incheck=0;
							do{
								unless ($newspot==0){$inspot--;
									#print "decrement to $inspot\n";
									($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
									if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
									elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
									elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
									elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
									elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
									elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
									elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
									elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
									else {$percover = 0;} #Need to go through binary search to find this one
									
									if ($head < $start_temp){
										push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
									}
									elsif($head >= $start_temp && $head <= $stop_temp){
										$incheck=1;
										if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
										$inspot--;
									}
									else{$incheck=1;}
								}
							}
							until($incheck==1 || $inspot==0);
							#$inspot--;
							($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
							if ($inspot <= 0){$dist5in = "StartChr\tStartChr";}
							else{
								$dist5 = ($head - $stop_temp)/1000;
								$dist5in = "$name\t$dist5";
							}
							#print "2, 5) $percover, $name\n";
							push (@dist5, $dist5in);
							
					#		print "3' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n";
							$inspot = $newspot;
							$incheck=0;
							do{
								unless ($newspot>=$#$chrom){$inspot++;
									($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
									if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
									elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
									elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
									elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
									elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
									elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
									elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
									elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
									else {$percover = 0;} #Need to go through binary search to find this one
									
									if ($tail > $stop_temp){
										push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover); 
									}
									elsif($tail >= $start_temp && $tail <= $stop_temp){
										$incheck=1;
										if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
										$inspot++;
									}
									else{
										$incheck=1;
									}
								}
							}
							until($incheck==1 || $inspot>=$#$chrom);
							#$inspot++;
							($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
							if ($inspot >= $#$chrom){$dist3in = "EndChr\tEndChr";}
							else{
								$dist3 = ($start_temp - $tail)/1000;
								$dist3in = "$name\t$dist3";
							}
							push (@dist3, $dist3in);
							#print "2, 3) $percover, $name\n";
							$found=1;
					}
					
					#Exon start is less than or equal to the region end and the exon end is greater than or equal to the region end
					#          ------------------------     exon
					#  ------------------------             region
					elsif ($start_temp <= $tail && $tail <= $stop_temp){ #exon 5' overlap
						#if ($ct == 1){ print "1) Head: $head, Tail: $tail, Start: $start_temp, Stop: $stop_temp\n";}
						#print "$newspot 5' overlap of exon, $start_temp > $head && $tail <= $stop_temp, let's see how far this goes\n";
						push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);
						
						$inspot = $newspot;
						$incheck=0;
						do{
							unless ($newspot==0){
								$inspot--;
								($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
									#if ($ct == 1){ print "2) Head: $head, Tail: $tail, Start: $start_temp, Stop: $stop_temp\n";}
									if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
									elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
									elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
									elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
									elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
									elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
									elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
									elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
									else {$percover = 0;} #Need to go through binary search to find this one
									#print "Per: $percover\n";
								if ($head < $start_temp){
									push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);
								}
								elsif($head >= $start_temp && $head <= $stop_temp){
									$incheck=1;
									if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
									$inspot--;
								}
								else{
									$incheck=1;
								}
							}
						}
						until($incheck==1 or $inspot<=0);
						$dist3 = ($start_temp2 - $tail)/1000;
						$dist3in = "$name2\t$dist3";
						push (@dist3, $dist3in);
						#print "3, 3) $percover, $name2\n";
						($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
						if ($inspot <= 0){$dist5in = "StartChr\tStartChr";}
						else{
							$dist5 = ($head - $stop_temp)/1000;
							$dist5in = "$name\t$dist5";
						}
						push (@dist5, $dist5in);
						#print "3, 5) $percover, $name\n";
						$found=1;
					}
					#Exon start is less than or equal to the region start and the exon end is greater than or equal to the region start
					#   -------------------               exon
					#          ----------------           region
					elsif ($start_temp <= $head && $head <= $stop_temp){ #exon 3' overlap
						#print "3' overlap of exon, $start_temp <= $head && $head <= $stop_temp, let's see how far this goes\n";
						push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);
						$inspot = $newspot;
						$incheck=0;
						do{
							unless ($inspot>=$#$chrom){$inspot++;
								($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
								if ($start_temp <= $head && $tail <= $stop_temp){$percover = 1;} #region inside of exon
								elsif ($start_temp >= $head && $tail >= $stop_temp){$percover = ( ($stop_temp - $start_temp) / $headtotail );} #exon inside of region
								elsif ($start_temp <= $tail && $tail <= $stop_temp){$percover = ( ($tail - $start_temp) / $headtotail );} #exon 5' overlap
								elsif ($start_temp <= $head && $head <= $stop_temp){$percover = ( ($stop_temp - $head) / $headtotail );} #exon 3' overlap
								elsif ($head > $stop_temp0 && $tail < $start_temp){$percover = 0;} #Not in an exon
								elsif ($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){$percover = 0;} #Not in an exon, end chr
								elsif ($head < $stop_temp && $tail < $start_temp && $newspot == 0){$percover = 0;} #Not in an exon, start chr
								elsif ($head > $stop_temp && $tail< $start_temp2 ){$percover = 0;} #Not in an exon
								else {$percover = 0;} #Need to go through binary search to find this one
									
								if ($tail > $stop_temp){
									push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);
								}
								elsif($tail >= $start_temp && $tail <= $stop_temp){
									$incheck=1;
									if($percover >= $sampleoverlap){push (@names, $name); push (@known, $na); push (@dups,$ga); push (@dels,$la); push (@tots,$ta); push (@nibs, $nib); push (@covers, $percover);}
									$inspot++;
								}
								else{
									$incheck=1;
								}
							}
						}
						until($incheck==1 || $inspot>=$#$chrom);
						$dist5 = ($head - $stop_temp0)/1000;
						$dist5in = "$name0\t$dist5";
						push (@dist5, $dist5in);
						#print "4, 5) $percover, $name0\n";
						($name, $chromo, $start_temp, $stop_temp, $na, $ga, $la, $ta, $nib) = split("\t", $$chrom[$inspot]);
						$dist3 = ($start_temp - $tail)/1000;
						$dist3in = "$name\t$dist3";
						push (@dist3, $dist3in);
						#print "4, 3) $percover, $name\n";
						$found=1;
					}
					
					#Exon end below is less than the region start and the current exon start is above the region end
					#  ------------                     Exon before
					#                -------            Region
					#                          -------- Exon current
					elsif($head > $stop_temp0 && $tail < $start_temp){
					#	print "here1 at $newspot and $#$chrom\n$head > $stop_temp0 && $tail< $start_temp\n $start_temp0\t$stop_temp0\n$start_temp\t$stop_temp\n$start_temp1\t$stop_temp1\n\n$$chromo[$newspot-3]\n\n$$chromo[$newspot-2]\n$$chromo[$newspot-1]\n$$chromo[$newspot]\n$$chromo[$newspot+1]\n$$chromo[$newspot+2]\n";
						$fivedist = ($head - $stop_temp0);
						$threedist = ($start_temp - $tail);
						$dist5 = ($head - $stop_temp0)/1000;
						$dist5in = "$name0\t$dist5";
						push (@dist5, $dist5in);
						#print "5, 5) $percover, $name0\n";
						$dist3 = ($start_temp - $tail)/1000;
						$dist3in = "$name\t$dist3";
						push (@dist3, $dist3in);
						#print "5, 3) $percover, $name\n";
						$found=1;
					}
					#Exon end is less than region start and exon start is less than region end and $newspot is at the end of the chromosome
					#ie no exon hit and end of the chromosome reached
					elsif($head > $stop_temp && $tail > $start_temp && $newspot==$#$chrom){
						#print "herenew at $newspot and $#$chrom\n";
						$fivedist = ($head - $stop_temp);
						$threedist = "ChrEnd";
						$dist5 = ($head - $stop_temp0)/1000;
						$dist5in = "$name0\t$dist5";
						push (@dist5, $dist5in);
						#print "6, 5) $percover, $name0\n";
						$dist3in = "ChrEnd\tChrEnd";
						push (@dist3, $dist3in);
						#print "6, 3) $percover, ChrEnd\n";
						$found=1;
					}
					#Exon start is greater than region end and exon end is greater than region end and $newspot is at the start of the chromosome
					#ie no exon hit and start of the chromosome reached
					elsif($head < $stop_temp && $tail < $start_temp && $newspot == 0){
					#	print "herenew at $newspot and $#$chrom\n";
						$fivedist = "ChrStart";
						$threedist = ($start_temp - $tail);
						$dist5in = "ChrStart\tChrStart";
						push (@dist5, $dist5in);
						#print "7, 5) $percover, ChrStart\n";
						$dist3 = ($start_temp2 - $tail)/1000;
						$dist3in = "$name2\t$dist3";
						push (@dist3, $dist3in);
						#print "7, 3) $percover, $name2\n";
						$found=1;
					}
					#Exon end is less that the region start and the next exon start is greater than the region end
					#  ------------                     Exon current
					#                -------            Region
					#                          -------- Exon next
					elsif($head > $stop_temp && $tail< $start_temp2 ){
					#	print "$head > $stop_temp && $tail< $start_temp2, here2\n";
						$fivedist = ($head - $stop_temp);
						$threedist = ($start_temp2 - $tail);
						$dist5 = ($head - $stop_temp)/1000;
						$dist5in = "$name\t$dist5";
						push (@dist5, $dist5in);
						#print "8, 5) $percover, $name\n";
						$dist3 = ($start_temp2 - $tail)/1000;
						$dist3in = "$name2\t$dist3";
						push (@dist3, $dist3in);
						#print "8, 3) $percover, $name2\n";
						$found=1;
					}
					
					#If there is no match then the binary search is repeated
					else{
						#print "Modifying newspot\n";
						#Exon start is greater than the region end
						if($tail < $start_temp){
							if($newspot != $oldspot){
								#print "test: $head is less than $stop_temp\n";
								#then we want the first half
								$oldspot=$newspot;
								$endspot = $newspot;
								#print "switching, so $endspot - $begspot is the new one\n";
								$newspot = ( $begspot + (($endspot - $begspot) / 2));
								#print "Preround1: $newspot, ";
								$newspot = &round($newspot);
								#print "Postround1: $newspot\n";
							#	print "$newspot lower array  $head $tail vs. $start_temp $stop_temp\n"; sleep(1);
							}
							else{
								#print "Going down\n";
								$newspot--;
							}
						}
						#Exon start is less than the region end
						else{
							if($newspot != $oldspot){
								$oldspot=$newspot;
								#print "test: $tail is greater than $start_temp\n";
								$begspot = $newspot;
								#print "we are making $endspot vs. $begspot\n";
								$newspot = (  ($begspot + ($endspot - $begspot) / 2 ));
								if ($newspot > $#$chrom){$newspot=$#$chrom; }
								#print "Preround2: $newspot, ";
								$newspot = &round($newspot);
								#print "Postround2: $newspot\n";
								#print "$newspot upper array $head $tail vs. $start_temp $stop_temp\n"; sleep(1);
							}
							else{
								#print "Going up\n";
								$newspot++;
							}
						}
					}
				}until($found==1);
			}
	#		shift (@known);
			$marks=0;
			
			if ($inputfiles[$ct] =~ /variation.hg18/ && $inputfiles[$ct] !~ /McC/){
				$keeper=''; foreach $dude (@known){
					@paperlist = split (/, /, $dude);
					foreach $paperhere (@paperlist){
						$uniquepaper{$paperhere} = 0;
					}
				} 
				@known = keys %uniquepaper;
				%uniquepaper = ();
			}
			@hitpapers = sort @known; 
			$paperlist=''; $numpapers=0;
			#find out how many unique functions are involved
			$papercount=0; #shift(@hitpapers);
			foreach $gn (@hitpapers){
				if($papercount==0){$paperlist=$gn;}
				else{$paperlist= "$paperlist, " . $gn;}
				$papercount++;
			}
			if ($papercount ==0){$paperlist="NA"}
			
			$totaldups=0;	$totaldels=0; $totalcnvs=0;
			
			#This is where the output files are made. If it contains variation.hg18 then maximum gains, losses and totals are made
			if ($inputfiles[$ct] =~ /variation.hg18/){
				#foreach $dude (@dups){if ($dude > $totaldups){$totaldups = $dude;}}
				#foreach $dude (@dels){if ($dude > $totaldels){$totaldels = $dude;}}
				$dudecounter = 0;
				#print "New\n";
				foreach $dude (@tots){
					if ($tots[$dudecounter] > $totalcnvs){
						#print "Tots: $tots[$dudecounter]\n";
						$totalcnvs = $tots[$dudecounter];
						$totaldups = $dups[$dudecounter];
						$totaldels = $dels[$dudecounter];
					}
					$dudecounter++;
				}
				#Unless it is the DGV file just Gains, losses and totals are printed
				if ($inputfiles[$ct] ne $tdgvfile){
					print OUT "$totaldups\t$totaldels\t$totalcnvs\t";
				}
				#If it is the DGV then paper count and names of papers are also printed
				else{
					print OUT "$totaldups\t$totaldels\t$totalcnvs\t$papercount\t$paperlist\t";
				}
			}
			#Otherwise it returns the names of the 1st, 5th, 6th, 7th and 9th columns as determined by the custom file
			else{
				$filehere=$inputfiles[$ct];
				@extracolumns = split(",", $colhash{$filehere}); chomp(@extracolumns);
	#			print "we believe we see: \n@covers\n and @extracolumns and @known\n"; 
				shift(@names); shift(@dups); shift(@dels); shift(@tots); shift(@covers); shift (@known); shift(@nibs); shift(@dist5); shift(@dist3);
				$alldudes=''; $allcovers='';
				$marks=0; $covhits=0; @combos='';
				
				foreach $dude (@names){
					$alldudes .= "$dude, ";
					$marks++;
				}
				#Percentage coverage of the first column is assessed here
				foreach $dude (@covers){
					$allcovers += $dude;
					#If you want the percentage per region then use the one below instead of the one above
					#$dude = sprintf ("%.0f",($dude * 100))."%";
					#$allcovers .= "$dude, ";
					$covhits++;
				}
				$allcovers = sprintf ("%.2f",($allcovers * 100))."%";
				$alldist = "$dist5[0]\t$dist3[0]";
				chomp $alldudes;
				@alldudes = split (/, /,$alldudes);
				foreach (@alldudes){
					$_ =~ s/\"//g;
					$uniquedudes{$_} = 0;
				}
				@alldudes = keys %uniquedudes;
				@alldudes = sort {$a cmp $b} @alldudes;
				$numdudes = @alldudes;
				$alldudes = join (', ',@alldudes);
				%uniquedudes = ();
				
				if ($firstcolhash{$filehere} =~ /y/i){
					if ($ct == 0){
						if ($exonlistouttype eq "Intergenic" and $alldudes ne ""){$exonlistouttype = "Intron";}
						if ($alldudes ne ''){
							print OUT "$alldudes\t$exonlistouttype\t$numdudes\t$exonlistout\t";
						}
						else {
							print OUT "0\t$exonlistouttype\t$numdudes\t$exonlistout\t";
						}
					}
					elsif ($alldudes ne ''){print OUT $alldudes."\t";}
					else{print OUT "0\t";}
				}
				if ($perhash{$filehere} =~ /y/i){
					if ($alldudes ne ''){print OUT $allcovers."\t";}
					else{print OUT "0%\t";}
				}
				if ($disthash{$filehere} =~ /y/i){
					print OUT $alldist."\t";;
				}
				
				#We now deal with column5
				#print "5) $filehere, $$filehere{5}, Known: @known\n";
				$output5 = 0; %output5 =();
				if (uc $$filehere{5} eq "N" or $$filehere{5} eq ''){}
				elsif ($$filehere{5} =~ /Total/i){
					foreach $dude (@known){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output5 += $tab;}
					}
					if ($output5 ne ''){print OUT $output5."\t";}
					else{print OUT "0\t";}
				}
				else {
					foreach $dude (@known){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output5{$tab} = 0;}
					}
					@output5 = keys %output5;
					if ($$filehere{5} =~ /y/i or $$filehere{5} =~ /Name/i){@output5 = sort {$a cmp $b} @output5; $output5 = join (', ', @output5);}
					elsif ($$filehere{5} =~ /Num/i){$output5 = @output5;}
					elsif ($$filehere{5} =~ /Max/i){@output5 = sort {$a <=> $b} @output5; $output5 = $output5[-1];}
					elsif ($$filehere{5} =~ /Min/i){@output5 = sort {$a <=> $b} @output5; $output5 = $output5[0];}
					if ($output5 ne ''){print OUT $output5."\t";}
					else {print OUT "0\t";}
				}
				
				#We now deal with column6
				#print "6) $filehere, $$filehere{6}, Known: @dups\n";
				$output6 = 0; %output6 =();
				if (uc $$filehere{6} eq "N" or $$filehere{6} eq ''){}
				elsif ($$filehere{6} =~ /Total/i){
					foreach $dude (@dups){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output6 += $tab;}
					}
					if ($output6 ne ''){print OUT $output6."\t";}
					else{print OUT "0\t";}
				}
				else {
					foreach $dude (@dups){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output6{$tab} = 0;}
					}
					@output6 = keys %output6;
					if ($$filehere{6} =~ /y/i or $$filehere{6} =~ /Name/i){@output6 = sort {$a cmp $b} @output6; $output6 = join (', ', @output6);}
					elsif ($$filehere{6} =~ /Num/i){$output6 = @output6;}
					elsif ($$filehere{6} =~ /Max/i){@output6 = sort {$a <=> $b} @output6; $output6 = $output6[-1];}
					elsif ($$filehere{6} =~ /Min/i){@output6 = sort {$a <=> $b} @output6; $output6 = $output6[0];}
					if ($output6 ne ''){print OUT $output6."\t";}
					else {print OUT "0\t";}
				}
				
				#We now deal with column7
				#print "7) $filehere, $$filehere{7}, Known: @dels\n";
				$output7 = 0; %output7 =();
				if (uc $$filehere{7} eq "N" or $$filehere{7} eq ''){}
				elsif ($$filehere{7} =~ /Total/i){
					foreach $dude (@dels){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output7 += $tab;}
					}
					if ($output7 ne ''){print OUT $output7."\t";}
					else{print OUT "0\t";}
				}
				else {
					foreach $dude (@dels){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output7{$tab} = 0;}
					}
					@output7 = keys %output7;
					if ($$filehere{7} =~ /y/i or $$filehere{7} =~ /Name/i){@output7 = sort {$a cmp $b} @output7; $output7 = join (', ', @output7);}
					elsif ($$filehere{7} =~ /Num/i){$output7 = @output7;}
					elsif ($$filehere{7} =~ /Max/i){@output7 = sort {$a <=> $b} @output7; $output7 = $output7[-1];}
					elsif ($$filehere{7} =~ /Min/i){@output7 = sort {$a <=> $b} @output7; $output7 = $output7[0];}
					if ($output7 ne ''){print OUT $output7."\t";}
					else {print OUT "0\t";}
				}
				
				#We now deal with column8
				$output8 = 0; %output8 =();
				if (uc $$filehere{8} eq "N" or $$filehere{8} eq ''){}
				elsif ($$filehere{8} =~ /Total/i){
					foreach $dude (@tots){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output8 += $tab;}
					}
					if ($output8 ne ''){print OUT $output8."\t";}
					else{print OUT "0\t";}
				}
				else {
					foreach $dude (@tots){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output8{$tab} = 0;}
					}
					@output8 = keys %output8;
					if ($$filehere{8} =~ /y/i or $$filehere{8} =~ /Name/i){@output8 = sort {$a cmp $b} @output8; $output8 = join (', ', @output8);}
					elsif ($$filehere{8} =~ /Num/i){$output8 = @output8;}
					elsif ($$filehere{8} =~ /Max/i){@output8 = sort {$a <=> $b} @output8; $output8 = $output8[-1];}
					elsif ($$filehere{8} =~ /Min/i){@output8 = sort {$a <=> $b} @output8; $output8 = $output8[0];}
					if ($output8 ne ''){print OUT $output8."\t";}
					else {print OUT "0\t";}
				}
				
				#We now deal with column9
				$output9 = 0; %output9 =();
				if (uc $$filehere{9} eq "N" or $$filehere{9} eq ''){}
				elsif ($$filehere{9} =~ /Total/i){
					foreach $dude (@nibs){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output9 += $tab;}
					}
					if ($output9 ne ''){print OUT $output9."\t";}
					else{print OUT "0\t";}
				}
				else {
					foreach $dude (@nibs){
						$dude =~ s/\"//g;
						@tab = split (/, /,$dude);
						foreach $tab (@tab){$output9{$tab} = 0;}
					}
					@output9 = keys %output9;
					if ($$filehere{9} =~ /y/i or $$filehere{9} =~ /Name/i){@output9 = sort {$a cmp $b} @output9; $output9 = join (', ', @output9);}
					elsif ($$filehere{9} =~ /Num/i){$output9 = @output9;}
					elsif ($$filehere{9} =~ /Max/i){@output9 = sort {$a <=> $b} @output9; $output9 = $output9[-1];}
					elsif ($$filehere{9} =~ /Min/i){@output9 = sort {$a <=> $b} @output9; $output9 = $output9[0];}
					if ($output9 ne ''){print OUT $output9."\t";}
					else {print OUT "0\t";}
				}
			}
		}
		print OUT "\n";
	}
	print OUT "\n\n\n";
	print OUT "All Genes Hit and how many times:\n";
	while( ($k, $v) = each %allhits) {
		print OUT "$k\t$v\n";
	}
	print OUT "\nAll 5' Genes and how many times:\n";
	while( ($k, $v) = each %allfives) {
		print OUT "$k\t$v\n";
	}
	print OUT "\nAll 3' Genes and how many times:\n";
	while( ($k, $v) = each %allthrees) {
		print OUT "$k\t$v\n";
	}
	
	$currenttime = &showtime;
	print "Finished at $currenttime\nEnjoy your data.\n";
	print LOG1 "$currenttime\tAnnotation\tINFO\tAnnotation complete\n";
}

sub MergeAnnotation{
	#print "We have entered the sub routine\n";
	$sampleID = 0;
	$mchromo = 1;
	$mstart = 2;
	$mstop = 3;
	$mcnvtype=4;
	$mother1=5;
	$mother2=6;
	$mother3=7;
	$mother4=8;
	
	#open output file
	open (OUTPREMERGE, ">$inputhere2") or die "Unable to open $inputhere2; may already be open or lack write permissions\n";
	#Put the CNVs into an array but with each one with an ID made up of the algo and uniqueID counters	
	
	############################################
	## Start of Merge Function for Custom Files
	############################################
	@remade = (); @newloci=(); %allsamp=(); @samps='';
	@newloci = @input;
	@newloci = sort { (split '\t', $a, 4)[$mstart] <=> (split '\t', $b, 4)[$mstart] } @newloci;
	@newloci = sort { (split '\t', $a, 4)[$mchromo] cmp (split '\t', $b, 4)[$mchromo] } @newloci;

	print "Merging $#newloci regions... ";
	#open (MOUT, ">>testout.txt");
	for ($pz=0; $pz<=$#newloci; $pz++){
		@this = split ("\t", $newloci[$pz]);
		$this[$mchromo] = (uc $this[$mchromo]);
		$newloci[$pz] = join ("\t", @this);
		$allsamp{$this[$sampleID]} = 0;
	}
	@samps = keys %allsamp;
	$total = @samps;
	#print "$total uniqueIDs detected...";

	$error = 0; $track=0;
	
	#Does each loci line up with the one below?
	foreach $line (@newloci){
		#print "Merge: $line\n"; sleep(1);
		@before = split ("\t", $newloci[$track-1]);
		@now = split ("\t", $newloci[$track]);
		@after = split ("\t",$newloci[$track+1]);
		$track++;
			
		#If this loci is from a differet sample or chromosome or if it does not overlap with the last loci or maxbase then start a new set
		if ($now[$mchromo] ne $before[$mchromo]
		or ($now[$mstart] > $before[$mstop]
		and $now[$mstart] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this loci overlaps with the last or with maxbase
		elsif ($now[$mchromo] eq $before[$mchromo]
		and ($now[$mstart] <= $before[$mstop]
		or $now[$mstart] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
		}
		
		#If the loci does not fit into either of the above groups then there is an error
		else{
			print "Error with the following region, its status could not be determined:\n$line\n";
			$error++;
		}
		
		#If this loci does not overlap with the next CNV then @match is processed
		if ($now[$mchromo] ne $after[$mchromo]
		or ($now[$mstop] < $after[$mstart]
		and $maxbase < $after[$mstart])
		or $after[$mstart] == ""){
			#Process @match
			#print "Found a match\n";
			&setexamineannot(@match);
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$mstop] > $maxbase){
			$maxbase = $now[$mstop];
		}
		
	}
	#print "$error errors found...";
	$loci = @remade;
	print "become $loci segments... Merge complete";
	foreach $line (@remade){
		print OUTPREMERGE $line."\n";
	}
	close OUTPREMERGE;
	
	#print "found $#remade total\n";
	return (@remade);
}

sub setexamineannot{
	@match = @_;
	$matches = @match;
	%coord = ();
	%locitype = ();
	%otherdata1= ();
	%otherdata2= ();
	%otherdata3= ();
	%otherdata4= ();
	%sizes = ();
	%snps = ();
	$counter = 0;
	
	#Put the start and stop positions into a new array and determine the characteristics of this group of matches
	foreach (@match){
		#print "SetE: $_\n"; sleep(1);
		@temp = split ("\t", $_);
		$temp[$mstart] =~ s/^\s+//;
		$temp[$mstop] =~ s/^\s+//;
		chomp $temp[$mstart];
		chomp $temp[$mstop];
		$coord{$temp[$mstart]} = 0;
		$coord{$temp[$mstop]} = 0;
		$locitype{$temp[$mcnvtype]} = 0;
		$otherdata1{$temp[$mother1]} = 0;
		$otherdata2{$temp[$mother2]} = 0;
		$otherdata3{$temp[$mother3]} = 0;
		$otherdata4{$temp[$mother4]} = 0;
		$chrm = $temp[$mchromo];
	}
	
	$chrm =~ s/chr//i;
	@coord = keys %coord;
	@coord = sort {$a <=> $b} @coord;
	
	$namecount = 0;
	%algos = ();
	%uid = ();
	%type = ();
	%other1 = ();
	%other2 = ();
	%other3 = ();
	%other4 = ();
	
	############################################################################################################################		
	#Look for matches at each line of the co-ord
	foreach $bp (@coord){
		$lastone = $coord[$counter - 1];
		$thisone = $coord[$counter];
		$nextone = $coord[$counter + 1];
		$testone = $coord[$counter + 2];
		$counter++;
		##########################################################################################################################
		
		#For each coord look for matches in the start and stop coords
		foreach $line4 (@match){
			@locim = split ("\t", $line4);
			#print "BP: $bp, This: $thisone, Start: $locim[$mstart], Stop: $locim[$mstop]\n";
			#print "$counter Locim: $line4\n"; sleep(1);
			#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
			if($thisone == $locim[$mstart]){
				$namecount++;
				chomp $locim[$sampleID];
				$uid{$locim[$sampleID]} = 0;
				#print "TypeA: $locim[$mcnvtype]\n";
				$type{$locim[$sampleID]} = $locim[$mcnvtype];
				$other1{$locim[$sampleID]} = $locim[$mother1];
				$other2{$locim[$sampleID]} = $locim[$mother2];
				$other3{$locim[$sampleID]} = $locim[$mother3];
				$other4{$locim[$sampleID]} = $locim[$mother4];
			}
			
			#If the coord matches the end of a CNV the uid and algo are removed from the hashes
			if ($thisone == $locim[$mstop]){
				$namecount = $namecount - 1;
				chomp $locim[$sampleID];
				delete ($uid{$locim[$sampleID]});
				delete ($type{$locim[$sampleID]});
				#print "TypeD: $locim[$mcnvtype]\n";
				delete ($other1{$locim[$sampleID]});
				delete ($other2{$locim[$sampleID]});
				delete ($other3{$locim[$sampleID]});
				delete ($other4{$locim[$sampleID]});
			}
		}

		@uidkeys = keys %uid; @typekeys = values %type; @other1keys = values %other1;
		@other2keys = values %other2; @other3keys = values %other3; @other4keys = values %other4;
		@uidkeys = sort @uidkeys;
		$sampresent = join (", ", @uidkeys);
		$typespresent = join (", ", @typekeys);
		$other1present= join (", ", @other1keys);
		$other2present= join (", ", @other2keys);
		$other3present= join (", ", @other3keys);
		$other4present= join (", ", @other4keys);
		
		#Printout this section of the match
		unless ($nextone == ""){
		
			#This prevents the same number being present at the start and end
			#This will have problems if something is duplicated 3 times or commas
			if ($#coord > 2){
				if ($counter>2 && $#uidkeys == 0 && $counter != $#coord){
					$nextone--; $thisone++;
				}
				elsif ($counter>2 && $#uidkeys == 0 && $counter == $#coord){
					$nextone--; 
				}
				elsif ($counter>1 && $#uidkeys == 0){
					$thisone++;
				}
				#This might be an issue
				elsif ($counter>1 && $#uidkeys == 1){
				}
				else{
					$nextone--;
				}
			}
			if ($thisone==$oldstart){$thisone++;}
			$retro = "$sampresent\tchr$chrm\t$thisone\t$nextone\t$typespresent\t$other1present\t$other2present\t$other3present\t$other4present\t$matches";
			#print "Retro: $retro\n";
			
			push (@remade, $retro);
			#print OUT "$retro\n";
			#print "$retro, $lastcall";
			$lastcall=0; $oldstart=$nextone;
		}
		if ($namecount>1){$lastcall=1; $oldsam=$sampresent; };
	}
##########################################
## End of Merge Function for Custom Files
##########################################
}

#The sample may be either a child or a parent
sub sampledeal_parent{
	@sample = @_;
	$oink = @sample;
	@tab = split ("\t", $sample[0]);
	$sample = $tab[$sampleID];
	print "$counter of $regions) The sample is $sample and there are $oink regions\n";
	$phenotype = $pheno{$sample};
	$family = $family{$sample};
	@members = split ("\t", $members{$family});
	@parents = split ("\t", $parents{$sample});
	#The parents are identified
	$father = $parents[0];
	$mother = $parents[1];
	if ($family eq "" or $father eq "" or $mother eq ""){
		print "Error: Missing family members\n";
	}
	
	@siblings = ();
	#The siblings are put into an array (not the sample)
	
	foreach $member (@members){
		#print "Sample parents: $parents{$sample} This sample parents: $parents{$member}\n";
		if ($parents{$member} eq $parents{$sample}
		and $sample ne $member){
			#print "Sample parents: $parents{$sample} This sample parents: $parents{$member}\n";
			push (@siblings, $member);
		}
	}
	
	@family = ();
	push (@family, $sample, $father, $mother);
	#print "@family\n";
	
	###################################Collect the data from the Final Report
	$membercount = 0;
	foreach $member (@family){
		#print "Looking for $member from family $family\n";
		if ($member eq 0){}
		elsif ($member eq ""){}
		else {
			$firep = $finalrep{$member};
			open (FR, "<$firep")|| print "FinalReport for sample $member is missing from this directory\n";
			@fr = <FR>; close FR;
			$regcount = 0;
			foreach $region (@sample){
				#If the plot setting is enabled then the wider regions are put into the corresponding logrw and bafw arrays
				$wide = 0;
				@region = split ("\t", $region);
				@lines = ();
				@lines = &linefind(@region);
				$bafw = "bafw\t$regcount\t$membercount";
				$position = "pos\t$regcount\t$membercount";
				push (@clearme, $bafw, $position);
				@$bafw = (); @$position = ();
				foreach $line (@lines){
					#print "$line\n"; sleep (1);
					@tab = split ("\t", $fr[$line]); chomp @tab;
					#print "$tab[0], $tab[1], $tab[2], $tab[3], $tab[$log2col], $tab[$ballele]\n"; sleep (1);
					if($membercount == 0){
						push (@$position, $tab[$snpcol]);
					}
					push (@$bafw, $tab[$ballele]);
				}
				#print "Input: pos\t$regcount\t$membercount: $$position[0]\n";
				@lines = ();
				$regcount++;
			}
		}
		$membercount++;
	}
	
	$regcount = 0;
	
	###################################Output the BAF for each family and region
	#%%This is where you need to make the link
	
	foreach $region (@sample){
		#print "Plot route\n";
		@tab = split ("\t", $region);
		$thefamily = $family{$tab[$sampleID]};
		$minifilename = 'sample.'.$thefamily.'_'.$regcount;
		$filename = $curr_dir.'05_Data/'.$minifilename.'.txt';
		open (OUT, ">$filename")||die "Could not open file $filename, may already be open";
		
		$position = "pos\t$regcount\t0";
		@data = (); @logr = (); @baf = ();
		foreach $line (@$position){
			#print "Position: $line\n";
			push (@data, $line);
		}
		print OUTPUT "$tab[$chromo]:$tab[$start]-$tab[$stop] $minifilename $tab[$typecol]\n";
		#print "Plot: pos\t$regcount\t0: $$position[0]\n";
		$membercount = 0;
		foreach $member (@family){
			$array = "bafw\t$regcount\t0";
			$bafw = "bafw\t$regcount\t$membercount";
			#print "Name:Reg:Mem $bafw\n";
			$counthere = 0;
			foreach (@$array){
				if ($baf[$counthere] eq ""){
					#print "I'm here\n";
					$baf[$counthere] = $$bafw[$counthere];
				}
				elsif ($member == 0){
					$baf[$counthere] = "$baf[$counthere]";
				}
				else{
					#print "I'm here3\n";
					$baf[$counthere] .= " $$bafw[$counthere]";
				}
				$counthere++;
			}
			$membercount++;
		}
		$counthere = 0;
		@sendme = ();
		foreach (@baf){
			$data[$counthere] = "$data[$counthere] $baf[$counthere]";
			print OUT "$data[$counthere]\n";
			push (@sendme, $data[$counthere]);
			$counthere++;
		}
		$answer = &parent_origin_sub(@sendme);
		print OUTHERE $region."\t".$answer."\n";
		$regcount++;
	}
	#Remove data from memory
	foreach $array (@clearme){
		@$array = ();
	}
	@clearme =();
}

#The sample may be either a child or a parent
sub sampledeal{
	@sample = @_;
	$oink = @sample;
	@tab = split ("\t", $sample[0]);
	$sample = $tab[$sampleID];
	print "$counter of $regions) The sample is $sample and there are $oink regions\n";
	$phenotype = $pheno{$sample};
	$family = $family{$sample};
	@members = split ("\t", $members{$family});
	@parents = split ("\t", $parents{$sample});
	#The parents are identified
	$father = $parents[0];
	$mother = $parents[1];
	
	@siblings = ();
	#The siblings are put into an array (not the sample)
	
	foreach $member (@members){
		#print "Sample parents: $parents{$sample} This sample parents: $parents{$member}\n";
		if ($parents{$member} eq $parents{$sample}
		and $sample ne $member){
			#print "Sample parents: $parents{$sample} This sample parents: $parents{$member}\n";
			push (@siblings, $member);
		}
	}
	
	@family = ();
	push (@family, $sample, $father, $mother, @siblings);
	#print "@family\n";
	$thegenderofsample = $gender{$sample};
	#print "Sample: $sample, Gender: $thegenderofsample\n";
	
	$membercount = 0;
	foreach $member (@family){
		#print "Looking for $member from family $family\n";
		if ($member eq 0){}
		elsif ($member eq ""){}
		else {
			$firep = $finalrep{$member};
			open (FR, "<$firep")|| print "FinalReport for sample $member is missing from this directory\n";
			@fr = <FR>; close FR;
			$regcount = 0;
			foreach $region (@sample){
				$wide = 0;
				@region = split ("\t", $region);
				@lines = ();
				@lines = &linefind(@region);
				$meanlr = "mean\t$regcount\t$membercount";
				$logr = "logr\t$regcount\t$membercount";
				$baf = "baf\t$regcount\t$membercount";
				$dnposition = "dnpos\t$regcount\t$membercount";
				@$logr = (); @$baf = (); @$meanlr = (); @$dnposition = ();
				#The data from the Final Report is put into the arrays logr and baf with numbering according to region and family member
				$total = 0;
				$totallines = @lines;
				foreach $line (@lines){
					@tab = split ("\t", $fr[$line]); chomp @tab;
					$total += $tab[$log2col];
					#print "LogR total is now $total\n";
					if($membercount == 0){
						push (@$dnposition, $tab[$chromodist]);
						#print "Position: $tab[$chromodist]\n";
					}
					push (@$logr, $tab[$log2col]);
					push (@$baf, $tab[$ballele]);
				}
				if ($totallines != 0){
					$mean = $total/$totallines;
				}
				else {
					$mean = 0;
				}
				#print "This gives a mean of $mean\n";
				#print "Mem: $membercount, Reg: $regcount, Lines: $totallines\n";
				push (@$meanlr, $mean);
				#If the plot setting is enabled then the wider regions are put into the corresponding logrw and bafw arrays
				if ($plot == 1){
					$wide = 1;
					@lines = ();
					@lines = &linefind(@region);
					$logrw = "logrw\t$regcount\t$membercount";
					$bafw = "bafw\t$regcount\t$membercount";
					$position = "pos\t$regcount\t$membercount";
					#print "$logrw\n";
					@$logrw = (); @$bafw = (); @$position = ();
					foreach $line (@lines){
						#print "$line\n"; sleep (1);
						@tab = split ("\t", $fr[$line]); chomp @tab;
						#print "$tab[0], $tab[1], $tab[2], $tab[3], $tab[$log2col], $tab[$ballele]\n"; sleep (1);
						if($membercount == 0){
							push (@$position, $tab[$chromodist]);
						}
						push (@$logrw, $tab[$log2col]);
						push (@$bafw, $tab[$ballele]);
					}
					#print "Input: pos\t$regcount\t$membercount: $$position[0]\n";
					@lines = ();
				}
				$regcount++;
			}
		}
		$membercount++;
	}
	
	$regcount = 0;
	foreach $region (@sample){
		#print "Non-plot route\n";
		@tab = split ("\t", $region);
		$chromohere = $tab[$chromo];
		$membercount = 0;
		$othercount = 0;
		$counthere = 0;
		$dnposition = "dnpos\t$regcount\t0";
		@dndata = (); @logr = (); @baf = ();
		foreach $line (@$dnposition){
			#print "Position: $line\n";
			push (@dndata, $line);
		}
		foreach $member (@family){
			$logr = "logr\t$regcount\t$membercount";
			$array = "logr\t$regcount\t0";
			#print "Name:Reg:Mem $logr\n";
			$baf = "baf\t$regcount\t$membercount";
			$counthere2++;
			$counthere = 0;
			foreach (@dndata){
				if ($member == 0 or $member == "" or $$logr[$counthere] eq ""){
					$logr[$counthere] = "$logr[$counthere]\tNo_data";
					$baf[$counthere] = "$baf[$counthere]\tNo_data";
				}
				else{
					$logr[$counthere] = "$logr[$counthere]\t$$logr[$counthere]";
					#print "LogR: $_\tCombo: $logr[$counthere]\n";
					$baf[$counthere] = "$baf[$counthere]\t$$baf[$counthere]";
				}
				$counthere++;
			}
			$membercount++;
		}
		
		$counthere = 0;
		foreach (@dndata){
			$input = "$dndata[$counthere]\t$logr[$counthere]\t$baf[$counthere]\n";
			$dndata[$counthere] = $input;
			$counthere++;
		}
		
		$output_for_dn = &process(@dndata);
		$regcount++;
		print OUTPUT "$region\t$output_for_dn\n";
		$output2 = "";
	}
	
	#This is where the data files are made for visualization
	if ($plot == 1){
		$regcount = 0;
		foreach $region (@sample){
			#print "Plot route\n";
			@tab = split ("\t", $region);
			$filename = $curr_dir."05_Data/$tab[$sampleID]_$tab[$chromo]_$tab[$start]_$tab[$stop].txt";
			open (OUT, ">$filename")||die "Could not open file $filename, may already be open";
			print OUT "Position\tCNV";
			$sibcounthere = 0;
			foreach (@family){
				print OUT "\tR_$pheno{$_}";
				if ($pheno{$_} eq Sibling){$sibcounthere++; print OUT $sibcounthere;}
			}
			$sibcounthere = 0;
			foreach (@family){
				print OUT "\tB_$pheno{$_}";
				if ($pheno{$_} eq Sibling){$sibcounthere++; print OUT $sibcounthere;}
			}
			$position = "pos\t$regcount\t0";
			@data = (); @logr = (); @baf = ();
			foreach $line (@$position){
				#print "Position: $line\n";
				push (@data, $line);
			}
			print OUT "\n$tab[$start]\t$family{$tab[$sampleID]}\n$tab[$stop]\n$$position[0]\t$tab[$chromo]:$tab[$start]-$tab[$stop]\n$$position[-1]\t$typenum\n";
			#print "Plot: pos\t$regcount\t0: $$position[0]\n";
			$membercount = 0;
			$logmin = 0;
			foreach $member (@family){
				$logrw = "logrw\t$regcount\t$membercount";
				$array = "logrw\t$regcount\t0";
				#print "Name:Reg:Mem $logrw\n";
				$bafw = "bafw\t$regcount\t$membercount";
				$counthere = 0;
				foreach (@$array){
					if ($logr[$counthere] eq ""){
						#print "I'm here\n";
						if ($$logrw[$counthere] < $logmin){$logmin = $$logrw[$counthere];}
						$logr[$counthere] = $$logrw[$counthere];
						$baf[$counthere] = $$bafw[$counthere];
					}
					elsif ($member == 0){
						$logr[$counthere] = "$logr[$counthere]\t ";
						$baf[$counthere] = "$baf[$counthere]\t ";
					}
					else{
						#print "I'm here3\n";
						$thislogr = $$logrw[$counthere] - $membercount;
						#print "This logR: $thislogr\n";
						$logr[$counthere] = "$logr[$counthere]\t$thislogr";
						#print "LogR: $_\tCombo: $logr[$counthere]\n";
						$baf[$counthere] = "$baf[$counthere]\t$$bafw[$counthere]";
						if ($thislogr < $logmin){$logmin = $thislogr;}
					}
					$counthere++;
				}
				$membercount++;
			}
			$counthere = 0;
			foreach (@logr){
				$data[$counthere] = "$data[$counthere]\t\t$logr[$counthere]\t$baf[$counthere]";
				print OUT "$data[$counthere]\n";
				$counthere++;
			}
			&rout(@family, $region);
			$regcount++;
		}
	}
}

#For an array of the terms of a single region it returns the narrow or wide logR positions
sub linefind{
	#print "Linefind! This is one of my favourite bits!\n";
	#Determines the chromosome, start and stop positions with margins (if needed) and CNV type
	@tab = @_;
	#print "$tab[$chromo]:$tab[$start]-$tab[$stop]\n";
	$chromoreg = $tab[$chromo];
	$chromoreg =~ s/chr//gi;
	$startreal = $tab[$start];
	$stopreal = $tab[$stop];
	$typenum = $tab[$typecol];
	if ($wide == 1){
		$size = $stopreal - $startreal;
		if($zoomout == 1){$margins = 25000 + $size;}
		elsif($zoomin == 1){$margins = $size/3;}
		else{$margins = 25000 + $size/3;}
		$startreg = $tab[$start] - $margins;
		$stopreg = $tab[$stop] + $margins;
		@widelines = ();
		$arraylinefind = "$chromoreg\t$startreg\t$stopreg\n";
		#print "Wide: $arraylinefind";
	}
	else {
		$arraylinefind = "$chromoreg\t$startreal\t$stopreal\n";
		#print "Narrow: $arraylinefind";
	}
	
	#The dandcdn subroutine is used to return the FinalReport array lines for this region
	@arraylines = ();
	@arraylines = &dandcdn($arraylinefind);
	@arraylines = sort { (split '\t', $a)[0] <=> (split '\t', $b)[0] } @arraylines;
	@lines = ();
	foreach (@arraylines){
		#print "$_\n";
		@here = split ("\t", $_);
		push (@lines, $here[1])
	}
	return (@lines)
}

#Subroutine used for divide and conquer in SNP counter
sub divide{
	$round++;
	$divison = 2**$round;
	#print "$linecount2) Chr: $chrm, Array location: $newspot, Round: $round, Divison: $divison\n";
	if ($round == 30) {sleep(5)};
	$herespot = $$chrm[$newspot];
	#If the chromosome does not have any SNPs it is picked up here
	if ($arraysize == 0){$zero = 1;};
	#If the region is has no SNPs within it then it is picked up here
	if ($herespot < $starth and $$chrm[($newspot + 1)] > $stoph){$zero = 1};
	if ($herespot > $stoph and $$chrm[($newspot - 1)] < $starth){$zero = 1};
	#If the region starts after the last SNP on the chromosome or before the first one it is picked up here
	if ($$chrm[-1] < $starth){$zero = 1};
	if ($$chrm[0] > $stoph){$zero = 1};
	#Otherwise newspot is adjusted depending on whether it is above or below the region of interest
	if ($herespot < $starth){$newspot = sprintf ("%.f",($newspot + ($halfspot/$divison)));};
	if ($herespot >= $starth and $herespot <= $stoph){$conquer = 1};
	if ($herespot > $stoph){$newspot = sprintf ("%.f",($newspot - ($halfspot/$divison)));};

	if ($zero == 1){
		&zero;
	}
	elsif ($conquer == 0){
		&divide;
	}
	elsif ($conquer == 1) {
		&conquer;
	}

}

#Subroutine used for divide and conquer in SNP counter
sub conquer{
	$snpcount = 0;
	$upspot = $newspot;
	$downspot = $newspot + 1;
	$marker1 = 0;
	$marker2 = 0;
	
	do{
		if ($upspot < 0){$marker1 = 1;}
		if ($$chrm[$upspot] >= $starth){
			$snpcount++;
			#print "$linecount2) Up:   SNP position: $$chrm[$upspot], SNP count: $snpcount\n";
			$upspot--;
		}
		if ($$chrm[$upspot] < $starth){
			$marker1 = 1;
		}

	}until ($marker1 == 1);
	
	do{
		if ($downspot > $#$chrm){$marker2 = 1;}
		if ($$chrm[$downspot] <= $stoph){
			$snpcount++;
			#print "$linecount2) Down: SNP position: $$chrm[$downspot], SNP count: $snpcount\n";
			$downspot++;
		}
		if ($$chrm[$downspot] > $stoph){
			$marker2 = 1;
		}

	}until ($marker2 == 1);
	#print "$chrm:$starth-$stoph SNP count is $snpcount\n";
}

#Subroutine used for divide and conquer in SNP counter
sub zero{
	$snpcount = 0;
	#print "Zero: SNP count is $snpcount\n";
}

#This subroutine returns the position and arrayline of every probe within a chr/start/stop term; it needs the dividedn and conquerdn parts to work
sub dandcdn{
	my ($term) = @_;
	#print "Term: $term\n";
	$linecount2++;
	@tabs = split ("\t", $term);
	$chrm = $tabs[0];
	$chrm =~ s/chr//gi;
	$chrm =~ tr/a-z/A-Z/;
	$chrm = "chr" . $chrm;
	$starth = $tabs[1];
	$stoph = $tabs[2];
	$count = 0;
	$mark = 0;
	
	
	$arraysize = @$chrm;
	$newspot = sprintf ("%.f",($arraysize / 2));
	$halfspot = $newspot;
	$round = 0;
	$conquerdn = 0;
	$zero = 0;
	#print "Array size: $arraysize, Round: $round\n";
	&dividedn;
	#print "Hey I'm back here\n";
	
}

#Required for the dandcdn subroutine (binary search algorithm to locate where the position is within a chromosome specific array)
sub dividedn{
	$round++;
	$divison = 2**$round;
	#print "$linecount2) Chr: $chrm, Array location: $newspot, Round: $round, Divison: $divison\n";
	if ($round == 30) {sleep(5)};
	#The numbers below are minus one to allow the first position in the array to be reached
	@here = split ("\t", $$chrm[$newspot - 1]);
	@above = split ("\t", $$chrm[$newspot]);
	@below = split ("\t", $$chrm[$newspot - 2]);
	@first = split ("\t", $$chrm[0]);
	@laster = split ("\t", $$chrm[-1]);
	$herespot = $here[0];
	#If the chromosome does not have any SNPs it is picked up here
	if ($arraysize == 0){$zero = 1;};
	#If the region is has no SNPs within it then it is picked up here
	if ($herespot < $starth and $above[0] > $stoph){$zero = 1};
	if ($herespot > $stoph and $below[0] < $starth){$zero = 1};
	#If the region starts after the last SNP on the chromosome or before the first one it is picked up here
	if ($laster[0] < $starth){$zero = 1};
	if ($first[0] > $stoph){$zero = 1};
	#Otherwise newspot is adjusted depending on whether it is above or below the region of interest
	if ($herespot < $starth){$newspot = sprintf ("%.f",($newspot + ($halfspot/$divison)));};
	if ($herespot >= $starth and $herespot <= $stoph){$conquerdn = 1};
	if ($herespot > $stoph){$newspot = sprintf ("%.f",($newspot - ($halfspot/$divison)));};

	if ($zero == 1){
		&zerodn;
	}
	elsif ($conquerdn == 0){
		&dividedn;
	}
	elsif ($conquerdn == 1) {
		&conquerdn;
	}
}

#Required for the dandcdn subroutine (extends from the 'hit' in the dividedn subroutine to find the SNPs within the specified area)
sub conquerdn{
	$snpcount = 0;
	$upspot = $newspot - 1;
	$downspot = $newspot;
	$marker1 = 0;
	$marker2 = 0;
	@returner = ();
	
	#Here the algorithm moves towards the start of the chromosome counting SNPs as it goes
	do{
		@upspot = split ("\t", $$chrm[$upspot]);
		if ($upspot < 0){$marker1 = 1;}
		elsif ($upspot[0] >= $starth){
			push (@returner, $$chrm[$upspot]);
			$snpcount++;
			#print "$linecount2) Spot: Up: $upspot, SNP position: $$chrm[$upspot], SNP count: $snpcount\n";
			$upspot--;
		}
		elsif ($upspot[0] < $starth){
			$marker1 = 1;
		}
	}until ($marker1 == 1);
	
	#Here the algorithm moves towards the end of the chromosome counting SNPs as it goes
	do{
		@downspot = split ("\t", $$chrm[$downspot]);
		if ($downspot > $#$chrm){$marker2 = 1;}
		elsif ($downspot[0] <= $stoph){
			$input = 
			push (@returner, $$chrm[$downspot]);
			$snpcount++;
			#print "$linecount2) Down: SNP position: $$chrm[$downspot], SNP count: $snpcount\n";
			$downspot++;
		}
		elsif ($downspot[0] > $stoph){
			$marker2 = 1;
		}
	}until ($marker2 == 1);
	#print "Return: @returner\n";
	return (@returner);
	#print "$chrm:$starth-$stoph SNP count is $snpcount\n";
}

#Required for the dandcdn subroutine (deals with the situation when there are no SNPs in the region)
sub zerodn{
	@retuner = ();
	$snpcount = 0;
	return (@returner);
	#print "Zero: SNP count is $snpcount\n";
}

#R output is written here for each sample
sub rout{
	$linesinfile = $#logr;
	if ($linesinfile <= 30){$cex = 1;}
	elsif ($linesinfile <= 200){$cex = 0.6;}
	else {$cex = 0.3;}
	$datafile = $filename;
	$filename =~ s/05_Data/06_Plots/gi;
	$filename =~ s/.txt/.pdf/gi;
	@abline = (0..$#family);
	$abline = 0;
	foreach (@abline){
		$abline = "$abline,-$_";
	}
	if ($logmin < -8){$ylimit = $logmin - 0.25;}
	elsif ($logmin < -4.25) {$ylimit = $logmin - 0.25;}
	else {$ylimit = -4.5;}
	#print "Logmin = $logmin, ylimit = $ylimit\n";
	print ROUT 'CNV<-read.delim("'.$datafile.'") #File with CNV logR and Ballele'."\n";
	print ROUT 'pdf("'.$filename.'") #output name of pdf file'."\n";
	print ROUT 'start<-CNV[1,1] #Start of CNV row 1, column 1'."\n";
	print ROUT 'stop<-CNV[2,1] #Stop of CNV row 2, column 1'."\n";
	print ROUT 'start2<-CNV[3,1] #Start of Region row 1, column 3'."\n";
	print ROUT 'stop2<-CNV[4,1] #Start of Region row 2, column 3'."\n";
	print ROUT 'fam<-CNV[1,2] #...family id'."\n";
	print ROUT 'chr<-CNV[3,2] #... loci coordinates'."\n";
	print ROUT 'type<-CNV[4,2] #...type'."\n";
	print ROUT 'header<-paste(fam,chr,type,sep="    ",collapse=NULL)'."\n";
	$counthere = 0; $sibcounthere=0;
	@printrout = ();
	foreach $member (@family){
		$col = $colours[$counthere];
		$pheno = $pheno{$member};
		#print "The Sample is $member and the phenotype is $pheno\n";
		if ($pheno =~ /Proband/){$col = "red";}
		elsif ($pheno =~ /Sibling/){$sibcounthere++; $pheno = $pheno.$sibcounthere;}
		if ($member eq $family[0]){
			print ROUT 'plot(CNV$R_'.$pheno.'~CNV$Position, ylim=range('.$ylimit.',1.5), xlim=c(start2,stop2), col="'.$col.'", pch=20, cex='.$cex.', yaxt="n", xlab="Position", ylab="Relative logR");abline(h=c('.$abline.'),v=c(start,stop))'."\n";
		}
		else{
			print ROUT 'par(new=T)'."\n";
			print ROUT 'plot(CNV$R_'.$pheno.'~CNV$Position, ylim=range('.$ylimit.',1.5), xlim=c(start2,stop2), col="'.$col.'", pch=20, cex='.$cex.', yaxt="n", xaxt="n", xlab="", ylab="")'."\n";
		}
		$input = 'plot(CNV$B_'.$pheno.'~CNV$Position,col="'.$col.'",pch=20, cex='.$cex.',xlim=c(start2,stop2), ylim=range(0,1), main="'.$member.'", xlab="Position", ylab="'.$pheno.' B-Allele", yaxp=c(0,1,2));abline(v=c(start,stop))'."\n";
		push (@printrout, $input);
		$counthere++;
	}
	print ROUT 'mtext(header,side=3)'."\n";
	print ROUT 'par(mfrow=c(3,2))'."\n";
	print ROUT @printrout;
	print ROUT 'dev.off()'."\n\n";
}

#This is part of the plot and denovo2 options, it determines the relatedness of the sample data
sub process{
	$specific_count++;
	@results = @_;
	$linespresent = @results;
	if ($giverealdata == 1){
		open (OUTER, ">$specific_count.txt")||die "Could not open $specific_count may already be open";
		print OUTER @results;
	}
	%logr=(); $dadlog=0; $mumlog=0; %homo=(); %het=(); %dup=(); %other=(); $inform=0; $missinherit=0;
	foreach (@results){
		@tab = split("\t\t", $_);
		@loger = split ("\t", $tab[1]);
		@bafer = split ("\t", $tab[2]);
		$membercounthere = 0;
		#This will be used to work out the mean logr and median logr
		foreach (@loger){
			if($_ eq "No_data"){$logr{$membercounthere} = "No_data";}
			else {
				$logr{$membercounthere} = $logr{$membercounthere} + $_;
			}
			push (@$membercounthere, $_);
			$membercounthere++;
		}
		
		#Looks if the logR for each SNP is the same as the proband
		if ($loger[1] eq "No_data"){$dadlog = "No_data";}
		elsif ($loger[0] <= 0
		and $loger[1] < ($loger[0]/2)){$dadlog++;}
		elsif ($loger[0] > 0
		and $loger[1] > ($loger[0]/2)){$dadlog++;}
		
		if ($loger[2] eq "No_data"){$mumlog = "No_data";}
		elsif ($loger[0] <= 0
		and $loger[2] < ($loger[0]/2)){$mumlog++;}
		elsif ($loger[0] > 0
		and $loger[2] > ($loger[0]/2)){$mumlog++;}
		
		#Looks for misinheritence
		if ($bafer[1] >= 0.9 or $bafer[2] >= 0.9 or $bafer[1] <= 0.1 or $bafer[2] <= 0.1 and $bafer[1] ne "No_data" and $bafer[2] ne "No_data"){
			$inform++;
			if ($bafer[0] <= 0.1 and ($bafer[1] >= 0.9 or $bafer[2] >= 0.9)){$missinherit++;} #If child is AA neither parent can be BB
			elsif ($bafer[0] >= 0.9 and ($bafer[1] <= 0.1 or $bafer[2] <= 0.1)){$missinherit++;} #If the child is BB neither parent can be AA
			elsif ($bafer[0] < 0.9 and $bafer[1] >= 0.9 and $bafer[2] >= 0.9){$missinherit++;} #If the child is not BB both parents cannot be BB
			elsif ($bafer[0] > 0.1 and $bafer[1] <= 0.1 and $bafer[2] <= 0.1){$missinherit++;} #If the child is not AA both parents cannot be AA
		}
		
		#Works out where each samples' BAF falls
		$membercounthere = 0;
		foreach (@bafer){
			if ($_ eq "No_data"){$homo{$membercounthere} = "No_data"; $het{$membercounthere} = "No_data"; $dup{$membercounthere} = "No_data"; $other{$membercounthere} = "No_data";}
			elsif ($_ <= 0.1 or $_ >= 0.9){$homo{$membercounthere} = $homo{$membercounthere} + 1;}
			elsif ($_ <= 0.58 and $_ >= 0.42){$het{$membercounthere} = $het{$membercounthere} + 1;}
			elsif ($_ >= 0.6 and $_ <= 0.75){$dup{$membercounthere} = $dup{$membercounthere} + 1;}
			elsif ($_ <= 0.4 and $_ >= 0.25){$dup{$membercounthere} = $dup{$membercounthere} + 1;}
			else {$other{$membercounthere} = $other{$membercounthere} + 1;}
			$membercounthere++;
		}
		$famsize = $#bafer;
	}
	
	$outhere = "";
	$typenames = "";
	foreach (0..$famsize){
		if ($logr{$_} eq "No_data"){$meanlogr = "No_data";}
		elsif ($linespresent != 0) {$meanlogr = $logr{$_}/$linespresent;}
		@median = sort {$a <=> $b} @$_;
		$stdevsum = 0;
		foreach $point (@median){
			$stdevsum += ($point - $meanlogr)*($point - $meanlogr);
		}
		if ($linespresent != 1){
			$stdev{$_} = sqrt ($stdevsum/($linespresent-1));
		}
		$min{$_} = $meanlogr - (1 * $stdev{$_});
		$max{$_} = $meanlogr + (1 * $stdev{$_});
		@$_ = ();
		#print @median;
		$half = $linespresent/2;
		if ($median[0] eq "No_data"){$medianlogr = "No_data"; $stdev{$_} = "No_data"; $min{$_} = "No_data"; $max{$_} = "No_data";}
		elsif ($half == int($half)){$medianlogr = $median[$half];}
		else {$medianlogr = ($median[$half - 0.5] + $median[$half + 0.5])/2;}
		#print "So the mean is $meanlogr and the median is $medianlogr, 0 = $median[0], -1 = $median[-1]\n";
		if ($homo{$_} eq "No_data"){$homo = "No_data"; $het = "No_data"; $dup = "No_data"; $other = "No_data";}
		elsif ($linespresent != 0) {
			$homo = $homo{$_}/$linespresent;
			$het = $het{$_}/$linespresent;
			$dup = $dup{$_}/$linespresent;
			$other = $other{$_}/$linespresent;
		}
		
		$uncertain = "";
		#Works out the type of CNV present
		if($medianlogr eq "No_data"){$thetype = "No_data";}
		#Homozygous deletion
		elsif($medianlogr <= -1.5){
			if ($homo < 0.5){$thetype = 0;}
			else {$thetype = 0; $uncertain = "?";}
		}
		#Heterozygous deletion
		elsif($medianlogr > -1.5 and $medianlogr <= -0.25){
			if ($homo >= 0.8){$thetype = 1;}
			else {$thetype = 1; $uncertain = "?";}
		}
		#Normal
		elsif($medianlogr > -0.25 and $medianlogr < 0.15){
			if (($het + $homo) >= 0.8){$thetype = 2;}
			else {$thetype = 2; $uncertain = "?";}
		}
		#Duplication
		elsif($medianlogr >= 0.15 and $medianlogr < 0.4){
			if ($dup >= $het){$thetype = 3;}
			elsif ($het > 0.2 and $dup < 0.1){$thetype = 2;}
			else {$thetype = 3; $uncertain = "?";}
		}
		elsif($medianlogr >= 0.4){
			if ($het < 0.15){$thetype = 3;}
			elsif ($het >= 0.15 and $other >= 0.1){$thetype = 4;}
			else {$thetype = 4; $uncertain = "?";}
		}
		
		
		#The output is prepared for each family member...
		$ta[$_] = $thetype;
		$ta2[$_] = $uncertain.$thetype;
		if ($thetype eq "No_data"){$ot[$_] = "No_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\t"; $outhere = "No_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\t";}
		else {
			$ot[$_] = sprintf("%.2f",$meanlogr)."\t".sprintf("%.2f",$stdev{$_})."\t".sprintf("%.2f",$min{$_})."\t".sprintf("%.2f",$max{$_})."\t".sprintf("%.2f",$medianlogr)."\t".sprintf("%02d",$homo*100)."%\t".sprintf("%02d",$het*100)."%\t".sprintf("%02d",$dup*100)."%\t".sprintf("%02d",$other*100)."%\t";
			$outhere = $outhere.sprintf("%.2f",$meanlogr)."\t".sprintf("%.2f",$medianlogr)."\t".sprintf("%02d",$homo*100)."%\t".sprintf("%02d",$het*100)."%\t".sprintf("%02d",$dup*100)."%\t".sprintf("%02d",$other*100)."%\t";
		}
		$typenames = "$typenames$uncertain$thetype\t";
	}
	
	if($dadlog eq "No_data"){$dadmatch = "No_data";}
	elsif ($linespresent != 0) {$dadmatch = $dadlog/$linespresent;}
	if($mumlog eq "No_data"){$mummatch = "No_data";}
	elsif ($linespresent != 0) {$mummatch = $mumlog/$linespresent;}
	if ($inform > 0){$misinherited = $missinherit/$inform;}
	else {$misinherited = "No_informative_SNPs";}
	
	#This section determines whether a CNV is de novo or inherited
	#print "Gender: $thegenderofsample\n";
	if ($dadmatch eq "No_data" and $mummatch eq "No_data"){$inherit = "No_data";}
	elsif ($ta[0] =~ /2/ and $chromohere =~ /x/gi and $thegenderofsample eq "Male") {
		if ($ta[2] =~ /3/ or $ta[2] =~ /4/){$inherit = "Maternal";}
		elsif ($ta[0] eq /2/ and $ta[2] eq /2/){$inherit = "De Novo";}
		else {$inherit = "?De Novo";}
		#print "I'm here: $inherit\n";
	}
	elsif ($ta[0] =~ /2/) {$inherit = "No_CNV";}
	elsif ($ta[0] =~ /0/) {
		if ($chromohere =~ /x/gi and ($ta[2] =~ /1/ or $ta[2] =~ /0/)){$inherit = "Homo_Mat";}
		elsif ($ta[1] =~ /1/ and $ta[2] =~ /1/){$inherit = "Homo_Both";}
		elsif ($ta[1] =~ /0/ and $ta[2] =~ /1/){$inherit = "Homo_Both";}
		elsif ($ta[1] =~ /1/ and $ta[2] =~ /0/){$inherit = "Homo_Both";}
		elsif ($ta[1] =~ /0/ and $ta[2] =~ /0/){$inherit = "Homo_Both";}
		elsif ($dadmatch ne "No_data" and $mummatch ne "No_data"){$inherit = "De Novo";}
		else {$inherit = "Missing parent";}
	}
	elsif ($dadmatch >= 0.75 or $mummatch >= 0.75){
		if ($ta[1] =~ /$ta[0]/ and $ta[2] =~ /$ta[0]/){$inherit = "Both";}
		elsif ($ta[1] =~ /$ta[0]/ and $ta[2] !~ /$ta[0]/){$inherit = "Paternal";}
		elsif ($ta[1] !~ /$ta[0]/ and $ta[2] =~ /$ta[0]/){$inherit = "Maternal";}
		elsif ($dadmatch ne "No_data" and $mummatch ne "No_data"){$inherit = "?Both";}
		else {$inherit = "Missing parent";}
	}
	elsif ($dadmatch >= 0.75 and $mummatch < 0.75){
		if ($ta[1] =~ /$ta[0]/){$inherit = "Paternal";}
		else {$inherit = "Paternal";}
	}
	elsif ($dadmatch < 0.75 and $mummatch >= 0.75){
		if ($ta[2] =~ /$ta[0]/){$inherit = "Maternal";}
		else {$inherit = "Maternal";}
	}
	elsif ($dadmatch >= 0.5 and $mummatch >= 0.5){
		if ($ta[1] =~ /$ta[0]/ and $ta[2] =~ /$ta[0]/){$inherit = "Both";}
		elsif ($ta[1] =~ /$ta[0]/ and $ta[2] !~ /$ta[0]/){$inherit = "Paternal";}
		elsif ($ta[1] !~ /$ta[0]/ and $ta[2] =~ /$ta[0]/){$inherit = "Maternal";}
		elsif ($dadmatch ne "No_data" and $mummatch ne "No_data"){$inherit = "?De Novo";}
		else {$inherit = "Missing parent";}
	}
	elsif ($dadmatch >= 0.5 and $mummatch < 0.5){
		if ($ta[1] =~ /$ta[0]/){$inherit = "Paternal";}
		else {$inherit = "?Paternal";}
	}
	elsif ($dadmatch < 0.5 and $mummatch >= 0.5){
		if ($ta[2] =~ /$ta[0]/){$inherit = "Maternal";}
		else {$inherit = "?Maternal";}
	}
	elsif ($dadmatch < 0.5 and $mummatch < 0.5 and ($dadmatch >= 0.25 or $mummatch >= 0.25)){
		if ($ta[1] =~ /$ta[0]/){$inherit = "?Paternal";}
		elsif ($ta[2] =~ /$ta[0]/){$inherit = "?Maternal";}
		elsif ($dadmatch ne "No_data" and $mummatch ne "No_data"){$inherit = "?De Novo";}
		else {$inherit = "Missing parent";}
	}
	elsif ($dadmatch < 0.25 and $mummatch < 0.25 and $dadmatch ne "No_data" and $mummatch ne "No_data"){
		if ($ta[1] =~ /$ta[0]/){$inherit = "?Paternal";}
		elsif ($ta[2] =~ /$ta[0]/){$inherit = "?Maternal";}
		else {$inherit = "De Novo";}
	}
	else {$inherit = "Missing parent";}
	
	$dadmatch = sprintf("%02d",$dadmatch*100)."%";
	$mummatch = sprintf("%02d",$mummatch*100)."%";
	$misinherited = sprintf("%02d",$misinherited*100)."%";
	
	#Inheritance from Chris' method
	if ($dadmatch eq "No_data" or $mummatch eq "No_data"){$inherit2 = "No_data";}
	else {
		if ($min{0} < $max{1} or $min{0} < $max{2}){$inherit2 = "Inherited";}
		elsif ($max{0} > $min{1} or $max{0} < $min{2}){$inherit2 = "Inherited";}
		else {$inherit2 = "De Novo";}
	}
	
	$outreturn = "$inherit\t$ta2[0]\t$ta2[1]\t$ta2[2]\t$ta2[3]\t\t$dadmatch\t$mummatch\t$misinherited\t\t$ot[0]\t$ot[1]\t$ot[2]\t$ot[3]";
	
	return ($outreturn);
}

#This is where the data files are made for visualization in eTaqman
sub plotpro{
	@poslogbaf = @_;
	$region = $region;
	$typehere = $typeonly;
	$typehere =~ s/\D//;
	
	#print "Plot route\n";
	@tab = split ("\t", $region);
	$filename = $curr_dir."05_Data/".$member."_"."$tab[$chromo]_$tab[$start]_$tab[$stop]_$typehere.txt";
	open (OUT, ">$filename")||die "Could not open file $filename, may already be open";
	print OUT "Position\tCNV\tR_$member\tB_$member";
	@tab2 = split (/\t/, $poslogbaf[0]);
	$realstart = $tab2[0];
	@tab2 = split (/\t/, $poslogbaf[-1]);
	$realstop = $tab2[0];
	
	print OUT "\n$tab[$start]\tFamily\n$tab[$stop]\n$realstart\t$tab[$chromo]:$tab[$start]-$tab[$stop]\n$realstop\t$typeonly\n";
	#print "Plot: pos\t$regcount\t0: $$position[0]\n";
	$logmin = 0;
	
	foreach $line (@poslogbaf){
		@tab2 = split (/\t/,$line);
		if ($tab2[1] < $logmin){$logmin = $tab2[1];}
		print OUT "$tab2[0]\t\t$tab2[1]\t$tab2[2]\n";
	}
	close OUT;
	
	#Set the size of the dots
	$linesinfile = @poslogbaf;
	if ($linesinfile <= 30){$cex = 1;}
	elsif ($linesinfile <= 200){$cex = 0.6;}
	else {$cex = 0.3;}
	#Set the boundaries
	if ($logmin < -8){$ylimit = $logmin - 0.25;}
	elsif ($logmin < -4.25) {$ylimit = $logmin - 0.25;}
	else {$ylimit = -4.5;}
	#Make the filename
	$datafile = $filename;
	$filename =~ s/05_Data/06_Plots/gi;
	$filename =~ s/.txt/.pdf/gi;
	#Set the logR 0 line
	$abline = 0;
	
	#Name of input and output file
	print ROUT 'CNV<-read.delim("'.$datafile.'") #File with CNV logR and Ballele'."\n";
	print ROUT 'pdf("'.$filename.'") #output name of pdf file'."\n";
	#Makes the two windows appear on the same page (2 vertical, 1 horizontal)
	print ROUT 'par(mfrow=c(2,1))'."\n";
	#Set the plot width and vertical lines
	print ROUT 'start<-CNV[1,1] #Start of CNV row 1, column 1'."\n";
	print ROUT 'stop<-CNV[2,1] #Stop of CNV row 2, column 1'."\n";
	print ROUT 'start2<-CNV[3,1] #Start of Region row 1, column 3'."\n";
	print ROUT 'stop2<-CNV[4,1] #Start of Region row 2, column 3'."\n";
	#Graph header is written here
	$headhere = "$member      $tab[$chromo]:$tab[$start]-$tab[$stop]      Type: $typeonly";
	#Colour of dots
	$col = "blue";
	print ROUT 'plot(CNV$R_'.$pheno.'~CNV$Position, ylim=range('.$ylimit.',1.5), main="'.$headhere.'",xlim=c(start2,stop2), col="'.$col.'", pch=20, cex='.$cex.', xlab="Position", ylab="Relative logR");abline(h=c('.$abline.'),v=c(start,stop))'."\n";
	#print ROUT 'mtext(header,side=3)'."\n";
	print ROUT 'plot(CNV$B_'.$pheno.'~CNV$Position,col="'.$col.'",pch=20, cex='.$cex.',xlim=c(start2,stop2), ylim=range(0,1), xlab="Position", ylab="'.$pheno.' B-Allele", yaxp=c(0,1,2));abline(v=c(start,stop))'."\n";
	print ROUT 'dev.off()'."\n\n";
	
}

#This is part of the in eTaqman option, it determines the CNV type in the logR and BAF data; Data is submitted in an array as: position\tlogR\tBAF
sub taqprocess{
	$specific_count++;
	@results = @_;
	$linespresent = @results;
	if ($giverealdata == 1){
		$outname = $member."_".$specific_count.".txt";
		open (OUTER, ">$outname")||die "Could not open $outname may already be open";
		print OUTER "Position\tLogR\tBAF\n";
		foreach (@results){
			print OUTER "$_\n";
		}
	}
	$homo=0; $het=0; $dup=0; $other=0; $totallogr = 0; @loglist = ();
	foreach (@results){
		@tab = split("\t", $_);
		$position = $tab[0];
		$logr = $tab[1];
		$bafer = $tab[2];
		$totallogr += $logr;
		push (@loglist, $logr);
		
		#Works out where each samples' BAF falls
		if ($bafer eq "No_data"){$homo = "No_data"; $het = "No_data"; $dup = "No_data"; $other = "No_data";}
		elsif ($bafer <= 0.1 or $bafer >= 0.9){$homo++;}
		elsif ($bafer <= 0.58 and $bafer >= 0.42){$het++;}
		elsif ($bafer >= 0.6 and $bafer <= 0.75){$dup++;}
		elsif ($bafer <= 0.4 and $bafer >= 0.25){$dup++;}
		else {$other++;}
	}
	
	#Calculate mean, median, stdev and min/max
	if ($linespresent != 0) {$meanlogr = $totallogr/$linespresent;}
	else {$meanlogr = "No_data";}
	@median = sort {$a <=> $b} @loglist;
	$stdevsum = 0;
	foreach $point (@median){
		$stdevsum += ($point - $meanlogr)*($point - $meanlogr);
	}
	if ($linespresent != 1 and $linespresent != 0){
		$stdev = sqrt ($stdevsum / ($linespresent-1) );
	}
	else {$stdev = $stdevsum;}
	$stfactor = 1;
	$min = $meanlogr - ($stfactor * $stdev);
	$max = $meanlogr + ($stfactor * $stdev);
	if ($min <= 0 and $max >= 0){$true = "No";}
	else {$true = "Yes";}
	#print @median;
	$half = $linespresent/2;
	if ($median[0] eq "No_data" or $median[0] eq ''){$medianlogr = "No_data"; $stdev = "No_data"; $min = "No_data"; $max = "No_data";}
	elsif ($half == int($half)){$medianlogr = $median[$half];}
	else {$medianlogr = ($median[$half - 0.5] + $median[$half + 0.5])/2;}
	#print "So the mean is $meanlogr and the median is $medianlogr, 0 = $median[0], -1 = $median[-1]\n";
	
	#Here the proportion of SNPs falling into each BAF group is calculated
	if ($homo eq "No_data"){$homo = "No_data"; $het = "No_data"; $dup = "No_data"; $other = "No_data";}
	elsif ($linespresent != 0) {
		$homo = $homo/$linespresent;
		$het = $het/$linespresent;
		$dup = $dup/$linespresent;
		$other = $other/$linespresent;
	}
	else {
		$homo = 0;
		$het = 0;
		$dup = 0;
		$other = 0;
	}
	
	#This is where the CNV typoe is calculated
	$uncertain = "";
	if($medianlogr eq "No_data"){$thetype = "No_data";}
	#Homozygous deletion
	elsif($medianlogr <= -1.5){
		if ($homo < 0.5){$thetype = 0;}
		else {$thetype = 0; $uncertain = "?";}
	}
	#Heterozygous deletion
	elsif($medianlogr > -1.5 and $medianlogr <= -0.25){
		if ($homo >= 0.8){$thetype = 1;}
		else {$thetype = 1; $uncertain = "?";}
	}
	#Normal
	elsif($medianlogr > -0.25 and $medianlogr < 0.15){
		if (($het + $homo) >= 0.8){$thetype = 2;}
		else {$thetype = 2; $uncertain = "?";}
	}
	#Duplication
	elsif($medianlogr >= 0.15 and $medianlogr < 0.4){
		if ($dup >= $het){$thetype = 3;}
		elsif ($het > 0.2 and $dup < 0.1){$thetype = 2;}
		else {$thetype = 3; $uncertain = "?";}
	}
	elsif($medianlogr >= 0.4){
		if ($het < 0.15){$thetype = 3;}
		elsif ($het >= 0.15 and $other >= 0.1){$thetype = 4;}
		else {$thetype = 4; $uncertain = "?";}
	}
	
	
	#The output is prepared then returned
	$typeout = $uncertain.$thetype;
	if ($thetype eq "No_data"){$theoutput = "No_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\tNo_data\t";}
	else {
		$theoutput = $typeout."\t".sprintf("%.2f",$meanlogr)."\t".sprintf("%.2f",$stdev)."\t".sprintf("%.2f",$min)."\t".sprintf("%.2f",$max)."\t".sprintf("%.2f",$medianlogr)."\t".sprintf("%02d",$homo*100)."%\t".sprintf("%02d",$het*100)."%\t".sprintf("%02d",$dup*100)."%\t".sprintf("%02d",$other*100)."%\t$true";
	}
	return ($theoutput);
}

#Very quickly assess all files with FinalReport in the title and returns an array with each file: filename, sampleID, SNPs, chip, bmp file
sub fr_name{
	&fr_columns;
	@files = <*>;
	foreach $file (@files){
		if ($file =~ /$namematch/){
			$count++;
			open (IN2, "<$file")|| die "Could not open file $file";
			if ($count - $lastcount >= 100){
				print "Doing number $count\n";
				$lastcount = $count;
			}
			$count2 = 0;
			while (<IN2>){
				$count2++;
				if($count2 == 4){
					@tab = split ("\t", $_);
					$bigchip = $tab[2];
					chomp $bigchip;
				}
				if($count2 == 5){
					@tab = split ("\t", $_);
					$snps = $tab[1];
					chomp $snps;
					$chip = &chiper($snps);
				}
				if($count2 == 20){
					@tab = split ("\t", $_);
					$sample = $tab[$patname];
					$output2 = "$file\t$sample\t$snps\t$chip\t$bigchip\n";
					#print OUT "$file\t$sample\t$snps\t$chip\n";
					push (@output, $output2);
					#print "$file\t$sample\n";
					close IN2
				}
			}
		}
	}
	return (@output);
}

#This subroutine opens the AIMs file and makes a hash (called hash) of the lines with SNP name as the key and the line of the file as the value
sub buildHash(){
	%hash;
	print "Loading ancestry informative markers (AIMs)\n";
	$aimsfile = $ANloc."hg18_AIMs.txt";
	open(AIM, "<$aimsfile") or die "Could not open $aimsfile; check it is present\n";
	while(<AIM>){
		@array = split(/\t/, $_);
		$snpLoc = $array[2];
		$hash{$snpLoc} = $_ or die "this failed\n";
	}
	return %hash;
}

#Checks if the AIM is present in the hash and makes an array of the line of the AIMs file
sub parseAIMS(){
	@array = split("\t", $hash{$_[0]}) or return 0;
	return @array;
}

#This subroutine processes a pedfile to make the following hashes:
#pedfile - key: sampleID, value: all 6 columns of pedfile
#family_member - key: familyID, value: list of samples in that family
#which_family - key: sampleID, value: familyID
#which_mother - key: sampleID, value: mother's sampleID
#which_father - key: sampleID, value: father's sampleID
#which_child - key: either parent's sampleID, value: list of children's sampleID
#which_sibling - key: sampleID, value: list of sibling sampleIDs
#gender - key: sampleID, value: gender (0, 1, 2)
#gendername - key: sampleID, value: gender (Unknown, Male, Female)
#phenotype - key: sampleID, value: phenotype (0, 1, 2)
#namepheno - key: sampleID, value: Proband, Sibling, Unaffected child, Control, Father, Mother, Unknown
sub propedfile {
	@data = @_;
	@pederror = ();
	foreach $line (@data){
		@tab = split (/\t/, $line);
		$family = $tab[0];
		$sample = $tab[1];
		$father = $tab[2];
		$mother = $tab[3];
		$gender = $tab[4];
		$phenotype = $tab[5];
		
		if ($gender == 1){
			$genname = "Male";
		}
		elsif ($gender == 2){
			$genname = "Female";
		}
		elsif ($gender == 0){
			$genname = "Unknown";
		}
		else {
			$genname = "Unknown";
		}
		
		if ($pedfile{$sample} eq ''){
			$pedfile{$sample} = $line;
		}
		else {
			push (@pederror, $sample);
		}
		$family_member{$family} = $family_member{$family}.$sample."\t";
		$which_family{$sample} = $family;
		$which_mother{$sample} = $mother;
		$which_father{$sample} = $father;
		unless ($father == 0){$which_child{$father} = $which_child{$father}.$sample."\t";}
		unless ($mother == 0){$which_child{$mother} = $which_child{$mother}.$sample."\t";}
		$gender{$sample} = $gender;
		$gendername{$sample} = $genname;
		$phenotype{$sample} = $phenotype;
	}
	
	#This works out who the siblings are (samples from the same family who have the same parents)
	@samplelist = keys %gender;
	foreach $sample (@samplelist){
		$family = $which_family{$sample};
		$members = $family_member{$family};
		@members = split (/\t/,$members);
		foreach $member (@members){
			if ($member eq $sample){}
			elsif ($member eq $which_mother{$sample}){}
			elsif ($member eq $which_father{$sample}){}
			else {
				if ($which_mother{$sample} eq $which_mother{$member}
				and $which_father{$sample} eq $which_father{$member}
				and $which_mother{$sample} ne 0
				and $which_father{$sample} ne 0){
					$which_sibling{$sample} = $which_sibling{$sample}.$member.",";
				}
			}
		}
		if ($phenotype{$sample} == 2){
			$namepheno{$sample} = "Proband";
			unless ($phenotype{$which_father{$sample}} == 2 or $which_father{$sample} eq 0){
				$namepheno{$which_father{$sample}} = "Father";
			}
			unless ($phenotype{$which_mother{$sample}} == 2 or $which_mother{$sample} eq 0){
				$namepheno{$which_mother{$sample}} = "Mother";
			}
			@siblings = split (/,/,$which_sibling{$sample});
			foreach $sib (@siblings){
				unless ($phenotype{$sib} == 2){
					$namepheno{$sib} = "Sibling";
				}
			}
		}
		elsif ($phenotype{$sample} == 1 and $namepheno{$sample} ne "Sibling"){
			#print "But I am here\n";
			unless ($namepheno{$sample} eq "Sibling" or $namepheno{$sample} eq "Father" or $namepheno{$sample} eq "Mother" or $namepheno{$sample} eq "Unaffected Child"){
				$namepheno{$sample} = "Control";
				#print "Samp: $sample\tName: $namepheno{$sample}\n";
			}
			#print "I am here\n";
			if (($namepheno{$which_father{$sample}} eq '' or $namepheno{$which_father{$sample}} eq 'Control' or $namepheno{$which_father{$sample}} eq 'Father') and $which_father{$sample} ne 0){
				#print "Dad: $which_father{$sample}\n";
				$namepheno{$which_father{$sample}} = "Father";
				$namepheno{$sample} = "Unaffected Child";
				#print "I'm here\n";
			}
			#print "Samp: $sample\tName: $namepheno{$sample}\n";
			if (($namepheno{$which_mother{$sample}} eq '' or $namepheno{$which_mother{$sample}} eq 'Control' or $namepheno{$which_mother{$sample}} eq 'Mother') and $which_mother{$sample} ne 0){
				$namepheno{$which_mother{$sample}} = "Mother";
				$namepheno{$sample} = "Unaffected Child";
				#print "I'm here\n";
			}
			#print "Samp: $sample\tName: $namepheno{$sample}\n";
			@siblings = split (/,/,$which_sibling{$sample});
			foreach $sib (@siblings){
				if ($namepheno{$sib} eq ''){
					$namepheno{$sib} = "Unaffected Child";
				}
			}
			#print "Samp: $sample\tName: $namepheno{$sample}\n";
		}
		elsif ($phenotype{$sample} == 0){
			$namepheno{$sample} = "Unknown";
		}
	}
	$allpederror = @pederror;
	if ($allpederror >= 1){
		print "ERROR There were $allpederror samples with multiple entries in the pedfile\nThe following samples were duplicated:";
		foreach (@pederror){
			print "$_\n";
		}
		$listpederror = join (",",@pederror);
		print LOG1 "$timeanddate\tProPED\tERROR\tThere were $allpederror samples with multiple entries in the pedfile\n";
		print LOG1 "$timeanddate\tProPED\tERROR\tThe duplicated samples are: $listpederror\n";
	}
}

#Subroutine used in frequency assessment
sub freqsub {
	@CNVfreq = ();
	$error = 0;
	$maxbase = 0;
	@match = ();
	@merge =();
	%allsamp = ();
	$track = 0;
	$total = 0;
	#Need to submit $sampleID, $chromo, $start, $stop values, FOUT open
	if ($chromo eq $start
	or $start eq $stop
	or $sampleID eq $chromo){
		print "ERROR  Frequency finder does not know which columns are which\n";
		$error++;
	}
	
	foreach (@_){
		@tmp = split ("\t", $_);
		$allsamp{$tmp[$sampleID]} = 0;
		
		if ($thres == 1){
			#print "Line: $_\n";
			#print "Values: SNP: $tmp[$numsnpcol] T:$snpthresnow Algonum: $tmp[$algonumcol] T:$algothresnow Type: $tmp[$cnvtypecol] T: $typethresnow\n";
			if ($tmp[$numsnpcol] >= $snpthresnow
			and $tmp[$algonumcol] >= $algothresnow){
				if ($typethresnow eq "del"
				and $tmp[$cnvtypecol] !~ /2/
				and $tmp[$cnvtypecol] !~ /3/
				and $tmp[$cnvtypecol] !~ /4/){
					push (@CNVfreq, $_);
					#print "deletion\n";
				}
				elsif ($typethresnow eq "dup"
				and $tmp[$cnvtypecol] !~ /0/
				and $tmp[$cnvtypecol] !~ /1/){
					push (@CNVfreq, $_);
					#print "duplication\n";
				}
				elsif ($typethresnow eq "both"){
					push (@CNVfreq, $_);
					#print "both\n";
				}
				#else {print "Not added to array\n";}
			}
			#else{print "Not added to the array\n";}
		}
		else {push (@CNVfreq, $_);}
	}
	@_ = ();
	
	print "Sorting CNVs by chromosome and start position\n";
	
	#Sort by Start (numbers)
	@CNVfreq = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNVfreq;
	
	#Sort by Chromosome (string)
	@CNVfreq = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNVfreq;
	
	@samps = keys %allsamp;
	$total = @samps;
	print "$total samples detected\n";
	print FOUT "Chr\tStart\tStop\tNumber of calls\tFrequency\tSampleID\n";
	
	#print OUT "@CNVall";
	
	print "Looking for CNVs that match\n";
	
	#Does each CNV line up with the one below?
	foreach $line (@CNVfreq){
		@before = split ("\t", $CNVfreq[$track-1]);
		@now = split ("\t", $CNVfreq[$track]);
		@after = split ("\t",$CNVfreq[$track+1]);
		$track++;
		#print "@now\n";
	
		#If this CNV is from a different sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print "\nERROR The identity of the following CNV could not be determined:\n$line\n";
			$error++;
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			#Process @match
			#print "Found a match\n";
			&setexaminef;
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	

	
	sub setexaminef{
		
		$matches = @match;
		
		%coord = ();
		%locitype = ();
		%sizes = ();
		%snps = ();
		$counter = 0;
		
		#Put the start and stop positions into a new array and determine the characteristics of this group of matches
		foreach (@match){
			@temp = split ("\t", $_);
			$temp[$start] =~ s/^\s+//;
			$temp[$stop] =~ s/^\s+//;
			chomp $temp[$start];
			chomp $temp[$stop];
			$coord{$temp[$start]} = 0;
			$coord{$temp[$stop]} = 0;
			$locitype{$temp[$cnvtype]} = 0;
			$chrm = $temp[$chromo];
		}
		
		$chrm =~ s/chr//i;
		@coord = keys %coord;
		@coord = sort {$a <=> $b} @coord;
		
		$algocount = 0;
		%algos = ();
		%uid = ();
		%type = ();
		
		############################################################################################################################
		
		#Look for matches at each line of the co-ord
		foreach $bp (@coord){
			$lastone = $coord[$counter - 1];
			$thisone = $coord[$counter];
			$nextone = $coord[$counter + 1];
			
			$counter++;
			
			##########################################################################################################################
			
			#For each coord look for matches in the start and stop coords
			foreach $line4 (@match){
				@cnvm = split ("\t", $line4);
				#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
				if($thisone == $cnvm[$start]){
					$algocount++;
					chomp $cnvm[$sampleID];
					$uid{$cnvm[$sampleID]} = "$cnvm[$algo]\t$cnvm[$cnvtype]\t$cnvm[$size]\t$cnvm[$numsnps]\t";
				}
				
				#If the coord matches the end of a CNV the uid and algo are removed from the hashes
				if ($thisone == $cnvm[$stop]){
					$algocount = $algocount - 1;
					chomp $cnvm[$sampleID];
					delete ($uid{$cnvm[$sampleID]});
				}
			}
			
			%type = ();
			%algoname = ();
			@features = ();
			
			@uidkeys = keys %uid;
			
			@uidkeys = sort @uidkeys;
			$sampresent = join (", ", @uidkeys);
			$freak = $algocount / $total;
			
			#Printout this section of the match
			unless ($nextone == ""){
				$outhere = "chr$chrm\t$thisone\t$nextone\t$algocount\t$freak\t$sampresent";
				print FOUT "$outhere\n";
				push (@merge, $outhere);
				if ($javamethod == 0){print "\rDoing chr$chrm   ";}
			}
		}
	}
	print FOUT "\n\n$total samples used to calculate population frequency\n";
	return ($total);
}

#Subroutine used in frequency assessment of DGV files
sub dgvfreqsub {
	
	$maxbase = 0;
	@match = ();
	@merge =();
	$track = 0;
	
	@CNVfreq = @_;
	
	print "Sorting CNVs by chromosome and start position\n";
	
	#Sort by Start (numbers)
	@CNVfreq = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNVfreq;
	
	#Sort by Chromosome (string)
	@CNVfreq = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNVfreq;
	
	
	print FOUT "Chr\tStart\tStop\tNumber of lines\tNumber of DGV calls\tFrequency ($total controls)\tSampleID\n";
	
	#print OUT "@CNVall";
	
	print "Looking for CNVs that match\n";
	
	#Does each CNV line up with the one below?
	foreach $line (@CNVfreq){
		@before = split ("\t", $CNVfreq[$track-1]);
		@now = split ("\t", $CNVfreq[$track]);
		@after = split ("\t",$CNVfreq[$track+1]);
		$track++;
		#print "@now\n";
	
		#If this CNV is from a different sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print "\nERROR The identity of the following CNV could not be determined:\n$line\n";
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			#Process @match
			#print "Found a match\n";
			&dgvsetexamine(@match);
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	
	
	sub dgvsetexamine{
		
		$matches = @_;
		
		%coord = ();
		%dgvcount = ();
		$counter = 0;
		
		#Put the start and stop positions into a new array and determine the characteristics of this group of matches
		foreach (@match){
			@temp = split ("\t", $_);
			$temp[$start] =~ s/^\s+//;
			$temp[$stop] =~ s/^\s+//;
			chomp $temp[$start];
			chomp $temp[$stop];
			$coord{$temp[$start]} = 0;
			$coord{$temp[$stop]} = 0;
			$dgvcount{$temp[$dgvinput]} = 0;
			$chrm = $temp[$chromo];
		}
		
		$chrm =~ s/chr//i;
		@coord = keys %coord;
		@coord = sort {$a <=> $b} @coord;
		
		$algocount = 0;
		%uid = ();
		
		############################################################################################################################
		
		#Look for matches at each line of the co-ord
		foreach $bp (@coord){
			$lastone = $coord[$counter - 1];
			$thisone = $coord[$counter];
			$nextone = $coord[$counter + 1];
			
			$counter++;
			
			##########################################################################################################################
			
			#For each coord look for matches in the start and stop coords
			foreach $line4 (@match){
				@cnvm = split ("\t", $line4);
				#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
				if($thisone == $cnvm[$start]){
					$algocount++;
					$dgvcount = $dgvcount + $cnvm[$dgvinput];
					chomp $cnvm[$sampleID];
					$uid{$cnvm[$sampleID]} = $cnvm[$dgvinput];
				}
				
				#If the coord matches the end of a CNV the uid and algo are removed from the hashes
				if ($thisone == $cnvm[$stop]){
					$algocount = $algocount - 1;
					$dgvcount = $dgvcount - $cnvm[$dgvinput];
					chomp $cnvm[$sampleID];
					delete ($uid{$cnvm[$sampleID]});
				}
			}
			
			@uidkeys = keys %uid;
			
			@uidkeys = sort @uidkeys;
			$sampresent = join (", ", @uidkeys);
			$freak = $algocount / $total * 100;
			$freak2 = $dgvcount / $total * 100;
			
			#Printout this section of the match
			unless ($nextone == ""){
				$outhere = "chr$chrm\t$thisone\t$nextone\t$algocount\t$dgvcount\t$freak2\t$sampresent";
				print FOUT "$outhere\n";
				print BED "chr$chrm\t$thisone\t$nextone\t$freak2\n";
				push (@merge, $outhere);
				if ($javamethod == 0){print "\rDoing chr$chrm   ";}
			}
		}
	}
	print FOUT "\n\n$total samples used to calculate population frequency\n";
}

# A subroutine of the denovo1 algorithm that works out the loci of the parents
sub locimakerd{
	@parents = @_;
	#print "Parents: @parents\n";
	@others = ();
	$thefather = $parents[0];
	$themother = $parents[1];
	push (@others, @$thefather);
	push (@others, @$themother);
	$othersize = @others;
	#print "Other: $othersize\n";
	#Sort by Start (numbers)
	@others = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @others;
	
	#Sort by Chromosome (string)
	@others = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @others;
	
	#Make the reference input file into loci
	$tracker = 0; @before = (); @now = (); @after = (); $maxbase = 0;
	#print "Start: $start, End: $stop, Chr: $chromo\n";
	foreach (@others){
		@before = split ("\t", $others[$tracker - 1]);
		@now = split ("\t", $others[$tracker]);
		@after = split ("\t", $others[$tracker + 1]);
		$tracker++;
		
		#print "\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
		
		#If this CNV is from a different chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New chromosome or set\n";
			push (@match, $_);
		}
		
		#If it is the same chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $_);
			#print "This was a match\n";
			#print "Maxbase = $maxbase\n"
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print LOG1 "$timeanddate\tDenovo\tERROR\tThe status of this CNV could not be determined:\t$line\n";
			print "Error with the following CNV, its status could not be determined:\n$line\n";
			$error++;
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			
			#Process @match
			%bps = ();
			foreach (@match){
				@bp = split ("\t", $_);
				$bps{$bp[$start]} = 0;
				$bps{$bp[$stop]} = 0;
				$chrm = $bp[$chromo];
			}
			@bps = keys %bps;
			@bps = sort {$a <=> $b} @bps;
			$st = $bps[0];
			$en = $bps[-1];
	
			$arrayin = $chrm."\t".$st."\t".$en."\n";
			push (@input, $arrayin);
			#print "Last match in a set, dealing with matches\n@match\nInput: $arrayin\n";
			@match = ();
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
	
	@others = ();
	@DGV2 = ();
	#open (IN, ">Modified_DGV.txt") || die "couldn't open file Modified_DGV.txt\n";
	#print IN @input;
	
	#Modify DGV array to same format as CNV file and make unique sampleID
	foreach $line (@input){
		@essex = split ("\t", $line);
		chomp @essex;
		$dcounter++;
		@newarray[$sampleID] = "$common$dcounter";
		@newarray[$chromo] = $essex[0];
		@newarray[$start] = $essex[1];
		@newarray[$stop] = $essex[2];
		$moddgv = join ("\t", @newarray);
		#print "$moddgv\n";
		push (@DGV2, "$moddgv\n");
	}
	@input = ();
	return (@DGV2);
	#print IN @DGV2;
}

# A subroutine of the denovo1 algorithm that processes the family. If two parents are present the parental loci are calculated and then submitted for setexamination otherwise printed out here
sub familyprod{
	@familymembers = @_;
	$famsize = @familymembers;
	$samplecount = $samplecount + $famsize;
	#print "Family: @familymembers\n";
	
	#If the sample is not identified as a parent it must be a 'child'
	foreach $member (@familymembers){
		#print "Member: $member\n";
		if ($which_father{$member} ne 0 and $which_mother{$member} ne 0){
			#This is a proband or sibling
			$thefather = $which_father{$member};
			$themother = $which_mother{$member};
			$fathersize = @$thefather;
			$mothersize = @$themother;
			#print "Fathersize = $fathersize, Mothersize = $mothersize\n";
			if ($fathersize > 0 and $mothersize > 0){
				@parents = ($which_father{$member}, $which_mother{$member});
				#print "@parents\n";
				@commonarray = &locimakerd(@parents);
				$commonsize1 = @commonarray;
				#print "Commonsize1 = $commonsize1\n";
				push (@commonarray, @$member);
				$commonsize2 = @commonarray;
				#print "Commonsize2 = $commonsize2\n";
				&samplepro2d(@commonarray);
			}
			else {
				print "ERROR no parental CNVs found for $member. Father: $fathersize CNVs, Mother: $mothersize CNVs\n";
				print LOG1 "$timeanddate\tDenovo\tERROR\tNo parental CNVs found for $member. Father: $fathersize CNVs, Mother: $mothersize CNVs\n";
				foreach $line (@$member){
					print OUT "$gendername{$member}\t$namepheno{$member}\t$famsize\t \t$line";
					$samplelog{$member} = "$namepheno{$member}\t$famsize\tNo";
				}
			}
		}
		else {
			#This is a parental sample
			foreach $line (@$member){
				print OUT "$gendername{$member}\t$namepheno{$member}\t$famsize\t \t$line";
				$samplelog{$member} = "$namepheno{$member}\t$famsize\tNo";
			}
		}
	}
	foreach $member (@familymembers){
		@$member = ();
	}
}

# A subroutine of the denovo1 algorithm the works out the common areas between parents and children in the array submitted to it
sub samplepro2d{
	@combine = @_;
	$setcount++;
	print "$setcount) Family $famnow, $famsize members, $commonsize1 parent loci, $commonsize2 child CNVs\n";
	
	#Sort by Start (numbers)
	@combine = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @combine;
	
	#Sort by Chromosome (string)
	@combine = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @combine;
	
	$track = 0;
	#Does each CNV line up with the one below?
	foreach $line (@combine){
		@before = split ("\t", $combine[$track - 1]);
		@now = split ("\t", $combine[$track]);
		@after = split ("\t",$combine[$track + 1]);
		$track++;
		
		#print "\n\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
			
		#If this CNV is from a differet sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if (lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
		elsif (lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print LOG1 "$timeanddate\tDenovo\tERROR\tThe status of this CNV could not be determined:\t$line\n";
			print "Error with the following CNV, its status could not be determined:\n$line\n";
			$error++;
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if (lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			#Process @match
			&setexamined(@match);
			@match = ();
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
	}
}

# A subroutine of the denovo1 algorithm that looks at overlapping loci
sub setexamined{
	@match = @_;
	$matches = @match;
	%coord = ();
	%locitype = ();
	%sizes = ();
	%snps = ();
	$counter = 0;
	
	#Put the start and stop positions into a new array and determine the characteristics of this group of matches
	foreach (@match){
		@temp = split ("\t", $_);
		$temp[$start] =~ s/^\s+//;
		$temp[$stop] =~ s/^\s+//;
		chomp $temp[$start];
		chomp $temp[$stop];
		$coord{$temp[$start]} = 0;
		$coord{$temp[$stop]} = 0;
		if ($temp[$sampleID] !~ /$common/){
			$sample = $temp[$sampleID];
			$chrm = $temp[$chromo];
		}
	}
	
	#Sort and determine the loci start and end
	@coord = keys %coord;
	@coord = sort {$a <=> $b} @coord;
	
	$algocount = 0;
	%algos = ();
	%uid = ();
	%type = ();
	$counter = 0;
	
	############################################################################################################################
	
	#Look for matches at each line of the co-ord
	foreach $bp (@coord){
		$lastone = $coord[$counter - 1];
		$thisone = $coord[$counter];
		$nextone = $coord[$counter + 1];
		
		$counter++;
		
		##########################################################################################################################
		
		#For each coord look for matches in the start and stop coords
		foreach $line4 (@match){
			@cnvm = split ("\t", $line4);
			#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
			if($thisone == $cnvm[$start]){
				$algocount++;
				chomp $cnvm[$sampleID];
				$uid{$cnvm[$sampleID]} = "$line4";
			}
			
			#If the coord matches the end of a CNV the uid and algo are removed from the hashes
			if ($thisone == $cnvm[$stop]){
				$algocount = $algocount - 1;
				chomp $cnvm[$sampleID];
				delete ($uid{$cnvm[$sampleID]});
			}
		}
		
		#Work out the size of this section
		$musize = $nextone - $thisone;
		
		@uidkeys = keys %uid;
		$commie = 0;
		$interest = 0;
		
		#print "New set\n";
		foreach $key (@uidkeys){
			#print "$key\n";
			if ($key =~ /$common/){
				$commie = 1;
				}
			else {
				@resize = split ("\t", $uid{$key});
				$resize[$start] = $thisone;
				$resize[$stop] = $nextone;
				$resize[$size] = $musize;
				$samplepoint = $key;
				#print "Sample ID: $samplepoint Phenotype: $pheno{$samplepoint}\n";
				$interest = 1;
			}
		}
		
		#Printout this section of the match
		$printout = join ("\t", @resize);
		#print "$printout\n";
		unless ($nextone eq ""
		or $interest == 0){
			print OUT "$gendername{$samplepoint}\t$namepheno{$samplepoint}\t$famsize\t$commie\t$printout";
			$samplelog{$samplepoint} = "$namepheno{$samplepoint}\t$famsize\tYes";
			#$uniquespot = "$resize[$sampleID]\t$resize[$chromo]\t$resize[$lstart]";
		}
	}
}

#Subroutine used to detect homozygous deletions
sub homodelsub {
	#Initial threshold
	$logfirst = -3;
	
	#Secondary threshold
	$logsecond = -1;
	
	#Loads the column order for FinalReports
	&fr_columns;
	
	@files = @_;
	
	foreach $file (@files){
		if ($file =~ /$namematch/){
			$filecount++;
			@homos = ();
			open (NEWFILE, "<$file" || die "unable to open $file\n");
			@newie = <NEWFILE>; close NEWFILE;
			
			if ($filecount == 1){
				print "Loading first FinalReport that will serve as a template for all who follow\n";
				@snp = @newie; chomp @snp;
				@random = split ("\t", $snp[30]);
				chomp @random;
				#print "Random: @random\n";
				$sample = $random[$patname];
				#print "Sample: $sample\n";
				
				#This section makes a hash called %rs with the SNP names as keys and line number in the FinalReport as the value
				print "Looking for position of each SNP in Final Report\n";
				$linenum = 0;
				foreach $line (@snp){
					@bits = split ("\t", $line, 4);
					chomp @bits;
					$rs{$bits[$snpcol]} = $linenum;
					$linenum++;
				}
				
				print "Sorting Final Report\n";
				#Sort by Position (numbers)
				@snp = sort { (split '\t', $a, 4)[$chromodist] <=> (split '\t', $b, 4)[$chromodist] } @snp;
			
				#Sort by Chromosome (string)
				@snp = sort { (split '\t', lc $a, 4)[$chromocol] <=> (split '\t', lc $b, 4)[$chromocol] } @snp;
				
				#This makes two hashes:
				# 1) A hash called %rsabove with the current SNP name as the key and the SNP name of the SNP above in the sorted FR as the value
				# 2) A hash called %rsbelow with the current SNP name as the key and the SNP name of the SNP below in the sorted FR as the value
				print "Looking for neighbouring SNPs in sorted Final Report\n";
				$linenum2 = 0;
				foreach $line (@snp){
					@before = split ("\t", $snp[$linenum2 - 1], 4);
					@now = split ("\t", $snp[$linenum2], 4);
					@after = split ("\t", $snp[$linenum2 + 1], 4);
					chomp @now;
					chomp @before;
					chomp @after;
					$rsabove{$now[$snpcol]} = $before[$snpcol];
					$rsbelow{$now[$snpcol]} = $after[$snpcol];
					$linenum2++;
				}
				
				$rs = keys %rs;
				$rsa = keys %rsabove;
				#print "Hash rs: $rs, Hash rsabove: $rsa\n";
			}
			
			
			#Homozygous del SNPs are put into the array @homos
			$tracker = 0;
			$het = 0;
			$homo = 0;
			$chroX = 'X';
			foreach $line (@newie){
				#print "$line\n";
				$tracker++;
				@here = split("\t", @newie[$tracker], 10);
				
				if ($here[$chromocol] =~ /$chroX/){
					if($here[$gtcol1] =~ /-/) {}
					else{
						if ($here[$gtcol1] eq $here[$gtcol2]){$homo++}
						else {$het++}
					}
				}
				
				if($here[$log2col] <= $logfirst) {
					push (@homos, @newie[$tracker]);
					$homocount++;
					#print "\rHomo number $homocount found\n";
				}
			}
			
			if(($homo+$het)>0){
				$homop = $homo / ($homo+$het)*100;
				$hetp = $het / ($homo+$het)*100;
			}
			if ($hetp > 10){
				$sex = 'Female';
				$sexn = 2;
			}
			else{
				$sex = 'Male';
				$sexn = 1;
			}
			print "$filecount) $file\tSex: $sex\t";
			
			#print "Now sorting that sample\n";
			#Sort the array by chr and position
			#Sort by Position (numbers)
			@homos = sort { (split '\t', $a)[$chromodist] <=> (split '\t', $b)[$chromodist] } @homos;
			
			#Sort by Chromosome (string)
			@homos = sort { (split '\t', lc $a)[$chromocol] <=> (split '\t', lc $b)[$chromocol] } @homos;
			
			$hom = @homos;
			#print "Found $hom SNPs with a LogR <-3\n";
			
			$tracker2 = 0;
			
			#Here SNPs with a logR less than the threshold are processed
			foreach (@homos){
				#print "Looking at SNP: $_\n";
				@now = split ("\t", @homos[$tracker2]);
				#print "Now: @homos[$tracker2]\nAfter: @homos[$tracker2 + 1]\n";
				@after = split ("\t", @homos[$tracker2 + 1]);
				push (@consec, @homos[$tracker2]);
				$tracker2++;
				
				#Extends the call until it finds SNPs less than logR -1 at start
				#print "After: $after[$snpcol], RSbelow: $rsbelow{$now[$snpcol]}\n";
				if ($after[$snpcol] ne $rsbelow{$now[$snpcol]} or $after[$snpcol] eq ""){
					$mark = 0;
					do{
						@first1 = split ("\t", $consec[0]);
						$rsstart1 = $first1[$snpcol];
						$rsbefore1 = $rsabove{$rsstart1};
						@before1 = split ("\t", $newie[$rs{$rsbefore1}]);
						chomp @before1;
						if ($before1[$log2col] <= $logsecond
						and $before1[$chromocol] == $first1[$chromocol]){
							unshift (@consec, $newie[$rs{$rsbefore1}]);
							#print "LogR $before1[$log2col] is less than -1\n";
						}
						else {
							($mark = 1);
							#print "This should now stop\n";
						}
					}until ($mark == 1);
					
					$mark = 0;
					do{
						@final1 = split ("\t", $consec[-1]);
						$rsstop1 = $final1[$snpcol];
						$rsafter1 = $rsbelow{$rsstop1};
						@after1 = split ("\t", $newie[$rs{$rsafter1}]);
						#print "Looking at the SNP above $rsstop: $rsafter1 in position $rs{$rsafter1}\n";
						#print "The SNP in this position is called $after1[$snpcol]\n";
						chomp @after1;
						if ($after1[$log2col] <= $logsecond
						and $after1[$chromocol] == $final1[$chromocol]){
							push (@consec, $newie[$rs{$rsafter1}]);
							#print "LogR $after1[$log2col] is less than -1\n";
						}
						else {
							($mark = 1);
							#print "This should now stop\n";
						}
					}until ($mark == 1);
					
					#print "Mismatch: Below list: $after[$snpcol] Below hash: $rsbelow{$now[$snpcol]}\n";
					@first = split ("\t", $consec[0]);
					@final = split ("\t", $consec[-1]);
					$samp = $first[$patname];
					$chro = $first[$chromocol];
					$rsstart = $first[$snpcol];
					$rsstop = $final[$snpcol];
					$start = $first[$chromodist];
					$stop = $final[$chromodist];
					$number = @consec;
					$size = $stop - $start;
					#Works out the total logR in preparation for working out the mean
					foreach (@consec){
						@bit = split ("\t", $_);
						chomp @bit;
						push (@logshere, $bit[$log2col]);
						$total = $total + $bit[$log2col];
					}
					$rsbefore = $rsabove{$rsstart};
					$rsafter = $rsbelow{$rsstop};
					@before = split ("\t", $newie[$rs{$rsbefore}]);
					@after = split ("\t", $newie[$rs{$rsafter}]);
					chomp @before;
					chomp @after;
					$startbefore = $before[$chromodist];
					unshift (@logshere, $before[$log2col]);
					$stopafter = $after[$chromodist];
					push (@logshere, $after[$log2col]);
					if ($number > 0){
						$mean = $total/$number;
					}
					else {$mean = 0}
					#print "Start: $start, Chromo: $chro, Number: $number, Mean: $mean\n";
					$logout = join (", ", @logshere);
					$chry = 'Y';
					if ($sexn == 1
					or $chro ne $chry){
						#print "I'm here\n";
						$outie = "$samp\tchr$chro\t$start\t$stop\t$rsstart\t$rsstop\t$size\t$number\t$startbefore\t$stopafter\t$rsbefore\t$rsafter\t$mean\t$logout\n";
						$output{$outie} = 1;
						$outputcount++;
					}
					$total = 0;
					@logshere = ();
					#print "$samp\t$chro\t$start\t$stop\t$number\n";
					@consec = ();
				}
			}
			@outputer = keys %output;
			#Sort by Position (numbers)
			@outputer = sort { (split '\t', $a)[2] <=> (split '\t', $b)[2] } @outputer;
			
			#Sort by Chromosome (string)
			@outputer = sort { (split '\t', lc $a)[1] cmp (split '\t', lc $b)[1] } @outputer;
			$here = @outputer;
			#print "Outputer has $here entries\n";
			print OUT @outputer;
			print "Deletions: $here\n";
			$totdel += $here;
			%output = ();
			@outputer = ();
			
			$outputcount = 0;
		}
	}
	$timeanddate = &showtime;
	$meandel = sprintf ("%.0f", $totdel / $filecount);
	print "\n$filecount $namematch files examined; $meandel homozygous deletions found on average\n";
	print LOG1 "$timeanddate\tHomodel\tINFO\tHomodel detection complete; $filecount $namematch files examined; mean of $meandel homozygous deletions found per sample\n";
	
	if ($meandel < 1){
		print "ERROR: Very low number of homozygous deleltions found, may not have worked\n";
		print LOG1 "$timeanddate\tHomodel\tERROR\tVery low number of homozygous deleltions found, may not have worked\n";
	}
	else{print "Everything is ok\n";}
}

#Subroutine used for small inquire
sub smallinquire {
	
	$input = $_[0];
	@slash = split (/\//,$input);
	if ($method == 0){
		$output = $merge_dir.$tag."_Sum.txt";
	}
	else {
		$output = "Sum_".$slash[-1];
	}
	open (INPUT, "<$input") || die "Cannot open file $input, check it is in the right directory\n";
	open (OUT, ">$output") || die "Cannot open file $output, may already be open\n";
	print LISTOUT $curr_dir."$output\n";
	print OUT "FID\tSampleID\tChr\tLoci Start\tLoci Stop\tType\tMax Score\tMax SNP\tSize\t#Algos\tAlgos\t%Three Algs\t%Two Algs\t%One Alg\n";
	@CNV = <INPUT>; close (INPUT);
	shift @CNV;
	
	$extra = $_[1];
	
	$famID = 0 + $extra;
	$sampleID = 1 + $extra;
	$chromo = 2 + $extra;
	$lstart = 3 + $extra;
	$lstop = 4 + $extra;
	$cnvtype= 5 + $extra;
	$maxsize = 6 + $extra;
	$maxsnp = 7 + $extra;
	$lsize = 8 + $extra;
	$start = 10 + $extra;
	$stop = 11 + $extra;
	$size = 12 + $extra;
	$type = 13 + $extra;
	$algonum = 14 + $extra;
	$algos = 15 + $extra;
	
	###############################################################################################
	print "Calculating numbers of loci present\n";
	#print "Sorting input file by Sample, Chromosome and Loci Start\n";
	#Sort by Start (numbers)
	@CNV = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNV;
	
	#Sort by Chromosome (string)
	@CNV = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNV;
	
	#Sort by SampleID (string)
	@CNV = sort { (split '\t', lc $a)[$sampleID] cmp (split '\t', lc $b)[$sampleID] } @CNV;
	
	#Split CNV list into loci then process each sample as it ends
	#print "CNVs sorted, now examining each loci\n";
	$tracker = 0;
	foreach (@CNV){
		@before = split ("\t", $CNV[$tracker - 1]);
		@now = split ("\t", $CNV[$tracker]);
		@after = split ("\t", $CNV[$tracker + 1]);
		$tracker++;
		
		#print "\nNew CNV:\n";
		#print "Before: @before\n";
		#print "Now: @now\n";
		#print "After: @after\n";
		
		push (@loci, $_);
		#print "Now: $now[$lstart], After: $after[$lstart]\n";
		
		if ($now[$sampleID] ne $after[$sampleID]
		or $now[$chromo] ne $after[$chromo]
		or $now[$lstart] != $after[$lstart]
		or $after[$sampleID] eq ""){
			#print "Now: $now[$sampleID], After: $after[$sampleID]\n";
			#print "This is the next loci:\n";
			&lociprosi;
			$locicount++;
			@loci = ();
		}
		
		if ($now[$sampleID] ne $after[$sampleID]){
			$totalloci{$now[$sampleID]} = $totalloci;
			$xtotalloci{$now[$sampleID]} = $xtotalloci;
			$samploci1{$now[$sampleID]} = $samploci1count;
			$samploci2{$now[$sampleID]} = $samploci2count;
			$samploci3{$now[$sampleID]} = $samploci3count;
			$samploci1count = 0;
			$samploci2count = 0;
			$samploci3count = 0;
			$totalloci = 0;
			$xtotalloci = 0;
		}
	}
	
	sub lociprosi{
		$locicountx++;
		$three = 0;
		$two = 0;
		$one = 0;
		%maxalgo = ();
		
		foreach $line (@loci){
			@tabs = split ("\t", $line);
			
			if ($tabs[$algonum] == 3){
				$three = $three + $tabs[$size];
				if ($firstthree == 0){
					$threestart = $tabs[$start];
					$firstthree = 1;
				}
				$threeend = $tabs[$stop];
			}
			if ($tabs[$algonum] == 2){
				$two = $two + $tabs[$size];
			}
			if ($tabs[$algonum] == 1){
				$one = $one + $tabs[$size];
			}
			$maxalgo{$tabs[$algonum]} = $tabs[$algos];
		}
		
		@tab = split ("\t", $loci[0]);
		
		#Percent rare1, de novo and number of algos
		
		if ($tab[$lsize] > 0){
			$pthree = sprintf ("%0d", $three / $tab[$lsize] * 100);
			$ptwo = sprintf ("%0d", $two / $tab[$lsize] * 100);
			$pone = sprintf ("%0d", $one / $tab[$lsize] * 100);
		}
		
		@algo = keys %maxalgo;
		@algo = sort {$a <=> $b} @algo;
		$maxal = $algo[-1];
		$maxalgs = $maxalgo{$maxal};
		
		#These counters are used to work out the number of loci with different numbers of algorithms
		if ($tab[$chromo] =~ /x/i){
			$xtotalloci++;
		}
		$totalloci++;
		if ($maxal == 1){
			$samploci1count++;
		}
		if ($maxal == 2){
			$samploci2count++;
		}
		if ($maxal >= 3){
			$samploci3count++;
		}
		
		print OUT "$tab[$famID]\t$tab[$sampleID]\t$tab[$chromo]\t$tab[$lstart]\t$tab[$lstop]\t$tab[$cnvtype]\t$tab[$maxsize]\t$tab[$maxsnp]\t$tab[$lsize]\t$maxal\t$maxalgs\t$pthree%\t$ptwo%\t$pone%\n";
		if ($javamethod == 0){print "\rChecking loci number $locicountx";}
	}
	print "\n";
}

#This subroutine finds the SampleID, Gender and chip type of every FinalReport present, the results are printed out in a file called Name_ID_sex.txt
sub sexsiftroutine{
	$chroX = 'X';
	&fr_columns;
	#Find filenames and IDs
	unless (-e "Name_ID_sex.txt" or -e "Name_ID_sex_1.txt"){
		$outputname = "Name_ID_sex.txt";
		open (OUT, ">$outputname") || die "Could not open the results file $outputname, may already be open\n";
	}
	print "Finding Sample IDs and gender\n";
	@files = @_;
	foreach $file (@files){
		if ($file =~ /$namematch/){
			$filecount++;
			$het=0;
			$homo=0;
			$nc = 0;
			$goodcall = 0;
			print "$filecount) ";
			open (NEWFILE, "<$file" || die "unable to open $file\n");
			@newie = <NEWFILE>;
			
			@chips3 = split ("\t", $newie[3]);
			$chip3 = $chips3[2];
	
			@temp=split("\t", $newie[4]);
			$probes = $temp[1];
			$chip2 = &chiper($temp[1]);
			
			@temp2=split("\t", $newie[12]);
			$guy = $temp2[$patname];
			
			chomp $probes;
			chomp $chip3;
			chomp $chip2;
			#print "$probes\t$chip3\t$chip2\n";
	
			foreach $line (@newie){
				#print "$line\n";
				if ($line =~ /$chroX/){
				#print "$line\n";
				@temp = split("\t", $line);
					if($temp[$gtcol1] =~ /-/) {}
					else{
						if ($temp[$gtcol1] eq $temp[$gtcol2]){$homo++}
						else {$het++}
					}
				}
			}
			if(($homo+$het)>0){
				$homop = $homo / ($homo+$het)*100;
				$hetp = $het / ($homo+$het)*100;
				$hetout = sprintf ("%02d", $hetp);
		}
			if ($hetp > 10){
				$sex = 'Female';
				$sexn = 2;
				$femcount++;
			}
			else{
				$sex = 'Male';
				$sexn = 1;
				$mancount++;
			}
			$sexlog{$guy} = $sex;
			$chiplog{$chip2} = 0;
			print "Het: $het\tHomo: $homo\tHet%: $hetout\tSex: $sex\tChip: $chip2\n";
			print OUT "$file\t$guy\t0\t0\t$sexn\t0\t$sex\t$hetp\t$chip3\t$probes\t$chip2\n";
		}
	}
	close OUT;
	@samplespresent = keys %sexlog;
	$totalsamp = @samplespresent;
	print LOG1 "$timeanddate\tSexsift\tINFO\t$filecount FinalReport files analysed from $totalsamp individuals, $mancount male and $femcount female\n";
	print "$filecount FinalReport files analysed from $totalsamp individuals, $mancount male and $femcount female\n";
	@chiplog = keys %chiplog;
	$chiplog = @chiplog;
	if($chiplog > 1){
		print "ERROR: Multiple chip types present. Check Name_Sex_ID.txt.\n";
		print LOG1 "$timeanddate\tSexsift\tERROR\tMultiple chip types present, Check Name_Sex_ID.txt:\t";
		print "Chips present: ";
		foreach (@chiplog){
			print "$_\t";
			print LOG1 "$_\t";
		}
		print "\n";
		print LOG1 "\n";
		if ($method == 0){
			print "\nDo you want to continue with multiple chip types (not recommended)? (\'y\' or \'n\')      ";
			$user_refer = <STDIN>; chomp $user_refer;
			if ($user_refer =~ /n/i){
				print LOG1 "$timedate\tSexsift\tERROR\t$progname will now end\n";
				die "Program terminating due to multiple chip types\n";
			}
		}
	}
}

sub uniq {
	my %h;
	return grep { !$h{$_}++ } @_;
}

sub merge2{
	#Two columns have been added so numbers are corrected
	$unique=0;
	$algo=1;
	$famID=2;
	$sampleID=3;
	$chromo=4;
	$start=5;
	$stop=6;
	$cnvtype=7;
	$score=8;
	$numsnps=9;
	$size=10;
	
	#I then need to sort the CNVs by sample, chr, start
	#print "Sorting CNVs\n";
	
	#Sort by Start (numbers)
	@CNVall = sort { (split '\t', $a)[$start] <=> (split '\t', $b)[$start] } @CNVall;
	
	#Sort by Chromosome (string)
	@CNVall = sort { (split '\t', lc $a)[$chromo] cmp (split '\t', lc $b)[$chromo] } @CNVall;
	
	#Sort by SampleID (string)
	@CNVall = sort { (split '\t', lc $a)[$sampleID] cmp (split '\t', lc $b)[$sampleID] } @CNVall;
	
	
	#print "@CNVall";
	
	#print "Looking for CNVs that match\n";
	
	$error = 0;
	$track = 0;
	
	#Does each CNV line up with the one below?
	foreach $line (@CNVall){
			@before = split ("\t", $CNVall[$track - 1]);
			@now = split ("\t", $CNVall[$track]);
			@after = split ("\t",$CNVall[$track + 1]);
			$track++;
			
			#print "\n\nNew CNV:\n";
			#print "Before: @before\n";
			#print "Now: @now\n";
			#print "After: @after\n";
			
		#If this CNV is from a differet sample or chromosome or if it does not overlap with the last CNV or maxbase then start a new set
		if ($now[$sampleID] ne $before[$sampleID]
		or lc $now[$chromo] ne lc $before[$chromo]
		or ($now[$start] > $before[$stop]
		and $now[$start] > $maxbase)){
			$maxbase = 0;
			@match = ();
			#print "New sample, chromosome or set\n";
			push (@match, $line);
		}
		
		#If it is the same sample and chromosome and this CNV overlaps with the last or with maxbase
		elsif ($now[$sampleID] eq $before[$sampleID] 
		and lc $now[$chromo] eq lc $before[$chromo]
		and ($now[$start] <= $before[$stop]
		or $now[$start] <= $maxbase)){
			
			#All matches get put into an array called @match
			push (@match, $line);
			#print "This was a match\n";
		}
		
		#If the CNV does not fit into either of the above groups then there is an error
		else{
			print "Error with the following CNV, its status could not be determined:\n$line\n";
			$error++;
		}
		
		#If this CNV does not overlap with the next CNV then @match is processed
		if ($now[$sampleID] ne $after[$sampleID]
		or lc $now[$chromo] ne lc $after[$chromo]
		or ($now[$stop] < $after[$start]
		and $maxbase < $after[$start])
		or $after[$start] == ""){
			#Process @match
			#print "\nMatch\n";
			#print @match;
			&setexamine53;
			$locicounter++;
			$samplocicount++;
			#print "Last match in a set, dealing with matches\n@match\n";
			}
		
		if ($now[$sampleID] ne $after[$sampleID]){
			$samploci{$now[$sampleID]} = $samplocicount;
			$samploci2{$now[$sampleID]} = $samploci2count;
			$samploci3{$now[$sampleID]} = $samploci3count;
			$samplocicount = 0;
			$samploci2count = 0;
			$samploci3count = 0;
		}
		
		#Makes Maxbase the highest it can be
		if ($now[$stop] > $maxbase){
			$maxbase = $now[$stop];
		}
		
	}
	
	#print "$error errors found\n";
	
	
	#print "Merge complete\n";
	
	sub setexamine53{
		
		$matches = @match;
		
		#If there is only one CNV it is simply printed out
		if ($matches == 1){
			#print "Single entry\n";
			$now[$chromo] =~ s/chr//i;
			push (@mergeout, "$now[$famID]\t$now[$sampleID]\tchr$now[$chromo]\t$now[$start]\t$now[$stop]\t$now[$cnvtype]\t$now[$size]\t$now[$numsnps]\t$now[$size]\t\t$now[$start]\t$now[$stop]\t$now[$size]\t$now[$cnvtype]\t1\t$now[$algo]\t\t$now[$algo]\t$now[$cnvtype]\t$now[$numsnps]\t$now[$size]\t$now[$unique]\n");
			@match = ();
		}
		
		%coord = ();
		%locitype = ();
		%sizes = ();
		%snps = ();
		$counter = 0;
		
		#Put the start and stop positions into a new array and determine the characteristics of this group of matches
		foreach (@match){
			@temp = split ("\t", $_);
			$temp[$start] =~ s/^\s+//;
			$temp[$stop] =~ s/^\s+//;
			chomp $temp[$start];
			chomp $temp[$stop];
			$coord{$temp[$start]} = 0;
			$coord{$temp[$stop]} = 0;
			$locitype{$temp[$cnvtype]} = 0;
			$sizes{$temp[$size]} = 0;
			$snps{$temp[$numsnps]} = 0;
			$family = $temp[$famID];
			$sample = $temp[$sampleID];
			$chrm = $temp[$chromo];
		}
		
		$chrm =~ s/chr//i;
		
		#Sort and determine the loci start and end
		@coord = keys %coord;
		@coord = sort {$a <=> $b} @coord;
		$locistart = $coord[0];
		$locistop = $coord[$#coord];
		$locisize = $locistop - $locistart;
		
		#Sort and determine the loci types present
		@locitypes = keys %locitype;
		@locitypes = sort {$a <=> $b} @locitypes;
		$locitypes = $types = join (", ", @locitypes);
		
		
		#Sort and determine the largest sized call
		@sizes = keys %sizes;
		@sizes = sort {$a <=> $b} @sizes;
		$maxsize = $sizes[-1];
		
		#Sort and determine the call with the most snps
		@snps = keys %snps;
		@snps = sort {$a <=> $b} @snps;
		$maxsnp = $snps[-1];
		
		#print "Within this set: Start = $locistart, End = $locistop, Types = $locitypes, Maxsize = $maxsize, MaxSNP = $maxsnp\n";
		
		$algocount = 0;
		%algos = ();
		%uid = ();
		%type = ();
		
		############################################################################################################################
		
		#Look for matches at each line of the co-ord
		foreach $bp (@coord){
			$lastone = $coord[$counter - 1];
			$thisone = $coord[$counter];
			$nextone = $coord[$counter + 1];
			
			$counter++;
			
			##########################################################################################################################
			
			#For each coord look for matches in the start and stop coords
			foreach $line4 (@match){
				@cnvm = split ("\t", $line4);
				#If the coord matches the start of a cnv the uid and features are added to hashes as the key and value
				if($thisone == $cnvm[$start]){
					$algocount++;
					chomp $cnvm[$unique];
					$uid{$cnvm[$unique]} = "$cnvm[$algo]\t$cnvm[$cnvtype]\t$cnvm[$numsnps]\t$cnvm[$size]\t$cnvm[$unique]";
				}
				
				#If the coord matches the end of a CNV the uid and algo are removed from the hashes
				if ($thisone == $cnvm[$stop]){
					$algocount = $algocount - 1;
					chomp $cnvm[$unique];
					delete ($uid{$cnvm[$unique]});
				}
			}
			
			%type = ();
			%algoname = ();
			@features = ();
			
			@uidkeys = keys %uid;
			
			foreach $key (@uidkeys){
				$temp = $uid{$key};
				chomp $temp;
				push (@features, $temp);
				@bits = split ("\t", $temp);
				$type{$bits[1]} = 1;
				$algoname{$bits[0]} = 1;
			}
			
			#Work out the size of this section
			$musize = $nextone - $thisone;
			
			#Work out the types present
			@type = keys %type; 
			chomp @type;
			@type = sort @type;
			$types = join (", ", @type);
			
			#Work out the algorithms present
			@algoname = keys %algoname;
			@algoname = sort @algoname;
			$algonames = join (", ", @algoname);
			$algonamesize = @algoname;
			if ($algonamesize > 1){
				foreach $key (@uidkeys){
					$uidmatch{$key} = 1;
				}
			}
			
			#Printout this section of the match
			$printout1 = "$family\t$sample\tchr$chrm\t$locistart\t$locistop\t$locitypes\t$maxsize\t$maxsnp\t$locisize\t\t$thisone\t$nextone\t$musize\t$types\t$algocount\t$algonames\t\t";
			#print "Print\n$printout1\n";
			chomp @features;
			$printout2 = join ("\t", @features);
			unless ($nextone == ""){
				push (@mergeout, "$printout1$printout2\n");
				#print "This is the output: $printout1$printout2\n";
			}
		}
	}
}

sub checkmatch {
	@match = @_;
	$matchsize = @match;
	@starts = (); @stops = (); @thres = (); @types = (); $typehash = (); $realsize = 0;
	unless ($matchsize == 0){
		foreach $cnv (@match){
			#print "CNV: $cnv\n";
			@tab = split (/\t/,$cnv);
			$realsize = $realsize + ($tab[$stop] - $tab[$start]);
			push (@starts, $tab[$start]);
			push (@stops, $tab[$stop]);
			push (@thres, $tab[$thres]);
			@typesplit = split (/, /,$tab[$type]);
			foreach $type (@typesplit){
				$typehash{$type} = 0;
			}
			$samplehere = $tab[$sampleID];
			$famhere = $tab[$familyID];
			$chrhere = $tab[$chromo];
		}
		@starts = sort  { $a <=> $b } @starts;
		@stops = sort  { $a <=> $b } @stops;
		@thres = sort  { $a <=> $b } @thres;
		@typeout = keys %typehash;
		@typeout = sort  { $a <=> $b } @typeout;
		$typeout = join (", ",@typeout);
		$maxthres = $thres[-1];
		$sizehere = $stops[-1] - $starts[0];
		$inputnum = @starts;
		if ($sizehere > 0){
			$perreal = $realsize / $sizehere * 100;
		}
		else {
			$perreal = 0;
		}
		$output = "$famhere\t$samplehere\t$chrhere\t$starts[0]\t$stops[-1]\t$typeout\t$maxthres\t$sizehere\t$perreal%\t$inputnum";
		#print "$output\n";
		if ($sizehere >= $threshold and $maxthres >= $thresthres){
			return $output;
		}
	}
}

#Subroutine used to convert a frequency file into merge format
sub reformat{
	#Puts the frequency file into the Merge input format
	@mergein = ();
	print "Reformatting frequency output\n";
	$chrcol = 0;
	$startcol = 1;
	$stopcol = 2;
	$callcol = 3;
	$freqcol = 4;
	
	foreach (@_){
		@tmp = split ("\t", $_);
		$size = $tmp[$stopcol] - $tmp[$startcol];
		$reformat = "NA1000\tBLANK10\t$tmp[$chrcol]\t$tmp[$startcol]\t$tmp[$stopcol]\t$tmp[$callcol]\t0\t$tmp[$freqcol]\t$size\n";
		push (@mergein, $reformat);
	}
}

#Rounds a number to the nearest integer
sub round {
    my($number) = shift;
    return int($number + .5);
}

#Gives functions the order of FinalReport columns. In theory this allows FR format to be changed. This has not been tested...
sub fr_columns{
	$patname    = 0;
	$snpcol     = 1;
	$chromocol  = 2;
	$chromodist = 3;
	$gtcol1     = 4;
	$gtcol2     = 5;
	$theta      = 6;
	$rvalue     = 7;
	$ballele    = 8;
	$log2col    = 9;
}

sub complement {
	$comp{A} = T;
	$comp{T} = A;
	$comp{G} = C;
	$comp{C} = G;
}

sub findthesex{
	#Makes a hash called sexysex with sample IDs as the keys and gender (Male/Female) as the value using the Name_ID_sex file
	
	#Columns in the Name_ID_sex file
	$families = 0;
	$samples = 1;
	$fathers = 2;
	$mothers = 3;
	$gender = 4;
	$phenotype = 5;
	$writgender = 6;
	$heterozyg = 7;
	$bpmfile = 8;
	$smallchip = 9;
	
	@files = <*>;
	$input = "Name_ID_sex.txt";

	unless (-e $input){
		#print "This simply ain't gonna swing. That pesky file just ain't here.\n";
		print "The --sexsift command has not been run so there is no file to determine sex from; this is not a problem I'll run the --sexsift command now\n";
		&sexsiftroutine(@files);
	}
	
	
	open (IN, "<$input")|| die "Could not open file $input, you may need to run the --sexsift command\n";
	while (<IN>){
		@tab = split (/\t/,$_);
		$sexysex{$tab[$samples]} = $tab[$writgender];
		#$chipspresent{$tab[$smallchip]} = $tab[$writgender];
	}
	@thekeys = keys %sexysex; $thekeys = @thekeys;
	@thevalues = values %sexysex; $thevalues = @thevalues;
	print "Genders loaded from $input for $thekeys samples, checking FinalReports\n";
	#print LOG1 "$timeanddate\tGNOSIS\tINFO\tGenders loaded from $input for $thekeys samples, checking FinalReports\n";
	
	#Checks to make sure every FinalReport has been identified
	@filespresent = &fr_name;
	#print @filespresent;
	$filespresent = @filespresent;
	$errorcount = 0; $mancount = 0; $womancount = 0;
	foreach (@filespresent){
		#print "Line: $_\n";
		@tab = split (/\t/,$_);
		$samplename = $tab[1];
		if ($sexysex{$samplename} eq "Male"){$mancount++;}
		elsif ($sexysex{$samplename} eq "Female"){$womancount++;}
		else {$errorcount++;}
	}
	print "\nThere are $filespresent FinalReports present; $mancount male, $womancount female and $errorcount unknown\n";
	#print LOG1 "$timeanddate\tGNOSIS\tINFO\tThere are $filespresent FinalReports present; $mancount male, $womancount female and $errorcount unknown\n";
	
	#If there is a mismatch of the FinalReports and Name_ID_sex file then --sexsift is repeated to correct it
	if ($errorcount > 0){
		print "ERROR: FinalReports of unknown gender present, repeating --sexsift routine\n";
		print LOG1 "$timeanddate\tGNOSIS\tERROR\tFinalReports of unknown gender present, repeating --sexsift routine\n";
		@files = <*>;
		&sexsiftroutine(@files);
		open (IN, "<$input")|| die "Could not open file $input, you may need to run the --sexsift command\n";
		while (<IN>){
			@tab = split (/\t/,$_);
			$sexysex{$tab[$samples]} = $tab[$writgender];
			#$chipspresent{$tab[$smallchip]} = $tab[$writgender];
		}
		@thekeys = keys %sexysex; $thekeys = @thekeys;
		@thevalues = values %sexysex; $thevalues = @thevalues;
		print "Genders loaded from $input for $thekeys samples\n";
		while (<@filespresent>){
			@tab = split (/\t/,$_);
			$samplename = $tab[1];
			if ($sexysex{$samplename} eq "Male"){$mancount++;}
			elsif ($sexysex{$samplename} eq "Female"){$womancount++;}
			else {$errorcount++;}
		}
		print "There are $filespresent FinalReports present; $mancount male, $womancount female and $errorcount unknown\n";
		#print LOG1 "$timeanddate\tGNOSIS\tINFO\tOn repeat there are $filespresent FinalReports present; $mancount male, $womancount female and $errorcount unknown\n";
	}
	#foreach (@thekeys){
	#	print "Sample $_ has a gender of $sexysex{$_}\n";
	#}
}

sub log2 {
	my $n = shift;
	$change = (log($n)/log(2));
	return $change;
}

#Sorts by number
sub sortNumbers() {
        $b<=>$a;
}

sub tail {
	#Sort all data
	@distribution = sort {$b<=>$a} (@distribution2);

	#For the homozygote calls#
	$up = ($pvalue);
	$down = (1- $pvalue);
	$upper_bound = ($up * 2 * $#distribution);
	$lower_bound = ($down * $#distribution);
	$upper_homo_bound = &round($upper_bound);
	$lower_homo_bound = &round($lower_bound);

	#For the heterozygote calls#
	$up = ($otherp);
	$down = (1- $otherp);
	$upper_bound = ($up * 2 * $#distribution);
	$lower_bound = ($down * $#distribution);
	$upper_het_bound = &round($upper_bound);
	$lower_het_bound = &round($lower_bound);

	#For the winodw calls#
	$up = ($windowp);
	$down = (1- $windowp);
	$upper_bound = ($up * 2 * $#distribution);
	$lower_bound = ($down * $#distribution);
	$upper_win_bound = &round($upper_bound);
	$lower_win_bound = &round($lower_bound);


	$uhmb= sprintf("%0.3f", $distribution[$upper_homo_bound]);
	$uhtb= sprintf("%0.3f", $distribution[$upper_het_bound]);
	$uwinb= sprintf("%0.3f", $distribution[$upper_win_bound]);

	$lhmb= sprintf("%0.3f", $distribution[$lower_homo_bound]);
	$lhtb= sprintf("%0.3f", $distribution[$lower_het_bound]);
	$lwinb= sprintf("%0.3f", $distribution[$lower_win_bound]);

#	print "cutoffs: ($uhmb, $uhtb, $lhtb, $lhmb)...";
	
	return (
	$uhmb, $lhmb, $uhtb, $lhtb, $uwinb, $lwinb
	);
}

#Makes Affy files into FinalReports
if ($out_FinalReports == 1){
	my $samplenums = &output_FinalReports; 
	print "\n\nTotal number of Final Reports generated = $samplenums\n";
}

sub output_FinalReports{
	print "\n**********< Generating Final Report Files.. >**************\n";
	#open files needed here
	#my @sigs = split/(?:\.)/,$output;
	$baffile = $ARGV[0];
	print "NOTICE: Beginning reading from the $baffile file\n";
	my @frids; my @filehandles;
 	open(BFILE, $baffile) || die ("Cannot open $baffile\n");
	#my @bfs = <BFILE>;
	do {
		$linecount2 = 0;
		while (my $line=<BFILE>){
			if ($linecount2 == 0){
				$bafheader = $line;
			}
			$linecount2++;
		}
	} until ($linecount2 >= 1);
	#my $bafheader = shift(@bfs);
	#print "$bafheader\n";
	my @hcells = split("\t", $bafheader);
	print"Done reading header information\n";
	my $count = 0;
	for(my $i = 3; $i < @hcells; $i = $i+2){
		$count++;
	my @ids = split/(?:\.)/,$hcells[$i];
	push(@frids,$ids[0]);
	my $fname = $ids[0]."_FinalReport_$count.txt";
	push(@filehandles, $fname);
	}
	my $snps = scalar(@bfs); my $samples = scalar(@frids); my $filenum = 0;  
	my $processdate = strftime "%m-%d-%Y", localtime;
 	my %handles = get_filew_handles(@filehandles);
	for(my $i = 0; $i < scalar(@filehandles); $i++){
		$filenum++;
		print {$handles{$filehandles[$i]}} "[Header]\nBSGT Version\tNone\nProcessing Date\t$processdate\nContent\tNormalized_affy6.0\nNum SNPs\t$snps\nTotal SNPs\t$snps\nNum Samples\t$samples\nTotal Samples\t$samples\nFile\t$filenum of $samples\n[Data]\n";
		print {$handles{$filehandles[$i]}} "Sample ID\tSNP Name\tChr\tPosition\tAllele1 - Top\tAllele2 - Top\tTheta\tR\tB Allele Freq\tLog R Ratio\n";
		#print {$handles{$filehandles[$i]}} "SampleId\tSNPname\tChr\tPosition\tAllel1-Top\tAllel2-Top\tTheta\tR\tB-AllelFrequency\tLog R Ratio\n";
	}
	my $linecount = 0;
	seek (BFILE,0,0);
	while (my $bfln=<BFILE>){
	#foreach my $bfln(@bfs){
		chomp($bfln);
		$linecount++;
		if ($linecount > 1){
			$thistarget++;
			if ($thistarget > 987){
				print  "processing baf file line:$linecount\r";
				$thistarget = 0;
			}
			my @tabs = split("\t", $bfln);
			my @sliceone = splice(@tabs,0,3);
			my $idx1 = 0; my $idx2 = 1; my $R = 0; my $Theta = 0;
			for(my $i = 0; $i < scalar(@filehandles); $i++){
				print {$handles{$filehandles[$i]}} $frids[$i]."\t".$sliceone[0]."\t".$sliceone[1]."\t".$sliceone[2]."\t";
				my ($GT1, $GT2, $Baf) = &get_genotypes($tabs[$idx2]);
				$Baf = sprintf("%.4f", $Baf);
				print {$handles{$filehandles[$i]}} "$GT1\t$GT2\t$Theta\t$R\t$Baf\t$tabs[$idx1]\n"; 
				$idx1 += 2; $idx2 += 2;
			}
		}
	}
	return $samples;
}

sub get_genotypes(){
	my($bafreq) = @_;
	my $gt1; my $gt2; my $baf;
	if(($bafreq >= 0.4) && ($bafreq <= 0.6)){$gt1 = 'A'; $gt2 ='T';$baf = $bafreq;}
	elsif(($bafreq >= 0.9) && ($bafreq <= 1)){$gt1 = 'T'; $gt2 = 'T';$baf = $bafreq;}
	elsif($bafreq < 0.1){$gt1 = 'A'; $gt2 = 'A'; $baf = $bafreq;}
	elsif($bafreq == 2){$gt1 = '-'; $gt2 = '-'; $baf = 0;}
	else {$gt1 = '-'; $gt2 = '-'; $baf = $bafreq;}
	return($gt1, $gt2, $baf);
}

sub get_filew_handles {
  my @file_names = @_;
  my %file_handles;
  foreach (@file_names) {
    open my $fh, '>', $_ or next;
    $file_handles{$_} = $fh;
  }
  return %file_handles;
}

sub parent_origin_sub {
	@inputparent = @_;
	$pro = 1;
	$fat = 2;
	$mot = 3;

	$linecount = 0; $dupcount = 0; $delcount = 0; $hetcount = 0; $othercount = 0; $dupmot = 0; $dupfat = 0; $delmot = 0; $delfat = 0; $confidence = 0;
	foreach $line (@inputparent){
		$linecount++;
		@tab = split (/\t| /, $line);
		#print "$line\t$tab[$pro]\n";
		#Duplication (AAB)
		if ($tab[$pro] > 0.25 and $tab[$pro] < 0.4){
			$dupcount++;
			#Informative and father gave the 'B' and mother gave the 'A'
			if ($tab[$fat] > 0.9 and $tab[$mot] < 0.6){
				$dupmot++;
			}
			#Informative and mother gave the 'B' and father gave the 'A'
			if ($tab[$mot] > 0.9 and $tab[$fat] < 0.6){
				$dupfat++;
			}
		}
		#Duplication (ABB)
		elsif ($tab[$pro] > 0.6 and $tab[$pro] < 0.75){
			$dupcount++;
			#Informative and father gave the 'A' and mother gave the 'B'
			if ($tab[$fat] < 0.1 and $tab[$mot] > 0.4){
				$dupmot++;
			}
			#Informative and mother gave the 'A' and father gave the 'B'
			if ($tab[$mot] < 0.1 and $tab[$fat] > 0.4){
				$dupfat++;
			}
		}
		#Deletion (A)
		elsif ($tab[$pro] < 0.1){
			$delcount++;
			#Informative and mother gave the 'A' and father gave the 'missing B'
			if ($tab[$mot] < 0.6 and $tab[$fat] > 0.9){
				$delfat++;
			}
			#Informative and father gave the 'A' and mother gave the 'missing B'
			if ($tab[$fat] < 0.6 and $tab[$mot] > 0.9){
				$delmot++;
			}
		}
		#Deletion (B)
		elsif ($tab[$pro] > 0.9){
			$delcount++;
			#Informative and mother gave the 'missing A' and father gave the 'B'
			if ($tab[$mot] < 0.1 and $tab[$fat] > 0.4){
				$delmot++;
			}
			#Informative and father gave the 'missing A' and mother gave the 'B'
			if ($tab[$fat] < 0.1 and $tab[$mot] > 0.4){
				$delfat++;
			}
		}
		elsif ($tab[$pro] > 0.4 and $tab[$pro] < 0.6){
			$hetcount++;
		}
		else {
			$othercount++;
		}
	}
	$deldup = eval {$delcount/$dupcount} + 0;
	$hetdel = eval {$hetcount/$delcount} + 0;
	if ($deldup > 0 and $deldup < 20){
		$type = "Duplication";
		if ($dupfat > $dupmot){
			$parent = "Father";
			$confidence = ($dupfat - $dupmot);
		}
		elsif ($dupmot > $dupfat){
			$parent = "Mother";
			$confidence = ($dupmot - $dupfat);
		}
		else {
			$parent = "???";
		}
	}
	elsif ($hetdel < 0.1){
		$type = "Deletion";
		if ($delfat > $delmot){
			$parent = "Father";
			$confidence = ($delfat - $delmot);
		}
		elsif ($delmot > $delfat){
			$parent = "Mother";
			$confidence = ($delmot - $delfat);
		}
		else {
			$parent = "???";
		}
	}
	else {
		$type = "???";
		$parent = "???";
	}
	$result = $linecount."\t".$dupcount."\t".$delcount."\t".$hetcount."\t".$othercount."\t".$dupmot."\t".$dupfat."\t".$delmot."\t".$delfat."\t".$type."\t".$parent."\t".$confidence;

	return ($result);
}
 
#unless ($setmeup == 1 or $help == 1){
#	$visor = $JVloc."CNVision.jar";
#	if (-e $visor){
#		unlink $jvlocfile;
#	}
#}

if ($help == 1){
  print "\nTry entering the following commands:\n";
  print "CNV Pipeline:\n";
  print "--start -------> Makes a batch file to run CNV algorithms\n"; 
  print "--sexsift -----> Looks for \'FinalReport\' and determines sex\n";
  print "--chiperror ---> Looks for extreme logR values in FinalReports\n";
  print "--merge -------> Merges files in format Family/Sample/Chr/Start/Stop/Type/Score\n";
  print "                 /SNPs/Size; list files for merging after the command\n";
  print "--rare --------> Compares Merged output to a list of loci; list Merged output\n";
  print "                 then the list of loci after the command\n";
  print "--denovo ------> Compares children to parents using a pedfile (with \'pedfile\' in";
  print "                 the filename); list the Merged or Rare output file after the\n";
  print "                 command\n";
  print "--denovo2 -----> Uses the FinalReports and a pedfile to work out whether a list\n";
  print "                 CNVs are de novo or inherited; list the file with the CNVs\n";
  print "--inquire -----> Analyses the denovo output and makes a BED UCSC input; requires";
  print "                 a list of badchips (Badchips.txt) and a pedfile (with \'pedfile\'";
  print "                 in the filename); list the Denovo output file\n";
  print "--annotate ----> Annotates which for genes, common variants and regions of\n";
  print "                 interest; list the file of CNV regions\n";
  
  print "\nCNV analysis:\n";
  print "--homodel -----> Looks for \'FinalReport\' and finds homozygous deletions\n";
  print "--bigcnv ------> Not done yet\n";
  print "--homozyg -----> Looks for \'FinalReport\' and shows the %homozygosity per\n";
  print "                 chromosome; useful for detecting uniparental disomy\n";
  print "--freq --------> Determines the frequency of different loci; list the\n";
  print "                 the Merged output file after the command\n";
  print "--relfreq -----> Works out the relative frequency of CNVs from two files; list\n";
  print "                 the Merged output after the command\n";
  print "--snpcount ----> Works out the number of SNPs in regions; list the file of \n";
  print "                 regions; needs a FinalReport file for the chip in the folder\n";
  
  print "\nFinalReport Manipulation:\n";
  print "--fr_name -----> Quickly finds the sample ID and chip type from FinalReports\n";
	print "--fr_consensus > Makes a list of the SNPs common to different chip types in\n";
  print "                 FinalReports; put in a folder with an example of each chip\n";
  print "--fr_prune ----> Removes SNPs that are not present in the consensus list from\n";
  print "                 --fr_consensus out of FinalReports\n";
  
  print "\nFile Management:\n";
  print "--filelist ----> Makes a list of every file in the folder called \'File_list.txt\'";
  print "--movefiles ---> Moves files in a text file to a new folder; list the text file\n";
  print "--copyfiles ---> Copies files in a text file to a new folder; list the text file";
  print "--vlookup -----> Looks up terms in a reference file to add a new column; list\n";
  print "                 original file then reference file\n";
  print "--split -------> Looks for differences in a specified column and splits the file";
  print "                 accordingly\n";
  print "--combo -------> Combines files that have columns in the same order; list the\n";
  print "                 files to be combined\n";
  print "--sorter ------> Sorts the file by up to three columns; list the file to be \n";
  print "                 sorted\n";
  print "--testfile ----> Makes a file that shows the first few lines of another file to\n";
  print "                 use as a test; list the file to make the testfile from\n";

  print "\nOther tools:\n";
  print "--loci --------> Takes a list of Chr/Start/End and reduces it to non-overlapping";
  print "                 loci; list the input file after the command\n";
	print "--snp_position > Looks up the chr and position of each SNP ID; list the file\n";
  print "                 with the list SNP names\n";
  print "--gene_position> Looks up the chr and position of each UCSC Gene; list the \n";
  print "                 file with the list Gene names\n";
  print "--plot --------> Uses the FinalReports and a pedfile to create a data file and\n";
  print "                 R script to visualise CNVs; list the CNV file then copy the\n";
  print "                 text file in the \'Plot\' folder into R\n";

}







